@@ -20,7 +20,6 @@ import net.sf.saxon.expr.parser.Location;
 import net.sf.saxon.expr.parser.PathMap;
 import net.sf.saxon.expr.sort.GroupIterator;
 import net.sf.saxon.functions.AccessorFn;
-import net.sf.saxon.functions.IriToUri;
 import net.sf.saxon.lib.*;
 import net.sf.saxon.om.*;
 import net.sf.saxon.regex.RegexIterator;
@@ -37,13 +36,13 @@ import net.sf.saxon.type.Type;
 import net.sf.saxon.type.Untyped;
 import net.sf.saxon.value.DateTimeValue;
 import net.sf.saxon.value.SequenceType;
+import net.sf.saxon.z.IntHashMap;
 
 import javax.xml.transform.ErrorListener;
 import javax.xml.transform.Source;
 import javax.xml.transform.Transformer;
 import javax.xml.transform.URIResolver;
 import javax.xml.transform.sax.SAXSource;
-import java.io.File;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.HashMap;
@@ -87,7 +86,7 @@ public class Controller implements ContextOriginator {
     private Configuration config;
     protected Executable executable;
 
-    protected Item globalContextItem;
+    protected Item<?> globalContextItem;
     private boolean globalContextItemPreset;
     private Map<PackageData, Bindery> binderies;
     private GlobalParameterSet globalParameters;
@@ -100,12 +99,12 @@ public class Controller implements ContextOriginator {
     private URIResolver userURIResolver;
     protected Receiver principalResult;
     protected String principalResultURI;
-    private String cookedPrincipalResultURI;
     private UnparsedTextURIResolver unparsedTextResolver;
     private String defaultCollectionURI;
     protected UnfailingErrorListener errorListener;
     private TreeModel treeModel = TreeModel.TINY_TREE;
     private DocumentPool sourceDocumentPool;
+    private IntHashMap<Map<Long, KeyIndex>> localIndexes;
     private HashMap<String, Object> userDataTable;
     private NodeInfo lastRememberedNode = null;
     private int lastRememberedNumber = -1;
@@ -121,8 +120,9 @@ public class Controller implements ContextOriginator {
     public final static String ANONYMOUS_PRINCIPAL_OUTPUT_URI = "dummy:/anonymous/principal/result";
     private StylesheetCache stylesheetCache = null;
 
-    private java.util.function.Function<SequenceIterator, FocusIterator> focusTrackerFactory =
-            FocusTrackingIterator::new;
+    private Function<SequenceIterator<?>, FocusTrackingIterator<?>> focusTrackerFactory = FocusTrackingIterator::new;
+
+    private Function<SequenceIterator<?>, FocusTrackingIterator<?>> multiThreadedFocusTrackerFactory;
 
     /**
      * Create a Controller and initialise variables. Note: XSLT applications should
@@ -177,6 +177,8 @@ public class Controller implements ContextOriginator {
 
     public void reset() {
         globalParameters = new GlobalParameterSet();
+        focusTrackerFactory = config.getFocusTrackerFactory(executable, false);
+        multiThreadedFocusTrackerFactory = config.getFocusTrackerFactory(executable, true);
         standardURIResolver = config.getSystemURIResolver();
         userURIResolver = config.getURIResolver();
         unparsedTextResolver = config.getUnparsedTextURIResolver();
@@ -223,6 +225,7 @@ public class Controller implements ContextOriginator {
         lastRememberedNode = null;
         lastRememberedNumber = -1;
         stylesheetCache = null;
+        localIndexes = null;
         if (!globalContextItemPreset) {
             globalContextItem = null;
         }
@@ -248,7 +251,7 @@ public class Controller implements ContextOriginator {
      * even if there is a default defined in the stylesheet or query.
      */
 
-    public Sequence getParameter(StructuredQName name) {
+    public Sequence<?> getParameter(StructuredQName name) {
         return globalParameters.get(name);
     }
 
@@ -264,9 +267,10 @@ public class Controller implements ContextOriginator {
      * against the required type.
      */
 
-    public GroundedValue getConvertedParameter(StructuredQName name, SequenceType requiredType, XPathContext context)
+    public GroundedValue<?> getConvertedParameter(StructuredQName name, SequenceType requiredType, XPathContext context)
             throws XPathException {
-        GroundedValue val = globalParameters.convertParameterValue(name, requiredType, convertParameters, context);
+        GroundedValue<?> val =
+                globalParameters.convertParameterValue(name, requiredType, convertParameters, context);
         if (val != null) {
 
             // Check that any nodes belong to the right configuration
@@ -338,31 +342,6 @@ public class Controller implements ContextOriginator {
         return principalResultURI;
     }
 
-    /**
-     * Get the base output URI after processing. The processing consists of (a) defaulting
-     * to the current user directory if no base URI is available and if the stylesheet is trusted,
-     * and (b) applying IRI-to-URI escaping
-     *
-     * @return the base output URI after processing.
-     */
-
-    /*@Nullable*/
-    public String getCookedBaseOutputURI() {     // TODO: no longer used
-        if (cookedPrincipalResultURI == null) {
-            String base = getBaseOutputURI();
-            if (base == null && config.getBooleanProperty(Feature.ALLOW_EXTERNAL_FUNCTIONS)) {
-                // if calling external functions is allowed, then the stylesheet is trusted, so
-                // we allow it to write to files relative to the current directory
-                base = new File(System.getProperty("user.dir")).toURI().toString();
-            }
-            if (base != null) {
-                base = IriToUri.iriToUri(base).toString();
-            }
-            cookedPrincipalResultURI = base;
-        }
-        return cookedPrincipalResultURI;
-    }
-
     /**
      * Get the principal result destination.
      * <p>This method is intended for internal use only. It is typically called by Saxon during the course
@@ -390,17 +369,6 @@ public class Controller implements ContextOriginator {
         return new SequenceOutputter(pipe, size);
     }
 
-    /**
-     * Accept a SequenceOutputter that is now available for reuse
-     *
-     * @param out the SequenceOutputter that is available for reuse
-     */
-
-    public void reuseSequenceOutputter(SequenceOutputter out) {
-        SequenceOutputter reusableSequenceOutputter = out;
-    }
-
-
     ///////////////////////////////////////////////////////////////////////////////
 
     /**
@@ -568,7 +536,7 @@ public class Controller implements ContextOriginator {
      * @return the Bindery (in which values of all variables for the requested package are held)
      */
 
-    public Bindery getBindery(PackageData packageData) {
+    public synchronized Bindery getBindery(PackageData packageData) {
         Bindery b = binderies.get(packageData);
         if (b == null) {
             b = new Bindery(packageData);
@@ -649,7 +617,7 @@ public class Controller implements ContextOriginator {
      */
 
     /*@Nullable*/
-    public Item getGlobalContextItem() {
+    public Item<?> getGlobalContextItem() {
         return globalContextItem;
         // See W3C bug 5224, which points out that the rules for XQuery 1.0 weren't clearly defined
     }
@@ -1168,7 +1136,8 @@ public class Controller implements ContextOriginator {
      * users (it is done automatically when transform() is invoked). However, it is available as a low-level API
      * especially for use with XQuery.
      *
-     * @param params the values of stylesheet parameters. This must include static parameters as well as non-static parameters.
+     * @param params the values of stylesheet parameters. Changed in 9.9.0.2 so this no longer includes
+     *               static parameters (which are already available in the {@link PreparedStylesheet}).
      * @throws XPathException if an error occurs, for example if a required parameter is not supplied.
      */
 
@@ -1182,7 +1151,7 @@ public class Controller implements ContextOriginator {
         // if parameters were supplied, set them up
 
         try {
-            executable.checkAllRequiredParamsArePresent(params);
+            executable.checkSuppliedParameters(params);
         } catch (XPathException e) {
             if (!e.hasBeenReported()) {
                 getErrorListener().fatalError(e);
@@ -1226,7 +1195,7 @@ public class Controller implements ContextOriginator {
      * @return the value of the required property
      */
 
-    public Object getUserData(Object key, String name) {
+    public synchronized Object getUserData(Object key, String name) {
         String keyValue = key.hashCode() + " " + name;
         // System.err.println("getUserData " + name + " on object returning " + userDataTable.get(key));
         return userDataTable.get(keyValue);
@@ -1247,7 +1216,7 @@ public class Controller implements ContextOriginator {
      *             for the key is removed.
      */
 
-    public void setUserData(Object key, String name, /*@Nullable*/ Object data) {
+    public synchronized void setUserData(Object key, String name, /*@Nullable*/ Object data) {
         // System.err.println("setUserData " + name + " on object to " + data);
         String keyVal = key.hashCode() + " " + name;
         if (data == null) {
@@ -1257,6 +1226,24 @@ public class Controller implements ContextOriginator {
         }
     }
 
+    /**
+     * Get the table of local indexes supporting xsl:key (or implicit keys created
+     * by the optimizer). Indexes are held at Controller level (rather than being
+     * shared across transformations) if the key definition is dependent on local
+     * information, for example stylesheet parameters.
+     * @return the index of indexes. The master index is created if it does not
+     * already exist. The master index is a two-level index: the first level is indexed
+     * by the integer fingerprint of the key name; the second level is indexed by
+     * the document number (a long) for the specific document or temporary tree.
+     */
+
+    public synchronized IntHashMap<Map<Long, KeyIndex>> getLocalIndexes() {
+        if (localIndexes == null) {
+            localIndexes = new IntHashMap<>();
+        }
+        return localIndexes;
+    }
+
     /**
      * Set the last remembered node, for node numbering purposes.
      * <p>This method is strictly for internal use only.</p>
@@ -1322,6 +1309,7 @@ public class Controller implements ContextOriginator {
             ((SAXSource) source).setXMLReader(null);
         }
         Builder sourceBuilder = makeBuilder();
+        sourceBuilder.setUseEventLocation(true);
         if (sourceBuilder instanceof TinyBuilder) {
             ((TinyBuilder) sourceBuilder).setStatistics(Statistics.SOURCE_DOCUMENT_STATISTICS);
         }
@@ -1563,25 +1551,42 @@ public class Controller implements ContextOriginator {
      * Get the factory function that is used to create new instances of FocusTrackingIterator.
      * The standard function for instantiating a FocusTrackingIterator can be overridden to deliver
      * one with extra diagnostic capability for use in debuggers
+     * @param multithreaded true if the focus tracker must be suitable for executing a multi-threaded
+     *                      xsl:for-each iteration
      * @return a factory function that is used to create FocusTrackingIterator instances
      */
 
-    public Function<SequenceIterator, FocusIterator> getFocusTrackerFactory() {
-        return focusTrackerFactory;
+    public Function<SequenceIterator<?>, FocusTrackingIterator<?>> getFocusTrackerFactory(boolean multithreaded) {
+        return multithreaded && multiThreadedFocusTrackerFactory != null ?
+                multiThreadedFocusTrackerFactory :
+                focusTrackerFactory;
     }
 
     /**
      * Set a factory function that will be used to create new instances of FocusTrackingIterator.
      * The standard function for instantiating a FocusTrackingIterator can be overridden to deliver
-     * one with extra diagnostic capability for use in debuggers
+     * one with extra diagnostic capability for use in debuggers.
      *
      * @param focusTrackerFactory a factory function that is used to create FocusTrackingIterator instances
      */
 
-    public void setFocusTrackerFactory(Function<SequenceIterator, FocusIterator> focusTrackerFactory) {
+    public void setFocusTrackerFactory(Function<SequenceIterator<?>, FocusTrackingIterator<?>> focusTrackerFactory) {
         this.focusTrackerFactory = focusTrackerFactory;
     }
 
+    /**
+     * Set a factory function that will be used to create new instances of FocusTrackingIterator for
+     * multithreaded operation.
+     * The standard function for instantiating a FocusTrackingIterator can be overridden to deliver
+     * one with extra diagnostic capability for use in debuggers.
+     *
+     * @param focusTrackerFactory a factory function that is used to create FocusTrackingIterator instances
+     */
+
+    public void setMultithreadedFocusTrackerFactory(Function<SequenceIterator<?>, FocusTrackingIterator<?>> focusTrackerFactory) {
+        this.multiThreadedFocusTrackerFactory = focusTrackerFactory;
+    }
+
     /**
      * Set the focus tracker factory function to a function that creates a memoizing focus tracker, which
      * has the effect that all items read by the focus iterator are accessible to a debugger at any stage
@@ -1590,16 +1595,19 @@ public class Controller implements ContextOriginator {
 
     public void setMemoizingFocusTrackerFactory() {
         setFocusTrackerFactory(base -> {
+            FocusTrackingIterator<?> fti;
             if ((base.getProperties() & GROUNDED) == 0 &&
                     !(base instanceof GroupIterator) && !(base instanceof RegexIterator)) {
                 try {
-                    return new FocusTrackingIterator(new MemoSequence(base).iterate());
+                    MemoSequence<?> ms = new MemoSequence<>(base);
+                    fti = FocusTrackingIterator.track(ms.iterate());
                 } catch (XPathException e) {
-                    return new FocusTrackingIterator(base);
+                    fti = FocusTrackingIterator.track(base);
                 }
             } else {
-                return new FocusTrackingIterator(base);
+                fti = FocusTrackingIterator.track(base);
             }
+            return fti;
         });
     }
 
