@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,12 +7,14 @@
 
 package net.sf.saxon;
 
+import net.sf.saxon.expr.instruct.Executable;
 import net.sf.saxon.expr.instruct.GlobalContextRequirement;
 import net.sf.saxon.expr.instruct.TerminationException;
 import net.sf.saxon.lib.*;
 import net.sf.saxon.s9api.*;
 import net.sf.saxon.style.StylesheetPackage;
 import net.sf.saxon.trace.AbstractTraceListener;
+import net.sf.saxon.trace.Instrumentation;
 import net.sf.saxon.trace.TimingCodeInjector;
 import net.sf.saxon.trace.TimingTraceListener;
 import net.sf.saxon.trans.CommandLineOptions;
@@ -39,15 +41,15 @@ import java.util.*;
 
 /**
  * This <b>Transform</b> class is the command-line entry point to the Saxon XSLT Processor.
- * <p/>
  * <p>It is possible to subclass this class to provide a customized command line interface. In writing such
  * a subclass:</p>
- * <p/>
  * <ul>
  * <li>The {@link #main} method should instantiate the class and call the {@link #doTransform} method, passing the
  * argument list. The argument list can be augmented or modified if required: for example, by adding a -config
  * argument to cause the configuration to be initialized from a configuration file.</li>
- * <li>The {@link #setFactoryConfiguration} method can be implemented
+ * <li>The {@link #initializeConfiguration} method can be implemented to customize the configuration in which
+ * the transformation will run.</li>
+ * </ul>
  */
 
 public class Transform {
@@ -64,19 +66,16 @@ public class Transform {
 
     /**
      * Main program, can be used directly from the command line.
-     * <p>The format is:</P>
-     * <p>java net.sf.saxon.Transform [options] <I>source-file</I> <I>style-file</I> &gt;<I>output-file</I></P>
+     * <p>The format is:</p>
+     * <p>java net.sf.saxon.Transform [options] <I>source-file</I> <I>style-file</I> &gt;<I>output-file</I></p>
      * <p>followed by any number of parameters in the form {keyword=value}... which can be
      * referenced from within the stylesheet.</p>
      * <p>This program applies the XSL style sheet in style-file to the source XML document in source-file.</p>
      *
      * @param args List of arguments supplied on operating system command line
-     * @throws java.lang.Exception Indicates that a compile-time or
-     *                             run-time error occurred
      */
 
-    public static void main(String args[])
-            throws java.lang.Exception {
+    public static void main(String args[]) {
         // the real work is delegated to another routine so that it can be used in a subclass
         new Transform().doTransform(args, "java net.sf.saxon.Transform");
     }
@@ -135,7 +134,7 @@ public class Transform {
         options.addRecognizedOption("o", CommandLineOptions.TYPE_FILENAME | CommandLineOptions.VALUE_REQUIRED,
                                     "Use specified file for primary output");
         options.addRecognizedOption("opt", CommandLineOptions.TYPE_STRING | CommandLineOptions.VALUE_REQUIRED,
-                                    "Enable/disable optimization options [-]cfgklmsvwx");
+                                    "Enable/disable optimization options [-]cfgjklmrsvwx");
         options.addRecognizedOption("or", CommandLineOptions.TYPE_CLASSNAME | CommandLineOptions.VALUE_REQUIRED,
                                     "Use named OutputURIResolver class");
         options.addRecognizedOption("outval", CommandLineOptions.TYPE_ENUMERATION | CommandLineOptions.VALUE_REQUIRED,
@@ -355,7 +354,7 @@ public class Transform {
             String value = options.getOptionValue("explain");
             if (value != null) {
                 explain = true;
-                processor.setConfigurationProperty(FeatureKeys.TRACE_OPTIMIZER_DECISIONS, true);
+                processor.setConfigurationProperty(Feature.TRACE_OPTIMIZER_DECISIONS, true);
                 jit = false;
                 compiler.setJustInTimeCompilation(jit);
                 if (!"".equals(value)) {
@@ -384,8 +383,12 @@ public class Transform {
             value = options.getOptionValue("jit");
             if (value != null) {
                 if ("on".equals(value) && exportOutputFileName == null && run) {
-                    //TODO the combination of the options jit, nogo and export should give an warning
-                    jit = true;
+                    if (export) {
+                        jit = false;
+                        System.err.println("Warning: -jit:on is ignored when -export:on is set");
+                    } else {
+                        jit = true;
+                    }
                 } else if ("off".equals(value)) {
                     jit = false;
                 }
@@ -396,7 +399,7 @@ public class Transform {
             value = options.getOptionValue("lib");
             if (value != null) {
                 StringTokenizer st = new StringTokenizer(value, File.pathSeparator);
-                Set<File> packs = new HashSet<File>();
+                Set<File> packs = new HashSet<>();
                 while (st.hasMoreTokens()) {
                     String n = st.nextToken();
                     packs.add(new File(n));
@@ -417,6 +420,10 @@ public class Transform {
 
             value = options.getOptionValue("nogo");
             if (value != null) {
+                if (jit) {
+                    jit = false;
+                    System.err.println("Warning: -jit:on is ignored when -nogo is set");
+                }
                 run = false;
                 compiler.setJustInTimeCompilation(false);
             }
@@ -450,7 +457,7 @@ public class Transform {
             if (value != null) {
                 System.err.println(config.getProductTitle());
                 System.err.println(Version.platform.getPlatformVersion());
-                processor.setConfigurationProperty(FeatureKeys.TIMING, true);
+                processor.setConfigurationProperty(Feature.TIMING, true);
                 showTime = true;
             }
 
@@ -461,19 +468,24 @@ public class Transform {
                 } else {
                     traceListener = config.makeTraceListener(value);
                 }
-                processor.setConfigurationProperty(FeatureKeys.TRACE_LISTENER, traceListener);
-                processor.setConfigurationProperty(FeatureKeys.LINE_NUMBERING, true);
+                processor.setConfigurationProperty(Feature.TRACE_LISTENER, traceListener);
+                processor.setConfigurationProperty(Feature.LINE_NUMBERING, true);
 
                 value = options.getOptionValue("Tlevel");
                 if (value != null && traceListener instanceof AbstractTraceListener) {
-                    if (value.equals("none")) {
-                        ((AbstractTraceListener)traceListener).setLevelOfDetail(0);
-                    } else if (value.equals("low")) {
-                        ((AbstractTraceListener) traceListener).setLevelOfDetail(1);
-                    } else if (value.equals("normal")) {
-                        ((AbstractTraceListener) traceListener).setLevelOfDetail(2);
-                    } else if (value.equals("high")) {
-                        ((AbstractTraceListener) traceListener).setLevelOfDetail(3);
+                    switch (value) {
+                        case "none":
+                            ((AbstractTraceListener) traceListener).setLevelOfDetail(0);
+                            break;
+                        case "low":
+                            ((AbstractTraceListener) traceListener).setLevelOfDetail(1);
+                            break;
+                        case "normal":
+                            ((AbstractTraceListener) traceListener).setLevelOfDetail(2);
+                            break;
+                        case "high":
+                            ((AbstractTraceListener) traceListener).setLevelOfDetail(3);
+                            break;
                     }
                 }
             }
@@ -481,16 +493,16 @@ public class Transform {
             value = options.getOptionValue("TB");
             if (value != null) {
                 // Trace hotspot byte code generation and produce a report.
-                config.setBooleanProperty(FeatureKeys.MONITOR_HOT_SPOT_BYTE_CODE, true);
+                config.setBooleanProperty(Feature.MONITOR_HOT_SPOT_BYTE_CODE, true);
             }
 
             value = options.getOptionValue("TP");
             if (value != null) {
                 traceListener = new TimingTraceListener();
-                processor.setConfigurationProperty(FeatureKeys.TRACE_LISTENER, traceListener);
-                processor.setConfigurationProperty(FeatureKeys.LINE_NUMBERING, true);
+                processor.setConfigurationProperty(Feature.TRACE_LISTENER, traceListener);
+                processor.setConfigurationProperty(Feature.LINE_NUMBERING, true);
                 compiler.getUnderlyingCompilerInfo().setCodeInjector(new TimingCodeInjector());
-                if (value.length() > 0) {
+                if (!value.isEmpty()) {
                     traceListener.setOutputDestination(
                             new StandardLogger(new File(value)));
                 }
@@ -502,15 +514,20 @@ public class Transform {
                     traceDestination = config.getLogger();
                 }
             } else {
-                if (value.equals("#err")) {
-                    traceDestination = new StandardLogger();
-                } else if (value.equals("#out")) {
-                    traceDestination = new StandardLogger(System.out);
-                } else if (value.equals("#null")) {
-                    traceDestination = null;
-                } else {
-                    traceDestination = new StandardLogger(new File(value));
-                    closeTraceDestination = true;
+                switch (value) {
+                    case "#err":
+                        traceDestination = new StandardLogger();
+                        break;
+                    case "#out":
+                        traceDestination = new StandardLogger(System.out);
+                        break;
+                    case "#null":
+                        traceDestination = null;
+                        break;
+                    default:
+                        traceDestination = new StandardLogger(new File(value));
+                        closeTraceDestination = true;
+                        break;
                 }
             }
 
@@ -522,7 +539,7 @@ public class Transform {
             value = options.getOptionValue("x");
             if (value != null) {
                 sourceParserName = value;
-                processor.setConfigurationProperty(FeatureKeys.SOURCE_PARSER_CLASS, sourceParserName);
+                processor.setConfigurationProperty(Feature.SOURCE_PARSER_CLASS, sourceParserName);
             }
 
             value = options.getOptionValue("xsd");
@@ -532,7 +549,7 @@ public class Transform {
 
             value = options.getOptionValue("xsdversion");
             if (value != null) {
-                processor.setConfigurationProperty(FeatureKeys.XSD_VERSION, value);
+                processor.setConfigurationProperty(Feature.XSD_VERSION, value);
             }
 
             value = options.getOptionValue("xsl");
@@ -543,7 +560,7 @@ public class Transform {
             value = options.getOptionValue("y");
             if (value != null) {
                 styleParserName = value;
-                processor.setConfigurationProperty(FeatureKeys.STYLE_PARSER_CLASS, value);
+                processor.setConfigurationProperty(Feature.STYLE_PARSER_CLASS, value);
             }
 
             value = options.getOptionValue("?");
@@ -615,7 +632,7 @@ public class Transform {
                 CommandLineOptions.loadAdditionalSchemas(config, additionalSchemas);
             }
 
-            List<Source> sources = new ArrayList<Source>();
+            List<Source> sources = new ArrayList<>();
             if (sourceFileName != null) {
                 boolean useSAXSource = sourceParserName != null || dtdValidation;
                 wholeDirectory = CommandLineOptions.loadDocuments(sourceFileName, useURLs, processor, useSAXSource, sources);
@@ -701,7 +718,11 @@ public class Transform {
 
 
                 if (sheet == null) {
-                    for (int j = 0; j < repeat; j++) {
+                    int repeatComp = repeat;
+                    if (repeatComp > 20) {
+                        repeatComp = 20;
+                    }
+                    for (int j = 0; j < repeatComp; j++) {
                         // Repeat loop is to get reliable performance data
 
                         options.applyStaticParams(compiler);
@@ -735,7 +756,7 @@ public class Transform {
                     out.setOutputProperty(Serializer.Property.METHOD, "xml");
                     out.setOutputProperty(Serializer.Property.INDENT, "yes");
                     out.setOutputProperty(Serializer.Property.OMIT_XML_DECLARATION, "yes");
-                    if (!"HE".equals(processor.getSaxonEdition())) {
+                    if (config.isLicensedFeature(Configuration.LicenseFeature.PROFESSIONAL_EDITION)) {
                         out.setOutputProperty(Serializer.Property.SAXON_INDENT_SPACES, "2");
                     }
                     sheet.explain(out);
@@ -743,47 +764,51 @@ public class Transform {
                 if (export) {
                     sheet.export(new FileOutputStream(exportOutputFileName));
                     if (showTime) {
-                        System.err.println("Stylesheet exported to: " + exportOutputFileName);
+                        System.err.println("Stylesheet exported to: " + new File(exportOutputFileName).getAbsolutePath());
                     }
                 }
 
                 if (run) {
 
-                    if (wholeDirectory) {
-                        if ((threadCount > 0) && (sources.size() > 1)) {
-                            if (threadCount > sources.size()) {
-                                threadCount = sources.size();
+                    try {
+                        if (wholeDirectory) {
+                            if ((threadCount > 0) && (sources.size() > 1)) {
+                                if (threadCount > sources.size()) {
+                                    threadCount = sources.size();
+                                }
+
+                                //calculate sources per thread
+                                int sourcesPerThread = (int) Math.floor(sources.size() / threadCount);
+
+                                //split remainder of sources amongst rem threads
+                                int rem = sources.size() % threadCount;
+                                th = new TransformThread[threadCount];
+                                // long elapsedTime = System.nanoTime();
+                                for (int i = 0, j = 0, z = 0; i < sources.size(); j++, i += sourcesPerThread + z) {
+                                    z = j < rem ? 1 : 0;  //split remainder of sources amongst rem threads
+                                    th[j] = new TransformThread(i, sheet, sources.subList(i, i + sourcesPerThread + z), outputFile, options, traceDestination);
+                                    th[j].start();
+                                }
+                                for (TransformThread aTh : th) {
+                                    aTh.join();
+                                }
+                                // elapsedTime = System.nanoTime() - elapsedTime;
+                                // System.out.println("Total Elapsed Time: "+elapsedTime/1e6 + "ms");
+
+
+                            } else {
+                                processDirectory(sources, sheet, outputFile, options, traceDestination);
                             }
-
-                            //calculate sources per thread
-                            int sourcesPerThread = (int) Math.floor(sources.size() / threadCount);
-
-                            //split remainder of sources amongst rem threads
-                            int rem = sources.size() % threadCount;
-                            th = new TransformThread[threadCount];
-                            // long elapsedTime = System.nanoTime();
-                            for (int i = 0, j = 0, z = 0; i < sources.size(); j++, i += sourcesPerThread + z) {
-                                z = j < rem ? 1 : 0;  //split remainder of sources amongst rem threads
-                                th[j] = new TransformThread(i, sheet, sources.subList(i, i + sourcesPerThread + z), outputFile, options, traceDestination);
-                                th[j].start();
-                            }
-                            for (TransformThread aTh : th) {
-                                aTh.join();
-                            }
-                            // elapsedTime = System.nanoTime() - elapsedTime;
-                            // System.out.println("Total Elapsed Time: "+elapsedTime/1e6 + "ms");
-
-
                         } else {
-                            processDirectory(sources, sheet, outputFile, options, traceDestination);
+                            Source source = sources == null || sources.isEmpty() ? null : sources.get(0);
+                            processFile(source, sheet, outputFile, options, traceDestination);
+                        }
+                    } finally {
+                        if (closeTraceDestination && traceDestination != null) {
+                            traceDestination.close();
                         }
-                    } else {
-                        Source source = sources == null || sources.isEmpty() ? null : sources.get(0);
-                        processFile(source, sheet, outputFile, options, traceDestination);
-                    }
-                    if (closeTraceDestination) {
-                        traceDestination.close();
                     }
+
                 }
                 if (options.getOptionValue("TB")!=null) {
                     // report on hotspot bytecode generation
@@ -795,14 +820,9 @@ public class Transform {
         } catch (SaxonApiException err) {
             //err.printStackTrace();
             quit(err.getMessage(), 2);
-        } catch (TransformerException err) {
+        } catch (TransformerException | LicenseException | TransformerFactoryConfigurationError err) {
             //err.printStackTrace();
             quit("Transformation failed: " + err.getMessage(), 2);
-        } catch (TransformerFactoryConfigurationError err) {
-            //err.printStackTrace();
-            quit("Transformation failed: " + err.getMessage(), 2);
-        } catch (LicenseException err) {
-            quit("Transformation failed: " + err.getMessage(), 2);
         } catch (Exception err2) {
             err2.printStackTrace();
             quit("Fatal error during transformation: " + err2.getClass().getName() + ": " +
@@ -951,7 +971,7 @@ public class Transform {
      */
 
     private File makeOutputFile(File directory, String localName, XsltExecutable sheet) {
-        String mediaType = sheet.getUnderlyingCompiledStylesheet().getDefaultOutputProperties().getProperty(OutputKeys.MEDIA_TYPE);
+        String mediaType = sheet.getUnderlyingCompiledStylesheet().getPrimarySerializationProperties().getProperty(OutputKeys.MEDIA_TYPE);
         String suffix = ".xml";
         if ("text/html".equals(mediaType)) {
             suffix = ".html";
@@ -1008,7 +1028,7 @@ public class Transform {
                         transformer.newSerializer(System.out) :
                         transformer.newSerializer(outFile);
         try {
-            options.setSerializationParams(serializer);
+            options.setSerializationProperties(serializer);
         } catch (IllegalArgumentException e) {
             quit(e.getMessage(), 2);
         }
@@ -1037,13 +1057,8 @@ public class Transform {
     protected Xslt30Transformer newTransformer(
             XsltExecutable sheet, CommandLineOptions options, Logger traceDestination) throws SaxonApiException {
         final Xslt30Transformer transformer = sheet.load30();
-        final Map<QName, XdmValue> params = new HashMap<QName, XdmValue>();
-        options.setParams(processor, new CommandLineOptions.ParamSetter() {
-            public void setParam(QName qName, XdmValue value) {
-                params.put(qName, value);
-            }
-        });
-        transformer.setStylesheetParameters(params);
+        final Map<QName, XdmValue> params = new HashMap<>();
+        Executable exec = transformer.getUnderlyingController().getExecutable();
         transformer.setTraceFunctionDestination(traceDestination);
         String initialMode = options.getOptionValue("im");
         if (initialMode != null) {
@@ -1053,7 +1068,7 @@ public class Transform {
         String now = options.getOptionValue("now");
         if (now != null) {
             try {
-                DateTimeValue currentDateTime = (DateTimeValue) DateTimeValue.makeDateTimeValue(
+                DateTimeValue currentDateTime = (DateTimeValue)DateTimeValue.makeDateTimeValue(
                         now, getConfiguration().getConversionRules()).asAtomic();
                 transformer.getUnderlyingController().setCurrentDateTime(currentDateTime);
             } catch (XPathException e) {
@@ -1177,25 +1192,27 @@ public class Transform {
                 traceDestination = null;
             }
             Xslt30Transformer transformer = newTransformer(sheet, options, traceDestination);
-            Serializer serializer =
-                    outputFile == null ?
-                            transformer.newSerializer(System.out) :
-                            transformer.newSerializer(outputFile);
+            Serializer serializer;
+            if (outputFile == null) {
+                transformer.setBaseOutputURI(new File(System.getProperty("user.dir")).toURI().toASCIIString());
+                serializer = transformer.newSerializer(System.out);
+            }else {
+                serializer = transformer.newSerializer(outputFile);
+            }
             try {
-                options.setSerializationParams(serializer);
+                options.setSerializationProperties(serializer);
             } catch (IllegalArgumentException e) {
                 quit(e.getMessage(), 2);
             }
 
             boolean buildResultTree;
-            Properties props = serializer.getCombinedOutputProperties();
-            String buildTreeProperty = props.getProperty(SaxonOutputKeys.BUILD_TREE);
+            String buildTreeProperty = serializer.getOutputProperty(Serializer.Property.BUILD_TREE);
             if ("yes".equals(buildTreeProperty)) {
                 buildResultTree = true;
             } else if ("no".equals(buildTreeProperty)) {
                 buildResultTree = false;
             } else {
-                String method = props.getProperty(OutputKeys.METHOD);
+                String method = serializer.getOutputProperty(Serializer.Property.METHOD);
                 buildResultTree = !("json".equals(method) || "adaptive".equals(method));
             }
 
@@ -1224,9 +1241,9 @@ public class Transform {
                             builder.setSchemaValidator(validator);
                         }
                     }
-                    builder.setDTDValidation(getConfiguration().getBooleanProperty(FeatureKeys.DTD_VALIDATION));
+                    builder.setDTDValidation(getConfiguration().getBooleanProperty(Feature.DTD_VALIDATION));
                     builder.setWhitespaceStrippingPolicy(sheet.getWhitespaceStrippingPolicy());
-                    if (getConfiguration().getBooleanProperty(FeatureKeys.DTD_VALIDATION_RECOVERABLE)) {
+                    if (getConfiguration().getBooleanProperty(Feature.DTD_VALIDATION_RECOVERABLE)) {
                         source = new AugmentedSource(source, getConfiguration().getParseOptions());
                     }
                     XdmNode node = builder.build(source);
@@ -1272,6 +1289,10 @@ public class Transform {
                     System.err.println("-------------------------------");
                     Runtime.getRuntime().gc();
                 }
+                if (Instrumentation.ACTIVE) {
+                    Instrumentation.report();
+                    Instrumentation.reset();
+                }
             }
             if (repeat == 999999 && totalTime > 60000) {
                 break;
