@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -22,12 +22,10 @@ import org.w3c.dom.*;
 /**
  * This class is an implementation of the DOM Element class that wraps a Saxon NodeInfo
  * representation of an element node.
- * <p/>
- * The class provides read-only access to the tree; methods that request updates all fail
- * with an UnsupportedOperationException.
- * <p/>
- * Note that contrary to the DOM specification, this implementation does not expose namespace
- * declarations as attributes.
+ * <p>The class provides read-only access to the tree; methods that request updates all fail
+ * with an UnsupportedOperationException.</p>
+ * <p>Note that contrary to the DOM specification, this implementation does not expose namespace
+ * declarations as attributes.</p>
  */
 
 public class ElementOverNodeInfo extends NodeOverNodeInfo implements Element {
@@ -90,7 +88,9 @@ public class ElementOverNodeInfo extends NodeOverNodeInfo implements Element {
             }
             if (att.getDisplayName().equals(name)) {
                 String val = att.getStringValue();
-                if (val == null) return "";
+                if (val == null) {
+                    return "";
+                }
                 return val;
             }
         }
@@ -167,8 +167,10 @@ public class ElementOverNodeInfo extends NodeOverNodeInfo implements Element {
      */
 
     public String getAttributeNS(String namespaceURI, String localName) {
-        String val = Navigator.getAttributeValue(node, (namespaceURI == null ? "" : namespaceURI), localName);
-        if (val == null) return "";
+        String val = Navigator.getAttributeValue(node, namespaceURI == null ? "" : namespaceURI, localName);
+        if (val == null) {
+            return "";
+        }
         return val;
     }
 
@@ -295,43 +297,36 @@ public class ElementOverNodeInfo extends NodeOverNodeInfo implements Element {
      */
 
     public boolean hasAttributeNS(String namespaceURI, String localName) {
-        return (Navigator.getAttributeValue(node, (namespaceURI == null ? "" : namespaceURI), localName) != null);
+        return Navigator.getAttributeValue(node, namespaceURI == null ? "" : namespaceURI, localName) != null;
     }
 
     /**
      * Mark an attribute as an ID. Always fails.
      *
-     * @throws DOMException
+     * @throws UnsupportedOperationException always
      */
 
-    public void setIdAttribute(String name,
-                               boolean isId)
-            throws DOMException {
+    public void setIdAttribute(String name, boolean isId) throws UnsupportedOperationException {
         disallowUpdate();
     }
 
     /**
      * Mark an attribute as an ID. Always fails.
      *
-     * @throws DOMException
+     * @throws UnsupportedOperationException always
      */
 
-    public void setIdAttributeNS(String namespaceURI,
-                                 String localName,
-                                 boolean isId)
-            throws DOMException {
+    public void setIdAttributeNS(String namespaceURI, String localName, boolean isId)
+            throws UnsupportedOperationException {
         disallowUpdate();
     }
 
     /**
      * Mark an attribute as an ID. Always fails.
-     *
-     * @throws DOMException
+     * @throws UnsupportedOperationException always
      */
 
-    public void setIdAttributeNode(Attr idAttr,
-                                   boolean isId)
-            throws DOMException {
+    public void setIdAttributeNode(Attr idAttr, boolean isId) throws UnsupportedOperationException {
         disallowUpdate();
     }
 
