@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -27,9 +27,8 @@ import java.util.List;
 
 /**
  * This class implements the DOM Node interface as a wrapper around a Saxon NodeInfo object.
- * <p/>
- * The class provides read-only access to the tree; methods that request updates all fail
- * with an UnsupportedOperationException.
+ * <p>The class provides read-only access to the tree; methods that request updates all fail
+ * with an UnsupportedOperationException.</p>
  */
 
 public abstract class NodeOverNodeInfo implements Node {
@@ -96,7 +95,7 @@ public abstract class NodeOverNodeInfo implements Node {
 
     public final boolean isSameNode(Node other) {
         return other instanceof NodeOverNodeInfo &&
-                node.isSameNodeInfo(((NodeOverNodeInfo) other).node);
+                node.equals(((NodeOverNodeInfo) other).node);
     }
 
     /**
@@ -336,7 +335,7 @@ public abstract class NodeOverNodeInfo implements Node {
 
     public NodeList getChildNodes() {
         try {
-            List<Node> nodes = new ArrayList<Node>(10);
+            List<Node> nodes = new ArrayList<>(10);
             SequenceIterator iter = node.iterateAxis(AxisInfo.CHILD);
             NodeInfo node;
             while ((node = (NodeInfo) iter.next()) != null) {
@@ -511,7 +510,7 @@ public abstract class NodeOverNodeInfo implements Node {
             return NamespaceConstant.XMLNS;
         }
         String uri = node.getURI();
-        return ("".equals(uri) ? null : uri);
+        return "".equals(uri) ? null : uri;
     }
 
     /**
@@ -553,7 +552,7 @@ public abstract class NodeOverNodeInfo implements Node {
      * @param other the other node.
      * @return Returns how the node is positioned relatively to the reference
      *         node.
-     * @throws org.w3c.dom.DOMException
+     * @throws org.w3c.dom.DOMException if an error occurs
      */
 
     public short compareDocumentPosition(Node other) throws DOMException {
@@ -593,7 +592,7 @@ public abstract class NodeOverNodeInfo implements Node {
      * in the case of document nodes.
      *
      * @return the string value of the node, or null in the case of document nodes.
-     * @throws org.w3c.dom.DOMException
+     * @throws org.w3c.dom.DOMException if a dynamic error occurs
      */
 
     public String getTextContent() throws DOMException {
@@ -608,10 +607,10 @@ public abstract class NodeOverNodeInfo implements Node {
      * Set the text content of a node. Always fails.
      *
      * @param textContent the new text content of the node
-     * @throws org.w3c.dom.DOMException
+     * @throws UnsupportedOperationException always
      */
 
-    public void setTextContent(String textContent) throws DOMException {
+    public void setTextContent(String textContent) throws UnsupportedOperationException {
         disallowUpdate();
     }
 
