@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,21 +13,18 @@ import net.sf.saxon.expr.parser.Location;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.lib.ParseOptions;
 import net.sf.saxon.lib.Validation;
-import net.sf.saxon.ma.arrays.ArrayItem;
-import net.sf.saxon.ma.map.MapItem;
 import net.sf.saxon.om.*;
 import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.XPathException;
-import net.sf.saxon.tree.util.Orphan;
 import net.sf.saxon.type.BuiltInAtomicType;
 import net.sf.saxon.type.SchemaType;
 import net.sf.saxon.type.SimpleType;
 import net.sf.saxon.type.Type;
-import net.sf.saxon.value.AtomicValue;
-import net.sf.saxon.value.ObjectValue;
 
 import java.util.Arrays;
 
+import static net.sf.saxon.event.RegularSequenceChecker.State.*;
+
 /**
  * This class is used for generating complex content, that is, the content of an
  * element or document node. It enforces the rules on the order of events within
@@ -35,11 +32,18 @@ import java.util.Arrays;
  * part of the namespace fixup rules, in particular, it ensures that there is a
  * namespace node for the namespace used in the element name and in each attribute
  * name.
- * <p/>
- * <p>The same ComplexContentOutputter may be used for generating an entire XML
- * document; it is not necessary to create a new outputter for each element node.</p>
+ * <p>
+ * The same ComplexContentOutputter may be used for generating an entire XML
+ * document; it is not necessary to create a new outputter for each element node.
+ * <p>
+ * From Saxon 9.9, the ComplexContentOutputter does not combine top-level events.
+ * Unless nested within a startDocument/endDocument or startElement/endElement pair,
+ * items such as atomic values, text nodes, attribute nodes, maps and arrays are
+ * passed through unchanged to the output. It is typically the responsibility
+ * of the Destination object to decide how to combine top-level events (whether
+ * to build a single document, whether to insert item separators, etc).
+ *
  *
- * @author Michael H. Kay
  */
 
 public final class ComplexContentOutputter extends SequenceReceiver {
@@ -47,11 +51,9 @@ public final class ComplexContentOutputter extends SequenceReceiver {
     private Receiver nextReceiver;
     // the next receiver in the output pipeline
 
-    private int pendingStartTagDepth = -2;
-    // -2 means we are at the top level, or immediately within a document node
-    // -1 means we are in the content of an element node whose start tag is complete
     private NodeName pendingStartTag = null;
-    private int level = -1; // records the number of startDocument or startElement events
+    private int level = -1;
+    // records the number of startDocument or startElement events
     // that have not yet been closed. Note that startDocument and startElement
     // events may be arbitrarily nested; startDocument and endDocument
     // are ignored unless they occur at the outermost level, except that they
@@ -76,61 +78,51 @@ public final class ComplexContentOutputter extends SequenceReceiver {
     private Location startElementLocationId = ExplicitLocation.UNKNOWN_LOCATION;
     private boolean declaresDefaultNamespace;
     private int hostLanguage = Configuration.XSLT;
-    private boolean serializing = false;
-
 
+    private RegularSequenceChecker.State state = Initial;
     /**
      * Create a ComplexContentOutputter
-     *
-     * @param pipe the pipeline configuration
+     * @param next the next receiver in the pipeline
      */
 
-    public ComplexContentOutputter(/*@NotNull*/ PipelineConfiguration pipe) {
-        super(pipe);
-        //System.err.println("ComplexContentOutputter init");
+    public ComplexContentOutputter(Receiver next) {
+        super(next.getPipelineConfiguration());
+        setReceiver(next);
+        setHostLanguage(next.getPipelineConfiguration().getHostLanguage());
     }
 
     /**
      * Static factory method to create an push pipeline containing a ComplexContentOutputter
      * @param receiver the destination to which the constructed complex content will be written
      * @param options options for validating the output stream; may be null
-     * @throws XPathException if any dynamic error occurs; and
-     *                        specifically, if an attempt is made to switch to a final output
-     *                        destination while writing a temporary tree or sequence @param isFinal true if the destination is a final result tree
-     *                        (either the principal output or a secondary result tree); false if  @param validation Validation to be performed on the output document
      */
 
-    public static SequenceReceiver makeComplexContentReceiver(Receiver receiver, ParseOptions options)
-        throws XPathException {
+    public static Receiver makeComplexContentReceiver(Receiver receiver, ParseOptions options) {
 //        System.err.println("CHANGE OUTPUT DESTINATION new=" + receiver);
 
         String systemId = receiver.getSystemId();
         boolean validate = options != null && options.getSchemaValidationMode() != Validation.PRESERVE;
 
         if (receiver instanceof ComplexContentOutputter && !validate) {
-            return (ComplexContentOutputter) receiver;
+            return receiver;
         }
 
-        PipelineConfiguration pipe = receiver.getPipelineConfiguration();
-        ComplexContentOutputter out = new ComplexContentOutputter(pipe);
-        out.setHostLanguage(pipe.getHostLanguage());
-
-        // add a filter to remove duplicate namespaces
-
-        NamespaceReducer ne = new NamespaceReducer(receiver);
-        ne.setSystemId(receiver.getSystemId());
-        receiver = ne;
-
         // add a validator to the pipeline if required
 
         if (validate) {
-            Configuration config = pipe.getConfiguration();
-            receiver = config.getDocumentValidator(ne, receiver.getSystemId(), options, null);
+            Configuration config = receiver.getPipelineConfiguration().getConfiguration();
+            receiver = config.getDocumentValidator(receiver, systemId, options, null);
         }
 
-        out.setReceiver(receiver);
-        out.setSystemId(systemId);
-        return out;
+        // add a filter to remove duplicate namespaces
+        NamespaceReducer ne = new NamespaceReducer(receiver);
+        ne.setSystemId(receiver.getSystemId());
+        receiver = ne;
+
+        receiver = new ComplexContentOutputter(ne);
+
+        receiver.setSystemId(systemId);
+        return receiver;
     }
 
     public void setPipelineConfiguration(/*@NotNull*/ PipelineConfiguration pipe) {
@@ -158,27 +150,6 @@ public final class ComplexContentOutputter extends SequenceReceiver {
         hostLanguage = language;
     }
 
-    /**
-     * Say whether this ComplexContentOutputter is performing serialization
-     * @param serializing true if this class implements the sequence normalization function of the serialization
-     *             spec; false if it is constructing document or element nodes in XSLT or XQuery
-     */
-
-    public void setSerializing(boolean serializing) {
-        this.serializing = serializing;
-    }
-
-    /**
-     * Ask whether this ComplexContentOutputter is performing serialization
-     *
-     * @return true if this class implements the sequence normalization function of the serialization
-     *                    spec; false if it is constructing document or element nodes in XSLT or XQuery
-     */
-
-    public boolean isSerializing() {
-        return serializing;
-    }
-
 
     /**
      * Set the receiver (to handle the next stage in the pipeline) directly
@@ -208,6 +179,7 @@ public final class ComplexContentOutputter extends SequenceReceiver {
     public void open() throws XPathException {
         nextReceiver.open();
         previousAtomic = false;
+        state = Open;
     }
 
     /**
@@ -218,15 +190,15 @@ public final class ComplexContentOutputter extends SequenceReceiver {
         level++;
         if (level == 0) {
             nextReceiver.startDocument(properties);
-        } else if (pendingStartTagDepth >= 0) {
+        } else if (state == StartTag) {
             startContent();
-            pendingStartTagDepth = -2;
         }
         previousAtomic = false;
         if (currentLevelIsDocument.length < level + 1) {
             currentLevelIsDocument = Arrays.copyOf(currentLevelIsDocument, level * 2);
         }
         currentLevelIsDocument[level] = true;
+        state = Content;
     }
 
     /**
@@ -239,6 +211,7 @@ public final class ComplexContentOutputter extends SequenceReceiver {
         }
         previousAtomic = false;
         level--;
+        state = level < 0 ? Open : Content;
     }
 
     /**
@@ -259,21 +232,23 @@ public final class ComplexContentOutputter extends SequenceReceiver {
      * requires it.
      *
      * @param s The String to be output
-     * @param locationId
+     * @param locationId the location of the node in the source, or of the instruction that created it
      * @throws XPathException for any failure
      */
 
     public void characters(CharSequence s, Location locationId, int properties) throws XPathException {
-        previousAtomic = false;
-        if (s == null) {
-            return;
-        }
-        int len = s.length();
-        if (len == 0) {
-            return;
-        }
-        if (pendingStartTagDepth >= 0) {
-            startContent();
+        if (level >= 0) {
+            previousAtomic = false;
+            if (s == null) {
+                return;
+            }
+            int len = s.length();
+            if (len == 0) {
+                return;
+            }
+            if (state == StartTag) {
+                startContent();
+            }
         }
         nextReceiver.characters(s, locationId, properties);
     }
@@ -284,13 +259,13 @@ public final class ComplexContentOutputter extends SequenceReceiver {
      * using attribute().
      *
      * @param elemName The element name
-     * @param location
+     * @param location the location of the element node (or the instruction that created it)
      */
 
     public void startElement(NodeName elemName, SchemaType typeCode, Location location, int properties) throws XPathException {
         //System.err.println("CCO " + this + "StartElement " + nameCode);
         level++;
-        if (pendingStartTagDepth >= 0) {
+        if (state == StartTag) {
             startContent();
         }
         startElementProperties = properties;
@@ -298,7 +273,7 @@ public final class ComplexContentOutputter extends SequenceReceiver {
         pendingAttListSize = 0;
         pendingNSListSize = 0;
         pendingStartTag = elemName;
-        pendingStartTagDepth = 1;
+        //pendingStartTagDepth = 1;
         elementIsInNullNamespace = null; // meaning not yet computed
         declaresDefaultNamespace = false;
         currentSimpleType = typeCode;
@@ -310,6 +285,7 @@ public final class ComplexContentOutputter extends SequenceReceiver {
             copyNamespacesStack = Arrays.copyOf(copyNamespacesStack, level * 2);
         }
         currentLevelIsDocument[level] = false;
+        state = StartTag;
     }
 
 
@@ -331,8 +307,8 @@ public final class ComplexContentOutputter extends SequenceReceiver {
         // Optimization for recursive shallow-copy added in 9.8 - see bug 3011.
         if (nsBindings instanceof InScopeNamespaces) {
             copyNamespacesStack[level] = (InScopeNamespaces)nsBindings;
-            if (level > 1 && copyNamespacesStack[level-1] instanceof InScopeNamespaces &&
-                    copyNamespacesStack[level - 1].getElement().isSameNodeInfo(((InScopeNamespaces)nsBindings).getElement())) {
+            if (level > 1 && copyNamespacesStack[level - 1] != null &&
+                    copyNamespacesStack[level - 1].getElement().equals(((InScopeNamespaces)nsBindings).getElement())) {
                 // Ignore these namespaces if they are the same as the namespaces for the parent element
                 return;
             }
@@ -340,13 +316,12 @@ public final class ComplexContentOutputter extends SequenceReceiver {
 
         // System.err.println("Write namespace prefix=" + (nscode>>16) + " uri=" + (nscode&0xffff));
         for (NamespaceBinding ns : nsBindings) {
-            if (pendingStartTagDepth < 0) {
+            if (state != StartTag) {
                 throw NoOpenStartTagException.makeNoOpenStartTagException(
                         Type.NAMESPACE,
                         ns.getPrefix(),
                         hostLanguage,
-                        pendingStartTagDepth == -2,
-                        isSerializing(),
+                        currentLevelIsDocument[level],
                     startElementLocationId);
             }
 
@@ -386,7 +361,7 @@ public final class ComplexContentOutputter extends SequenceReceiver {
             // It is an error to output a namespace node for the default namespace if the element
             // itself is in the null namespace, as the resulting element could not be serialized
 
-            if (ns.getPrefix().isEmpty() && (ns.getURI().length() != 0)) {
+            if (ns.getPrefix().isEmpty() && !ns.getURI().isEmpty()) {
                 declaresDefaultNamespace = true;
                 if (elementIsInNullNamespace == null) {
                     elementIsInNullNamespace = pendingStartTag.hasURI("");
@@ -417,22 +392,21 @@ public final class ComplexContentOutputter extends SequenceReceiver {
      *
      * @param attName    The name of the attribute
      * @param value      The value of the attribute
-     * @param locationId
+     * @param locationId the location of the node in the source, or of the instruction that created it
      *@param properties Bit fields containing properties of the attribute to be written  @throws XPathException if there is no start tag to write to (created using writeStartTag),
      *                        or if character content has been written since the start tag was written.
      */
 
     public void attribute(NodeName attName, SimpleType typeCode, CharSequence value, Location locationId, int properties) throws XPathException {
         //System.err.println("Write attribute " + nameCode + "=" + value + " to Outputter " + this);
-        if (pendingStartTagDepth < 0) {
+        if (level >= 0 && state != StartTag) {
             // The complexity here is in identifying the right error message and error code
 
             XPathException err = NoOpenStartTagException.makeNoOpenStartTagException(
                     Type.ATTRIBUTE,
                     attName.getDisplayName(),
                     hostLanguage,
-                    level < 0 || currentLevelIsDocument[level],
-                    isSerializing(),
+                    currentLevelIsDocument[level],
                 startElementLocationId);
             err.setLocator(locationId);
             throw err;
@@ -441,20 +415,22 @@ public final class ComplexContentOutputter extends SequenceReceiver {
         // if this is a duplicate attribute, overwrite the original, unless
         // the REJECT_DUPLICATES option is set.
 
-        for (int a = 0; a < pendingAttListSize; a++) {
-            if (pendingAttCode[a].equals(attName)) {
-                if (hostLanguage == Configuration.XSLT) {
-                    pendingAttType[a] = typeCode;
-                    pendingAttValue[a] = value.toString();
-                    // we have to copy the CharSequence, because some kinds of CharSequence are mutable.
-                    pendingAttLocation[a] = locationId;
-                    pendingAttProp[a] = properties;
-                    return;
-                } else {
-                    XPathException err = new XPathException("Cannot create an element having two attributes with the same name: " +
-                            Err.wrap(attName.getDisplayName(), Err.ATTRIBUTE));
-                    err.setErrorCode("XQDY0025");
-                    throw err;
+        if (level >= 0) {
+            for (int a = 0; a < pendingAttListSize; a++) {
+                if (pendingAttCode[a].equals(attName)) {
+                    if (hostLanguage == Configuration.XSLT) {
+                        pendingAttType[a] = typeCode;
+                        pendingAttValue[a] = value.toString();
+                        // we have to copy the CharSequence, because some kinds of CharSequence are mutable.
+                        pendingAttLocation[a] = locationId;
+                        pendingAttProp[a] = properties;
+                        return;
+                    } else {
+                        XPathException err = new XPathException("Cannot create an element having two attributes with the same name: " +
+                                Err.wrap(attName.getDisplayName(), Err.ATTRIBUTE));
+                        err.setErrorCode("XQDY0025");
+                        throw err;
+                    }
                 }
             }
         }
@@ -474,6 +450,11 @@ public final class ComplexContentOutputter extends SequenceReceiver {
             }
         }
 
+        // push top-level attribute nodes down the pipeline
+        if (level < 0) {
+            nextReceiver.attribute(attName, typeCode, value, locationId, properties);
+        }
+
         // otherwise, add this one to the list
 
         if (pendingAttListSize >= pendingAttCode.length) {
@@ -566,10 +547,10 @@ public final class ComplexContentOutputter extends SequenceReceiver {
 
     public void endElement() throws XPathException {
         //System.err.println("Write end tag " + this + " : " + name);
-        if (pendingStartTagDepth >= 0) {
+        if (state == StartTag) {
             startContent();
         } else {
-            pendingStartTagDepth = -2;
+            //pendingStartTagDepth = -2;
             pendingStartTag = null;
         }
 
@@ -578,6 +559,7 @@ public final class ComplexContentOutputter extends SequenceReceiver {
         nextReceiver.endElement();
         level--;
         previousAtomic = false;
+        state = level < 0 ? Open : Content;
     }
 
     /**
@@ -585,11 +567,13 @@ public final class ComplexContentOutputter extends SequenceReceiver {
      */
 
     public void comment(CharSequence comment, Location locationId, int properties) throws XPathException {
-        if (pendingStartTagDepth >= 0) {
-            startContent();
+        if (level >= 0) {
+            if (state == StartTag) {
+                startContent();
+            }
+            previousAtomic = false;
         }
         nextReceiver.comment(comment, locationId, properties);
-        previousAtomic = false;
     }
 
     /**
@@ -597,76 +581,37 @@ public final class ComplexContentOutputter extends SequenceReceiver {
      */
 
     public void processingInstruction(String target, CharSequence data, Location locationId, int properties) throws XPathException {
-        if (pendingStartTagDepth >= 0) {
-            startContent();
+        if (level >= 0) {
+            if (state == StartTag) {
+                startContent();
+            }
+            previousAtomic = false;
         }
         nextReceiver.processingInstruction(target, data, locationId, properties);
-        previousAtomic = false;
     }
 
     /**
      * Append an arbitrary item (node or atomic value) to the output
-     *  @param item           the item to be appended
+     *
+     * @param item           the item to be appended
      * @param locationId     the location of the calling instruction, for diagnostics
      * @param copyNamespaces if the item is an element node, this indicates whether its namespaces
- *                       need to be copied. Values are {@link NodeInfo#ALL_NAMESPACES},
- *                       {@link NodeInfo#LOCAL_NAMESPACES}, {@link NodeInfo#NO_NAMESPACES}
+     *                       need to be copied. Values are {@link ReceiverOptions#ALL_NAMESPACES},
+     *                       {@link ReceiverOptions#LOCAL_NAMESPACES}; the default (0) means
+     *                       no namespaces
      */
 
     public void append(Item item, Location locationId, int copyNamespaces) throws XPathException {
-        if (item != null) {
-            if (item instanceof AtomicValue || item instanceof ObjectValue) {
-                if (previousAtomic) {
-                    characters(" ", locationId, 0);
-                }
-                characters(item.getStringValueCS(), locationId, 0);
-                previousAtomic = true;
-            } else if (item instanceof ArrayItem) {
-                for (Sequence member : (ArrayItem) item) {
-                    SequenceIterator iter = member.iterate();
-                    Item it;
-                    while ((it = iter.next()) != null) {
-                        append(it, locationId, copyNamespaces);
-                    }
-                }
-            } else if (item instanceof Function) {
-                if (SequenceReceiver.isTrueSequenceReceiver(nextReceiver)) {
-                    ((SequenceReceiver)nextReceiver).append(item);
-                } else {
-                    String kind = "a function item";
-                    if (item instanceof MapItem) {
-                        kind = "a map";
-                    }
-                    boolean isXSLT = getPipelineConfiguration().getHostLanguage() == Configuration.XSLT;
-                    throw new XPathException("Cannot add " + kind + " to an XML tree", isXSLT ? "XTDE0450" : "FOTY0013");
-                }
-            } else if (((NodeInfo) item).getNodeKind() == Type.DOCUMENT) {
-                startDocument(0);
-                SequenceIterator iter = ((NodeInfo) item).iterateAxis(AxisInfo.CHILD);
-                Item it;
-                while ((it = iter.next()) != null) {
-                    append(it, locationId, copyNamespaces);
-                }
-                endDocument();
-                previousAtomic = false;
-            } else if (item instanceof Orphan && ((Orphan) item).isDisableOutputEscaping()) {
-                // see test case doe-0185 - needed for output buffered within try/catch
-                characters(item.getStringValueCS(), locationId, ReceiverOptions.DISABLE_ESCAPING);
-                previousAtomic = false;
-            } else {
-                int copyOptions = CopyOptions.TYPE_ANNOTATIONS;
-                if (copyNamespaces == NodeInfo.LOCAL_NAMESPACES) {
-                    copyOptions |= CopyOptions.LOCAL_NAMESPACES;
-                } else if (copyNamespaces == NodeInfo.ALL_NAMESPACES) {
-                    copyOptions |= CopyOptions.ALL_NAMESPACES;
-                }
-                ((NodeInfo) item).copy(this, copyOptions, locationId);
-                previousAtomic = false;
-            }
+        // Decompose the item into a sequence of node events if we're within a start/end element/document
+        // pair. Otherwise, send the item down the pipeline unchanged: it's the job of the Destination
+        // to deal with it (inserting item separators if appropriate)
+        if (level >= 0) {
+            decompose(item, locationId, copyNamespaces);
+        } else {
+            nextReceiver.append(item, locationId, copyNamespaces);
         }
     }
 
-
     /**
      * Close the output
      */
@@ -675,6 +620,7 @@ public final class ComplexContentOutputter extends SequenceReceiver {
         // System.err.println("Close " + this + " using emitter " + emitter.getClass());
         nextReceiver.close();
         previousAtomic = false;
+        state = Final;
     }
 
     /**
@@ -683,19 +629,14 @@ public final class ComplexContentOutputter extends SequenceReceiver {
 
     public void startContent() throws XPathException {
 
-        if (pendingStartTagDepth < 0) {
+        if (state != StartTag) {
             // this can happen if the method is called from outside,
             // e.g. from a SequenceOutputter earlier in the pipeline
             return;
         }
 
-        int props = startElementProperties;
-        NodeName elcode = pendingStartTag;
-        if (declaresDefaultNamespace || pendingStartTag.getPrefix().length() != 0) {
-            // skip this check if the element is unprefixed and no xmlns="abc" declaration has been encountered
-            elcode = checkProposedPrefix(pendingStartTag, 0);
-            props = startElementProperties | ReceiverOptions.NAMESPACE_OK;
-        }
+        NodeName elcode = checkProposedPrefix(pendingStartTag, 0);
+        int props = startElementProperties | ReceiverOptions.NAMESPACE_OK;
         nextReceiver.startElement(elcode, currentSimpleType, startElementLocationId, props);
 
         for (int a = 0; a < pendingAttListSize; a++) {
@@ -722,8 +663,9 @@ public final class ComplexContentOutputter extends SequenceReceiver {
 
         pendingAttListSize = 0;
         pendingNSListSize = 0;
-        pendingStartTagDepth = -1;
+        //pendingStartTagDepth = -1;
         previousAtomic = false;
+        state = Content;
     }
 
     /**
@@ -740,7 +682,7 @@ public final class ComplexContentOutputter extends SequenceReceiver {
 
     public void beforeBulkCopy() throws XPathException {
         level++;
-        if (pendingStartTagDepth >= 0) {
+        if (state == StartTag) {
             startContent();
         }
     }
