@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -21,23 +21,19 @@ import java.util.Stack;
 
 /**
  * This class sits in a receiver (push) pipeline and maintains the current path.
- *
- * @author Michael H. Kay
  */
 
 
 public class PathMaintainer extends ProxyReceiver {
 
-    private Stack<AbsolutePath.PathElement> path = new Stack<AbsolutePath.PathElement>();
-    private Stack<HashMap<NodeName, Integer>> siblingCounters =
-            new Stack<HashMap<NodeName, Integer>>();
+    private Stack<AbsolutePath.PathElement> path = new Stack<>();
+    private Stack<HashMap<NodeName, Integer>> siblingCounters = new Stack<>();
 
     public PathMaintainer(/*@NotNull*/ Receiver next) {
         super(next);
-        siblingCounters.push(new HashMap<NodeName, Integer>());
+        siblingCounters.push(new HashMap<>());
     }
 
-
     public void startElement(NodeName elemName, SchemaType type, Location location, int properties) throws XPathException {
         // System.err.println("startElement " + nameCode);
         nextReceiver.startElement(elemName, type, location, properties);
@@ -51,7 +47,7 @@ public class PathMaintainer extends ProxyReceiver {
             counters.put(elemName, 1);
         }
         path.push(new AbsolutePath.PathElement(Type.ELEMENT, elemName, index));
-        siblingCounters.push(new HashMap<NodeName, Integer>());
+        siblingCounters.push(new HashMap<>());
     }
 
     /**
@@ -79,14 +75,14 @@ public class PathMaintainer extends ProxyReceiver {
             fsb.append('/');
             if (useURIs) {
                 String uri = pe.getName().getURI();
-                if (uri.length() != 0) {
+                if (!uri.isEmpty()) {
                     fsb.append('"');
                     fsb.append(uri);
                     fsb.append('"');
                 }
             } else {
                 String prefix = pe.getName().getPrefix();
-                if (prefix.length() != 0) {
+                if (!prefix.isEmpty()) {
                     fsb.append(prefix);
                     fsb.append(':');
                 }
