@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -53,9 +53,12 @@ public abstract class Sender {
      *                 flag set. On the other hand, if it contains a URI that needs to be dereferenced to obtain
      *                 an InputStream, then the InputStream will be closed after use.
      * @param receiver the destination to which it is to be copied. The pipelineConfiguration
-     *                 of this receiver must have been initialized.
-     * @param options  Parse options. If source is an AugmentedSource, any options set in the
-     *                 AugmentedSource are used in preference to those set in options. If neither specifies
+     *                 of this receiver must have been initialized. The implementation sends a sequence
+     *                 of events to the {@code Receiver}, starting with an {@link Receiver#open()} call
+     *                 and ending with {@link Receiver#close()}; this will be a <b>regular event sequence</b>
+     *                 as defined by {@link RegularSequenceChecker}.
+     * @param options  Parsing options. If source is an {@link AugmentedSource}, any options set in the
+     *                 {@code AugmentedSource} are used in preference to those set in options. If neither specifies
      *                 a particular option, the defaults from the Configuration are used. If null is supplied,
      *                 the parse options from the PipelineConfiguration of the receiver are used.
      * @throws net.sf.saxon.trans.XPathException
@@ -87,7 +90,7 @@ public abstract class Sender {
         List<FilterFactory> filters = options.getFilters();
         if (filters != null) {
             for (int i = filters.size() - 1; i >= 0; i--) {
-                ProxyReceiver filter = filters.get(i).makeFilter(next);
+                Receiver filter = filters.get(i).makeFilter(next);
                 filter.setSystemId(source.getSystemId());
                 next = filter;
             }
@@ -220,6 +223,7 @@ public abstract class Sender {
      *                 node or an element node
      * @param receiver the destination to receive the events
      * @throws XPathException if any error occurs
+     * @throws IllegalArgumentException if the node is not a document or element node
      */
 
 
@@ -239,9 +243,19 @@ public abstract class Sender {
         receiver.open();
 
         // copy the contents of the document
-        receiver.startDocument(0);
-        top.copy(receiver, CopyOptions.ALL_NAMESPACES | CopyOptions.TYPE_ANNOTATIONS, location);
-        receiver.endDocument();
+        switch (top.getNodeKind()) {
+            case Type.DOCUMENT:
+                top.copy(receiver, CopyOptions.ALL_NAMESPACES | CopyOptions.TYPE_ANNOTATIONS, location);
+                break;
+            case Type.ELEMENT:
+                receiver.startDocument(0);
+                top.copy(receiver, CopyOptions.ALL_NAMESPACES | CopyOptions.TYPE_ANNOTATIONS, location);
+                receiver.endDocument();
+                break;
+            default:
+                throw new IllegalArgumentException("Expected document or element node");
+
+        }
 
         // end event stream
         receiver.close();
@@ -296,11 +310,9 @@ public abstract class Sender {
         if (!pipe.getParseOptions().isExpandAttributeDefaults()) {
             try {
                 parser.setFeature("http://xml.org/sax/features/use-attributes2", true);
-            } catch (SAXNotRecognizedException err) {
+            } catch (SAXNotRecognizedException | SAXNotSupportedException err) {
                 // ignore the failure, we did our best (Xerces gives us an Attribute2 even though it
                 // doesn't recognize this request!)
-            } catch (SAXNotSupportedException err) {
-                // ignore the failure, we did our best
             }
         }
 
@@ -319,9 +331,7 @@ public abstract class Sender {
                         try {
                             // This feature name is supported in Xerces 2.9.0
                             parser.setFeature(name, value);
-                        } catch (SAXNotRecognizedException err) {
-                            tryAgain = true;
-                        } catch (SAXNotSupportedException err) {
+                        } catch (SAXNotRecognizedException | SAXNotSupportedException err) {
                             tryAgain = true;
                         }
                         if (tryAgain) {
@@ -371,9 +381,7 @@ public abstract class Sender {
             try {
                 // This feature name is supported in the version of Xerces bundled with JDK 1.5
                 parser.setFeature("http://apache.org/xml/features/xinclude-aware", true);
-            } catch (SAXNotRecognizedException err) {
-                tryAgain = true;
-            } catch (SAXNotSupportedException err) {
+            } catch (SAXNotRecognizedException | SAXNotSupportedException err) {
                 tryAgain = true;
             }
             if (tryAgain) {
@@ -395,7 +403,7 @@ public abstract class Sender {
 
 
         receiver = makeValidator(receiver, source.getSystemId(), options);
-
+        
         // Reuse the previous ReceivingContentHandler if possible (it contains a useful cache of names)
 
         ReceivingContentHandler ce;
@@ -409,9 +417,8 @@ public abstract class Sender {
             parser.setDTDHandler(ce);
             try {
                 parser.setProperty("http://xml.org/sax/properties/lexical-handler", ce);
-            } catch (SAXNotSupportedException err) {    // this just means we won't see the comments
-                // ignore the error
-            } catch (SAXNotRecognizedException err) {
+            } catch (SAXNotSupportedException | SAXNotRecognizedException err) {
+                // this just means we won't see the comments
                 // ignore the error
             }
         }
