@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -27,12 +27,10 @@ import java.util.List;
  * It maintains details of the namespace context, and a full set of attribute information, on behalf
  * of other filters that need access to namespace information or need to process attributes in arbitrary
  * order.
- * <p/>
  * <p>StartTagBuffer also implements namespace fixup (the process of creating namespace nodes|bindings on behalf
  * of constructed element and attribute nodes). Although this would be done anyway, further down the pipeline,
  * it has to be done early in the case of a validating pipeline, because the namespace bindings must be created
  * before any namespace-sensitive attribute content is validated.</p>
- * <p/>
  * <p>The StartTagBuffer also allows error conditions to be buffered. This is because the XSIAttributeHandler
  * validates attributes such as xsi:type and xsi:nil before attempting to match its parent element against
  * a particle of its containing type. It is possible that the parent element will match a wildcard particle
@@ -57,6 +55,7 @@ public class StartTagBuffer extends ProxyReceiver implements NamespaceResolver {
     protected AttributeCollectionImpl bufferedAttributes;
     private boolean acceptAttributes;
     private boolean inDocument;
+    private boolean inStartTag;
 
     // We keep track of namespaces. The namespaces
     // array holds a list of all namespaces currently declared (organised as pairs of entries,
@@ -99,6 +98,7 @@ public class StartTagBuffer extends ProxyReceiver implements NamespaceResolver {
         }
         acceptAttributes = false;
         inDocument = true;      // we ought to clear this on endDocument, but it only affects diagnostics
+        inStartTag = false;
     }
 
 
@@ -119,6 +119,8 @@ public class StartTagBuffer extends ProxyReceiver implements NamespaceResolver {
 
     public void startElement(NodeName nameCode, SchemaType typeCode, Location location, int properties) throws XPathException {
 
+        maybeStartContent();
+        
         elementNameCode = nameCode;
         elementTypeCode = typeCode;
         elementLocationId = location.saveLocation();
@@ -144,6 +146,7 @@ public class StartTagBuffer extends ProxyReceiver implements NamespaceResolver {
             namespace(nameCode.getNamespaceBinding(), 0);
         }
         attCount = 0;
+        inStartTag = true;
     }
 
     public void namespace(NamespaceBindingSet namespaceBindings, int properties) throws XPathException {
@@ -153,7 +156,7 @@ public class StartTagBuffer extends ProxyReceiver implements NamespaceResolver {
                 throw NoOpenStartTagException.makeNoOpenStartTagException(
                         Type.NAMESPACE, ns.getPrefix(),
                         getPipelineConfiguration().getHostLanguage(),
-                        inDocument, false, ExplicitLocation.UNKNOWN_LOCATION);
+                        inDocument, ExplicitLocation.UNKNOWN_LOCATION);
             }
 
             // avoid duplicates
@@ -171,12 +174,15 @@ public class StartTagBuffer extends ProxyReceiver implements NamespaceResolver {
      * Notify an attribute. Attributes are notified after the startElement event, and before any
      * children. Namespaces and attributes may be intermingled.
      *
-     * @param attName    The name of the attribute, as held in the name pool
-     * @param typeCode   The type of the attribute, as held in the name pool
-     * @param locationId
-     *@param properties Bit significant value. The following bits are defined:
+     * @param attName    The name of the attribute
+     * @param typeCode   The type of the attribute
+     * @param locationId the location of the node in the source, or of the instruction that created it
+     * @param properties Bit significant value. The following bits are defined:
+     *                   <dl>
      *                   <dd>DISABLE_ESCAPING</dd>    <dt>Disable escaping for this attribute</dt>
-     *                   <dd>NO_SPECIAL_CHARACTERS</dd>      <dt>Attribute value contains no special characters</dt>  @throws IllegalStateException: attempt to output an attribute when there is no open element
+     *                   <dd>NO_SPECIAL_CHARACTERS</dd>      <dt>Attribute value contains no special characters</dt>
+     *                   </dl>
+     * @throws IllegalStateException: attempt to output an attribute when there is no open element
      *                                start tag
      */
 
@@ -187,7 +193,7 @@ public class StartTagBuffer extends ProxyReceiver implements NamespaceResolver {
             throw NoOpenStartTagException.makeNoOpenStartTagException(
                     Type.ATTRIBUTE, attName.getDisplayName(),
                     getPipelineConfiguration().getHostLanguage(),
-                    inDocument, false, ExplicitLocation.UNKNOWN_LOCATION);
+                    inDocument, ExplicitLocation.UNKNOWN_LOCATION);
         }
 
         // Perform namespace fixup for the attribute
@@ -228,6 +234,7 @@ public class StartTagBuffer extends ProxyReceiver implements NamespaceResolver {
      */
 
     public void startContent() throws XPathException {
+        inStartTag = false;
         int props = elementProperties | ReceiverOptions.NAMESPACE_OK;
         if (hasChildren) {
             props |= ReceiverOptions.HAS_CHILDREN;
@@ -247,6 +254,12 @@ public class StartTagBuffer extends ProxyReceiver implements NamespaceResolver {
         nextReceiver.startContent();
     }
 
+    private void maybeStartContent() throws XPathException {
+        if (inStartTag) {
+            startContent();
+        }
+    }
+
     protected void declareNamespacesForStartElement() throws XPathException {
         for (int i = namespacesSize - countStack[depth - 1]; i < namespacesSize; i++) {
             nextReceiver.namespace(namespaces[i], 0);
@@ -287,6 +300,7 @@ public class StartTagBuffer extends ProxyReceiver implements NamespaceResolver {
      */
 
     public void endElement() throws XPathException {
+        maybeStartContent();
         nextReceiver.endElement();
         undeclareNamespacesForElement();
     }
@@ -455,5 +469,44 @@ public class StartTagBuffer extends ProxyReceiver implements NamespaceResolver {
         return prefix + '_' + seq;
     }
 
+    /**
+     * Character data
+     *
+     * @param chars
+     * @param locationId the location of the node in the source, or of the instruction that created it
+     * @param properties
+     */
+    @Override
+    public void characters(CharSequence chars, Location locationId, int properties) throws XPathException {
+        maybeStartContent();
+        nextReceiver.characters(chars, locationId, properties);
+    }
+
+    /**
+     * Processing Instruction
+     *
+     * @param target
+     * @param data
+     * @param locationId the location of the node in the source, or of the instruction that created it
+     * @param properties
+     */
+    @Override
+    public void processingInstruction(String target, CharSequence data, Location locationId, int properties) throws XPathException {
+        maybeStartContent();
+        nextReceiver.processingInstruction(target, data, locationId, properties);
+    }
+
+    /**
+     * Output a comment
+     *
+     * @param chars
+     * @param locationId the location of the node in the source, or of the instruction that created it
+     * @param properties
+     */
+    @Override
+    public void comment(CharSequence chars, Location locationId, int properties) throws XPathException {
+        maybeStartContent();
+        nextReceiver.comment(chars, locationId, properties);
+    }
 }
 
