@@ -327,7 +327,7 @@ public final class Atomizer extends UnaryExpression {
      */
 
     /*@NotNull*/
-    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<?> iterate(XPathContext context) throws XPathException {
         SequenceIterator base = getBaseExpression().iterate(context);
         return getAtomizingIterator(base, untyped && operandItemType instanceof NodeTest);
     }
@@ -518,13 +518,13 @@ public final class Atomizer extends UnaryExpression {
         }
     }
 
-    public static AtomicSequence atomize(Sequence sequence) throws XPathException {
+    public static AtomicSequence atomize(Sequence<?> sequence) throws XPathException {
         if (sequence instanceof AtomicSequence) {
             return (AtomicSequence)sequence;
         } else if (sequence instanceof EmptySequence) {
             return EmptyAtomicSequence.getInstance();
         } else {
-            SequenceIterator iter = getAtomizingIterator(sequence.iterate(), false);
+            SequenceIterator<? extends AtomicValue> iter = getAtomizingIterator(sequence.iterate(), false);
             return new AtomicArray(iter);
         }
     }
