@@ -80,8 +80,9 @@ public class CastToList extends UnaryExpression {
             GroundedValue literalOperand = ((Literal) operand).getValue();
             if (literalOperand instanceof AtomicValue) {
                 try {
-                    SequenceIterator seq = iterate(visitor.getStaticContext().makeEarlyEvaluationContext());
-                    return Literal.makeLiteral(((SequenceIterator<Item>) seq).materialize(), this);
+                    SequenceIterator<?> seq =
+                            iterate(visitor.getStaticContext().makeEarlyEvaluationContext());
+                    return Literal.makeLiteral(seq.materialize(), this);
                 } catch (XPathException err) {
                     err.maybeSetErrorCode("FORG0001");
                     err.setLocation(getLocation());
@@ -227,7 +228,7 @@ public class CastToList extends UnaryExpression {
      * Evaluate the expression
      */
 
-    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<?> iterate(XPathContext context) throws XPathException {
         AtomicValue value = (AtomicValue) getBaseExpression().evaluateItem(context);
         if (value == null) {
             if (allowEmpty) {
