@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -68,14 +68,13 @@ public final class FirstItemExpression extends SingleItemFilter {
      * Convert this expression to an equivalent XSLT pattern
      *
      * @param config the Saxon configuration
-     * @param is30   true if this is XSLT 3.0
      * @return the equivalent pattern
      * @throws net.sf.saxon.trans.XPathException
      *          if conversion is not possible
      */
     @Override
-    public Pattern toPattern(Configuration config, boolean is30) throws XPathException {
-        Pattern basePattern = getBaseExpression().toPattern(config, is30);
+    public Pattern toPattern(Configuration config) throws XPathException {
+        Pattern basePattern = getBaseExpression().toPattern(config);
         ItemType type = basePattern.getItemType();
         if (type instanceof NodeTest) {
             Expression baseExpr = getBaseExpression();
