@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -30,7 +30,8 @@ import java.util.ArrayList;
 
 public class LetExpression extends Assignation implements TailCallReturner {
 
-    private int evaluationMode = ExpressionTool.UNDECIDED;
+    //private int evaluationMode = ExpressionTool.UNDECIDED;
+    private Evaluator evaluator = null;
     private boolean needsEagerEvaluation = false;
     private boolean needsLazyEvaluation = false;
     private boolean isInstruction;
@@ -108,7 +109,7 @@ public class LetExpression extends Assignation implements TailCallReturner {
     /**
      * Ask whether the expression can be lifted out of a loop, assuming it has no dependencies
      * on the controlling variable/focus of the loop
-     * @param forStreaming
+     * @param forStreaming true if we are handling streaming code (where rewrites need to be more cautious)
      */
     @Override
     public boolean isLiftable(boolean forStreaming) {
@@ -118,7 +119,11 @@ public class LetExpression extends Assignation implements TailCallReturner {
     @Override
     public void resetLocalStaticProperties() {
         super.resetLocalStaticProperties();
-        references = new ArrayList<VariableReference>(); // bug 3233
+        references = new ArrayList<>(); // bug 3233
+        if (evaluator == Evaluator.VARIABLE && !(getSequence() instanceof VariableReference)) {
+            evaluator = null;
+            setEvaluator();
+        }
     }
 
     /**
@@ -188,7 +193,6 @@ public class LetExpression extends Assignation implements TailCallReturner {
 
     /**
      * Perform optimisation of an expression and its subexpressions.
-     * <p/>
      * <p>This method is called after all references to functions and variables have been resolved
      * to the declaration of the function or variable, and after all type checking has been done.</p>
      *
@@ -204,7 +208,7 @@ public class LetExpression extends Assignation implements TailCallReturner {
 
     /*@NotNull*/
     public Expression optimize(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType) throws XPathException {
-        Optimizer opt = getConfiguration().obtainOptimizer();
+        Optimizer opt = visitor.obtainOptimizer();
 
         // if this is a construct of the form "let $j := EXP return $j" replace it with EXP
         // Remarkably, people do write this, and it can also be produced by previous rewrites
@@ -243,6 +247,12 @@ public class LetExpression extends Assignation implements TailCallReturner {
             }
         }
 
+        // If the initializing expression has (potential) side-effects, prevent optimizations such as elimination
+        // of unused variables
+        if (getSequence().hasSpecialProperty(StaticProperty.HAS_SIDE_EFFECTS)) {
+            needsEagerEvaluation = true;
+        }
+
         // Removal of redundant variables, and inlining of variables that are only used once, depends on accurate
         // knowledge of all references to the variable. The problem is that obtaining this knowledge can be expensive:
         // see bug 2707. On the other hand, failing to do these optimizations is not fatal. So the general approach
@@ -256,76 +266,73 @@ public class LetExpression extends Assignation implements TailCallReturner {
 
         hasLoopingReference |= removeDeadReferences();
 
-        // Force evaluation if the select expression is known to have side-effects
-        if ((getSequence().getSpecialProperties() & StaticProperty.HAS_SIDE_EFFECTS) != 0) {
-            needsEagerEvaluation = true;
-        }
+        if (!needsEagerEvaluation) {
 
-        // If there are less than two references, and none is in a loop, then there is potential
-        // for optimization. But we now need to be absolutely sure that we have an accurate list
-        // of references.
+            // If there are less than two references, and none is in a loop, then there is potential
+            // for optimization. But we now need to be absolutely sure that we have an accurate list
+            // of references.
 
-        boolean considerRemoval =
-                ((references != null && references.size() < 2) || getSequence() instanceof VariableReference) &&
-                        !isIndexedVariable &&
-                        !hasLoopingReference &&
-                        !needsEagerEvaluation;
+            boolean considerRemoval = ((references != null && references.size() < 2) || getSequence() instanceof VariableReference) &&
+                    !isIndexedVariable && !hasLoopingReference && !needsEagerEvaluation;
 
-        if (considerRemoval) {
-            verifyReferences();
-            // At this point the list of references is either accurate, or null
-            considerRemoval = references != null;
-        }
-
-        if (considerRemoval && references.isEmpty()) {
-            // variable is not used - no need to evaluate it
-            getActionOp().optimize(visitor, contextItemType);
-            opt.trace("Eliminated unused variable " + getVariableName(), getAction());
-            return getAction();
-        }
-
-        // Don't inline context-dependent variables in a streamable template. See strmode011.
-        // The reason for this is that a variable <xsl:variable><xsl:copy-of select="."/></xsl:variable>
-        // can be evaluated in streaming mode, but an arbitrary expression using copy() inline can't (e.g.
-        // if it appears in a path expression or as an operand of an arithmetic expression)
-
-        if (considerRemoval && references.size() == 1 && ExpressionTool.dependsOnFocus(getSequence()) ) {
-            if (visitor.isOptimizeForStreaming()) {
-                considerRemoval = false;
+            if (considerRemoval) {
+                verifyReferences();
+                // At this point the list of references is either accurate, or null
+                considerRemoval = references != null;
             }
-            // Disallow inlining if the focus of the variable reference differs from the containing binding.
-            Expression child = references.get(0);
-            Expression parent = child.getParentExpression();
-            while (parent != null && parent != this) {
-                if (!ExpressionTool.findOperand(parent, child).hasSameFocus()) {
+
+            if (considerRemoval && references.isEmpty()) {
+                // variable is not used - no need to evaluate it
+                getActionOp().optimize(visitor, contextItemType);
+                opt.trace("Eliminated unused variable " + getVariableName(), getAction());
+                return getAction();
+            }
+
+            // Don't inline context-dependent variables in a streamable template. See strmode011.
+            // The reason for this is that a variable <xsl:variable><xsl:copy-of select="."/></xsl:variable>
+            // can be evaluated in streaming mode, but an arbitrary expression using copy() inline can't (e.g.
+            // if it appears in a path expression or as an operand of an arithmetic expression)
+
+            if (considerRemoval && references.size() == 1 && ExpressionTool.dependsOnFocus(getSequence()) ) {
+                if (visitor.isOptimizeForStreaming()) {
                     considerRemoval = false;
-                    break;
                 }
-                child = parent;
-                parent = child.getParentExpression();
+                // Disallow inlining if the focus of the variable reference differs from the containing binding.
+                Expression child = references.get(0);
+                Expression parent = child.getParentExpression();
+                while (parent != null && parent != this) {
+                    Operand operand = ExpressionTool.findOperand(parent, child);
+                    assert operand != null;
+                    if (!operand.hasSameFocus()) {
+                        considerRemoval = false;
+                        break;
+                    }
+                    child = parent;
+                    parent = child.getParentExpression();
+                }
             }
-        }
 
-
-        if (considerRemoval && references.size() == 1) {
-            if (ExpressionTool.changesXsltContext(getSequence())) {
-                // Don't inline variables whose initializer might contain a call to xsl:result-document
-                considerRemoval = false;
-            } else if ((getSequence().getDependencies() & StaticProperty.DEPENDS_ON_CURRENT_GROUP) != 0) {
-                // Don't inline variables that depend on current-group() or current-grouping-key()
-                considerRemoval = false;
-            } else if (references.get(0).isInLoop()) {
-                considerRemoval = false;
+            if (considerRemoval && references.size() == 1) {
+                if (ExpressionTool.changesXsltContext(getSequence())) {
+                    // Don't inline variables whose initializer might contain a call to xsl:result-document
+                    considerRemoval = false;
+                } else if ((getSequence().getDependencies() & StaticProperty.DEPENDS_ON_CURRENT_GROUP) != 0) {
+                    // Don't inline variables that depend on current-group() or current-grouping-key()
+                    considerRemoval = false;
+                } else if (references.get(0).isInLoop()) {
+                    // Don't inline variables where the variable reference is evaluated repeatedly
+                    considerRemoval = false;
+                }
             }
-        }
 
-        if (considerRemoval && (references.size() == 1 || getSequence() instanceof Literal ||
-                                        getSequence() instanceof VariableReference) &&
-                opt.isOptionSet(OptimizerOptions.INLINE_VARIABLES)) {
-            inlineReferences();
-            opt.trace("Inlined references to $" + getVariableName(), getAction());
-            references = null;
-            return getAction().optimize(visitor, contextItemType);
+            if (considerRemoval && (references.size() == 1 || getSequence() instanceof Literal ||
+                                            getSequence() instanceof VariableReference) &&
+                    opt.isOptionSet(OptimizerOptions.INLINE_VARIABLES)) {
+                inlineReferences();
+                opt.trace("Inlined references to $" + getVariableName(), getAction());
+                references = null;
+                return getAction().optimize(visitor, contextItemType);
+            }
         }
 
         int tries = 0;
@@ -333,7 +340,7 @@ public class LetExpression extends Assignation implements TailCallReturner {
             Expression seq0 = getSequence();
             getSequenceOp().optimize(visitor, contextItemType);
             if (getSequence() instanceof Literal && !isIndexedVariable) {
-                return getAction().optimize(visitor, contextItemType);
+                return optimize(visitor, contextItemType);
             }
             if (seq0 == getSequence()) {
                 break;
@@ -352,6 +359,7 @@ public class LetExpression extends Assignation implements TailCallReturner {
                 if (references != null && references.size() < 2) {
                     if (references.isEmpty()) {
                         // We may have removed references to the variable; try again at eliminating this expression.
+                        hasLoopingReference = false;
                         return optimize(visitor, contextItemType);
                     } else {
                         // there is one remaining reference; try again at inlining if it's not in a loop
@@ -365,16 +373,20 @@ public class LetExpression extends Assignation implements TailCallReturner {
 
         // Don't use lazy evaluation for a variable that is referenced inside the "try" part of a contained try catch (XSLT3 test try-031)
 
-        if (needsEagerEvaluation) {
-            setEvaluationMode(ExpressionTool.eagerEvaluationMode(getSequence()));
-        } else if (isIndexedVariable()) {
-            setEvaluationMode(ExpressionTool.MAKE_CLOSURE);
-        } else if (evaluationMode == ExpressionTool.UNDECIDED) {
-            setEvaluationMode(ExpressionTool.lazyEvaluationMode(getSequence()));
-        }
+        setEvaluator();
         return this;
     }
 
+    public void setEvaluator() {
+        if (needsEagerEvaluation) {
+            setEvaluator(ExpressionTool.eagerEvaluator(getSequence()));
+        } else if (isIndexedVariable()) {
+            setEvaluator(Evaluator.MAKE_INDEXED_VARIABLE);
+        } else if (evaluator == null) {
+            setEvaluator(ExpressionTool.lazyEvaluator(getSequence(), getNominalReferenceCount() > 1));
+        }
+    }
+
     private void inlineReferences() {
         // Note that the list of references might include references that are no longer reachable on the tree.
         // We therefore take no action if (a) the parent of the reference is null, or (b) the reference is
@@ -413,7 +425,7 @@ public class LetExpression extends Assignation implements TailCallReturner {
      *         false otherwise. The value false may indicate "not known".
      */
 
-    private boolean allReferencesAreFlattened() throws XPathException {
+    private boolean allReferencesAreFlattened() {
         if (references != null) {
             for (VariableReference ref : references) {
                 if (!ref.isFlattened()) {
@@ -486,11 +498,11 @@ public class LetExpression extends Assignation implements TailCallReturner {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<?> iterate(XPathContext context) throws XPathException {
         // minimize stack consumption by evaluating nested LET expressions iteratively
         LetExpression let = this;
         while (true) {
-            Sequence val = let.eval(context);
+            Sequence<?> val = let.eval(context);
             context.setLocalVariable(let.getLocalSlotNumber(), val);
             if (let.getAction() instanceof LetExpression) {
                 let = (LetExpression) let.getAction();
@@ -509,22 +521,22 @@ public class LetExpression extends Assignation implements TailCallReturner {
      * @return the result of evaluating the expression that is bound to the variable
      */
 
-    public Sequence eval(XPathContext context) throws XPathException {
-        if (evaluationMode == ExpressionTool.UNDECIDED) {
-            setEvaluationMode(ExpressionTool.lazyEvaluationMode(getSequence()));
+    public Sequence<?> eval(XPathContext context) throws XPathException {
+        if (evaluator == null) {
+            setEvaluator(ExpressionTool.lazyEvaluator(getSequence(), getNominalReferenceCount() > 1));
         }
         try {
             int savedOutputState = context.getTemporaryOutputState();
             context.setTemporaryOutputState(StandardNames.XSL_VARIABLE);
-            Sequence result = ExpressionTool.evaluate(getSequence(), evaluationMode, context, getNominalReferenceCount());
+            Sequence<?> result = evaluator.evaluate(getSequence(), context);
             context.setTemporaryOutputState(savedOutputState);
             return result;
         } catch (ClassCastException e) {
             // Probably the evaluation mode is wrong, as a result of an expression rewrite. Try again.
-            setEvaluationMode(ExpressionTool.lazyEvaluationMode(getSequence()));
+            assert false;
             int savedOutputState = context.getTemporaryOutputState();
             context.setTemporaryOutputState(StandardNames.XSL_VARIABLE);
-            Sequence result = ExpressionTool.evaluate(getSequence(), evaluationMode, context, getNominalReferenceCount());
+            Sequence<?> result = Evaluator.EAGER_SEQUENCE.evaluate(getSequence(), context);
             context.setTemporaryOutputState(savedOutputState);
             return result;
         }
@@ -538,7 +550,7 @@ public class LetExpression extends Assignation implements TailCallReturner {
         // minimize stack consumption by evaluating nested LET expressions iteratively
         LetExpression let = this;
         while (true) {
-            Sequence val = let.eval(context);
+            Sequence<?> val = let.eval(context);
             context.setLocalVariable(let.getLocalSlotNumber(), val);
             if (let.getAction() instanceof LetExpression) {
                 let = (LetExpression) let.getAction();
@@ -565,7 +577,7 @@ public class LetExpression extends Assignation implements TailCallReturner {
         // minimize stack consumption by evaluating nested LET expressions iteratively
         LetExpression let = this;
         while (true) {
-            Sequence val = let.eval(context);
+            Sequence<?> val = let.eval(context);
             context.setLocalVariable(let.getLocalSlotNumber(), val);
             if (let.getAction() instanceof LetExpression) {
                 let = (LetExpression) let.getAction();
@@ -585,7 +597,7 @@ public class LetExpression extends Assignation implements TailCallReturner {
         // minimize stack consumption by evaluating nested LET expressions iteratively
         LetExpression let = this;
         while (true) {
-            Sequence val = let.eval(context);
+            Sequence<?> val = let.eval(context);
             context.setLocalVariable(let.getLocalSlotNumber(), val);
             if (let.getAction() instanceof LetExpression) {
                 let = (LetExpression) let.getAction();
@@ -614,7 +626,7 @@ public class LetExpression extends Assignation implements TailCallReturner {
      * inference rules defined in the XSLT 3.0 specification.
      *
      * @return the static item type of the expression according to the XSLT 3.0 defined rules
-     * @param contextItemType
+     * @param contextItemType the type of the context item
      */
     @Override
     public UType getStaticUType(UType contextItemType) {
@@ -641,8 +653,8 @@ public class LetExpression extends Assignation implements TailCallReturner {
     public int computeSpecialProperties() {
         int props = getAction().getSpecialProperties();
         int seqProps = getSequence().getSpecialProperties();
-        if ((seqProps & StaticProperty.NON_CREATIVE) == 0) {
-            props &= ~StaticProperty.NON_CREATIVE;
+        if ((seqProps & StaticProperty.NO_NODES_NEWLY_CREATED) == 0) {
+            props &= ~StaticProperty.NO_NODES_NEWLY_CREATED;
         }
         return props;
     }
@@ -660,7 +672,7 @@ public class LetExpression extends Assignation implements TailCallReturner {
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings variables whose binding needs to change
      */
 
     /*@NotNull*/
@@ -697,7 +709,7 @@ public class LetExpression extends Assignation implements TailCallReturner {
         // minimize stack consumption by evaluating nested LET expressions iteratively
         LetExpression let = this;
         while (true) {
-            Sequence val = let.eval(context);
+            Sequence<?> val = let.eval(context);
             context.setLocalVariable(let.getLocalSlotNumber(), val);
             if (let.getAction() instanceof LetExpression) {
                 let = (LetExpression) let.getAction();
@@ -726,7 +738,7 @@ public class LetExpression extends Assignation implements TailCallReturner {
         // minimize stack consumption by evaluating nested LET expressions iteratively
         LetExpression let = this;
         while (true) {
-            Sequence val = let.eval(context);
+            Sequence<?> val = let.eval(context);
             context.setLocalVariable(let.getLocalSlotNumber(), val);
             if (let.getAction() instanceof LetExpression) {
                 let = (LetExpression) let.getAction();
@@ -747,7 +759,7 @@ public class LetExpression extends Assignation implements TailCallReturner {
      */
 
     public String toString() {
-        return "let $" + getVariableEQName() + " := " + getSequence().toString() +
+        return "let $" + getVariableEQName() + " := " + getSequence() +
                 " return " + ExpressionTool.parenthesize(getAction());
     }
 
@@ -774,18 +786,25 @@ public class LetExpression extends Assignation implements TailCallReturner {
             out.emitAttribute("indexable", "true");
         }
         out.emitAttribute("slot", getLocalSlotNumber() + "");
-        out.emitAttribute("eval", getEvaluationMode() + "");
+        if (evaluator == null) {
+            setEvaluator(ExpressionTool.lazyEvaluator(getSequence(), getNominalReferenceCount() > 1));
+        }
+        out.emitAttribute("eval", getEvaluator().getCode() + "");
         getSequence().export(out);
         getAction().export(out);
         out.endElement();
     }
 
-    public void setEvaluationMode(int evaluationMode) {
-        this.evaluationMode = evaluationMode;
+    public void setEvaluator(Evaluator evaluator) {
+        this.evaluator = evaluator;
     }
 
-    public int getEvaluationMode() {
-        return evaluationMode;
+    public void setEvaluationMode(int mode) {
+        this.evaluator = Evaluator.evaluatorForCode(mode);
+    }
+
+    public Evaluator getEvaluator() {
+        return evaluator;
     }
 
 
