@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -64,7 +64,7 @@ public class NegateExpression extends UnaryExpression {
 
     /*@NotNull*/
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
-        typeCheckChildren(visitor, contextInfo);
+        getOperand().typeCheck(visitor, contextInfo);
         RoleDiagnostic role = new RoleDiagnostic(RoleDiagnostic.UNARY_EXPR, "-", 0);
         Expression operand = visitor.getConfiguration().getTypeChecker(backwardsCompatible).staticTypeCheck(
                 getBaseExpression(), SequenceType.OPTIONAL_NUMERIC,
@@ -73,7 +73,7 @@ public class NegateExpression extends UnaryExpression {
         if (operand instanceof Literal) {
             GroundedValue v = ((Literal) operand).getValue();
             if (v instanceof NumericValue) {
-                return Literal.makeLiteral(((NumericValue) v).negate());
+                return Literal.makeLiteral(((NumericValue) v).negate(), this);
             }
         }
         return this;
@@ -81,10 +81,9 @@ public class NegateExpression extends UnaryExpression {
 
     /**
      * Determine the data type of the expression, if this is known statically
-     * <p/>
-     * <p/>
-     * /*@NotNull
      */
+
+    /*@NotNull*/
     public ItemType getItemType() {
         return getBaseExpression().getItemType();
     }
