@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -12,16 +12,18 @@ import net.sf.saxon.expr.parser.ExpressionTool;
 import net.sf.saxon.expr.parser.ExpressionVisitor;
 import net.sf.saxon.expr.parser.RebindingMap;
 import net.sf.saxon.om.AxisInfo;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.EmptyIterator;
 import net.sf.saxon.type.ErrorType;
 import net.sf.saxon.type.ItemType;
+import net.sf.saxon.value.SequenceType;
 
 
 /**
- * An SimpleStepExpression is a special case of a SlashExpression in which the
+ * A SimpleStepExpression is a special case of a SlashExpression in which the
  * start expression selects a single item (or nothing), and the step expression is
  * a simple AxisExpression. This is designed to avoid the costs of creating a new
  * dynamic context for expressions (common in XQuery) such as
@@ -37,6 +39,13 @@ public final class SimpleStepExpression extends SlashExpression {
         }
     }
 
+    private static OperandRole STEP_ROLE = new OperandRole(OperandRole.USES_NEW_FOCUS | OperandRole.HIGHER_ORDER, OperandUsage.TRANSMISSION, SequenceType.ANY_SEQUENCE);
+
+    @Override
+    protected OperandRole getOperandRole(int arg) {
+        return arg == 0 ? OperandRole.FOCUS_CONTROLLING_SELECT : STEP_ROLE;
+    }
+
     /*@NotNull*/
     @Override
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
@@ -107,7 +116,7 @@ public final class SimpleStepExpression extends SlashExpression {
      */
     /*@NotNull*/
     @Override
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<?> iterate(XPathContext context) throws XPathException {
         NodeInfo origin = null;
         try {
             origin = (NodeInfo) getStart().evaluateItem(context);
