@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,6 +9,7 @@ package net.sf.saxon.expr;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.expr.parser.*;
+import net.sf.saxon.ma.map.MapType;
 import net.sf.saxon.om.AtomicSequence;
 import net.sf.saxon.om.Item;
 import net.sf.saxon.om.SequenceIterator;
@@ -80,7 +81,7 @@ public final class SingletonAtomizer extends UnaryExpression {
 
     /*@NotNull*/
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
-        typeCheckChildren(visitor, contextInfo);
+        getOperand().typeCheck(visitor, contextInfo);
         Expression operand = getBaseExpression();
         ExpressionTool.resetStaticProperties(this);
         if (Literal.isEmptySequence(operand)) {
@@ -93,9 +94,11 @@ public final class SingletonAtomizer extends UnaryExpression {
         if (operandType.isPlainType()) {
             return operand;
         }
-        if (!operandType.isAtomizable()) {
+        if (!operandType.isAtomizable(visitor.getConfiguration().getTypeHierarchy())) {
             XPathException err;
-            if (operandType instanceof FunctionItemType) {
+            if (operandType instanceof MapType) {
+                err = new XPathException("Cannot atomize a map", "FOTY0013");
+            } else if (operandType instanceof FunctionItemType) {
                 err = new XPathException("Cannot atomize a function item", "FOTY0013");
             } else {
                 err = new XPathException(
@@ -103,6 +106,7 @@ public final class SingletonAtomizer extends UnaryExpression {
             }
             err.setIsTypeError(true);
             err.setLocation(getLocation());
+            err.setFailingExpression(getParentExpression());
             throw err;
         }
         return this;
@@ -128,19 +132,19 @@ public final class SingletonAtomizer extends UnaryExpression {
     /**
      * Determine the special properties of this expression
      *
-     * @return {@link StaticProperty#NON_CREATIVE}.
+     * @return {@link StaticProperty#NO_NODES_NEWLY_CREATED}.
      */
 
     public int computeSpecialProperties() {
         int p = super.computeSpecialProperties();
-        return p | StaticProperty.NON_CREATIVE;
+        return p | StaticProperty.NO_NODES_NEWLY_CREATED;
     }
 
     /**
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings variables that need to be re-bound
      */
 
     /*@NotNull*/
