@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -17,7 +17,6 @@ import net.sf.saxon.functions.registry.BuiltInFunctionSet;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.ma.map.MapFunctionSet;
 import net.sf.saxon.om.Sequence;
-import net.sf.saxon.om.SequenceTool;
 import net.sf.saxon.pattern.NodeSetPattern;
 import net.sf.saxon.pattern.Pattern;
 import net.sf.saxon.trace.ExpressionPresenter;
@@ -150,20 +149,17 @@ public class SystemFunctionCall extends StaticFunctionCall implements Negatable
             if ((properties & BuiltInFunctionSet.LAST) != 0) {
                 dep |= StaticProperty.DEPENDS_ON_LAST;
             }
-            return dep;
-        } else if ((properties & BuiltInFunctionSet.BASE) != 0) {
-            return dep | StaticProperty.DEPENDS_ON_STATIC_CONTEXT;
-        } else if ((properties & BuiltInFunctionSet.DCOLL) != 0) {
-            return dep | StaticProperty.DEPENDS_ON_STATIC_CONTEXT;
-        } else if (isCallOn(RegexGroup.class)) {
-            return dep | StaticProperty.DEPENDS_ON_CURRENT_GROUP;
-        } else if (isCallOnSystemFunction("current-merge-group")) {
-            return dep | StaticProperty.DEPENDS_ON_CURRENT_GROUP;
-        } else if (isCallOnSystemFunction("current-merge-key")) {
-            return dep | StaticProperty.DEPENDS_ON_CURRENT_GROUP;
-        } else {
-            return dep;
         }
+        if ((properties & BuiltInFunctionSet.BASE) != 0) {
+            dep |= StaticProperty.DEPENDS_ON_STATIC_CONTEXT;
+        }
+        if ((properties & BuiltInFunctionSet.DCOLL) != 0) {
+            dep |= StaticProperty.DEPENDS_ON_STATIC_CONTEXT;
+        }
+        if (isCallOn(RegexGroup.class) || isCallOn(CurrentMergeGroup.class) || isCallOn(CurrentMergeKey.class)) {
+            dep |= StaticProperty.DEPENDS_ON_CURRENT_GROUP;
+        }
+        return dep;
     }
 
     /**
@@ -224,10 +220,15 @@ public class SystemFunctionCall extends StaticFunctionCall implements Negatable
      * Ask whether the expression can be lifted out of a loop, assuming it has no dependencies
      * on the controlling variable/focus of the loop
      *
-     * @param forStreaming true if the expression is being optimized for streaming
+     * @param forStreaming true if we are optimizing for streamed evaluation
      */
     @Override
     public boolean isLiftable(boolean forStreaming) {
+        // xsl:map-entry is not liftable when streaming because of the special streamability
+        // rules for xsl:map; similarly XPath map constructor expressions.
+        // The tests for current-merge-group/key were added to fix bug 3652 - it seems
+        // an inelegant solution because it's being handled differently from other context
+        // dependencies, but it works.
         return super.isLiftable(forStreaming) &&
                 !isCallOn(CurrentMergeGroup.class) && !isCallOn(CurrentMergeKey.class) &&
                 (!forStreaming || !isCallOn(MapFunctionSet.MapEntry.class));
@@ -259,7 +260,7 @@ public class SystemFunctionCall extends StaticFunctionCall implements Negatable
                 // the condition eliminates concat, which is a special case.
                 for (int i = 0; i < getArity(); i++) {
                     if (Literal.isEmptySequence(getArg(i)) && details.resultIfEmpty[i] != null) {
-                        return Literal.makeLiteral(SequenceTool.toGroundedValue(details.resultIfEmpty[i]));
+                        return Literal.makeLiteral(details.resultIfEmpty[i].materialize(), this);
                     }
                 }
             }
@@ -278,7 +279,6 @@ public class SystemFunctionCall extends StaticFunctionCall implements Negatable
      * sequences, in general; this method determines the type of the items within the
      * sequence, assuming that (a) this is known in advance, and (b) it is the same for
      * all items in the sequence.
-     * <p/>
      * <p>This method should always return a result, though it may be the best approximation
      * that is available at the time.</p>
      *
@@ -292,8 +292,7 @@ public class SystemFunctionCall extends StaticFunctionCall implements Negatable
 
     /**
      * Copy an expression. This makes a deep copy.
-     *
-     * @param rebindings
+     * @param rebindings variables that need to be re-bound
      * @return the copy of the original expression
      */
     @Override
@@ -304,11 +303,10 @@ public class SystemFunctionCall extends StaticFunctionCall implements Negatable
         }
         SystemFunction target = getTargetFunction();
         if (target instanceof StatefulSystemFunction) {
-            target = ((StatefulSystemFunction)target).copy();
+            target = ((StatefulSystemFunction) target).copy();
         }
         return target.makeFunctionCall(args);
     }
-
     /**
      * For an expression that returns an integer or a sequence of integers, get
      * a lower and upper bound on the values of the integers that may be returned, from
@@ -391,7 +389,6 @@ public class SystemFunctionCall extends StaticFunctionCall implements Negatable
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -424,22 +421,20 @@ public class SystemFunctionCall extends StaticFunctionCall implements Negatable
      * Convert this expression to an equivalent XSLT pattern
      *
      * @param config the Saxon configuration
-     * @param is30   true if this is XSLT 3.0
      * @return the equivalent pattern
      * @throws net.sf.saxon.trans.XPathException if conversion is not possible
      */
     @Override
-    public Pattern toPattern(Configuration config, boolean is30) throws XPathException {
+    public Pattern toPattern(Configuration config) throws XPathException {
         SystemFunction fn = getTargetFunction();
         if (fn instanceof Root_1) {
-            if (is30 &&
-                    (getArg(0) instanceof ContextItemExpression ||
+            if (getArg(0) instanceof ContextItemExpression ||
                              (getArg(0) instanceof ItemChecker &&
-                                      ((ItemChecker) getArg(0)).getBaseExpression() instanceof ContextItemExpression))) {
+                                      ((ItemChecker) getArg(0)).getBaseExpression() instanceof ContextItemExpression)) {
                 return new NodeSetPattern(this);
             }
         }
-        return super.toPattern(config, is30);
+        return super.toPattern(config);
     }
 
     public Sequence[] evaluateArguments(XPathContext context) throws XPathException {
@@ -477,7 +472,7 @@ public class SystemFunctionCall extends StaticFunctionCall implements Negatable
      * Diagnostic print of expression structure. The abstract expression tree
      * is written to the supplied output destination.
      *
-     * @param out
+     * @param out the destination of the output
      */
     @Override
     public void export(ExpressionPresenter out) throws XPathException {
@@ -493,7 +488,7 @@ public class SystemFunctionCall extends StaticFunctionCall implements Negatable
         } else {
             // Function was implemented as an IntegratedFunctionCall in 9.7 and we retain the same export format
             out.startElement("ifCall", this);
-            out.emitAttribute("name", getFunctionName().getEQName());
+            out.emitAttribute("name", getFunctionName());
             out.emitAttribute("type", getTargetFunction().getFunctionItemType().getResultType().toExportString());
             getTargetFunction().exportAttributes(out);
             for (Operand o : operands()) {
