@@ -35,7 +35,7 @@ public abstract class VariableReference extends Expression implements BindingRef
     /*@Nullable*/ protected Binding binding = null;     // This will be null until fixup() is called; it will also be null
     // if the variable reference has been inlined
     protected SequenceType staticType = null;
-    protected GroundedValue constantValue = null;
+    protected GroundedValue<?> constantValue = null;
     private StructuredQName variableName = null;
     private boolean flattened = false;
     private boolean inLoop = false;
@@ -116,13 +116,12 @@ public abstract class VariableReference extends Expression implements BindingRef
      * as supplying the static type, it may also supply a compile-time value for the variable.
      * As well as the type information, other static properties of the value are supplied:
      * for example, whether the value is an ordered node-set.
-     *
-     * @param type       the static type of the variable
+     *  @param type       the static type of the variable
      * @param value      the value of the variable if this is a compile-time constant, or null otherwise
      * @param properties static properties of the expression to which the variable is bound
      */
 
-    public void setStaticType(SequenceType type, /*@Nullable*/ GroundedValue value, int properties) {
+    public void setStaticType(SequenceType type, GroundedValue<?> value, int properties) {
         // System.err.println(this + " Set static type = " + type);
         if (type == null) {
             type = SequenceType.ANY_SEQUENCE;
@@ -259,6 +258,14 @@ public abstract class VariableReference extends Expression implements BindingRef
             Optimizer.trace(visitor.getConfiguration(), "Replaced variable " + getDisplayName() + " by its value", result);
             return result;
         }
+        if (binding instanceof GlobalParam && ((GlobalParam)binding).isStatic()) {
+            Expression select = ((GlobalParam)binding).getSelectExpression();
+            if (select instanceof Literal) {
+                binding = null;
+                Optimizer.trace(visitor.getConfiguration(), "Replaced static parameter " + getDisplayName() + " by its value", select);
+                return select;
+            }
+        }
         return this;
     }
 
@@ -287,7 +294,7 @@ public abstract class VariableReference extends Expression implements BindingRef
      * @param properties    additional static properties of the variable's initializer
      */
 
-    public void refineVariableType(ItemType type, int cardinality, /*@Nullable*/ GroundedValue constantValue, int properties) {
+    public void refineVariableType(ItemType type, int cardinality, GroundedValue<?> constantValue, int properties) {
         TypeHierarchy th = getConfiguration().getTypeHierarchy();
         ItemType oldItemType = getItemType();
         ItemType newItemType = oldItemType;
@@ -544,9 +551,9 @@ public abstract class VariableReference extends Expression implements BindingRef
      */
 
     /*@NotNull*/
-    public SequenceIterator<? extends Item> iterate(XPathContext c) throws XPathException {
+    public SequenceIterator<?> iterate(XPathContext c) throws XPathException {
         try {
-            Sequence<? extends Item> actual = evaluateVariable(c);
+            Sequence<?> actual = evaluateVariable(c);
             assert actual != null;
             return actual.iterate();
         } catch (XPathException err) {
@@ -569,7 +576,7 @@ public abstract class VariableReference extends Expression implements BindingRef
 
     public Item evaluateItem(XPathContext c) throws XPathException {
         try {
-            Sequence actual = evaluateVariable(c);
+            Sequence<?> actual = evaluateVariable(c);
             assert actual != null;
             return actual.head();
         } catch (XPathException err) {
@@ -580,7 +587,7 @@ public abstract class VariableReference extends Expression implements BindingRef
 
     public void process(XPathContext c) throws XPathException {
         try {
-            SequenceIterator<? extends Item> iter = evaluateVariable(c).iterate();
+            SequenceIterator<?> iter = evaluateVariable(c).iterate();
             Receiver out = c.getReceiver();
             Location loc = getLocation();
             iter.forEachOrFail(item -> out.append(item, loc, ReceiverOptions.ALL_NAMESPACES));
@@ -599,7 +606,7 @@ public abstract class VariableReference extends Expression implements BindingRef
      */
 
     /*@NotNull*/
-    public Sequence<? extends Item> evaluateVariable(XPathContext c) throws XPathException {
+    public Sequence<?> evaluateVariable(XPathContext c) throws XPathException {
         try {
             return binding.evaluateVariable(c);
         } catch (NullPointerException err) {
