@@ -1,3 +1,10 @@
+////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+// Copyright (c) 2018 Saxonica Limited.
+// This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
+// If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
+// This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
+////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
+
 package net.sf.saxon.expr.compat;
 
 import net.sf.saxon.Configuration;
@@ -31,7 +38,7 @@ import java.util.List;
 /**
  * GeneralComparison10: a boolean expression that compares two expressions
  * for equals, not-equals, greater-than or less-than. This implements the operators
- * =, !=, <, >, etc. This version of the class implements general comparisons
+ * =, !=, &lt;, &gt;, etc. This version of the class implements general comparisons
  * in XPath 1.0 backwards compatibility mode.
  */
 
@@ -74,7 +81,8 @@ public class GeneralComparison10 extends BinaryExpression implements Callable {
     /*@NotNull*/
     public Expression typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo) throws XPathException {
 
-        typeCheckChildren(visitor, contextInfo);
+        getLhs().typeCheck(visitor, contextInfo);
+        getRhs().typeCheck(visitor, contextInfo);
 
         StaticContext env = visitor.getStaticContext();
         StringCollator comp = visitor.getConfiguration().getCollation(env.getDefaultCollationName());
@@ -88,7 +96,7 @@ public class GeneralComparison10 extends BinaryExpression implements Callable {
         // evaluate the expression now if both arguments are constant
 
         if ((getLhsExpression() instanceof Literal) && (getRhsExpression() instanceof Literal)) {
-            return Literal.makeLiteral(evaluateItem(context));
+            return Literal.makeLiteral(evaluateItem(context), this);
         }
 
         return this;
@@ -110,7 +118,8 @@ public class GeneralComparison10 extends BinaryExpression implements Callable {
         Configuration config = visitor.getConfiguration();
         StaticContext env = visitor.getStaticContext();
 
-        optimizeChildren(visitor, contextInfo);
+        getLhs().optimize(visitor, contextInfo);
+        getRhs().optimize(visitor, contextInfo);
 
         // Neither operand needs to be sorted
 
@@ -121,7 +130,7 @@ public class GeneralComparison10 extends BinaryExpression implements Callable {
 
         if ((getLhsExpression() instanceof Literal) && (getRhsExpression() instanceof Literal)) {
             return Literal.makeLiteral(
-                    evaluateItem(env.makeEarlyEvaluationContext()));
+                    evaluateItem(env.makeEarlyEvaluationContext()), this);
         }
 
         final TypeHierarchy th = config.getTypeHierarchy();
@@ -156,7 +165,7 @@ public class GeneralComparison10 extends BinaryExpression implements Callable {
                     GeneralComparison gc = new GeneralComparison20(getLhsExpression(), operator, getRhsExpression());
                     gc.setRetainedStaticContext(getRetainedStaticContext());
                     gc.setAtomicComparer(comparer);
-                    Expression binExp = config.obtainOptimizer()
+                    Expression binExp = visitor.obtainOptimizer()
                             .optimizeGeneralComparison(visitor, gc, false, contextInfo);
                     ExpressionTool.copyLocationInfo(this, binExp);
                     return binExp.typeCheck(visitor, contextInfo).optimize(visitor, contextInfo);
@@ -164,7 +173,7 @@ public class GeneralComparison10 extends BinaryExpression implements Callable {
             } else if (numeric0 && numeric1) {
                 GeneralComparison gc = new GeneralComparison20(getLhsExpression(), operator, getRhsExpression());
                 gc.setRetainedStaticContext(getRetainedStaticContext());
-                Expression binExp = config.obtainOptimizer()
+                Expression binExp = visitor.obtainOptimizer()
                         .optimizeGeneralComparison(visitor, gc, false, contextInfo);
                 ExpressionTool.copyLocationInfo(this, binExp);
                 return binExp.typeCheck(visitor, contextInfo).optimize(visitor, contextInfo);
@@ -195,7 +204,7 @@ public class GeneralComparison10 extends BinaryExpression implements Callable {
      * @throws XPathException if a dynamic error occurs during the evaluation of the expression
      */
 
-    public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+    public BooleanValue call(XPathContext context, Sequence[] arguments) throws XPathException {
         return BooleanValue.get(effectiveBooleanValue(arguments[0].iterate(), arguments[1].iterate(), context));
     }
 
@@ -414,11 +423,11 @@ public class GeneralComparison10 extends BinaryExpression implements Callable {
         // convert it to the type of the other operand, and compare
 
         if (t0.equals(BuiltInAtomicType.UNTYPED_ATOMIC)) {
-            a0 = t1.getStringConverter(rules).convert(a0).asAtomic();
+            a0 = t1.getStringConverter(rules).convert((StringValue)a0).asAtomic();
         }
 
         if (t1.equals(BuiltInAtomicType.UNTYPED_ATOMIC)) {
-            a1 = t0.getStringConverter(rules).convert(a1).asAtomic();
+            a1 = t0.getStringConverter(rules).convert((StringValue)a1).asAtomic();
         }
 
         return ValueComparison.compare(a0, operator, a1, comparer, false);
@@ -436,10 +445,6 @@ public class GeneralComparison10 extends BinaryExpression implements Callable {
     }
 
 
-//    protected String displayOperator() {
-//        return "many-to-many (1.0) " + super.displayOperator();
-//    }
-
     protected void explainExtraAttributes(ExpressionPresenter out) {
         out.emitAttribute("cardinality", "many-to-many (1.0)");
         out.emitAttribute("comp", comparer.save());
@@ -451,6 +456,3 @@ public class GeneralComparison10 extends BinaryExpression implements Callable {
     }
 }
 
-// Copyright (c) 2017 Saxonica Limited
-// Earlier versions of this module are available under the Mozilla Public License.
-
