@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,7 +13,6 @@ import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.value.Int64Value;
 import net.sf.saxon.value.SequenceExtent;
-import net.sf.saxon.value.SequenceType;
 
 import java.util.ArrayList;
 import java.util.List;
@@ -29,12 +28,12 @@ public class WindowClausePull extends TuplePull {
     private SequenceIterator baseIterator;
     private boolean finished = false;
     private XPathContext context;
-    /*@Nullable*/ private Item previousPrevious = null;
     private Item previous = null;
     private Item current = null;
     private Item next = null;
     private int position = -1;
-    private List<WindowClause.Window> currentWindows = new ArrayList<WindowClause.Window>();
+    private List<WindowClause.Window> currentWindows = new ArrayList<>();
+
 
     public WindowClausePull(TuplePull source, WindowClause windowClause, XPathContext context) {
         this.windowClause = windowClause;
@@ -48,15 +47,13 @@ public class WindowClausePull extends TuplePull {
      *
      * @param context the dynamic evaluation context
      * @return true if another tuple has been generated; false if the tuple stream is exhausted. If the
-     *         method returns false, the values of the local variables corresponding to this tuple stream
-     *         are undefined.
+     * method returns false, the values of the local variables corresponding to this tuple stream
+     * are undefined.
      */
     @Override
     public boolean nextTuple(XPathContext context) throws XPathException {
 
-        if (baseIterator == null) {
-            baseIterator = windowClause.getSequence().iterate(context);
-        }
+        boolean deliver = false;
 
         // First see if there are any windows waiting to be delivered
 
@@ -65,13 +62,29 @@ public class WindowClausePull extends TuplePull {
             return true;
         }
 
-        // Otherwise advance the input sequence
+        // If we're not currently processing an input sequence, try to get a new one
 
-        boolean autoclose = (windowClause.isTumblingWindow() && windowClause.getEndCondition() == null);
-        boolean deliver = false;
+        if (finished || baseIterator == null) {
+            // advance the previous step in the pipeline
+            if (source.nextTuple(context)) {
+                // get an iterator over the input sequence
+                baseIterator = windowClause.getSequence().iterate(context);
+                finished = false;
+                previous = null;
+                position = -1;
+                current = null;
+                next = null;
+            }
+        }
 
         while (!finished) {
-            previousPrevious = previous;
+
+            // advance the input sequence
+
+            boolean autoclose = windowClause.isTumblingWindow() && windowClause.getEndCondition() == null;
+            deliver = false;
+
+            Item oldPrevious = previous;
             previous = current;
             current = next;
             next = baseIterator.next();
@@ -81,42 +94,50 @@ public class WindowClausePull extends TuplePull {
             }
             position++;
             if (position > 0) {
-
+                // See if we need to start a new window
                 if ((windowClause.isSlidingWindow() || currentWindows.isEmpty() || autoclose) &&
                         windowClause.matchesStart(previous, current, next, position, context)) {
+                    // See if we need to end the previous window
                     if (autoclose && !currentWindows.isEmpty()) {
                         // automatically end the previous window
                         WindowClause.Window w = currentWindows.get(0);
                         w.endItem = previous;
-                        w.endPreviousItem = previousPrevious;
+                        w.endPreviousItem = oldPrevious;
                         w.endNextItem = current;
                         w.endPosition = position - 1;
                         deliver = despatch(w, context);
                         currentWindows.clear();
                     }
+                    // Create the new window
                     WindowClause.Window window = new WindowClause.Window();
                     window.startPosition = position;
                     window.startItem = current;
                     window.startPreviousItem = previous;
                     window.startNextItem = next;
-                    window.contents = new ArrayList<Item>();
+                    window.contents = new ArrayList<>();
                     currentWindows.add(window);
                 }
+                // Add the current item to all active windows
                 for (WindowClause.Window active : currentWindows) {
                     if (!active.isFinished()) {
                         active.contents.add(current);
                     }
                 }
-                if (windowClause.getEndCondition() != null) {
-                    List<WindowClause.Window> removals = new ArrayList<WindowClause.Window>();
+                // See if this item marks the end of any active windows
+                boolean explicitEndCondition = windowClause.getEndCondition() != null;
+                boolean implicitEndCondition = finished && windowClause.isIncludeUnclosedWindows();
+                if (explicitEndCondition || implicitEndCondition) {
+                    List<WindowClause.Window> removals = new ArrayList<>();
                     for (WindowClause.Window w : currentWindows) {
-                        if (!w.isFinished() && windowClause.matchesEnd(w, previous, current, next, position, context)) {
+                        if (!w.isFinished() &&
+                                (implicitEndCondition ||
+                                         windowClause.matchesEnd(w, previous, current, next, position, context))) {
                             w.endItem = current;
                             w.endPreviousItem = previous;
                             w.endNextItem = next;
                             w.endPosition = position;
                             if (!deliver) {
-                                deliver |= despatch(w, context);
+                                deliver = despatch(w, context);
                                 if (w.isDespatched()) {
                                     removals.add(w);
                                 }
@@ -127,44 +148,27 @@ public class WindowClausePull extends TuplePull {
                         currentWindows.remove(w);
                     }
                 }
+                // if there's a window ready to be delivered, deliver it
                 if (deliver) {
                     return true;
                 }
             }
         }
-        // on completion, despatch unclosed windows if required
-        if (windowClause.isIncludeUnclosedWindows()) {
-            for (WindowClause.Window w : currentWindows) {
-                w.endItem = current;
-                w.endPreviousItem = previous;
-                w.endNextItem = null;
-                w.endPosition = position;
-                if (despatch(w, context)) {
-                    return true;
-                }
-            }
-        }
-        return deliver;
+        return false;
     }
 
     /**
      * Deliver a tuple (representing a window)
      *
      * @param w       a Window whose end conditions have just been satisfied
-     * @param context
+     * @param context dynamic evaluation context
      * @return true if a Tuple has been created by writing variables to the context stack; false otherwise
-     * @throws XPathException
+     * @throws XPathException if anything goes wrong
      */
 
     private boolean despatch(WindowClause.Window w, XPathContext context) throws XPathException {
 
-        // Do a dynamic check of the window contents against the required type
-        // TODO: optimize this out based on knowledge of the type of the input sequence
-        SequenceType requiredType = windowClause.getVariableBinding(WindowClause.WINDOW_VAR).getRequiredType();
-        if (requiredType != SequenceType.ANY_SEQUENCE &&
-                !requiredType.matches(SequenceExtent.makeSequenceExtent(w.contents), context.getConfiguration().getTypeHierarchy())) {
-            throw new XPathException("Contents of sliding/tumbling window do not match the required type", "XPTY0004");
-        }
+        windowClause.checkWindowContents(w);
 
         // In ordered mode, we must despatch windows in order of start position not in order of end position.
         // So we don't despatch it yet if there are unfinished windows with an earlier start position
@@ -177,7 +181,7 @@ public class WindowClausePull extends TuplePull {
      * than any other window) that is available to be despatched to the output tuple stream
      *
      * @return true if an earliest window was found; false if there are no windows ready to be despatched
-     * @throws XPathException
+     * @throws XPathException if anything goes wrong
      */
 
     private boolean lookForEarliest() throws XPathException {
@@ -208,7 +212,7 @@ public class WindowClausePull extends TuplePull {
      *
      * @param w       the window to be despatched
      * @param context the dynamic evaluation context
-     * @throws XPathException
+     * @throws XPathException if anything goes wrong
      */
 
     public void processWindow(WindowClause.Window w, XPathContext context) throws XPathException {
