@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,7 +13,6 @@ import net.sf.saxon.om.SequenceIterator;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.value.Int64Value;
 import net.sf.saxon.value.SequenceExtent;
-import net.sf.saxon.value.SequenceType;
 
 import java.util.ArrayList;
 import java.util.List;
@@ -26,7 +25,7 @@ public class WindowClausePush extends TuplePush {
 
     private WindowClause windowClause;
     private TuplePush destination;
-    /*@Nullable*/ List<WindowClause.Window> currentWindows = new ArrayList<WindowClause.Window>();
+    /*@Nullable*/ List<WindowClause.Window> currentWindows = new ArrayList<>();
 
     public WindowClausePush(TuplePush destination, WindowClause windowClause) {
         this.windowClause = windowClause;
@@ -42,8 +41,8 @@ public class WindowClausePush extends TuplePush {
      */
     @Override
     public void processTuple(XPathContext context) throws XPathException {
-        currentWindows = new ArrayList<WindowClause.Window>();
-        boolean autoclose = (windowClause.isTumblingWindow() && windowClause.getEndCondition() == null);
+        currentWindows = new ArrayList<>();
+        boolean autoclose = windowClause.isTumblingWindow() && windowClause.getEndCondition() == null;
         Item previousPrevious = null;
         Item previous = null;
         Item current = null;
@@ -79,7 +78,7 @@ public class WindowClausePush extends TuplePush {
                     window.startItem = current;
                     window.startPreviousItem = previous;
                     window.startNextItem = next;
-                    window.contents = new ArrayList<Item>();
+                    window.contents = new ArrayList<>();
                     currentWindows.add(window);
                 }
                 for (WindowClause.Window active : currentWindows) {
@@ -88,7 +87,7 @@ public class WindowClausePush extends TuplePush {
                     }
                 }
                 if (windowClause.getEndCondition() != null) {
-                    List<WindowClause.Window> removals = new ArrayList<WindowClause.Window>();
+                    List<WindowClause.Window> removals = new ArrayList<>();
                     for (WindowClause.Window w : currentWindows) {
                         if (!w.isFinished() && windowClause.matchesEnd(w, previous, current, next, position, context)) {
                             w.endItem = current;
@@ -126,20 +125,12 @@ public class WindowClausePush extends TuplePush {
      *
      * @param w       the window to be despatched
      * @param context the dynamic evaluation context
-     * @throws XPathException
+     * @throws XPathException if anything goes wrong, for example if the window contents have the wrong type
      */
 
     private void despatch(WindowClause.Window w, XPathContext context) throws XPathException {
 
-        // System.err.println("despatch window from " + w.startPosition + " to " + w.endPosition);
-
-        // Do a dynamic check of the window contents against the required type
-        // TODO: optimize this out based on knowledge of the type of the input sequence
-        SequenceType requiredType = windowClause.getVariableBinding(WindowClause.WINDOW_VAR).getRequiredType();
-        if (requiredType != SequenceType.ANY_SEQUENCE &&
-                !requiredType.matches(new SequenceExtent(w.contents), context.getConfiguration().getTypeHierarchy())) {
-            throw new XPathException("Contents of sliding/tumbling window do not match the required type", "XPTY0004");
-        }
+        windowClause.checkWindowContents(w);
 
         // In ordered mode, we must despatch windows in order of start position not in order of end position.
         // So we don't despatch it yet if there are unfinished windows with an earlier start position
