@@ -8,6 +8,7 @@
 package net.sf.saxon.expr.instruct;
 
 import net.sf.saxon.Configuration;
+import net.sf.saxon.event.PipelineConfiguration;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.om.FocusIterator;
@@ -383,6 +384,9 @@ public class AnalyzeString extends Instruction implements ContextOriginator {
         FocusIterator focusIter = c2.trackFocus(iter);
         c2.setCurrentRegexIterator(iter);
 
+        PipelineConfiguration pipe = c2.getReceiver().getPipelineConfiguration();
+        pipe.setXPathContext(c2);
+
         Item it;
         while ((it = focusIter.next()) != null) {
             if (iter.isMatching()) {
@@ -396,6 +400,7 @@ public class AnalyzeString extends Instruction implements ContextOriginator {
             }
         }
 
+        pipe.setXPathContext(context);
         return null;
 
     }
@@ -416,11 +421,8 @@ public class AnalyzeString extends Instruction implements ContextOriginator {
             String flagstr = getFlags().evaluateAsString(context).toString();
             String dialect = "XP30";
             StringValue regexString = (StringValue)getRegex().evaluateItem(context);
-            re = regexString.getPrecompiledRegularExpression();
-            if (re == null || !re.getFlags().equals(flagstr)) {
-                re = context.getConfiguration().compileRegularExpression(
+            re = context.getConfiguration().compileRegularExpression(
                         getRegex().evaluateAsString(context), flagstr, dialect, null);
-            }
             if ((dialect.equals("XP20") || !useXsltErrorCodes) && re.matches("")) {
                 dynamicError("The regular expression must not be one that matches a zero-length string",
                         useXsltErrorCodes ? "XTDE1150" : "FORX0003", context);
@@ -430,9 +432,6 @@ public class AnalyzeString extends Instruction implements ContextOriginator {
         return re.analyze(input);
     }
 
-    // TODO: use the new RegexStringValue mechanism for cases where the regex is known statically, to
-    // avoid having two separate mechanism.
-
     /**
      * Return an Iterator to iterate over the values of a sequence. The value of every
      * expression can be regarded as a sequence, so this method is supported for all
@@ -449,7 +448,7 @@ public class AnalyzeString extends Instruction implements ContextOriginator {
      */
 
     /*@NotNull*/
-    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<?> iterate(XPathContext context) throws XPathException {
         RegexIterator iter = getRegexIterator(context);
         XPathContextMajor c2 = context.newContext();
         c2.setOrigin(this);
