@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,7 +7,7 @@
 
 package net.sf.saxon.expr.instruct;
 
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.Receiver;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.ContextItemStaticInfo;
 import net.sf.saxon.expr.parser.ExpressionTool;
@@ -85,13 +85,12 @@ public final class Comment extends SimpleNodeConstructor {
      *
      * @param value   the string value of the new node
      * @param context the dynamic evaluation context
-     * @throws XPathException
+     * @throws XPathException if a dynamic error occurs
      */
 
     public void processValue(CharSequence value, XPathContext context) throws XPathException {
-        //String comment = expandChildren(context).toString();
         String comment = checkContent(value.toString(), context);
-        SequenceReceiver out = context.getReceiver();
+        Receiver out = context.getReceiver();
         out.comment(comment, getLocation(), 0);
     }
 
