@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -226,7 +226,6 @@ public class CopyOf extends Instruction implements ValidatingInstruction {
 
     /**
      * Say whether accumulator values should be copied from the source document
-     *
      * @param copy true if values should be copied
      */
 
@@ -470,7 +469,6 @@ public class CopyOf extends Instruction implements ValidatingInstruction {
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -509,7 +507,7 @@ public class CopyOf extends Instruction implements ValidatingInstruction {
 
     public TailCall processLeavingTail(XPathContext context) throws XPathException {
 
-        SequenceReceiver out = context.getReceiver();
+        Receiver out = context.getReceiver();
 
         if (copyAccumulators) {
             // Try to create a virtual copy if we can. This is cheaper
@@ -518,57 +516,42 @@ public class CopyOf extends Instruction implements ValidatingInstruction {
                 // Test case accumulators-070
                 // We have to create a physical copy because of the validation requirement, but this makes
                 // it difficult to copy the accumulator values.
-                SequenceIterator iter = getSelect().iterate(context);
-                Item item;
-                while ((item = iter.next()) != null) {
+                getSelect().iterate(context).forEachOrFail(item -> {
                     if (item instanceof NodeInfo) {
                         TinyBuilder builder = new TinyBuilder(out.getPipelineConfiguration());
                         builder.open();
-                        copyOneNode(context, new TreeReceiver(builder), (NodeInfo)item, CopyOptions.ALL_NAMESPACES);
+                        copyOneNode(context, new TreeReceiver(builder), (NodeInfo) item, CopyOptions.ALL_NAMESPACES);
                         builder.close();
-                        TinyNodeImpl copy = (TinyNodeImpl)builder.getCurrentRoot();
-                        copy.getTree().setCopiedFrom((NodeInfo)item);
+                        TinyNodeImpl copy = (TinyNodeImpl) builder.getCurrentRoot();
+                        copy.getTree().setCopiedFrom((NodeInfo) item);
                         out.append(copy);
                     } else {
                         out.append(item);
                     }
-                }
-
+                });
             } else {
                 // Use the iterate() method to create a virtual copy.
-                SequenceIterator iter = iterate(context);
-                Item item;
-                while ((item = iter.next()) != null) {
-                    out.append(item);
-                }
+                iterate(context).forEachOrFail(out::append);
             }
         } else {
 
-            int copyOptions = CopyOptions.TYPE_ANNOTATIONS;
-            if (copyNamespaces) {
-                copyOptions |= CopyOptions.ALL_NAMESPACES;
-            }
+            int copyOptions = CopyOptions.TYPE_ANNOTATIONS
+                    | (copyNamespaces ? CopyOptions.ALL_NAMESPACES : 0)
+                    | (copyForUpdate ? CopyOptions.FOR_UPDATE : 0);
 
-            if (copyForUpdate) {
-                copyOptions |= CopyOptions.FOR_UPDATE;
-            }
-            //int whichNamespaces = (copyNamespaces ? NodeInfo.ALL_NAMESPACES : NodeInfo.NO_NAMESPACES);
-
-            SequenceIterator iter = getSelect().iterate(context);
-            Item item;
-            while ((item = iter.next()) != null) {
+            getSelect().iterate(context).forEachOrFail(item -> {
                 if (item instanceof NodeInfo) {
                     copyOneNode(context, out, (NodeInfo) item, copyOptions);
-
                 } else {
-                    out.append(item, getLocation(), NodeInfo.ALL_NAMESPACES);
+                    out.append(item, getLocation(), ReceiverOptions.ALL_NAMESPACES);
                 }
-            }
+            });
+
         }
         return null;
     }
 
-    private void copyOneNode(XPathContext context, SequenceReceiver out, NodeInfo item, int copyOptions) throws XPathException {
+    private void copyOneNode(XPathContext context, Receiver out, NodeInfo item, int copyOptions) throws XPathException {
         Controller controller = context.getController();
         boolean copyBaseURI = out.getSystemId() == null;
         int kind = item.getNodeKind();
@@ -810,6 +793,7 @@ public class CopyOf extends Instruction implements ValidatingInstruction {
             if (err != null) {
                 err.setMessage("Attribute being copied does not match the required type. " +
                                        err.getMessage());
+                err.setErrorCode("XTTE1510");
                 throw err.makeException();
             }
             annotation = schemaType;
@@ -841,62 +825,61 @@ public class CopyOf extends Instruction implements ValidatingInstruction {
     }
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<?> iterate(XPathContext context) throws XPathException {
         final Controller controller = context.getController();
         assert controller != null;
         if (schemaType == null /*&& copyNamespaces*/ && !copyForUpdate) {
             if (validation == Validation.PRESERVE) {
                 // create a virtual copy of the underlying nodes
-                ItemMappingFunction copier = new ItemMappingFunction() {
-                    public Item mapItem(Item item) throws XPathException {
-                        if (item instanceof NodeInfo) {
-                            if (((NodeInfo) item).getTreeInfo().isTyped()) {
-                                if (!copyNamespaces && ((NodeInfo) item).getNodeKind() == Type.ELEMENT) {
-                                    // A lot of extra work here just to check for error XTTE0950, but the conditions are rare
-                                    Sink sink = new Sink(controller.makePipelineConfiguration());
-                                    ((NodeInfo) item).copy(sink, CopyOptions.TYPE_ANNOTATIONS, getLocation());
-                                }
-                                if (((NodeInfo) item).getNodeKind() == Type.ATTRIBUTE &&
-                                        ((SimpleType) ((NodeInfo) item).getSchemaType()).isNamespaceSensitive()) {
-                                    throw new XPathException("Cannot copy an attribute with namespace-sensitive content except as part of its containing element", "XTTE0950");
-                                }
+                ItemMappingFunction<Item<?>, Item<?>> copier = item -> {
+                    if (item instanceof NodeInfo) {
+                        if (((NodeInfo) item).getTreeInfo().isTyped()) {
+                            if (!copyNamespaces && ((NodeInfo) item).getNodeKind() == Type.ELEMENT) {
+                                // A lot of extra work here just to check for error XTTE0950, but the conditions are rare
+                                Sink sink = new Sink(controller.makePipelineConfiguration());
+                                ((NodeInfo) item).copy(sink, CopyOptions.TYPE_ANNOTATIONS, getLocation());
                             }
-                            VirtualCopy vc = VirtualCopy.makeVirtualCopy((NodeInfo) item);
-                            vc.setDropNamespaces(!copyNamespaces);
-                            vc.getTreeInfo().setCopyAccumulators(copyAccumulators);
-                            if (((NodeInfo) item).getNodeKind() == Type.ELEMENT) {
-                                vc.setSystemId(computeNewBaseUri((NodeInfo) item, getStaticBaseURIString()));
+                            if (((NodeInfo) item).getNodeKind() == Type.ATTRIBUTE &&
+                                    ((SimpleType) ((NodeInfo) item).getSchemaType()).isNamespaceSensitive()) {
+                                throw new XPathException("Cannot copy an attribute with namespace-sensitive content except as part of its containing element", "XTTE0950");
                             }
-                            return vc;
-                        } else {
-                            return item;
                         }
-                    }
-                };
-                return new ItemMappingIterator(getSelect().iterate(context), copier, true);
-            } else if (validation == Validation.STRIP) {
-                // create a virtual copy of the underlying nodes
-                ItemMappingFunction copier = new ItemMappingFunction() {
-                    public Item mapItem(Item item) {
-                        if (!(item instanceof NodeInfo)) {
-                            return item;
-                        }
-                        VirtualCopy vc = VirtualUntypedCopy.makeVirtualUntypedTree((NodeInfo) item, (NodeInfo) item);
-                        vc.getTreeInfo().setCopyAccumulators(copyAccumulators);
+                        VirtualCopy vc = VirtualCopy.makeVirtualCopy((NodeInfo) item);
                         vc.setDropNamespaces(!copyNamespaces);
+                        vc.getTreeInfo().setCopyAccumulators(copyAccumulators);
                         if (((NodeInfo) item).getNodeKind() == Type.ELEMENT) {
                             vc.setSystemId(computeNewBaseUri((NodeInfo) item, getStaticBaseURIString()));
                         }
                         return vc;
+                    } else {
+                        return item;
                     }
                 };
-                return new ItemMappingIterator(getSelect().iterate(context), copier, true);
+                return new ItemMappingIterator<>(getSelect().iterate(context), copier, true);
+            } else if (validation == Validation.STRIP) {
+                // create a virtual copy of the underlying nodes
+                ItemMappingFunction<Item<?>, Item<?>> copier = item -> {
+                    if (!(item instanceof NodeInfo)) {
+                        return item;
+                    }
+                    VirtualCopy vc = VirtualUntypedCopy.makeVirtualUntypedTree((NodeInfo) item, (NodeInfo) item);
+                    vc.getTreeInfo().setCopyAccumulators(copyAccumulators);
+                    vc.setDropNamespaces(!copyNamespaces);
+                    if (((NodeInfo) item).getNodeKind() == Type.ELEMENT) {
+                        vc.setSystemId(computeNewBaseUri((NodeInfo) item, getStaticBaseURIString()));
+                    }
+                    return vc;
+                };
+                return new ItemMappingIterator<>(getSelect().iterate(context), copier, true);
             }
         }
-        SequenceReceiver saved = context.getReceiver();
+        Receiver saved = context.getReceiver();
         PipelineConfiguration pipe = controller.makePipelineConfiguration();
-        pipe.setLocationIsCodeLocation(true);
+        pipe.setXPathContext(context);
         SequenceOutputter out = new SequenceOutputter(pipe);
+        if (copyForUpdate) {
+            out.setTreeModel(TreeModel.LINKED_TREE);
+        }
         pipe.setHostLanguage(getPackageData().getHostLanguage());
         context.setReceiver(out);
         try {
