@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,10 +9,7 @@ package net.sf.saxon.expr.instruct;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.Controller;
-import net.sf.saxon.event.Builder;
-import net.sf.saxon.event.ComplexContentOutputter;
-import net.sf.saxon.event.PipelineConfiguration;
-import net.sf.saxon.event.SequenceReceiver;
+import net.sf.saxon.event.*;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.ExpressionTool;
 import net.sf.saxon.expr.parser.RebindingMap;
@@ -41,7 +38,6 @@ import net.sf.saxon.value.UntypedAtomicValue;
  * instruction in XSLT. It is also used to support the document node constructor
  * expression in XQuery, and is generated implicitly within an xsl:variable
  * that constructs a temporary tree.
- * <p/>
  * <p>Conceptually it represents an XSLT instruction xsl:document-node,
  * with no attributes, whose content is a complex content constructor for the
  * children of the document node.</p>
@@ -238,7 +234,7 @@ public class DocumentInstr extends ParentNodeConstructor {
 
     public TailCall processLeavingTail(XPathContext context) throws XPathException {
         if (preservingTypes && !textOnly) {
-            SequenceReceiver out = context.getReceiver();
+            Receiver out = context.getReceiver();
             out.startDocument(0);
             getContentExpression().process(context);
             out.endDocument();
@@ -246,8 +242,8 @@ public class DocumentInstr extends ParentNodeConstructor {
         } else {
             Item item = evaluateItem(context);
             if (item != null) {
-                SequenceReceiver out = context.getReceiver();
-                out.append(item, getLocation(), NodeInfo.ALL_NAMESPACES);
+                Receiver out = context.getReceiver();
+                out.append(item, getLocation(), ReceiverOptions.ALL_NAMESPACES);
             }
             return null;
         }
@@ -278,9 +274,14 @@ public class DocumentInstr extends ParentNodeConstructor {
             root = new TextFragmentValue(controller.getConfiguration(), textValue, getStaticBaseURIString());
         } else {
             try {
-                SequenceReceiver saved = context.getReceiver();
+                Receiver saved = context.getReceiver();
+                PipelineConfiguration pipe = controller.makePipelineConfiguration();
+                pipe.setXPathContext(context);
+
+                Builder builder;
+                builder = controller.makeBuilder();
+                builder.setUseEventLocation(false);
 
-                Builder builder = controller.makeBuilder();
                 if (builder instanceof TinyBuilder) {
                     ((TinyBuilder) builder).setStatistics(Statistics.SOURCE_DOCUMENT_STATISTICS);
                 }
@@ -288,13 +289,10 @@ public class DocumentInstr extends ParentNodeConstructor {
                 builder.setBaseURI(getStaticBaseURIString());
                 builder.setTiming(false);
 
-                PipelineConfiguration pipe = controller.makePipelineConfiguration();
                 pipe.setHostLanguage(getPackageData().getHostLanguage());
-                pipe.setLocationIsCodeLocation(true);
-                //pipe.setBaseURI(baseURI);
                 builder.setPipelineConfiguration(pipe);
 
-                SequenceReceiver out = ComplexContentOutputter.makeComplexContentReceiver(builder, getValidationOptions());
+                Receiver out = ComplexContentOutputter.makeComplexContentReceiver(builder, getValidationOptions());
                 context.setReceiver(out);
                 out.open();
                 out.startDocument(0);
@@ -352,7 +350,7 @@ public class DocumentInstr extends ParentNodeConstructor {
         }
         final SchemaType schemaType = getSchemaType();
         if (schemaType != null) {
-            out.emitAttribute("type", schemaType.getEQName());
+            out.emitAttribute("type", schemaType.getStructuredQName());
         }
         getContentExpression().export(out);
         out.endElement();
