@@ -285,6 +285,29 @@ public class Executable {
         outputDeclarations.put(qName, properties);
     }
 
+    /**
+     * Get the properties for xsl:output. The object returned will
+     * be a clone of the internal values, and thus it can be mutated
+     * without mutating the Executable object.
+     * <p>The returned properties object is a new, empty, Properties object that is
+     * backed by the live properties to supply default values for missing properties.
+     * This means that the property values must be read using the getProperty() method.
+     * Calling the get() method on the underlying Hashtable will return null.</p>
+     * <p>This method gets the output properties for the unnamed output
+     * format in the stylesheet.</p>
+     *
+     * @return A Properties object reflecting the output properties defined
+     * for the default (unnamed) output format in the stylesheet. It may
+     * be mutated and supplied to the setOutputProperties() method of the
+     * Transformer, without affecting other transformations that use the
+     * same stylesheet.
+     * @see javax.xml.transform.Transformer#setOutputProperties
+     */
+
+    public Properties getOutputProperties() {
+        return new Properties(defaultOutputProperties);
+    }
+    
     /**
      * Get a named output format
      *
@@ -500,29 +523,33 @@ public class Executable {
      * @return a list of all the stylesheet parameters (XSLT) or external variables (XQuery)
      */
 
-    public Iterable<GlobalParam> getGlobalParameters() {
-        return globalParams.values();
+    public Map<StructuredQName, GlobalParam> getGlobalParameters() {
+        return globalParams;
     }
 
     /**
-     * Check that all required parameters have been supplied. Used in XSLT only.
+     * Get the global parameter with a given name, if there is one
+     * @param name the parameter name
+     * @return the global parameter with this name, if it exists, or null otherwise
+     */
+
+    public GlobalParam getGlobalParameter(StructuredQName name) {
+        return globalParams.get(name);
+    }
+
+    /**
+     * Check the supplied parameters. Checks that all required parameters have been supplied, and that
+     * no parameters supplied at run-time are declared static in the stylesheet, or were supplied
+     * eagerly at compile-time.
+     *
+     * <p>Used in XSLT only.</p>
      *
      * @param params the set of parameters that have been supplied (null represents an empty set)
      * @throws XPathException if there is a required parameter for which no value has been supplied
      */
 
-    public void checkAllRequiredParamsArePresent(GlobalParameterSet params) throws XPathException {
-        for (Map.Entry<StructuredQName, GlobalParam> entry : globalParams.entrySet()) {
-            if (entry.getValue().isRequiredParam()) {
-                StructuredQName req = entry.getKey();
-                if (params == null || params.get(req) == null) {
-                    XPathException err = new XPathException("No value supplied for required parameter " +
-                        req.getDisplayName());
-                    err.setErrorCode(getHostLanguage() == Configuration.XQUERY ? "XPDY0002" : "XTDE0050");
-                    throw err;
-                }
-            }
-        }
+    public void checkSuppliedParameters(GlobalParameterSet params) throws XPathException {
+
     }
 
 
