@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,10 +8,14 @@
 package net.sf.saxon.expr.instruct;
 
 import net.sf.saxon.Controller;
+import net.sf.saxon.event.PipelineConfiguration;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.lib.TraceListener;
-import net.sf.saxon.om.*;
+import net.sf.saxon.om.FocusIterator;
+import net.sf.saxon.om.Item;
+import net.sf.saxon.om.SequenceIterator;
+import net.sf.saxon.om.StandardNames;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.ErrorType;
@@ -27,7 +31,7 @@ import net.sf.saxon.value.Cardinality;
  * is known that the rhs delivers atomic values.
  */
 
-public class ForEach extends Instruction implements ContextMappingFunction, ContextSwitchingExpression {
+public class ForEach extends Instruction implements ContextMappingFunction<Item<?>>, ContextSwitchingExpression {
 
     protected boolean containsTailCall;
     protected Operand selectOp;
@@ -52,7 +56,7 @@ public class ForEach extends Instruction implements ContextMappingFunction, Cont
      * @param select           the select expression
      * @param action           the body of the xsl:for-each loop
      * @param containsTailCall true if the body of the loop contains a tail call on the containing function
-     * @param threads          if >1 causes multithreaded execution (Saxon-EE only)
+     * @param threads          if &gt;1 causes multithreaded execution (Saxon-EE only)
      */
 
     public ForEach(Expression select, Expression action, boolean containsTailCall, Expression threads) {
@@ -240,7 +244,7 @@ public class ForEach extends Instruction implements ContextMappingFunction, Cont
      * inference rules defined in the XSLT 3.0 specification.
      *
      * @return the static item type of the expression according to the XSLT 3.0 defined rules
-     * @param contextItemType
+     * @param contextItemType static type of the context item
      */
     @Override
     public UType getStaticUType(UType contextItemType) {
@@ -260,7 +264,7 @@ public class ForEach extends Instruction implements ContextMappingFunction, Cont
 
     public final boolean mayCreateNewNodes() {
         int props = getAction().getSpecialProperties();
-        return (props & StaticProperty.NON_CREATIVE) == 0;
+        return (props & StaticProperty.NO_NODES_NEWLY_CREATED) == 0;
     }
 
     /*@NotNull*/
@@ -303,7 +307,7 @@ public class ForEach extends Instruction implements ContextMappingFunction, Cont
         }
 
         if (threadsOp != null && !Literal.isEmptySequence(getThreads())) {
-            return visitor.getConfiguration().obtainOptimizer().generateMultithreadedInstruction(this);
+            return visitor.obtainOptimizer().generateMultithreadedInstruction(this);
         }
         return this;
     }
@@ -326,7 +330,6 @@ public class ForEach extends Instruction implements ContextMappingFunction, Cont
     /**
      * Add a representation of this expression to a PathMap. The PathMap captures a map of the nodes visited
      * by an expression in a source tree.
-     * <p/>
      * <p>The default implementation of this method assumes that an expression does no navigation other than
      * the navigation done by evaluating its subexpressions, and that the subexpressions are evaluated in the
      * same context as the containing expression. The method must be overridden for any expression
@@ -353,7 +356,7 @@ public class ForEach extends Instruction implements ContextMappingFunction, Cont
      * Copy an expression. This makes a deep copy.
      *
      * @return the copy of the original expression
-     * @param rebindings
+     * @param rebindings variables that need to be re-bound
      */
 
     /*@NotNull*/
@@ -364,25 +367,6 @@ public class ForEach extends Instruction implements ContextMappingFunction, Cont
         return f2;
     }
 
-    /**
-     * Compute the dependencies of an expression, as the union of the
-     * dependencies of its subexpressions. (This is overridden for path expressions
-     * and filter expressions, where the dependencies of a subexpression are not all
-     * propogated). This method should be called only once, to compute the dependencies;
-     * after that, getDependencies should be used.
-     *
-     * @return the depencies, as a bit-mask
-     */
-
-//    public int computeDependencies() {
-//        // Some of the dependencies aren't relevant. Note that the sort keys are absorbed into the select
-//        // expression.
-//        int dependencies = 0;
-//        dependencies |= getSelect().getDependencies();
-//        dependencies |= getAction().getDependencies() & ~StaticProperty.DEPENDS_ON_FOCUS;
-//        return dependencies;
-//    }
-
     /**
      * Get the static properties of this expression (other than its type). The result is
      * bit-signficant. These properties are used for optimizations. In general, if
@@ -406,6 +390,33 @@ public class ForEach extends Instruction implements ContextMappingFunction, Cont
         return (getAction() instanceof Instruction) && ((Instruction)getAction()).alwaysCreatesNewNodes();
     }
 
+    /**
+     * Determine whether this is an updating expression as defined in the XQuery update specification
+     *
+     * @return true if this is an updating expression
+     */
+
+    public boolean isUpdatingExpression() {
+        return getAction().isUpdatingExpression();
+    }
+
+    /**
+     * Check to ensure that this expression does not contain any inappropriate updating subexpressions.
+     * This check is overridden for those expressions that permit updating subexpressions.
+     *
+     * @throws XPathException if the expression has a non-permitted updating subexpression
+     */
+
+    public void checkForUpdatingSubexpressions() throws XPathException {
+        if (getSelect().isUpdatingExpression()) {
+            XPathException err = new XPathException(
+                    "Updating expression appears in a context where it is not permitted", "XUST0001");
+            err.setLocation(getSelect().getLocation());
+            throw err;
+        }
+    }
+
+
     /**
      * An implementation of Expression must provide at least one of the methods evaluateItem(), iterate(), or process().
      * This method indicates which of these methods is provided. This implementation provides both iterate() and
@@ -431,13 +442,13 @@ public class ForEach extends Instruction implements ContextMappingFunction, Cont
     public TailCall processLeavingTail(XPathContext context) throws XPathException {
         Controller controller = context.getController();
         assert controller != null;
-        FocusIterator iter = new FocusTrackingIterator(getSelect().iterate(context));
 
         XPathContextMajor c2 = context.newContext();
         c2.setOrigin(this);
-        c2.setCurrentIterator(iter);
+        FocusIterator<?> iter = c2.trackFocus(getSelect().iterate(context));
         c2.setCurrentTemplateRule(null);
 
+        Expression action = getAction();
         if (containsTailCall) {
             if (controller.isTracing()) {
                 TraceListener listener = controller.getTraceListener();
@@ -447,7 +458,7 @@ public class ForEach extends Instruction implements ContextMappingFunction, Cont
                     return null;
                 }
                 listener.startCurrentItem(item);
-                TailCall tc = ((TailCallReturner) getAction()).processLeavingTail(c2);
+                TailCall tc = ((TailCallReturner) action).processLeavingTail(c2);
                 listener.endCurrentItem(item);
                 return tc;
             } else {
@@ -455,23 +466,24 @@ public class ForEach extends Instruction implements ContextMappingFunction, Cont
                 if (item == null) {
                     return null;
                 }
-                return ((TailCallReturner) getAction()).processLeavingTail(c2);
+                return ((TailCallReturner) action).processLeavingTail(c2);
             }
         } else {
+            PipelineConfiguration pipe = c2.getReceiver().getPipelineConfiguration();
+            pipe.setXPathContext(c2);
             if (controller.isTracing()) {
                 TraceListener listener = controller.getTraceListener();
                 assert listener != null;
                 Item item;
                 while ((item = iter.next()) != null) {
                     listener.startCurrentItem(item);
-                    getAction().process(c2);
+                    action.process(c2);
                     listener.endCurrentItem(item);
                 }
             } else {
-                while (iter.next() != null) {
-                    getAction().process(c2);
-                }
+                iter.forEachOrFail(item -> action.process(c2));
             }
+            pipe.setXPathContext(context);
         }
         return null;
     }
@@ -487,11 +499,10 @@ public class ForEach extends Instruction implements ContextMappingFunction, Cont
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate(XPathContext context) throws XPathException {
-        FocusIterator master = new FocusTrackingIterator(getSelect().iterate(context));
+    public SequenceIterator<?> iterate(XPathContext context) throws XPathException {
         XPathContextMinor c2 = context.newMinorContext();
-        c2.setCurrentIterator(master);
-        return new ContextMappingIterator<Item>(this, c2);
+        c2.trackFocus(getSelect().iterate(context));
+        return new ContextMappingIterator<>(this, c2);
     }
 
     /**
@@ -503,10 +514,31 @@ public class ForEach extends Instruction implements ContextMappingFunction, Cont
      *         item maps to
      */
 
-    public SequenceIterator map(XPathContext context) throws XPathException {
+    public SequenceIterator<?> map(XPathContext context) throws XPathException {
         return getAction().iterate(context);
     }
 
+    /**
+     * Evaluate an updating expression, adding the results to a Pending Update List.
+     * The default implementation of this method, which is used for non-updating expressions,
+     * throws an UnsupportedOperationException
+     *
+     * @param context the XPath dynamic evaluation context
+     * @param pul     the pending update list to which the results should be written
+     * @throws net.sf.saxon.trans.XPathException if evaluation fails
+     * @throws UnsupportedOperationException     if the expression is not an updating expression
+     */
+
+    public void evaluatePendingUpdates(XPathContext context, PendingUpdateList pul) throws XPathException {
+        XPathContextMinor c2 = context.newMinorContext();
+        c2.trackFocus(getSelect().iterate(context));
+        SequenceIterator iter = c2.getCurrentIterator();
+        Item item;
+        while ((item = iter.next()) != null) {
+            getAction().evaluatePendingUpdates(c2, pul);
+        }
+    }
+
     /**
      * Diagnostic print of expression structure. The abstract expression tree
      * is written to the supplied output destination.
@@ -527,7 +559,6 @@ public class ForEach extends Instruction implements ContextMappingFunction, Cont
     /**
      * <p>The toString() method for an expression attempts to give a representation of the expression
      * in an XPath-like form.</p>
-     * <p/>
      * <p>For subclasses of Expression that represent XPath expressions, the result should always be a string that
      * parses as an XPath 3.0 expression</p>
      *
