@@ -8,6 +8,7 @@
 package net.sf.saxon.expr.instruct;
 
 import net.sf.saxon.Controller;
+import net.sf.saxon.event.PipelineConfiguration;
 import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.expr.sort.*;
@@ -35,7 +36,7 @@ import java.net.URISyntaxException;
  */
 
 public class ForEachGroup extends Instruction
-        implements SortKeyEvaluator, ContextMappingFunction<Item>, ContextSwitchingExpression {
+        implements SortKeyEvaluator, ContextMappingFunction<Item<?>>, ContextSwitchingExpression {
 
     public static final int GROUP_BY = 0;
     public static final int GROUP_ADJACENT = 1;
@@ -575,6 +576,7 @@ public class ForEachGroup extends Instruction
     public TailCall processLeavingTail(XPathContext context) throws XPathException {
         Controller controller = context.getController();
         assert controller != null;
+        PipelineConfiguration pipe = context.getReceiver().getPipelineConfiguration();
 
         GroupIterator groupIterator = getGroupIterator(context);
         XPathContextMajor c2 = context.newContext();
@@ -582,6 +584,7 @@ public class ForEachGroup extends Instruction
         FocusIterator focusIterator = c2.trackFocus(groupIterator);
         c2.setCurrentGroupIterator(groupIterator);
         c2.setCurrentTemplateRule(null);
+        pipe.setXPathContext(c2);
 
         if (controller.isTracing()) {
             TraceListener listener = controller.getTraceListener();
@@ -598,6 +601,7 @@ public class ForEachGroup extends Instruction
             }
         }
 
+        pipe.setXPathContext(context);
         return null;
     }
 
@@ -654,7 +658,7 @@ public class ForEachGroup extends Instruction
                     coll = getCollator(context);
                 }
                 XPathContext c2 = context.newMinorContext();
-                FocusIterator<? extends Item> population = c2.trackFocus(select.iterate(context));
+                FocusIterator<?> population = c2.trackFocus(select.iterate(context));
                 groupIterator = new GroupByIterator(population, getGroupingKey(), c2, coll, composite);
                 break;
             }
@@ -712,7 +716,7 @@ public class ForEachGroup extends Instruction
      */
 
     /*@NotNull*/
-    public SequenceIterator<? extends Item> iterate(XPathContext context) throws XPathException {
+    public SequenceIterator<?> iterate(XPathContext context) throws XPathException {
         GroupIterator master = getGroupIterator(context);
         XPathContextMajor c2 = context.newContext();
         c2.setOrigin(this);
@@ -732,7 +736,7 @@ public class ForEachGroup extends Instruction
      *         sequence.
      */
 
-    public SequenceIterator<? extends Item> map(XPathContext context) throws XPathException {
+    public SequenceIterator<?> map(XPathContext context) throws XPathException {
         return getActionExpression().iterate(context);
     }
 
