@@ -52,6 +52,7 @@ public class GlobalVariable extends Actor
     private GlobalVariable originalVariable;
     private int binderySlotNumber;
     private boolean isRequiredParam;
+    private boolean isStatic;
 
     /**
      * Create a global variable
@@ -104,6 +105,24 @@ public class GlobalVariable extends Actor
         return select;
     }
 
+    /**
+     * Say whether this variable is declared to be static
+     * @param declaredStatic true if the variable is declared with static="yes"
+     */
+
+    public void setStatic(boolean declaredStatic) {
+        isStatic = declaredStatic;
+    }
+
+    /**
+     * Ask whether this variable is declared to be static
+     * @return true if the variable is declared with static="yes"
+     */
+
+    public boolean isStatic() {
+        return this.isStatic;
+    }
+
     /**
      * Set the required type of this variable
      *
@@ -396,18 +415,14 @@ public class GlobalVariable extends Actor
     }
 
     /**
-     * Create a compiled representation of this global variable
+     * Create a compiled representation of this global variable. Used in XQuery only.
      *
      * @param exec the executable
      * @param slot the slot number allocated to this variable
      * @throws XPathException if compile-time errors are found.
      */
 
-    public void compile(/*@NotNull*/ Executable exec, int slot) throws XPathException {
-        // Not used in XSLT
-//        if (references.isEmpty()) {
-//            return;
-//        }
+    public void compile(Executable exec, int slot) throws XPathException {
         TypeHierarchy th = getConfiguration().getTypeHierarchy();
 
         setBinderySlotNumber(slot);
@@ -417,7 +432,7 @@ public class GlobalVariable extends Actor
         SequenceType type = getRequiredType();
         for (BindingReference ref : references) {
             ref.fixup(this);
-            GroundedValue constantValue = null;
+            GroundedValue<?> constantValue = null;
             int properties = 0;
             Expression select = getSelectExpression();
             if (select instanceof Literal && !(this instanceof GlobalParam)) {
@@ -480,7 +495,7 @@ public class GlobalVariable extends Actor
                     final ItemType itemType = value.getItemType();
                     final int cardinality = value.getCardinality();
                     setRequiredType(SequenceType.makeSequenceType(itemType, cardinality));
-                    GroundedValue constantValue = null;
+                    GroundedValue<?> constantValue = null;
                     if (value2 instanceof Literal) {
                         constantValue = ((Literal) value2).getValue();
                     }
@@ -589,7 +604,7 @@ public class GlobalVariable extends Actor
                 ((GlobalVariable) b).lookForCycles(referees, globalFunctionLibrary);
             }
         }
-        List<SymbolicName> flist = new ArrayList<SymbolicName>();
+        List<SymbolicName> flist = new ArrayList<>();
         ExpressionTool.gatherCalledFunctionNames(body, flist);
         for (SymbolicName s : flist) {
             XQueryFunction qf = globalFunctionLibrary.getDeclarationByKey(s);
@@ -611,7 +626,7 @@ public class GlobalVariable extends Actor
      * @throws XPathException if a dynamic error occurs during evaluation
      */
 
-    public GroundedValue getSelectValue(XPathContext context, Component target) throws XPathException {
+    public GroundedValue<?> getSelectValue(XPathContext context, Component target) throws XPathException {
         if (select == null) {
             throw new AssertionError("*** No select expression for global variable $" +
                     getVariableQName().getDisplayName() + "!!");
@@ -623,7 +638,8 @@ public class GlobalVariable extends Actor
                 XPathContextMajor c2 = context.newCleanContext();
                 c2.setOrigin(this);
                 if (target == null || target.getDeclaringPackage().isRootPackage()) {
-                    ManualIterator mi = new ManualIterator(context.getController().getGlobalContextItem());
+                    ManualIterator<?> mi =
+                            new ManualIterator<>(context.getController().getGlobalContextItem());
                     c2.setCurrentIterator(mi);
                 } else {
                     c2.setCurrentIterator(null);
@@ -635,7 +651,7 @@ public class GlobalVariable extends Actor
                 int savedOutputState = c2.getTemporaryOutputState();
                 c2.setTemporaryOutputState(StandardNames.XSL_VARIABLE);
                 c2.setCurrentOutputUri(null);
-                GroundedValue result;
+                GroundedValue<?> result;
                 if (indexed) {
                     result = c2.getConfiguration().makeSequenceExtent(select, FilterExpression.FILTERED, c2);
                 } else {
@@ -656,12 +672,12 @@ public class GlobalVariable extends Actor
      * Evaluate the variable
      */
 
-    public GroundedValue<? extends Item> evaluateVariable(XPathContext context) throws XPathException {
+    public GroundedValue<?> evaluateVariable(XPathContext context) throws XPathException {
         final Controller controller = context.getController();
         assert controller != null;
         final Bindery b = controller.getBindery(getPackageData());
 
-        final GroundedValue<? extends Item> v = b.getGlobalVariable(getBinderySlotNumber());
+        final GroundedValue<?> v = b.getGlobalVariable(getBinderySlotNumber());
 
         if (v != null) {
             return v;
@@ -704,11 +720,10 @@ public class GlobalVariable extends Actor
      * @throws XPathException if evaluation fails
      */
 
-    protected GroundedValue<? extends Item> actuallyEvaluate(XPathContext context, Component target) throws XPathException {
+    protected GroundedValue<?> actuallyEvaluate(XPathContext context, Component target) throws XPathException {
         final Controller controller = context.getController();
         assert controller != null;
         final Bindery b = controller.getBindery(getPackageData());
-        //System.err.println("EVAL GV " + this + " " + getVariableQName().getDisplayName() + " in slot " + getBinderySlotNumber());
 
         try {
             // This is the first reference to a global variable; try to evaluate it now.
@@ -723,7 +738,7 @@ public class GlobalVariable extends Actor
                 return b.getGlobalVariable(getBinderySlotNumber());
             }
 
-            GroundedValue value = getSelectValue(context, target);
+            GroundedValue<?> value = getSelectValue(context, target);
             if (indexed) {
                 value = controller.getConfiguration().obtainOptimizer().makeIndexedValue(value.iterate());
             }
