@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,6 +8,7 @@
 package net.sf.saxon.expr.number;
 
 import net.sf.saxon.lib.Numberer;
+import net.sf.saxon.regex.EmptyString;
 import net.sf.saxon.regex.UnicodeString;
 import net.sf.saxon.tree.util.FastStringBuffer;
 
@@ -88,7 +89,7 @@ public abstract class AbstractNumberer implements Numberer {
                                String ordinal) {
 
         return format(number, picture,
-                new RegularGroupFormatter(groupSize, groupSeparator, UnicodeString.EMPTY_STRING),
+                new RegularGroupFormatter(groupSize, groupSeparator, EmptyString.THE_INSTANCE),
                 letterValue, ordinal);
     }
 
@@ -412,7 +413,6 @@ public abstract class AbstractNumberer implements Numberer {
         return sb.toString();
     }
 
-
     /**
      * Construct the ordinal suffix for a number, for example "st", "nd", "rd". The default
      * (language-neutral) implementation returns a zero-length string
