@@ -297,7 +297,8 @@ public class ExpressionTool {
      *                        expression
      */
 
-    public static Sequence evaluate(Expression exp, int evaluationMode, XPathContext context, int ref)
+    public static Sequence<?> evaluate(
+            Expression exp, int evaluationMode, XPathContext context, int ref)
             throws XPathException {
         switch (evaluationMode) {
 
@@ -326,7 +327,7 @@ public class ExpressionTool {
                 return EmptySequence.getInstance();
 
             case EVALUATE_AND_MATERIALIZE_VARIABLE:
-                Sequence<? extends Item> v = ((VariableReference) exp).evaluateVariable(context);
+                Sequence<?> v = ((VariableReference) exp).evaluateVariable(context);
                 if (v instanceof Closure) {
                     return v.iterate().materialize();
                 } else {
@@ -358,10 +359,10 @@ public class ExpressionTool {
             case LAZY_TAIL_EXPRESSION: {
                 TailExpression tail = (TailExpression) exp;
                 VariableReference vr = (VariableReference) tail.getBaseExpression();
-                Sequence base = evaluate(vr, EVALUATE_VARIABLE, context, ref);
+                Sequence<?> base = evaluate(vr, EVALUATE_VARIABLE, context, ref);
                 if (base instanceof MemoClosure) {
-                    SequenceIterator it = base.iterate();
-                    base = ((SequenceIterator<Item>) it).materialize();
+                    SequenceIterator<?> it = base.iterate();
+                    base = it.materialize();
                 }
                 if (base instanceof IntegerRange) {
                     long start = ((IntegerRange) base).getStart() + tail.getStart() - 1;
@@ -423,7 +424,7 @@ public class ExpressionTool {
      *                        expression
      */
 
-    public static Sequence lazyEvaluate(Expression exp, XPathContext context, boolean repeatable) throws XPathException {
+    public static Sequence<?> lazyEvaluate(Expression exp, XPathContext context, boolean repeatable) throws XPathException {
         Evaluator evaluator = lazyEvaluator(exp, repeatable);
         return evaluator.evaluate(exp, context);
     }
@@ -438,7 +439,7 @@ public class ExpressionTool {
      *                                           expression
      */
 
-    public static GroundedValue<? extends Item> eagerEvaluate(Expression exp, XPathContext context) throws XPathException {
+    public static GroundedValue<?> eagerEvaluate(Expression exp, XPathContext context) throws XPathException {
         Evaluator evaluator = eagerEvaluator(exp);
         return evaluator.evaluate(exp, context).materialize();
     }
@@ -761,7 +762,8 @@ public class ExpressionTool {
      * @throws XPathException if a dynamic error occurs
      */
 
-    public static SequenceIterator getIteratorFromProcessMethod(Expression exp, XPathContext context) throws XPathException {
+    public static SequenceIterator<?> getIteratorFromProcessMethod(
+            Expression exp, XPathContext context) throws XPathException {
         Controller controller = context.getController();
         assert controller != null;
         Receiver saved = context.getReceiver();
