@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -16,9 +16,8 @@ import java.util.List;
 
 /**
  * Tokenizer for expressions and inputs.
- * <p/>
- * This code was originally derived from James Clark's xt, though it has been greatly modified since.
- * See copyright notice at end of file.
+ * <p>This code was originally derived from James Clark's xt, though it has been greatly modified since.
+ * See copyright notice at end of file.</p>
  */
 
 
@@ -691,9 +690,8 @@ public final class Tokenizer {
                     boolean allowE = true;
                     boolean allowSign = false;
                     boolean allowDot = true;
-                    boolean endOfNum = false;
                     numloop:
-                    while (!endOfNum) {
+                    while (true) {
                         switch (c) {
                             case '0':
                             case '1':
@@ -826,6 +824,8 @@ public final class Tokenizer {
                 case '\r':
                     nextTokenStartOffset = inputOffset;
                     break;
+                case '\u00B6':
+
                 case 'Q':
                     if (inputOffset < inputLength && input.charAt(inputOffset) == '{') {
                         // EQName, revised syntax as per bug 15399
@@ -835,6 +835,9 @@ public final class Tokenizer {
                         }
                         String uri = input.substring(inputOffset, close);
                         uri = Whitespace.collapseWhitespace(uri).toString(); // Bug 29708
+                        if (uri.contains("{")) {
+                            throw new XPathException("EQName must not contain opening brace");
+                        }
                         inputOffset = close + 1;
                         int start = inputOffset;
                         boolean isStar = false;
@@ -1035,7 +1038,7 @@ public final class Tokenizer {
                     return Token.MODIFY;
                 }
                 if (s.equals("orElse") && allowSaxonExtensions) {
-                    return Token.MODIFY;
+                    return Token.OR_ELSE;
                 }
                 break;
             case 7:
