@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,23 +8,27 @@
 package net.sf.saxon.expr.sort;
 
 import net.sf.saxon.expr.LastPositionFinder;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.om.SequenceIterator;
+import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.ListIterator;
 
 import java.util.ArrayList;
+import java.util.List;
 
 /**
  * DocumentOrderIterator takes as input an iteration of nodes in any order, and
  * returns as output an iteration of the same nodes in document order, eliminating
- * any duplicates.
+ * any duplicates. An error occurs if the input sequence contains items that are
+ * not nodes.
  */
 
-public final class DocumentOrderIterator implements SequenceIterator, Sortable {
+public final class DocumentOrderIterator implements SequenceIterator<NodeInfo>, Sortable {
 
-    private SequenceIterator iterator;
-    private ArrayList<NodeInfo> sequence;
+    private SequenceIterator<NodeInfo> iterator;
+    private List<NodeInfo> sequence;
     private ItemOrderComparer comparer;
     private NodeInfo current = null;
 
@@ -32,26 +36,31 @@ public final class DocumentOrderIterator implements SequenceIterator, Sortable {
      * Iterate over a sequence in document order.
      * @param base the input sequence to be sorted
      * @param comparer the comparer used for comparing node positions
-     * @throws XPathException if the input sequence cannot be evaluated
+     * @throws XPathException if the input sequence cannot be evaluated, or if it contains an item
+     * that is not a node
      */
 
-    public DocumentOrderIterator(SequenceIterator base, ItemOrderComparer comparer) throws XPathException {
+    public DocumentOrderIterator(SequenceIterator<?> base, ItemOrderComparer comparer) throws XPathException {
 
         this.comparer = comparer;
 
         int len = (base.getProperties() & SequenceIterator.LAST_POSITION_FINDER) == 0
                 ? 50
                 : ((LastPositionFinder) base).getLength();
-        sequence = new ArrayList<NodeInfo>(len);
-        NodeInfo node;
-        while ((node = (NodeInfo)base.next()) != null) {
-            sequence.add(node);
-        }
-        //System.err.println("sort into document order: sequence length = " + sequence.getLength());
+        sequence = new ArrayList<>(len);
+        base.forEachOrFail(item -> {
+            if (item instanceof NodeInfo) {
+                sequence.add((NodeInfo) item);
+            } else {
+                throw new XPathException("Item in input for sorting is not a node: " + Err.depict(item), "XPTY0004");
+            }
+        });
+
+        //System.err.println("SORT into document order: sequence length = " + sequence.size());
         if (sequence.size() > 1) {
             GenericSorter.quickSort(0, sequence.size(), this);
         }
-        iterator = new ListIterator(sequence);
+        iterator = new ListIterator<>(sequence);
     }
 
     /**
@@ -67,7 +76,7 @@ public final class DocumentOrderIterator implements SequenceIterator, Sortable {
      */
 
     public int compare(int a, int b) {
-        //System.err.println("compare " + a + " with " + b);
+        //System.err.println("compare " + a + " with " + b + " = " + comparer.compare(sequence.get(a), sequence.get(b)));
         return comparer.compare(sequence.get(a), sequence.get(b));
     }
 
@@ -86,12 +95,12 @@ public final class DocumentOrderIterator implements SequenceIterator, Sortable {
 
     public NodeInfo next() throws XPathException {
         while (true) {
-            NodeInfo next = (NodeInfo)iterator.next();
+            NodeInfo next = iterator.next();
             if (next == null) {
                 current = null;
                 return null;
             }
-            if (current == null || !next.isSameNodeInfo(current)) {
+            if (!next.equals(current)) {
                 current = next;
                 return current;
             }
