@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,6 +11,7 @@ import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.lib.StringCollator;
 import net.sf.saxon.om.StandardNames;
 import net.sf.saxon.trans.NoDynamicContextException;
+import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.BuiltInAtomicType;
 import net.sf.saxon.type.Type;
 import net.sf.saxon.value.AtomicValue;
@@ -146,7 +147,7 @@ public class GenericAtomicComparer implements AtomicComparer {
      *          interface.
      * @param b the second object to be compared. This must be comparable with the first object: for
      *          example, if one is a string, they must both be strings.
-     * @return <0 if a < b, 0 if a = b, >0 if a > b
+     * @return &lt;0 if a &lt; b, 0 if a = b, &gt;0 if a &gt; b
      * @throws ClassCastException        if the objects are not comparable
      * @throws NoDynamicContextException if this comparer required access to dynamic context information,
      *                                   notably the implicit timezone, and this information is not available. In general this happens if a
@@ -175,8 +176,9 @@ public class GenericAtomicComparer implements AtomicComparer {
             Comparable ac = (Comparable) a.getXPathComparable(true, collator, implicitTimezone);
             Comparable bc = (Comparable) b.getXPathComparable(true, collator, implicitTimezone);
             if (ac == null || bc == null) {
-                throw new ClassCastException("Objects are not comparable (" +
-                        Type.displayTypeName(a) + ", " + Type.displayTypeName(b) + ')');
+                XPathException e = new XPathException("Objects are not comparable (" +
+                        Type.displayTypeName(a) + ", " + Type.displayTypeName(b) + ')', "XPTY0004");
+                throw new ComparisonException(e);
             } else {
                 return ac.compareTo(bc);
             }
@@ -192,7 +194,7 @@ public class GenericAtomicComparer implements AtomicComparer {
      *          collator is used to compare the values, otherwise the value must implement the equals() method.
      * @param b the second object to be compared. This must be comparable with the first object: for
      *          example, if one is a string, they must both be strings.
-     * @return <0 if a<b, 0 if a=b, >0 if a>b
+     * @return true if the values are equal, false if not
      * @throws ClassCastException if the objects are not comparable
      */
 
@@ -223,6 +225,19 @@ public class GenericAtomicComparer implements AtomicComparer {
     public String save() {
         return "GAC|" + collator.getCollationURI();
     }
+
+    @Override
+    public int hashCode() {
+        return collator.hashCode();
+    }
+
+    @Override
+    public boolean equals(Object obj) {
+        // In considering whether two GenericAtomicComparers are equal, we ignore the dynamic context, because this
+        // is only ever used to test the implicit timezone, and in all reasonable scenarios, the implicit timezone
+        // is global.
+        return obj instanceof GenericAtomicComparer && collator.equals(((GenericAtomicComparer)obj).collator);
+    }
 }
 
 
