@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -19,6 +19,7 @@ import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.StringConverter;
 import net.sf.saxon.type.ValidationFailure;
+import net.sf.saxon.value.SequenceType;
 import net.sf.saxon.value.StringValue;
 import net.sf.saxon.value.Whitespace;
 
@@ -26,13 +27,13 @@ import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Objects;
 import java.util.Properties;
 
 /**
- * A SortKeyDefinition defines one component of a sort key. <BR>
- * <p/>
- * Note that most attributes defining the sort key can be attribute value templates,
- * and can therefore vary from one invocation to another. We hold them as expressions.
+ * A SortKeyDefinition defines one component of a sort key.
+ * <p>Note that most attributes defining the sort key can be attribute value templates,
+ * and can therefore vary from one invocation to another. We hold them as expressions.</p>
  */
 
 // TODO: optimise also for the case where the attributes depend only on global variables
@@ -75,7 +76,7 @@ public class SortKeyDefinition extends PseudoExpression {
     /**
      * Ask whether the expression can be lifted out of a loop, assuming it has no dependencies
      * on the controlling variable/focus of the loop
-     * @param forStreaming
+     * @param forStreaming true if we are optimizing for streamed evaluation
      */
 
     public boolean isLiftable(boolean forStreaming) {
@@ -92,7 +93,15 @@ public class SortKeyDefinition extends PseudoExpression {
      */
 
     public void setSortKey(Expression exp, boolean setContext) {
-        sortKey = new Operand(this, exp, setContext ? OperandRole.FOCUS_CONTROLLED_ACTION : OperandRole.ATOMIC_SEQUENCE);
+        OperandRole opRole;
+        if (setContext) {
+            opRole = new OperandRole(OperandRole.HAS_SPECIAL_FOCUS_RULES | OperandRole.HIGHER_ORDER,
+                                     OperandUsage.TRANSMISSION,
+                                     SequenceType.ANY_SEQUENCE);
+        } else {
+            opRole = OperandRole.ATOMIC_SEQUENCE;
+        }
+        sortKey = new Operand(this, exp, opRole);
         setContextForSortKey = setContext;
     }
 
@@ -374,7 +383,9 @@ public class SortKeyDefinition extends PseudoExpression {
      * Copy this SortKeyDefinition
      *
      * @return a copy of this SortKeyDefinition
-     * @param rm
+     * @param rm a mutable list of (old binding, new binding) pairs
+     *                     that is used to update the bindings held in any
+     *                     local variable references that are copied.
      */
 
     public SortKeyDefinition copy(RebindingMap rm) {
@@ -409,7 +420,7 @@ public class SortKeyDefinition extends PseudoExpression {
      */
 
     public SortKeyDefinition typeCheck(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType) throws XPathException {
-        for (Operand o : operands()) {
+        for (Operand o : checkedOperands()) {
             if (o.hasSameFocus()) {
                 o.typeCheck(visitor, contextItemType);
             }
@@ -417,7 +428,7 @@ public class SortKeyDefinition extends PseudoExpression {
         }
         Expression lang = getLanguage();
         if (lang instanceof StringLiteral && ((StringLiteral) lang).getStringValue().length() != 0) {
-            ValidationFailure vf = StringConverter.STRING_TO_LANGUAGE.validate(((StringLiteral) lang).getStringValue());
+            ValidationFailure vf = StringConverter.StringToLanguage.INSTANCE.validate(((StringLiteral) lang).getStringValue());
             if (vf != null) {
                 throw new XPathException("The lang attribute of xsl:sort must be a valid language code", "XTDE0030");
             }
@@ -427,7 +438,7 @@ public class SortKeyDefinition extends PseudoExpression {
 
     @Override
     public Iterable<Operand> operands() {
-        List<Operand> list = new ArrayList<Operand>(8);
+        List<Operand> list = new ArrayList<>(8);
         list.add(sortKey);
         list.add(order);
         if (dataTypeExpression != null) {
@@ -507,12 +518,12 @@ public class SortKeyDefinition extends PseudoExpression {
             boolean firstParam = true;
             Properties props = new Properties();
             if (languageX.length() != 0) {
-                ValidationFailure vf = StringConverter.STRING_TO_LANGUAGE.validate(languageX);
+                ValidationFailure vf = StringConverter.StringToLanguage.INSTANCE.validate(languageX);
                 if (vf != null) {
                     throw new XPathException("The lang attribute of xsl:sort must be a valid language code", "XTDE0030");
                 }
                 props.setProperty("lang", languageX);
-                uri += (firstParam ? "?" : ";") + "lang=" + languageX;
+                uri += "?lang=" + languageX;
                 firstParam = false;
             }
             if (!caseOrderX.equals("#default")) {
@@ -532,20 +543,21 @@ public class SortKeyDefinition extends PseudoExpression {
             }
         } else {
             String dataType = dataTypeExpression.getChildExpression().evaluateAsString(context).toString();
-            if (dataType.equals("text")) {
-                atomicComparer = AtomicSortComparer.makeSortComparer(stringCollator,
-                        StandardNames.XS_STRING, context);
-                atomicComparer = new TextComparer(atomicComparer);
-            } else if (dataType.equals("number")) {
-                if (context.getConfiguration().getXsdVersion() == Configuration.XSD10) {
-                    atomicComparer = NumericComparer.getInstance();
-                } else {
-                    atomicComparer = NumericComparer11.getInstance();
-                }
-            } else {
-                XPathException err = new XPathException("data-type on xsl:sort must be 'text' or 'number'");
-                err.setErrorCode("XTDE0030");
-                throw err;
+            switch (dataType) {
+                case "text":
+                    atomicComparer = AtomicSortComparer.makeSortComparer(stringCollator,
+                                                                         StandardNames.XS_STRING, context);
+                    atomicComparer = new TextComparer(atomicComparer);
+                    break;
+                case "number":
+                    atomicComparer = context.getConfiguration().getXsdVersion() == Configuration.XSD10
+                            ? NumericComparer.getInstance()
+                            : NumericComparer11.getInstance();
+                    break;
+                default:
+                    XPathException err = new XPathException("data-type on xsl:sort must be 'text' or 'number'");
+                    err.setErrorCode("XTDE0030");
+                    throw err;
             }
         }
 
@@ -561,14 +573,15 @@ public class SortKeyDefinition extends PseudoExpression {
             }
         }
 
-        if (orderX.equals("ascending")) {
-            return atomicComparer;
-        } else if (orderX.equals("descending")) {
-            return new DescendingComparer(atomicComparer);
-        } else {
-            XPathException err1 = new XPathException("order must be 'ascending' or 'descending'");
-            err1.setErrorCode("XTDE0030");
-            throw err1;
+        switch (orderX) {
+            case "ascending":
+                return atomicComparer;
+            case "descending":
+                return new DescendingComparer(atomicComparer);
+            default:
+                XPathException err1 = new XPathException("order must be 'ascending' or 'descending'");
+                err1.setErrorCode("XTDE0030");
+                throw err1;
         }
     }
 
@@ -576,7 +589,6 @@ public class SortKeyDefinition extends PseudoExpression {
      * Set the comparator which is used to compare two values according to this sort key. The comparator makes the final
      * decision whether one value sorts before or after another: this takes into account the data type, the collation,
      * whether empty comes first or last, whether the sort order is ascending or descending.
-     * <p/>
      * <p>This method is called at compile time if all these factors are known at compile time.
      * It must not be called at run-time, except to reconstitute a finalComparator that has been
      * lost by virtue of serialization .</p>
@@ -631,35 +643,25 @@ public class SortKeyDefinition extends PseudoExpression {
     public boolean equals(Object other) {
         if (other instanceof SortKeyDefinition) {
             SortKeyDefinition s2 = (SortKeyDefinition) other;
-            return eq(getSortKey(), s2.getSortKey()) &&
-                    eq(getOrder(), s2.getOrder()) &&
-                    eq(getLanguage(), s2.getLanguage()) &&
-                    eq(getDataTypeExpression(), s2.getDataTypeExpression()) &&
-                    eq(getStable(), s2.getStable()) &&
-                    eq(getCollationNameExpression(), s2.getCollationNameExpression());
+            return Objects.equals(getSortKey(), s2.getSortKey()) &&
+                    Objects.equals(getOrder(), s2.getOrder()) &&
+                    Objects.equals(getLanguage(), s2.getLanguage()) &&
+                    Objects.equals(getDataTypeExpression(), s2.getDataTypeExpression()) &&
+                    Objects.equals(getStable(), s2.getStable()) &&
+                    Objects.equals(getCollationNameExpression(), s2.getCollationNameExpression());
         } else {
             return false;
         }
     }
 
-    private static boolean eq(Expression a, Expression b) {
-        if (a == null) {
-            return b == null;
-        } else {
-            return a.equals(b);
-        }
-    }
-
     /**
      * Get a hashcode to reflect the equals() method
      *
      * @return a hashcode based sortkey attribute values.
      */
 
-    @Override
     public int computeHashCode() {
         int h = 0;
-        h ^= getSortKey().hashCode();
         h ^= getOrder().hashCode();
         h ^= getCaseOrder().hashCode();
         h ^= getLanguage().hashCode();
