@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,7 +13,6 @@ import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.parser.ContextItemStaticInfo;
 import net.sf.saxon.expr.parser.ExpressionVisitor;
 import net.sf.saxon.expr.sort.AtomicComparer;
-import net.sf.saxon.om.AbstractItem;
 import net.sf.saxon.om.AtomicSequence;
 import net.sf.saxon.om.Function;
 import net.sf.saxon.query.AnnotationList;
@@ -25,7 +24,7 @@ import java.util.Arrays;
 /**
  * Abstract superclass (and factory class) for implementations of Function
  */
-public abstract class AbstractFunction extends AbstractItem implements Function {
+public abstract class AbstractFunction implements Function {
 
     /**
      * Get the roles of the arguments, for the purposes of streaming
@@ -120,6 +119,18 @@ public abstract class AbstractFunction extends AbstractItem implements Function
         // default: no action
     }
 
+    /**
+     * Prepare an XPathContext object for evaluating the function
+     *
+     * @param callingContext the XPathContext of the function calling expression
+     * @return a suitable context for evaluating the function (which may or may
+     * not be the same as the caller's context)
+     */
+
+    public XPathContext makeNewContext(XPathContext callingContext) {
+        return callingContext;
+    }
+
     /**
      * Test whether this FunctionItem is deep-equal to another function item,
      * under the rules of the deep-equal function
@@ -134,13 +145,7 @@ public abstract class AbstractFunction extends AbstractItem implements Function
      * Output information about this function item to the diagnostic explain() output
      */
     public void export(ExpressionPresenter out) throws XPathException {
-        out.startElement("abstractFunction");
-        out.emitAttribute("class", getClass().getSimpleName());
-        if (getFunctionName() != null) {
-            out.emitAttribute("name", getFunctionName());
-        }
-        out.emitAttribute("arity", getArity() + "");
-        out.endElement();
+        throw new UnsupportedOperationException("export() not implemented for " + this.getClass());
     }
 
     /**
