@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -30,7 +30,7 @@ public class BaseUri_1 extends SystemFunction implements Callable {
         if (s == null) {
             return ZeroOrOne.empty();
         }
-        return new ZeroOrOne<AnyURIValue>(new AnyURIValue(s));
+        return new ZeroOrOne<>(new AnyURIValue(s));
 
     }
 
