@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,10 +11,8 @@ import net.sf.saxon.Configuration;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.sort.AtomicComparer;
 import net.sf.saxon.expr.sort.GenericAtomicComparer;
-import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.om.*;
 import net.sf.saxon.pattern.SameNameTest;
-import net.sf.saxon.trans.NoDynamicContextException;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.iter.AtomicIterator;
 import net.sf.saxon.tree.iter.AxisIterator;
@@ -156,31 +154,25 @@ public class DeepEqual extends CollatingFunctionFixed {
                 }
 
                 if (item1 instanceof Function || item2 instanceof Function) {
+                    if (!(item1 instanceof Function && item2 instanceof Function)) {
+                        reason = "if one item is a function then both must be functions (position " + pos1 + ")";
+                        return false;
+                    }
                     // two maps or arrays can be deep-equal
-                    //XPathContext context = new EarlyEvaluationContext(config, config.getCollationMap());
                     boolean fe = ((Function) item1).deepEquals((Function) item2, context, comparer, flags);
                     if (!fe) {
                         result = false;
-                        reason = "maps at position " + pos1 + " differ";
+                        reason = "functions at position " + pos1 + " differ";
                         break;
                     }
-                    return fe;
+                    continue;
                 }
 
                 if (item1 instanceof ObjectValue || item2 instanceof ObjectValue) {
-                    if (item1 instanceof ObjectValue && item2 instanceof ObjectValue) {
-                        boolean oe = item1.equals(item2);
-                        if (!oe) {
-                            result = false;
-                            reason = "external objects at position " + pos1 + " differ";
-                            break;
-                        }
-                        return oe;
-                    } else {
-                        result = false;
-                        reason = "external object at position " + pos1;
-                        break;
-                    }
+                    if (!item1.equals(item2)) {
+                        return false;
+                    };
+                    continue;
                 }
 
                 if (item1 instanceof NodeInfo) {
@@ -220,15 +212,6 @@ public class DeepEqual extends CollatingFunctionFixed {
             //err.printStackTrace();
             result = false;
             reason = "sequences contain non-comparable values";
-        } catch (NoDynamicContextException err) {
-            throw err;
-        } catch (XPathException err) {
-            // comparison errors are masked
-            if ("FOTY0015".equals(err.getErrorCodeLocalPart()) && NamespaceConstant.ERR.equals(err.getErrorCodeNamespace())) {
-                throw err;
-            }
-            result = false;
-            reason = "error occurred while comparing two values (" + err.getMessage() + ')';
         }
 
         if (!result) {
@@ -249,7 +232,7 @@ public class DeepEqual extends CollatingFunctionFixed {
                                       AtomicComparer comparer, Configuration config, int flags)
             throws XPathException {
         // shortcut: a node is always deep-equal to itself
-        if (n1.isSameNodeInfo(n2)) {
+        if (n1.equals(n2)) {
             return true;
         }
 
@@ -261,8 +244,8 @@ public class DeepEqual extends CollatingFunctionFixed {
         switch (n1.getNodeKind()) {
             case Type.ELEMENT:
                 if (!Navigator.haveSameName(n1, n2)) {
-                    explain(config, "element names differ: " + NameOfNode.makeName(n1).getStructuredQName().getClarkName() +
-                            " != " + NameOfNode.makeName(n2).getStructuredQName().getClarkName(), flags, n1, n2);
+                    explain(config, "element names differ: " + NameOfNode.makeName(n1).getStructuredQName().getEQName() +
+                            " != " + NameOfNode.makeName(n2).getStructuredQName().getEQName(), flags, n1, n2);
                     return false;
                 }
                 if (((flags & INCLUDE_PREFIXES) != 0) && !n1.getPrefix().equals(n2.getPrefix())) {
@@ -285,20 +268,20 @@ public class DeepEqual extends CollatingFunctionFixed {
 
                     if (att2 == null) {
                         explain(config, "one element has an attribute " +
-                            NameOfNode.makeName(att1).getStructuredQName().getClarkName() +
+                            NameOfNode.makeName(att1).getStructuredQName().getEQName() +
                                 ", the other does not", flags, n1, n2);
                         return false;
                     }
                     if (!deepEquals(att1, att2, comparer, config, flags)) {
                         deepEquals(att1, att2, comparer, config, flags);
                         explain(config, "elements have different values for the attribute " +
-                            NameOfNode.makeName(att1).getStructuredQName().getClarkName(), flags, n1, n2);
+                            NameOfNode.makeName(att1).getStructuredQName().getEQName(), flags, n1, n2);
                         return false;
                     }
                 }
                 if ((flags & INCLUDE_NAMESPACES) != 0) {
-                    HashSet<NamespaceBinding> ns1 = new HashSet<NamespaceBinding>(10);
-                    HashSet<NamespaceBinding> ns2 = new HashSet<NamespaceBinding>(10);
+                    HashSet<NamespaceBinding> ns1 = new HashSet<>(10);
+                    HashSet<NamespaceBinding> ns2 = new HashSet<>(10);
                     AxisIterator it1 = n1.iterateAxis(AxisInfo.NAMESPACE);
                     NodeInfo nn1;
                     while ((nn1 = it1.next()) != null) {
@@ -350,7 +333,8 @@ public class DeepEqual extends CollatingFunctionFixed {
                         explain(config, "one element has a simple type, the other does not", flags, n1, n2);
                         return false;
                     }
-                    if (isSimple1 && isSimple2) {
+                    if (isSimple1) {
+                        assert isSimple2;
                         final AtomicIterator v1 = n1.atomize().iterate();
                         final AtomicIterator v2 = n2.atomize().iterate();
                         return deepEqual(v1, v2, comparer, config.getConversionContext(), flags);
@@ -401,8 +385,8 @@ public class DeepEqual extends CollatingFunctionFixed {
             case Type.ATTRIBUTE:
                 if (!Navigator.haveSameName(n1, n2)) {
                     explain(config, "attribute names differ: " +
-                        NameOfNode.makeName(n1).getStructuredQName().getClarkName() +
-                            " != " + NameOfNode.makeName(n1).getStructuredQName().getClarkName(), flags, n1, n2);
+                        NameOfNode.makeName(n1).getStructuredQName().getEQName() +
+                            " != " + NameOfNode.makeName(n1).getStructuredQName().getEQName(), flags, n1, n2);
                     return false;
                 }
                 if (((flags & INCLUDE_PREFIXES) != 0) && !n1.getPrefix().equals(n2.getPrefix())) {
@@ -537,7 +521,7 @@ public class DeepEqual extends CollatingFunctionFixed {
     }
 
     private static SequenceIterator mergeAdjacentTextNodes(SequenceIterator in) throws XPathException {
-        List<Item> items = new ArrayList<Item>(20);
+        List<Item> items = new ArrayList<>(20);
         boolean prevIsText = false;
         FastStringBuffer textBuffer = new FastStringBuffer(FastStringBuffer.C64);
         while (true) {
@@ -566,7 +550,7 @@ public class DeepEqual extends CollatingFunctionFixed {
             textNode.setStringValue(textBuffer.toString()); // must copy the buffer before reusing it
             items.add(textNode);
         }
-        SequenceExtent extent = new SequenceExtent(items);
+        SequenceExtent extent = new SequenceExtent<>(items);
         return extent.iterate();
     }
 
@@ -577,7 +561,7 @@ public class DeepEqual extends CollatingFunctionFixed {
      * @param arguments the values of the arguments, supplied as Sequences.
      * @return the result of the evaluation, in the form of a Sequence. It is the responsibility
      *         of the callee to ensure that the type of result conforms to the expected result type.
-     * @throws XPathException
+     * @throws XPathException (should not happen)
      */
 
     public BooleanValue call(XPathContext context, Sequence[] arguments) throws XPathException {
