@@ -97,7 +97,7 @@ public class DocumentFn extends SystemFunction implements Callable {
      * @throws net.sf.saxon.trans.XPathException
      *          if a dynamic error occurs during the evaluation of the expression
      */
-    public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+    public Sequence<?> call(XPathContext context, Sequence[] arguments) throws XPathException {
         int numArgs = getArity();
 
         SequenceIterator hrefSequence = arguments[0].iterate();
@@ -295,6 +295,7 @@ public class DocumentFn extends SystemFunction implements Callable {
                 newdoc = startNode.getTreeInfo();
             } else {
                 Builder b = controller.makeBuilder();
+                b.setUseEventLocation(true);
                 if (b instanceof TinyBuilder) {
                     ((TinyBuilder) b).setStatistics(Statistics.SOURCE_DOCUMENT_STATISTICS);
                 }
@@ -629,6 +630,7 @@ public class DocumentFn extends SystemFunction implements Callable {
         PipelineConfiguration pipe = out.getPipelineConfiguration();
         if (pipe == null) {
             pipe = c.getController().makePipelineConfiguration();
+            pipe.setXPathContext(c);
             out.setPipelineConfiguration(pipe);
         }
 
