@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -15,13 +15,12 @@ import net.sf.saxon.lib.Numberer;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.om.ZeroOrOne;
-import net.sf.saxon.regex.Categories;
+import net.sf.saxon.regex.charclass.Categories;
 import net.sf.saxon.regex.UnicodeString;
 import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.value.*;
-import net.sf.saxon.value.StringValue;
 
 import java.util.Arrays;
 import java.util.TimeZone;
@@ -35,21 +34,14 @@ import java.util.regex.Pattern;
 
 public class FormatDate extends SystemFunction implements Callable {
 
-    static String[] knownCalendars = {"AD", "AH", "AME", "AM", "AP", "AS", "BE", "CB", "CE", "CL", "CS", "EE", "FE", "ISO", "JE",
+    static final String[] knownCalendars = {"AD", "AH", "AME", "AM", "AP", "AS", "BE", "CB", "CE", "CL", "CS", "EE", "FE", "ISO", "JE",
             "KE", "KY", "ME", "MS", "NS", "OS", "RS", "SE", "SH", "SS", "TE", "VE", "VS"};
 
-    private boolean is30 = false;
-
-
     private CharSequence adjustCalendar(StringValue calendarVal, CharSequence result, XPathContext context) throws XPathException {
         StructuredQName cal;
         try {
             String c = calendarVal.getStringValue();
-            if (c.startsWith("Q{")) {
-                cal = StructuredQName.fromEQName(c);
-            } else {
-                cal = StructuredQName.fromLexicalQName(c, true, is30, getRetainedStaticContext());
-            }
+            cal = StructuredQName.fromLexicalQName(c, true, true, getRetainedStaticContext());
         } catch (XPathException e) {
             XPathException err = new XPathException("Invalid calendar name. " + e.getMessage());
             err.setErrorCode("FOFD1340");
@@ -62,7 +54,7 @@ public class FormatDate extends SystemFunction implements Callable {
             if (calLocal.equals("AD") || calLocal.equals("ISO")) {
                 // no action
             } else if (Arrays.binarySearch(knownCalendars, calLocal) >= 0) {
-                result = "[Calendar: AD]" + result.toString();
+                result = "[Calendar: AD]" + result;
             } else {
                 XPathException err = new XPathException("Unknown no-namespace calendar: " + calLocal);
                 err.setErrorCode("FOFD1340");
@@ -70,7 +62,7 @@ public class FormatDate extends SystemFunction implements Callable {
                 throw err;
             }
         } else {
-            result = "[Calendar: AD]" + result.toString();
+            result = "[Calendar: AD]" + result;
         }
         return result;
     }
@@ -82,13 +74,13 @@ public class FormatDate extends SystemFunction implements Callable {
      * @param value    the value to be formatted
      * @param format   the supplied format picture
      * @param language the chosen language
-     * @param country  the chosen country
+     * @param place  the chosen country
      * @param context  the XPath dynamic evaluation context
      * @return the formatted date/time
      * @throws XPathException if a dynamic error occurs
      */
 
-    private static CharSequence formatDate(CalendarValue value, String format, String language, String country, XPathContext context)
+    private static CharSequence formatDate(CalendarValue value, String format, String language, String place, XPathContext context)
             throws XPathException {
 
         Configuration config = context.getConfiguration();
@@ -97,11 +89,20 @@ public class FormatDate extends SystemFunction implements Callable {
         if (language == null) {
             language = config.getDefaultLanguage();
         }
-        if (country == null) {
-            country = config.getDefaultCountry();
+        if (place == null) {
+            place = config.getDefaultCountry();
         }
 
-        Numberer numberer = config.makeNumberer(language, country);
+        // if the value has a timezone and the place is a timezone name, the value is adjusted to that timezone
+        if (value.hasTimezone() && place.contains("/")) {
+            TimeZone tz = TimeZone.getTimeZone(place);
+            if (tz != null) {
+                int milliOffset = tz.getOffset(value.toDateTime().getCalendar().getTime().getTime());
+                value = value.adjustTimezone(milliOffset / 60000);
+            }
+        }
+
+        Numberer numberer = config.makeNumberer(language, place);
         FastStringBuffer sb = new FastStringBuffer(FastStringBuffer.C64);
         if (numberer.getClass() == Numberer_en.class && !"en".equals(language) && !languageDefaulted) {
             sb.append("[Language: en]");
@@ -144,7 +145,7 @@ public class FormatDate extends SystemFunction implements Callable {
                 }
                 String componentFormat = format.substring(i, close);
                 sb.append(formatComponent(value, Whitespace.removeAllWhitespace(componentFormat),
-                                          numberer, country, context));
+                                          numberer, place, context));
                 i = close + 1;
             }
         }
@@ -210,7 +211,7 @@ public class FormatDate extends SystemFunction implements Callable {
                 } else {
                     int year = dtvalue.getYear();
                     if (year < 0) {
-                        year = 1 - year;
+                        year = 0 - year;
                     }
                     return formatNumber(component, year, format, defaultFormat, numberer, context);
                 }
@@ -272,6 +273,7 @@ public class FormatDate extends SystemFunction implements Callable {
                     throw error;
                 } else {
                     Int64Value hour = (Int64Value) value.getComponent(AccessorFn.Component.HOURS);
+                    assert hour != null;
                     return formatNumber(component, (int) hour.longValue(), format, defaultFormat, numberer, context);
                 }
             case 'h':       // hour in half-day (12 hour clock)
@@ -282,6 +284,7 @@ public class FormatDate extends SystemFunction implements Callable {
                     throw error;
                 } else {
                     Int64Value hour = (Int64Value) value.getComponent(AccessorFn.Component.HOURS);
+                    assert hour != null;
                     int hr = (int) hour.longValue();
                     if (hr > 12) {
                         hr = hr - 12;
@@ -298,8 +301,9 @@ public class FormatDate extends SystemFunction implements Callable {
                     error.setXPathContext(context);
                     throw error;
                 } else {
-                    Int64Value min = (Int64Value) value.getComponent(AccessorFn.Component.MINUTES);
-                    return formatNumber(component, (int) min.longValue(), format, defaultFormat, numberer, context);
+                    Int64Value minutes = (Int64Value) value.getComponent(AccessorFn.Component.MINUTES);
+                    assert minutes != null;
+                    return formatNumber(component, (int) minutes.longValue(), format, defaultFormat, numberer, context);
                 }
             case 's':       // seconds
                 if (ignoreTime) {
@@ -308,8 +312,9 @@ public class FormatDate extends SystemFunction implements Callable {
                     error.setXPathContext(context);
                     throw error;
                 } else {
-                    IntegerValue sec = (IntegerValue) value.getComponent(AccessorFn.Component.WHOLE_SECONDS);
-                    return formatNumber(component, (int) sec.longValue(), format, defaultFormat, numberer, context);
+                    IntegerValue seconds = (IntegerValue) value.getComponent(AccessorFn.Component.WHOLE_SECONDS);
+                    assert seconds != null;
+                    return formatNumber(component, (int) seconds.longValue(), format, defaultFormat, numberer, context);
                 }
             case 'f':       // fractional seconds
                 // ignore the format
@@ -319,12 +324,33 @@ public class FormatDate extends SystemFunction implements Callable {
                     error.setXPathContext(context);
                     throw error;
                 } else {
-                    int micros = (int) ((Int64Value) value.getComponent(AccessorFn.Component.MICROSECONDS)).longValue();
-                    return formatNumber(component, micros, format, defaultFormat, numberer, context);
+                    Int64Value micros = (Int64Value) value.getComponent(AccessorFn.Component.MICROSECONDS);
+                    assert micros != null;
+                    return formatNumber(component, (int)micros.longValue(), format, defaultFormat, numberer, context);
                 }
             case 'z':
             case 'Z':
-                return formatTimeZone(value.toDateTime(), component.charAt(0), format, country);
+                DateTimeValue dtv;
+                if (value instanceof TimeValue) {
+                    // See bug 3761. We need to pad the time with a date. 1972-12-31 or 1970-01-01 won't do because
+                    // timezones were different then (Alaska changed in 1983, for example). Today's date isn't ideal
+                    // because it's better to choose a date that isn't in summer time. We'll choose the first of
+                    // January in the current year, unless that's in summer time in the country in question, in which
+                    // case we'll choose first of July.
+                    DateTimeValue now = DateTimeValue.getCurrentDateTime(context);
+                    int year = now.getYear();
+                    int tzoffset = value.getTimezoneInMinutes();
+                    DateTimeValue baseDate =
+                            new DateTimeValue(year, (byte)1, (byte)1, (byte)0, (byte)0, (byte)0, 0, tzoffset, false);
+                    Boolean b = NamedTimeZone.inSummerTime(baseDate, country);
+                    if (b != null && b) {
+                        baseDate = new DateTimeValue(year, (byte) 7, (byte) 1, (byte) 0, (byte) 0, (byte) 0, 0, tzoffset, false);
+                    }
+                    dtv = DateTimeValue.makeDateTimeValue(baseDate.toDateValue(), (TimeValue)value);
+                } else {
+                    dtv = value.toDateTime();
+                }
+                return formatTimeZone(dtv, component.charAt(0), format, country);
 
             case 'F':       // day of week
                 if (ignoreDate) {
@@ -383,6 +409,7 @@ public class FormatDate extends SystemFunction implements Callable {
     private static Pattern digitsOrOptionalDigitsPattern =
             Pattern.compile("[#\\p{Nd}]+");
 
+
     private static Pattern fractionalDigitsPattern =
             Pattern.compile("\\p{Nd}+#*");
 
@@ -430,7 +457,7 @@ public class FormatDate extends SystemFunction implements Callable {
                     int c = uPrimary.uCharAt(i);
                     if (c == '#') {
                         max++;
-                    } else if ((c >= '0' && c <= '9') || Categories.ESCAPE_d.matches(c)) {
+                    } else if ((c >= '0' && c <= '9') || Categories.ESCAPE_d.test(c)) {
                         min++;
                         max++;
                     }
@@ -448,13 +475,14 @@ public class FormatDate extends SystemFunction implements Callable {
             min = range[0];
             //max = Integer.MAX_VALUE;
 
-            String s = numberer.format(value, UnicodeString.makeUnicodeString(primary), null, letterValue, ordinal);
-            int len = StringValue.getStringLength(s);
+            String roman = numberer.format(value, UnicodeString.makeUnicodeString(primary), null, letterValue, ordinal);
+            StringBuilder s = new StringBuilder(roman);
+            int len = StringValue.getStringLength(roman);
             while (len < min) {
-                s = s + ' ';
+                s.append(' ');
                 len++;
             }
-            return s;
+            return s.toString();
         } else if (!widths.isEmpty()) {
             int[] range = getWidths(widths);
             min = Math.max(min, range[0]);
@@ -487,7 +515,7 @@ public class FormatDate extends SystemFunction implements Callable {
             }
         } else if ("Y".equals(component)) {
             if (max < Integer.MAX_VALUE) {
-                value = value % (int)Math.pow(10, max);
+                value = value % (int) Math.pow(10, max);
             }
         } else if ("f".equals(component)) {
             // value is supplied as integer number of microseconds
@@ -499,9 +527,9 @@ public class FormatDate extends SystemFunction implements Callable {
             // if there are grouping separators, handle as a reverse integer as described in the 3.1 spec
             if (!digitsOrOptionalDigitsPattern.matcher(primary).matches()) {
                 UnicodeString reverseFormat = reverse(uFormat);
-                UnicodeString reverseValue = reverse(UnicodeString.makeUnicodeString(""+value));
+                UnicodeString reverseValue = reverse(UnicodeString.makeUnicodeString("" + value));
                 CharSequence reverseResult = formatNumber("s",
-                     Integer.parseInt(reverseValue.toString()), reverseFormat.toString(), false, numberer, context);
+                                                          Integer.parseInt(reverseValue.toString()), reverseFormat.toString(), false, numberer, context);
                 UnicodeString correctedResult = reverse(UnicodeString.makeUnicodeString(reverseResult));
                 if (correctedResult.uLength() > max) {
                     correctedResult = correctedResult.uSubstring(0, max);
@@ -511,21 +539,21 @@ public class FormatDate extends SystemFunction implements Callable {
             if (!fractionalDigitsPattern.matcher(primary).matches()) {
                 throw new XPathException("Invalid picture for fractional seconds: " + primary, "FOFD1340");
             }
-            String s;
+            StringBuilder s;
             if (value == 0) {
-                s = "0";
+                s = new StringBuilder("0");
             } else {
-                s = ((1000000 + value) + "").substring(1);
+                s = new StringBuilder(((1000000 + value) + "").substring(1));
                 if (s.length() > max) {
                     // Spec bug 29749 says we should truncate rather than rounding
-                    s = s.substring(0, max);
+                    s = new StringBuilder(s.substring(0, max));
                 }
             }
             while (s.length() < min) {
-                s = s + '0';
+                s.append('0');
             }
             while (s.length() > min && s.charAt(s.length() - 1) == '0') {
-                s = s.substring(0, s.length() - 1);
+                s = new StringBuilder(s.substring(0, s.length() - 1));
             }
             // for non standard decimal digit family
             int zeroDigit = Alphanumeric.getDigitFamily(uFormat.uCharAt(0));
@@ -534,11 +562,11 @@ public class FormatDate extends SystemFunction implements Callable {
                 for (int z = 0; z <= 9; z++) {
                     digits[z] = zeroDigit + z;
                 }
-                long n = Long.parseLong(s);
+                long n = Long.parseLong(s.toString());
                 int requiredLength = s.length();
-                s = AbstractNumberer.convertDigitSystem(n, digits, requiredLength).toString();
+                s = new StringBuilder(AbstractNumberer.convertDigitSystem(n, digits, requiredLength).toString());
             }
-            return s;
+            return s.toString();
         }
 
         if ("N".equals(primary) || "n".equals(primary) || "Nn".equals(primary)) {
@@ -590,7 +618,7 @@ public class FormatDate extends SystemFunction implements Callable {
 
     private static UnicodeString reverse(UnicodeString in) {
         int[] out = new int[in.uLength()];
-        for (int i=in.uLength()-1, j=0; i>=0; i--,j++) {
+        for (int i = in.uLength() - 1, j = 0; i >= 0; i--, j++) {
             out[j] = in.uCharAt(i);
         }
         return UnicodeString.makeUnicodeString(out);
@@ -627,7 +655,7 @@ public class FormatDate extends SystemFunction implements Callable {
                 }
             }
 
-            if (min > max && max != -1) {
+            if (min > max) {
                 XPathException e = new XPathException("Minimum width in date/time picture exceeds maximum width");
                 e.setErrorCode("FOFD1340");
                 throw e;
@@ -657,7 +685,7 @@ public class FormatDate extends SystemFunction implements Callable {
                 return "";
             }
         }
-        if (format.isEmpty() && widthModifier.length() > 0) {
+        if (format.isEmpty() && !widthModifier.isEmpty()) {
             int[] widths = getWidths(widthModifier);
             int min = widths[0];
             int max = widths[1];
@@ -804,7 +832,7 @@ public class FormatDate extends SystemFunction implements Callable {
         if (calendarVal != null) {
             result = adjustCalendar(calendarVal, result, context);
         }
-        return new ZeroOrOne<StringValue>(new StringValue(result));
+        return new ZeroOrOne<>(new StringValue(result));
     }
 
 }
