@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -14,35 +14,35 @@ import net.sf.saxon.expr.number.*;
 import net.sf.saxon.lib.Numberer;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.regex.ARegularExpression;
-import net.sf.saxon.regex.charclass.Categories;
 import net.sf.saxon.regex.RegularExpression;
 import net.sf.saxon.regex.UnicodeString;
+import net.sf.saxon.regex.charclass.Categories;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.value.IntegerValue;
 import net.sf.saxon.value.StringValue;
 import net.sf.saxon.z.IntHashSet;
-import net.sf.saxon.z.IntPredicate;
 import net.sf.saxon.z.IntSet;
 
 import java.util.ArrayList;
 import java.util.List;
+import java.util.function.IntPredicate;
 
 public class FormatInteger extends SystemFunction {
 
-    //TODO - initialise Numberer and prepare the picture at compile time if possible
+    //TODO - PERF - initialise Numberer and prepare the picture at compile time if possible
 
-    private static RegularExpression badHashPattern;
-    private static RegularExpression modifierPattern;
-    private static RegularExpression decimalDigitPattern;
+    private static final RegularExpression badHashPattern;
+    private static final RegularExpression modifierPattern;
+    private static final RegularExpression decimalDigitPattern;
 
     public static final String preface = "In the picture string for format-integer, ";
 
     static {
         try {
-            badHashPattern = new ARegularExpression("((\\d+|\\w+)#+.*)|(#+[^\\d]+)", "", "XP20", null);
-            modifierPattern = new ARegularExpression("([co](\\(.*\\))?)?[at]?", "", "XP20", null);
-            decimalDigitPattern = new ARegularExpression("^((\\p{Nd}|#|[^\\p{N}\\p{L}])+?)$", "", "XP20", null);
+            badHashPattern = new ARegularExpression("((\\d+|\\w+)#+.*)|(#+[^\\d]+)", "", "XP20", null, null);
+            modifierPattern = new ARegularExpression("([co](\\(.*\\))?)?[at]?", "", "XP20", null, null);
+            decimalDigitPattern = new ARegularExpression("^((\\p{Nd}|#|[^\\p{N}\\p{L}])+?)$", "", "XP20", null, null);
         } catch (Exception e) {
             throw new AssertionError(e);
         }
@@ -128,7 +128,7 @@ public class FormatInteger extends SystemFunction {
         IntPredicate isDecimalDigit = Categories.getCategory("Nd");
         boolean isDecimalDigitPattern = false;
         for (int i = 0; i < primary.uLength(); i++) {
-            if (isDecimalDigit.matches(primary.uCharAt(i))) {
+            if (isDecimalDigit.test(primary.uCharAt(i))) {
                 isDecimalDigitPattern = true;
                 break;
             }
@@ -164,7 +164,7 @@ public class FormatInteger extends SystemFunction {
 
         UnicodeString picExpanded = UnicodeString.makeUnicodeString(pic);
         IntSet groupingPositions = new IntHashSet(5);
-        List<Integer> separatorList = new ArrayList<Integer>();
+        List<Integer> separatorList = new ArrayList<>();
         int groupingPosition = 0; // number of digits to the right of a grouping separator
         int firstGroupingPos = 0; // number of digits to the right of the first grouping separator
         int lastGroupingPos = 0;
@@ -256,7 +256,7 @@ public class FormatInteger extends SystemFunction {
         }
 
         UnicodeString adjustedPic = extractSeparators(picExpanded, groupingPositions);
-        if (groupingPositions.size() == 0) {
+        if (groupingPositions.isEmpty()) {
             return new RegularGroupFormatter(0, "", adjustedPic);
         }
 
