@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -30,9 +30,9 @@ public class Idref extends SystemFunction {
     public int getSpecialProperties(Expression[] arguments) {
         int prop = StaticProperty.ORDERED_NODESET |
                 StaticProperty.SINGLE_DOCUMENT_NODESET |
-                StaticProperty.NON_CREATIVE;
+                StaticProperty.NO_NODES_NEWLY_CREATED;
         if ((getArity() == 1) ||
-                (arguments[1].getSpecialProperties() & StaticProperty.CONTEXT_DOCUMENT_NODESET) != 0) {
+                arguments[1].hasSpecialProperty(StaticProperty.CONTEXT_DOCUMENT_NODESET)) {
             prop |= StaticProperty.CONTEXT_DOCUMENT_NODESET;
         }
         return prop;
@@ -49,14 +49,14 @@ public class Idref extends SystemFunction {
      * @throws XPathException if a dynamic error occurs
      */
 
-    public static SequenceIterator getIdrefMultiple(TreeInfo doc, SequenceIterator keys, XPathContext context)
+    public static SequenceIterator<NodeInfo> getIdrefMultiple(TreeInfo doc, SequenceIterator<StringValue> keys, XPathContext context)
             throws XPathException {
         IdrefMappingFunction map = new IdrefMappingFunction();
         map.document = doc;
         map.keyContext = context;
         map.keyManager = context.getController().getKeyManager();
         map.keySet = map.keyManager.getKeyDefinitionSet(StandardNames.getStructuredQName(StandardNames.XS_IDREFS));
-        SequenceIterator allValues = new MappingIterator(keys, map);
+        SequenceIterator<? extends NodeInfo> allValues = new MappingIterator<>(keys, map);
         return new DocumentOrderIterator(allValues, LocalOrderComparer.getInstance());
     }
 
@@ -69,7 +69,7 @@ public class Idref extends SystemFunction {
      * @throws net.sf.saxon.trans.XPathException
      *          if a dynamic error occurs during the evaluation of the expression
      */
-    public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+    public Sequence<?> call(XPathContext context, Sequence[] arguments) throws XPathException {
         NodeInfo start = arguments.length == 1 ? getContextNode(context) : (NodeInfo) arguments[1].head();
         NodeInfo arg2 = start.getRoot();
         if (arg2.getNodeKind() != Type.DOCUMENT) {
@@ -79,7 +79,7 @@ public class Idref extends SystemFunction {
         return SequenceTool.toLazySequence(getIdrefMultiple(arg2.getTreeInfo(), arguments[0].iterate(), context));
     }
 
-    private static class IdrefMappingFunction implements MappingFunction {
+    private static class IdrefMappingFunction implements MappingFunction<StringValue, NodeInfo> {
         public TreeInfo document;
         public XPathContext keyContext;
         public KeyManager keyManager;
@@ -89,8 +89,8 @@ public class Idref extends SystemFunction {
          * Implement the MappingFunction interface
          */
 
-        public SequenceIterator map(Item item) throws XPathException {
-            return keyManager.selectByKey(keySet, document, (StringValue)item, keyContext);
+        public SequenceIterator<NodeInfo> map(StringValue item) throws XPathException {
+            return keyManager.selectByKey(keySet, document, item, keyContext);
         }
     }
 
