@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -78,6 +78,22 @@ public abstract class Minimax extends CollatingFunctionFixed {
         return argumentType;
     }
 
+    /**
+     * Static analysis: preallocate a comparer if possible
+     */
+
+    @Override
+    public void supplyTypeInformation(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType, Expression[] arguments) {
+        ItemType type = arguments[0].getItemType();
+        argumentType = type.getAtomizedItemType();
+        if (argumentType instanceof AtomicType) {
+            if (argumentType == BuiltInAtomicType.UNTYPED_ATOMIC) {
+                argumentType = BuiltInAtomicType.DOUBLE;
+            }
+            preAllocateComparer((AtomicType) argumentType, (AtomicType) argumentType, visitor.getStaticContext());
+        }
+    }
+
     /*@NotNull*/
     public ItemType getResultItemType(Expression[] args) {
         TypeHierarchy th = getRetainedStaticContext().getConfiguration().getTypeHierarchy();
@@ -87,23 +103,7 @@ public abstract class Minimax extends CollatingFunctionFixed {
         }
         return base.getPrimitiveItemType();
     }
-
-    /**
-     * Static analysis: preallocate a comparer if possible
-     */
-
-    @Override
-    public void supplyTypeInformation(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType, Expression[] arguments) throws XPathException {
-        ItemType type = arguments[0].getItemType();
-        argumentType = type.getAtomizedItemType();
-        if (argumentType instanceof AtomicType) {
-            if (argumentType == BuiltInAtomicType.UNTYPED_ATOMIC) {
-                argumentType = BuiltInAtomicType.DOUBLE;
-            }
-            preAllocateComparer((AtomicType) argumentType, (AtomicType) argumentType, visitor.getStaticContext(), false);
-        }
-    }
-
+    
     /**
      * Determine the cardinality of the function.
      */
@@ -145,10 +145,7 @@ public abstract class Minimax extends CollatingFunctionFixed {
         if (type.equals(BuiltInAtomicType.UNTYPED_ATOMIC)) {
             type = BuiltInAtomicType.DOUBLE;
         }
-        if (!(type instanceof AtomicType)) {
-            type = BuiltInAtomicType.ANY_ATOMIC;
-        }
-        BuiltInAtomicType prim = (BuiltInAtomicType) type.getPrimitiveItemType();
+        BuiltInAtomicType prim = (BuiltInAtomicType) type;
         return GenericAtomicComparer.makeAtomicComparer(prim, prim, getStringCollator(), context);
     }
 
@@ -329,8 +326,8 @@ public abstract class Minimax extends CollatingFunctionFixed {
      * @throws net.sf.saxon.trans.XPathException if a dynamic error occurs during the evaluation of the expression
      */
     public ZeroOrOne<AtomicValue> call(XPathContext context, Sequence[] arguments) throws XPathException {
-        return new ZeroOrOne<AtomicValue>(
-            minimax(arguments[0].iterate(), isMaxFunction(), getAtomicComparer(context), ignoreNaN, context));
+        return new ZeroOrOne<>(
+                minimax(arguments[0].iterate(), isMaxFunction(), getAtomicComparer(context), ignoreNaN, context));
     }
 
     @Override
