@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -10,9 +10,9 @@ package net.sf.saxon.functions;
 import net.sf.saxon.expr.Callable;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.om.Sequence;
+import net.sf.saxon.om.ZeroOrOne;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.value.*;
-import net.sf.saxon.value.StringValue;
 
 import java.util.ArrayList;
 import java.util.List;
@@ -32,12 +32,12 @@ public class ParseIetfDate extends SystemFunction implements Callable {
      * @throws net.sf.saxon.trans.XPathException
      *          if a dynamic error occurs during the evaluation of the expression
      */
-    public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+    public ZeroOrOne<DateTimeValue> call(XPathContext context, Sequence[] arguments) throws XPathException {
         StringValue stringValue = (StringValue) arguments[0].head();
         if (stringValue == null) {
-            return EmptySequence.getInstance();
+            return ZeroOrOne.empty();
         }
-        return parse(stringValue.getStringValue(), context);
+        return new ZeroOrOne<>(parse(stringValue.getStringValue(), context));
     }
 
     private String[] dayNames = new String[]{
@@ -126,7 +126,7 @@ public class ParseIetfDate extends SystemFunction implements Callable {
             "UT","UTC","GMT","EST","EDT","CST","CDT","MST","MDT","PST","PDT"
     };
 
-    private boolean isTimezoneName(String string){
+    private boolean isTimezoneName(String string) {
         for (String s: timezoneNames){
             if (s.equalsIgnoreCase(string)){
                 return true;
@@ -172,7 +172,7 @@ public class ParseIetfDate extends SystemFunction implements Callable {
         int year = 0;
         byte month = 0;
         byte day = 0;
-        List<TimeValue> timeValue = new ArrayList<TimeValue>();
+        List<TimeValue> timeValue = new ArrayList<>();
         int i = 0;
         String currentToken = tokens.get(i);
         if (currentToken.matches("[A-Za-z]+") && isDayName(currentToken)){
@@ -208,18 +208,11 @@ public class ParseIetfDate extends SystemFunction implements Callable {
                 badDate("Space missing after time string", input);
             }
             currentToken = tokens.get(++i);
-            if (!currentToken.matches("[0-9]+")){
+            if (currentToken.matches("[0-9]+")) {
+                year = checkTwoOrFourDigits(input, currentToken);
+            } else {
                 badDate("Year number expected after time", input);
             }
-            else if (currentToken.length() == 4){
-                year = Integer.parseInt(currentToken);
-            }
-            else if (currentToken.length() == 2){
-                year = Integer.parseInt(currentToken) +1900;
-            }
-            else if (currentToken.length() != 4 && currentToken.length() != 4){
-                badDate("Year number must be two or four digits", input);
-            }
         }
         else if (currentToken.matches("[0-9]+")) {
             if (currentToken.length() > 2){
@@ -234,18 +227,11 @@ public class ParseIetfDate extends SystemFunction implements Callable {
             month = getMonthNumber(currentToken);
             i = requireDSep(tokens, ++i, input);
             currentToken = tokens.get(i);
-            if (!currentToken.matches("[0-9]+")){
+            if (currentToken.matches("[0-9]+")) {
+                year = checkTwoOrFourDigits(input, currentToken);
+            } else {
                 badDate("Year number expected after month name", input);
             }
-            else if (currentToken.length() == 4){
-                year = Integer.parseInt(currentToken);
-            }
-            else if (currentToken.length() == 2){
-                year = Integer.parseInt(currentToken) +1900;
-            }
-            else if (currentToken.length() != 4 && currentToken.length() != 4){
-                badDate("Year number must be two or four digits", input);
-            }
             currentToken = tokens.get(++i);
             if (!" ".equals(currentToken)){
                 badDate("Space missing after year number", input);
@@ -272,6 +258,19 @@ public class ParseIetfDate extends SystemFunction implements Callable {
         return DateTimeValue.makeDateTimeValue(date, time);
     }
 
+    private int checkTwoOrFourDigits(String input, String currentToken) throws XPathException {
+        int year;
+        if (currentToken.length() == 4){
+            year = Integer.parseInt(currentToken);
+        } else if (currentToken.length() == 2){
+            year = Integer.parseInt(currentToken) +1900;
+        } else {
+            badDate("Year number must be two or four digits", input);
+            year = 0;
+        }
+        return year;
+    }
+
     /**
      * Parse part of a string (already tokenized) to obtain a TimeValue
      *
@@ -289,89 +288,89 @@ public class ParseIetfDate extends SystemFunction implements Callable {
         int tz = 0; /*the timezone displacement in minutes from UTC.*/
         int i = currentPosition;
         int n = currentPosition; /* the final token index, returned by the method */
-        String currentToken = tokens.get(i);
-        if (!currentToken.matches("[0-9]+")){
+        StringBuilder currentToken = new StringBuilder(tokens.get(i));
+        if (!currentToken.toString().matches("[0-9]+")){
             badDate("Hour number expected", input);
         }
         if (currentToken.length() > 2){
             badDate("Hour number exceeds two digits", input);
         }
-        hour = (byte) Integer.parseInt(currentToken);
-        currentToken = tokens.get(++i);
-        if (!":".equals(currentToken)){
+        hour = (byte) Integer.parseInt(currentToken.toString());
+        currentToken = new StringBuilder(tokens.get(++i));
+        if (!":".equals(currentToken.toString())){
             badDate("Separator ':' missing after hour", input);
         }
-        currentToken = tokens.get(++i);
-        if (!currentToken.matches("[0-9]+")){
+        currentToken = new StringBuilder(tokens.get(++i));
+        if (!currentToken.toString().matches("[0-9]+")){
             badDate("Minutes expected after hour", input);
         }
         if (currentToken.length() != 2){
             badDate("Minutes must be exactly two digits", input);
         }
-        minute = (byte) Integer.parseInt(currentToken);
-        currentToken = tokens.get(++i);
+        minute = (byte) Integer.parseInt(currentToken.toString());
+        currentToken = new StringBuilder(tokens.get(++i));
         boolean finished = false;
 
-        if (currentToken.equals(EOF)){
+        if (currentToken.toString().equals(EOF)){
         /* seconds, microseconds, timezones not given*/
             n = i - 1;
             finished = true;
         }
-        else if (":".equals(currentToken)){
-            currentToken = tokens.get(++i);
-            if (!currentToken.matches("[0-9]+")){
+        else if (":".equals(currentToken.toString())){
+            currentToken = new StringBuilder(tokens.get(++i));
+            if (!currentToken.toString().matches("[0-9]+")){
                 badDate("Seconds expected after ':' separator after minutes", input);
             }
             if (currentToken.length() != 2){
                 badDate("Seconds number must have exactly two digits (before decimal point)", input);
             }
-            second = (byte) Integer.parseInt(currentToken);
-            currentToken = tokens.get(++i);
-            if (currentToken.equals(EOF)){
+            second = (byte) Integer.parseInt(currentToken.toString());
+            currentToken = new StringBuilder(tokens.get(++i));
+            if (currentToken.toString().equals(EOF)){
             /* microseconds, timezones not given*/
                 n = i - 1;
                 finished = true;
             }
-            else if (".".equals(currentToken)){
-                currentToken = tokens.get(++i);
-                if (!currentToken.matches("[0-9]+")){
+            else if (".".equals(currentToken.toString())){
+                currentToken = new StringBuilder(tokens.get(++i));
+                if (!currentToken.toString().matches("[0-9]+")){
                     badDate("Fractional part of seconds expected after decimal point", input);
                 }
                 int len = Math.min(6, currentToken.length());
-                currentToken = currentToken.substring(0, len);
+                currentToken = new StringBuilder(currentToken.substring(0, len));
                 while (currentToken.length() < 6){
-                    currentToken = currentToken + "0";
+                    currentToken.append("0");
                 }
-                microsecond = Integer.parseInt(currentToken);
+                microsecond = Integer.parseInt(currentToken.toString());
                 if (i < tokens.size() - 1) {
-                    currentToken = tokens.get(++i);
+                    currentToken = new StringBuilder(tokens.get(++i));
                 }
             }
         }
         if (!finished) {
-            if (" ".equals(currentToken)) {
-                currentToken = tokens.get(++i);
-                if (currentToken.matches("[0-9]+")) {
+            if (" ".equals(currentToken.toString())) {
+                currentToken = new StringBuilder(tokens.get(++i));
+                if (currentToken.toString().matches("[0-9]+")) {
                 /* no timezone is given in the time, we must have reached a year */
                     n = i - 2;
                     finished = true;
                 }
             }
             if (!finished) {
-                if (currentToken.matches("[A-Za-z]+")){
-                    if (!isTimezoneName(currentToken)){
+                if (currentToken.toString().matches("[A-Za-z]+")){
+                    if (!isTimezoneName(currentToken.toString())){
                         badDate("Timezone name not recognised", input);
                     }
-                    tz = getTimezoneOffsetFromName(currentToken);
+                    tz = getTimezoneOffsetFromName(currentToken.toString());
                     n = i;
                     finished = true;
 
-                } else if ("+".equals(currentToken)|"-".equals(currentToken)) {
-                    String sign = currentToken;
+                } else if ("+".equals(currentToken.toString())|"-".equals(currentToken.toString())) {
+                    String sign = currentToken.toString();
                     int tzOffsetHours = 0;
                     int tzOffsetMinutes = 0;
-                    currentToken = tokens.get(++i);
-                    if (!currentToken.matches("[0-9]+")){
+                    currentToken = new StringBuilder(tokens.get(++i));
+                    if (!currentToken.toString().matches("[0-9]+")){
                         badDate("Parsing timezone offset, number expected after '" + sign + "'", input);
                     }
                     int tLength = currentToken.length();
@@ -381,19 +380,20 @@ public class ParseIetfDate extends SystemFunction implements Callable {
                     else if (tLength >= 3){
                         tzOffsetHours = Integer.parseInt(currentToken.substring(0, tLength - 2));
                         tzOffsetMinutes = Integer.parseInt(currentToken.substring(tLength - 2, tLength));
-                        currentToken = tokens.get(++i);
+                        currentToken = new StringBuilder(tokens.get(++i));
                     }
                     else {
-                        tzOffsetHours = Integer.parseInt(currentToken);
-                        currentToken = tokens.get(++i);
-                        if (":".equals(currentToken)) {
-                            currentToken = tokens.get(++i);
-                            if (currentToken.matches("[0-9]+")){
+                        tzOffsetHours = Integer.parseInt(currentToken.toString());
+                        currentToken = new StringBuilder(tokens.get(++i));
+                        if (":".equals(currentToken.toString())) {
+                            currentToken = new StringBuilder(tokens.get(++i));
+                            if (currentToken.toString().matches("[0-9]+")){
                                 if (currentToken.length() != 2) {
                                     badDate("Parsing timezone offset, minutes must be two digits", input);
+                                } else {
+                                    tzOffsetMinutes = Integer.parseInt(currentToken.toString());
                                 }
-                                else tzOffsetMinutes = Integer.parseInt(currentToken);
-                                currentToken = tokens.get(++i);
+                                currentToken = new StringBuilder(tokens.get(++i));
                             }
                         }
                     }
@@ -404,36 +404,36 @@ public class ParseIetfDate extends SystemFunction implements Callable {
                     if (sign.equals("-")){
                         tz = -tz;
                     }
-                    if (currentToken.equals(EOF)){
+                    if (currentToken.toString().equals(EOF)){
                         n = i - 1;
                         finished = true;
                     }
-                    else if (" ".equals(currentToken)) {
-                        currentToken = tokens.get(++i);
-                        if (currentToken.matches("[0-9]+")) {
+                    else if (" ".equals(currentToken.toString())) {
+                        currentToken = new StringBuilder(tokens.get(++i));
+                        if (currentToken.toString().matches("[0-9]+")) {
                         /* we must have reached the year */
                             n = i - 2;
                             finished = true;
                         }
                     }
-                    if (!finished && "(".equals(currentToken)) {
-                        currentToken = tokens.get(++i);
-                        if (" ".equals(currentToken)) {
-                            currentToken = tokens.get(++i);
+                    if (!finished && "(".equals(currentToken.toString())) {
+                        currentToken = new StringBuilder(tokens.get(++i));
+                        if (" ".equals(currentToken.toString())) {
+                            currentToken = new StringBuilder(tokens.get(++i));
                         }
-                        if (!currentToken.matches("[A-Za-z]+")) {
+                        if (!currentToken.toString().matches("[A-Za-z]+")) {
                             badDate("Timezone name expected after '('", input);
                         }
-                        else if (currentToken.matches("[A-Za-z]+")) {
-                            if (!isTimezoneName(currentToken)){
+                        else if (currentToken.toString().matches("[A-Za-z]+")) {
+                            if (!isTimezoneName(currentToken.toString())){
                                 badDate("Timezone name not recognised", input);
                             }
-                            currentToken = tokens.get(++i);
+                            currentToken = new StringBuilder(tokens.get(++i));
                         }
-                        if (" ".equals(currentToken)) {
-                            currentToken = tokens.get(++i);
+                        if (" ".equals(currentToken.toString())) {
+                            currentToken = new StringBuilder(tokens.get(++i));
                         }
-                        if (!")".equals(currentToken)){
+                        if (!")".equals(currentToken.toString())){
                             badDate("Expected ')' after timezone name", input);
                         }
                         n = i;
@@ -442,7 +442,7 @@ public class ParseIetfDate extends SystemFunction implements Callable {
                     else if (!finished) {
                         badDate("Unexpected content after timezone offset", input);
                     }
-                } else if (!finished) {
+                } else {
                     badDate("Unexpected content in time (after minutes)", input);
                 }
             }
@@ -479,7 +479,7 @@ public class ParseIetfDate extends SystemFunction implements Callable {
 
 
     private List<String> tokenize(String input) throws XPathException{
-        List<String> tokens = new ArrayList<String>();
+        List<String> tokens = new ArrayList<>();
         input = input.trim();
         if (input.isEmpty()){
             badDate("Input is empty",input);
@@ -519,4 +519,4 @@ public class ParseIetfDate extends SystemFunction implements Callable {
 }
 
 
-// Copyright (c) 2017 Saxonica Limited.
\ No newline at end of file
+// Copyright (c) 2018 Saxonica Limited.
\ No newline at end of file
