@@ -125,7 +125,7 @@ public class TransformFn extends SystemFunction implements Callable {
      * options with recognized names and type-valid values.
      */
 
-    private void checkTransformOptions(Map<String, Sequence> options, XPathContext context, boolean isXslt30Processor) throws XPathException {
+    private void checkTransformOptions(Map<String, Sequence<?>> options, XPathContext context, boolean isXslt30Processor) throws XPathException {
         if (options.isEmpty()) {
             throw new XPathException("No transformation options supplied", "FOXT0002");
         }
@@ -137,11 +137,11 @@ public class TransformFn extends SystemFunction implements Callable {
         }
     }
 
-    private String checkStylesheetMutualExclusion(Map<String, Sequence> map) throws XPathException {
+    private String checkStylesheetMutualExclusion(Map<String, Sequence<?>> map) throws XPathException {
         return exactlyOneOf(map, "stylesheet-location", "stylesheet-node", "stylesheet-text");
     }
 
-    private String checkStylesheetMutualExclusion30(Map<String, Sequence> map) throws XPathException {
+    private String checkStylesheetMutualExclusion30(Map<String, Sequence<?>> map) throws XPathException {
         String styleOption = exactlyOneOf(map, "stylesheet-location", "stylesheet-node", "stylesheet-text",
                                           "package-name", "package-node", "package-location");
         if (styleOption.equals("package-location")) {
@@ -150,7 +150,7 @@ public class TransformFn extends SystemFunction implements Callable {
         return styleOption;
     }
 
-    private String checkInvocationMutualExclusion(Map<String, Sequence> options) throws XPathException {
+    private String checkInvocationMutualExclusion(Map<String, Sequence<?>> options) throws XPathException {
         return oneOf(options, "initial-mode", "initial-template");
     }
 
@@ -163,7 +163,7 @@ public class TransformFn extends SystemFunction implements Callable {
      * @throws XPathException if more than one of the keys is present
      */
 
-    private String oneOf(Map<String, Sequence> map, String... keys) throws XPathException {
+    private String oneOf(Map<String, Sequence<?>> map, String... keys) throws XPathException {
         String found = null;
         for (String s : keys) {
             if (map.get(s) != null) {
@@ -188,7 +188,7 @@ public class TransformFn extends SystemFunction implements Callable {
      * @throws XPathException if none of the keys is present or if more than one of the keys is present
      */
 
-    private String exactlyOneOf(Map<String, Sequence> map, String... keys) throws XPathException {
+    private String exactlyOneOf(Map<String, Sequence<?>> map, String... keys) throws XPathException {
         String found = oneOf(map, keys);
         if (found == null) {
             throw new XPathException("One of the following transform options must be present: " + enumerate(keys));
@@ -210,7 +210,7 @@ public class TransformFn extends SystemFunction implements Callable {
         return buffer.toString();
     }
 
-    private String checkInvocationMutualExclusion30(Map<String, Sequence> map) throws XPathException {
+    private String checkInvocationMutualExclusion30(Map<String, Sequence<?>> map) throws XPathException {
         return oneOf(map, "initial-mode", "initial-template", "initial-function");
     }
 
@@ -232,7 +232,7 @@ public class TransformFn extends SystemFunction implements Callable {
         throw new XPathException("Unrecognized boolean value " + value, "FOXT0002");
     }
 
-    private void setRequestedProperties(Map<String, Sequence> options, Processor processor) throws XPathException {
+    private void setRequestedProperties(Map<String, Sequence<?>> options, Processor processor) throws XPathException {
         MapItem requestedProps = (MapItem) options.get("requested-properties").head();
         AtomicIterator optionIterator = requestedProps.keys();
         while (true) {
@@ -357,7 +357,7 @@ public class TransformFn extends SystemFunction implements Callable {
         }
     }
 
-    private void setStaticParams(Map<String, Sequence> options, XsltCompiler xsltCompiler, boolean allowTypedNodes) throws XPathException {
+    private void setStaticParams(Map<String, Sequence<?>> options, XsltCompiler xsltCompiler, boolean allowTypedNodes) throws XPathException {
         MapItem staticParamsMap = (MapItem) options.get("static-params").head();
         AtomicIterator paramIterator = staticParamsMap.keys();
         while (true) {
@@ -379,10 +379,10 @@ public class TransformFn extends SystemFunction implements Callable {
         }
     }
 
-    private XsltExecutable getStylesheet(Map<String, Sequence> options, XsltCompiler xsltCompiler, String styleOptionStr, XPathContext context) throws XPathException {
+    private XsltExecutable getStylesheet(Map<String, Sequence<?>> options, XsltCompiler xsltCompiler, String styleOptionStr, XPathContext context) throws XPathException {
         Item styleOptionItem = options.get(styleOptionStr).head();
         URI stylesheetBaseUri = null;
-        Sequence seq;
+        Sequence<?> seq;
         if ((seq = options.get("stylesheet-base-uri")) != null) {
             StringValue styleBaseUri = (StringValue) seq.head();
             stylesheetBaseUri = URI.create(styleBaseUri.getStringValue());
@@ -563,18 +563,20 @@ public class TransformFn extends SystemFunction implements Callable {
     }
 
 
-    public Sequence<? extends Item> call(XPathContext context, Sequence[] arguments) throws XPathException {
-        Map<String, Sequence> options = getDetails().optionDetails.processSuppliedOptions((MapItem) arguments[0].head(), context);
+    public Sequence<?> call(XPathContext context, Sequence[] arguments) throws XPathException {
+        Map<String, Sequence<?>> options = getDetails().optionDetails.processSuppliedOptions((MapItem) arguments[0].head(), context);
 
-        Sequence vendorOptionsValue = options.get("vendor-options");
+        Sequence<?> vendorOptionsValue = options.get("vendor-options");
         MapItem vendorOptions = vendorOptionsValue == null ? null : (MapItem) vendorOptionsValue.head();
 
         Configuration targetConfig = context.getConfiguration();
         boolean allowTypedNodes = true;
+        int schemaValidation = Validation.DEFAULT;
+        
         if (vendorOptions != null) {
-            Sequence targetConfigValue = vendorOptions.get(new QNameValue("", NamespaceConstant.SAXON, "configuration"));
-            if (targetConfigValue != null) {
-                NodeInfo configFile = (NodeInfo) targetConfigValue.head();
+            Sequence<?> optionValue = vendorOptions.get(new QNameValue("", NamespaceConstant.SAXON, "configuration"));
+            if (optionValue != null) {
+                NodeInfo configFile = (NodeInfo) optionValue.head();
                 targetConfig = Configuration.readConfiguration(configFile, targetConfig);
                 targetConfig.importLicenseDetails(context.getConfiguration());
                 allowTypedNodes = false;
@@ -582,6 +584,11 @@ public class TransformFn extends SystemFunction implements Callable {
                     targetConfig.setBooleanProperty(Feature.ALLOW_EXTERNAL_FUNCTIONS, false);
                 }
             }
+            optionValue = vendorOptions.get(new QNameValue("", NamespaceConstant.SAXON, "schema-validation"));
+            if (optionValue != null) {
+                String valOption = optionValue.head().getStringValue();
+                schemaValidation = Validation.getCode(valOption);
+            }
         }
         Processor processor = new Processor(true);
         processor.setConfigurationProperty(Feature.CONFIGURATION, targetConfig);
@@ -658,10 +665,9 @@ public class TransformFn extends SystemFunction implements Callable {
         XdmValue[] functionParams = null;
         Function postProcessor = null;
         String principalResultKey = "output";
-        int schemaValidation = Validation.DEFAULT;
 
         for (String name : options.keySet()) {
-            Sequence value = options.get(name);
+            Sequence<?> value = options.get(name);
             Item head = value.head();
             switch (name) {
                 case "source-node":
@@ -707,10 +713,10 @@ public class TransformFn extends SystemFunction implements Callable {
                 }
                 case "global-context-item":
                     if (useXslt30Processor) {
-                        globalContextItem = (XdmItem) XdmValue.wrap(head);
-                        if (!allowTypedNodes && globalContextItem instanceof NodeInfo && ((NodeInfo) globalContextItem).getTreeInfo().isTyped()) {
+                        if (!allowTypedNodes && head instanceof NodeInfo && ((NodeInfo) head).getTreeInfo().isTyped()) {
                             throw new XPathException("Schema-validated nodes cannot be passed to fn:transform() when it runs under a different Saxon Configuration", "FOXT0002");
                         }
+                        globalContextItem = (XdmItem) XdmValue.wrap(head);
                     }
                     break;
                 case "template-params": {
@@ -730,7 +736,7 @@ public class TransformFn extends SystemFunction implements Callable {
                     ArrayItem functionParamsArray = (ArrayItem) head;
                     functionParams = new XdmValue[functionParamsArray.arrayLength()];
                     for (int i = 0; i < functionParams.length; i++) {
-                        Sequence argVal = functionParamsArray.get(i);
+                        Sequence<?> argVal = functionParamsArray.get(i);
                         if (!allowTypedNodes) {
                             checkSequenceIsUntyped(argVal);
                         }
@@ -767,7 +773,7 @@ public class TransformFn extends SystemFunction implements Callable {
         controller.setResultDocumentResolver(deliverer);
 
         Destination destination = deliverer.getPrimaryDestination(serializationParamsMap);
-        Sequence result;
+        Sequence<?> result;
         try {
             transformer.setStylesheetParameters(stylesheetParams);
             transformer.setBaseOutputURI(baseOutputUri);
@@ -810,9 +816,13 @@ public class TransformFn extends SystemFunction implements Callable {
                 if (initialMatchSelection == null && sourceNode != null) {
                     initialMatchSelection = XdmValue.wrap(sourceNode);
                 }
-                if (initialMatchSelection == null) {
+                if (initialMatchSelection == null && sourceLocation != null) {
                     StreamSource stream = new StreamSource(sourceLocation);
-                    transformer.applyTemplates(stream, destination);
+                    if (transformer.getUnderlyingController().getInitialMode().isDeclaredStreamable()) {
+                        transformer.applyTemplates(stream, destination);
+                    } else {
+                        transformer.transform(stream,destination);
+                    }
                     result = deliverer.getPrimaryResult();
                 } else {
                     transformer.applyTemplates(initialMatchSelection, destination);
@@ -864,7 +874,7 @@ public class TransformFn extends SystemFunction implements Callable {
                     throw new XPathException("The names of parameters must be supplied as QNames", "FOXT0002");
                 }
                 QName paramName = new QName(((QNameValue) param).getStructuredQName());
-                Sequence value = suppliedParams.get(param);
+                Sequence<?> value = suppliedParams.get(param);
                 if (!allowTypedNodes) {
                     checkSequenceIsUntyped(value);
                 }
@@ -1055,13 +1065,13 @@ public class TransformFn extends SystemFunction implements Callable {
          * @return the primary result, or null if there is no primary result (after post-processing if any)
          */
 
-        public abstract Sequence getPrimaryResult() throws XPathException;
+        public abstract Sequence<?> getPrimaryResult() throws XPathException;
 
         /**
          * Post-process the result if required
          */
 
-        public GroundedValue postProcess(String uri, Sequence result) throws XPathException {
+        public GroundedValue<?> postProcess(String uri, Sequence<?> result) throws XPathException {
             if (postProcessor != null) {
                 result = postProcessor.call(context.newCleanContext(), new Sequence[]{new StringValue(uri), result});
             }
@@ -1086,7 +1096,7 @@ public class TransformFn extends SystemFunction implements Callable {
         }
 
         @Override
-        public Sequence getPrimaryResult() throws XPathException {
+        public Sequence<?> getPrimaryResult() throws XPathException {
             XdmNode node = destination.getXdmNode();
             return node == null ? null : postProcess(baseOutputUri, node.getUnderlyingNode());
         }
@@ -1134,7 +1144,7 @@ public class TransformFn extends SystemFunction implements Callable {
         }
 
         @Override
-        public Sequence getPrimaryResult() throws XPathException {
+        public Sequence<?> getPrimaryResult() throws XPathException {
             String str = primaryWriter.toString();
             if (str.isEmpty()) {
                 return null;
@@ -1185,8 +1195,8 @@ public class TransformFn extends SystemFunction implements Callable {
         }
 
         @Override
-        public Sequence getPrimaryResult() throws XPathException {
-            Sequence actualResult = primaryDestination.getXdmValue().getUnderlyingValue();
+        public Sequence<?> getPrimaryResult() throws XPathException {
+            Sequence<?> actualResult = primaryDestination.getXdmValue().getUnderlyingValue();
             return postProcess(baseOutputUri, actualResult);
         }
 
