@@ -17,6 +17,7 @@ import net.sf.saxon.functions.OptionsParameter;
 import net.sf.saxon.functions.SystemFunction;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.om.Function;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.trans.SymbolicName;
@@ -35,7 +36,7 @@ import java.util.List;
  */
 public abstract class BuiltInFunctionSet implements FunctionLibrary {
 
-    public static Sequence EMPTY = EmptySequence.getInstance();
+    public static Sequence<?> EMPTY = EmptySequence.getInstance();
 
     /**
      * Categories of functions, bit significant
@@ -96,7 +97,7 @@ public abstract class BuiltInFunctionSet implements FunctionLibrary {
     public static final int NAV = 1 << 27;   // = usage NAVIGATION (function navigates from this node)
 
 
-    private HashMap<String, Entry> functionTable = new HashMap<String, Entry>(200);
+    private HashMap<String, Entry> functionTable = new HashMap<>(200);
     //private List<BuiltInFunctionSet> importedFunctions = new ArrayList<BuiltInFunctionSet>();
 
     /**
@@ -175,9 +176,6 @@ public abstract class BuiltInFunctionSet implements FunctionLibrary {
      *                     functions could be used.
      * @return An expression equivalent to a call on the specified function, if one is found;
      * null if no function was found matching the required name and arity.
-     * @throws XPathException if a function is found with the required name and arity, but
-     *                        the implementation of the function cannot be loaded or used; or if an error occurs
-     *                        while searching for the function.
      */
     @Override
     public Expression bind(SymbolicName.F symbolicName, Expression[] staticArgs, StaticContext env, List<String> reasons) {
@@ -342,7 +340,7 @@ public abstract class BuiltInFunctionSet implements FunctionLibrary {
             e.usage = new OperandUsage[1];
         } else {
             e.argumentTypes = new SequenceType[arity];
-            e.resultIfEmpty = new Sequence[arity];
+            e.resultIfEmpty = (Sequence<?>[])new Sequence[arity];
             e.usage = new OperandUsage[arity];
         }
         functionTable.put(name + "#" + arity, e);
@@ -366,7 +364,7 @@ public abstract class BuiltInFunctionSet implements FunctionLibrary {
             e.applicability = master.applicability;
             e.properties = master.properties;
             e.argumentTypes = new SequenceType[arity];
-            e.resultIfEmpty = new Sequence[arity];
+            e.resultIfEmpty = (Sequence<?>[])new Sequence[arity];
             e.usage = new OperandUsage[arity];
             for (int i=0; i<arity; i++) {
                 e.argumentTypes[i] = master.argumentTypes[i];
@@ -413,10 +411,6 @@ public abstract class BuiltInFunctionSet implements FunctionLibrary {
          * The arity of the function
          */
         public int arity;
-        /**
-         * The maximum number of arguments permitted
-         */
-        //public int maxArguments;
         /**
          * The item type of the result of the function
          */
@@ -442,7 +436,7 @@ public abstract class BuiltInFunctionSet implements FunctionLibrary {
          * as the value of this argument allows the result to be determined irrespective of the values of the
          * other arguments; null if there is no such calculation possible
          */
-        public Sequence[] resultIfEmpty;
+        public Sequence<?>[] resultIfEmpty;
         /**
          * Any additional properties. Various bit settings are defined: for example SAME_AS_FIRST_ARGUMENT indicates that
          * the result type is the same as the type of the first argument
@@ -469,7 +463,7 @@ public abstract class BuiltInFunctionSet implements FunctionLibrary {
          * @return this entry (to allow chaining)
          */
 
-        public Entry arg(int a, ItemType type, int options, /*@Nullable*/ Sequence resultIfEmpty) {
+        public Entry arg(int a, ItemType type, int options, Sequence<?> resultIfEmpty) {
             int cardinality = options & StaticProperty.CARDINALITY_MASK;
             OperandUsage usage = OperandUsage.NAVIGATION;
             if ((options & ABS) != 0) {
