@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -102,7 +102,7 @@ public class ParseOptions {
         invalidityHandler = p.invalidityHandler;
         stable = p.stable;
         if (p.filters != null) {
-            filters = new ArrayList<FilterFactory>(p.filters);
+            filters = new ArrayList<>(p.filters);
         }
         sourceIsXQJ = p.sourceIsXQJ;
         setExpandAttributeDefaults(p.expandAttributeDefaults);
@@ -111,12 +111,14 @@ public class ParseOptions {
         continueAfterValidationErrors = p.continueAfterValidationErrors;
         addCommentsAfterValidationErrors = p.addCommentsAfterValidationErrors;
         if (p.parserFeatures != null) {
-            parserFeatures = new HashMap<String, Boolean>(p.parserFeatures);
+            parserFeatures = new HashMap<>(p.parserFeatures);
         }
         if (p.parserProperties != null) {
-            parserProperties = new HashMap<String, Object>(p.parserProperties);
+            parserProperties = new HashMap<>(p.parserProperties);
         }
         applicableAccumulators = p.applicableAccumulators;
+        checkEntityReferences = p.checkEntityReferences;
+        validationStatisticsRecipient = p.validationStatisticsRecipient;
     }
 
     /**
@@ -176,19 +178,19 @@ public class ParseOptions {
         }
         if (options.filters != null) {
             if (filters == null) {
-                filters = new ArrayList<FilterFactory>();
+                filters = new ArrayList<>();
             }
             filters.addAll(options.filters);
         }
         if (options.parserFeatures != null) {
             if (parserFeatures == null) {
-                parserFeatures = new HashMap<String, Boolean>();
+                parserFeatures = new HashMap<>();
             }
             parserFeatures.putAll(options.parserFeatures);
         }
         if (options.parserProperties != null) {
             if (parserProperties == null) {
-                parserProperties = new HashMap<String, Object>();
+                parserProperties = new HashMap<>();
             }
             parserProperties.putAll(options.parserProperties);
         }
@@ -250,7 +252,7 @@ public class ParseOptions {
 
     public void addFilter(FilterFactory filterFactory) {
         if (filters == null) {
-            filters = new ArrayList<FilterFactory>(5);
+            filters = new ArrayList<>(5);
         }
         filters.add(filterFactory);
     }
@@ -342,7 +344,7 @@ public class ParseOptions {
      */
     public void addParserFeature(String uri, boolean value) {
         if (parserFeatures == null) {
-            parserFeatures = new HashMap<String, Boolean>();
+            parserFeatures = new HashMap<>();
         }
         parserFeatures.put(uri, value);
     }
@@ -355,7 +357,7 @@ public class ParseOptions {
      */
     public void addParserProperties(String uri, Object value) {
         if (parserProperties == null) {
-            parserProperties = new HashMap<String, Object>();
+            parserProperties = new HashMap<>();
         }
         parserProperties.put(uri, value);
     }
@@ -403,7 +405,7 @@ public class ParseOptions {
      *
      * @return one of {@link net.sf.saxon.event.Builder#TINY_TREE}, {@link net.sf.saxon.event.Builder#LINKED_TREE},
      * or {@link net.sf.saxon.event.Builder#TINY_TREE_CONDENSED},
-     * or {link Builder#UNSPECIFIED_TREE_MODEL} if no call on setTreeModel() has been made
+     * or {@link net.sf.saxon.event.Builder#UNSPECIFIED_TREE_MODEL} if no call on setTreeModel() has been made
      */
 
     public int getTreeModel() {
@@ -586,7 +588,6 @@ public class ParseOptions {
      *
      * @param option one of {@link Validation#STRICT},  {@link Validation#LAX},
      *               {@link Validation#STRIP}, {@link Validation#DEFAULT}.
-     *               <p/>
      *               <p>The value {@link Validation#LAX} indicates that DTD validation is
      *               requested, but validation failures are treated as warnings only.</p>
      */
@@ -602,7 +603,6 @@ public class ParseOptions {
      *
      * @return the validation mode requested, or {@link Validation#DEFAULT}
      * to use the default validation mode from the Configuration.
-     * <p/>
      * <p>The value {@link Validation#LAX} indicates that DTD validation is
      * requested, but validation failures are treated as warnings only.</p>
      */
@@ -804,11 +804,9 @@ public class ParseOptions {
 
     /**
      * <p>Set state of XInclude processing.</p>
-     * <p/>
      * <p>If XInclude markup is found in the document instance, should it be
      * processed as specified in <a href="http://www.w3.org/TR/xinclude/">
      * XML Inclusions (XInclude) Version 1.0</a>.</p>
-     * <p/>
      * <p>XInclude processing defaults to <code>false</code>.</p>
      *
      * @param state Set XInclude processing to <code>true</code> or
