@@ -29,9 +29,6 @@ import javax.xml.transform.TransformerException;
 import javax.xml.transform.sax.SAXResult;
 import javax.xml.transform.stax.StAXResult;
 import javax.xml.transform.stream.StreamResult;
-import java.io.OutputStream;
-import java.io.OutputStreamWriter;
-import java.io.UnsupportedEncodingException;
 import java.io.Writer;
 import java.util.*;
 import java.util.regex.Pattern;
@@ -130,9 +127,9 @@ public class SerializerFactory {
      * a serialization pipeline. The method can be overridden in a subclass; alternatively, the
      * subclass can override the various methods used to instantiate components of the serialization
      * pipeline.
-     * <p>This version of the method calls {@link #getReceiver(Result, SerializationProperties, PipelineConfiguration)
+     * <p>This version of the method calls {@link #getReceiver(Result, SerializationProperties, PipelineConfiguration)}
      * supplying default output properties, and a {@code PipelineConfiguration} newly constructed using
-     * {@link Configuration#makePipelineConfiguration()}}.</p>
+     * {@link Configuration#makePipelineConfiguration()}.</p>
      *
      * @param result The final destination of the serialized output. Usually a StreamResult,
      *               but other kinds of Result are possible.
@@ -148,8 +145,8 @@ public class SerializerFactory {
      * a serialization pipeline. The method can be overridden in a subclass; alternatively, the
      * subclass can override the various methods used to instantiate components of the serialization
      * pipeline.
-     * <p>This version of the method calls {@link #getReceiver(Result, SerializationProperties, PipelineConfiguration)
-     * supplying a {@code PipelineConfiguration} newly constructed using {@link Configuration#makePipelineConfiguration()}}.</p>
+     * <p>This version of the method calls {@link #getReceiver(Result, SerializationProperties, PipelineConfiguration)}
+     * supplying a {@code PipelineConfiguration} newly constructed using {@link Configuration#makePipelineConfiguration()}.</p>
      *
      * @param result The final destination of the serialized output. Usually a StreamResult,
      *               but other kinds of Result are possible.
@@ -257,18 +254,6 @@ public class SerializerFactory {
 
             Emitter emitter = null;
 
-            CharacterMapExpander characterMapExpander = null;  // TODO: drop this
-            String useMaps = props.getProperty(SaxonOutputKeys.USE_CHARACTER_MAPS);
-            if (useMaps != null) {
-                characterMapExpander = charMapIndex.makeCharacterMapExpander(useMaps, new Sink(pipe), this);
-            }
-
-            ProxyReceiver normalizer = null;  // TODO: drop this
-            String normForm = props.getProperty(SaxonOutputKeys.NORMALIZATION_FORM);
-            if (normForm != null && !normForm.equals("none")) {
-                normalizer = newUnicodeNormalizer(new Sink(pipe), props);
-            }
-
             switch (method) {
                 case "html": {
                     emitter = newHTMLEmitter(props);
@@ -303,34 +288,24 @@ public class SerializerFactory {
                     if (sortOrder != null) {
                         js.setPropertySorter(getPropertySorter(sortOrder));
                     }
+                    CharacterMapExpander characterMapExpander = makeCharacterMapExpander(pipe, props, charMapIndex);
+                    ProxyReceiver normalizer = makeUnicodeNormalizer(pipe, props);
                     return customizeJSONSerializer(js, props, characterMapExpander, normalizer);
 
                 }
                 case "adaptive": {
-                    TEXTEmitter te = new TEXTEmitter();
-                    te.setPipelineConfiguration(pipe);
-                    te.setOutputProperties(props);                          // TODO: is the TEXTEmitter used?
-                    Writer writer = ((StreamResult) result).getWriter();
-                    if (writer == null) {
-                        OutputStream os = ((StreamResult) result).getOutputStream();
-                        String encoding = props.getProperty("encoding");
-                        if (encoding == null) {
-                            encoding = "UTF-8";
-                        }
-                        try {
-                            writer = new OutputStreamWriter(os, encoding);
-                        } catch (UnsupportedEncodingException e) {
-                            throw new XPathException(e);
-                        }
-                    }
+                    ExpandedStreamResult esr = new ExpandedStreamResult(pipe.getConfiguration(), (StreamResult)result, props);
+                    Writer writer = esr.obtainWriter();
                     AdaptiveEmitter je = new AdaptiveEmitter(pipe, writer);
                     je.setOutputProperties(props);
-                    StreamResult sr = (StreamResult) result;
-                    te.setStreamResult(sr);
+                    CharacterMapExpander characterMapExpander = makeCharacterMapExpander(pipe, props, charMapIndex);
+                    ProxyReceiver normalizer = makeUnicodeNormalizer(pipe, props);
                     return customizeAdaptiveSerializer(je, props, characterMapExpander, normalizer);
                 }
                 default: {
                     if (method.startsWith("{" + NamespaceConstant.SAXON + "}")) {
+                        CharacterMapExpander characterMapExpander = makeCharacterMapExpander(pipe, props, charMapIndex);
+                        ProxyReceiver normalizer = makeUnicodeNormalizer(pipe, props);
                         target = createSaxonSerializationMethod(
                                 method, params, pipe, characterMapExpander, normalizer, (StreamResult)result);
                         if (target instanceof Emitter) {
@@ -367,6 +342,22 @@ public class SerializerFactory {
         }
     }
 
+    private ProxyReceiver makeUnicodeNormalizer(PipelineConfiguration pipe, Properties props) throws XPathException {
+        String normForm = props.getProperty(SaxonOutputKeys.NORMALIZATION_FORM);
+        if (normForm != null && !normForm.equals("none")) {
+            return newUnicodeNormalizer(new Sink(pipe), props);
+        }
+        return null;
+    }
+
+    private CharacterMapExpander makeCharacterMapExpander(PipelineConfiguration pipe, Properties props, CharacterMapIndex charMapIndex) throws XPathException {
+        String useMaps = props.getProperty(SaxonOutputKeys.USE_CHARACTER_MAPS);
+        if (useMaps != null) {
+            return charMapIndex.makeCharacterMapExpander(useMaps, new Sink(pipe), this);
+        }
+        return null;
+    }
+
     /**
      * Get a Receiver to handle a result other than a StreamResult. This will generally not involve
      * serialization.
@@ -595,7 +586,7 @@ public class SerializerFactory {
         target = injectUnicodeNormalizer(params, target);
         target = injectCharacterMapExpander(params, target, true);
         String cdataElements = props.getProperty(OutputKeys.CDATA_SECTION_ELEMENTS);
-        if (cdataElements != null && cdataElements.length() > 0) {
+        if (cdataElements != null && !cdataElements.isEmpty()) {
             target = newCDATAFilter(target, props);
         }
 
@@ -665,7 +656,7 @@ public class SerializerFactory {
             target = injectCharacterMapExpander(params, target, true);
         }
         String cdataElements = props.getProperty(OutputKeys.CDATA_SECTION_ELEMENTS);
-        if (cdataElements != null && cdataElements.length() > 0 && !canonical) {
+        if (cdataElements != null && !cdataElements.isEmpty() && !canonical) {
             target = newCDATAFilter(target, props);
         }
         if (canonical) {
@@ -829,17 +820,6 @@ public class SerializerFactory {
         return new TEXTEmitter();
     }
 
-    /**
-     * Create a new Adaptive Emitter. This method exists so that it can be overridden in a subclass.
-     *
-     * @return the newly created adaptive emitter.
-     */
-
-    protected SequenceWriter newAdaptiveEmitter(PipelineConfiguration pipe, Writer writer) {
-        return new AdaptiveEmitter(pipe, writer);
-    }
-
-
     /**
      * Create a new XML Indenter. This method exists so that it can be overridden in a subclass.
      *
@@ -974,7 +954,7 @@ public class SerializerFactory {
      * @return the newly created filter.
      */
 
-    protected Receiver newAttributeSorter(Receiver next, Properties outputProperties)  throws XPathException {
+    protected Receiver newAttributeSorter(Receiver next, Properties outputProperties) throws XPathException {
         return next;
     }
 
