@@ -19,6 +19,7 @@ import net.sf.saxon.functions.URIQueryParameters;
 import net.sf.saxon.functions.UnparsedTextFunction;
 import net.sf.saxon.om.*;
 import net.sf.saxon.pattern.NodeKindTest;
+import net.sf.saxon.resource.StandardCollectionFinder;
 import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.Maker;
 import net.sf.saxon.trans.XPathException;
@@ -29,9 +30,9 @@ import net.sf.saxon.tree.tiny.Statistics;
 import net.sf.saxon.tree.tiny.TinyBuilder;
 import net.sf.saxon.tree.util.Navigator;
 import net.sf.saxon.value.AnyURIValue;
+import net.sf.saxon.value.ExternalObject;
 import net.sf.saxon.value.ObjectValue;
 import net.sf.saxon.value.TextFragmentValue;
-import net.sf.saxon.z.IntPredicate;
 import org.xml.sax.XMLReader;
 
 import javax.xml.transform.Source;
@@ -40,6 +41,7 @@ import javax.xml.transform.stream.StreamSource;
 import java.io.*;
 import java.net.URI;
 import java.net.URISyntaxException;
+import java.util.function.IntPredicate;
 
 /**
  * This class implements the default collection URI Resolver.
@@ -59,10 +61,10 @@ import java.net.URISyntaxException;
  * Otherwise, the resolver attempts to dereference the URI to obtain a catalog file. This is an XML file
  * containing a list of documents, in the format: </p>
  * <pre>
- * &lt;collection>
- *   &lt;doc href="doc1.xml"/>
- *   &lt;doc href="doc2.xml"/>
- * &lt;/collection>
+ * &lt;collection&gt;
+ *   &lt;doc href="doc1.xml"/&gt;
+ *   &lt;doc href="doc2.xml"/&gt;
+ * &lt;/collection&gt;
  * </pre>
  */
 
@@ -89,14 +91,10 @@ public class StandardCollectionURIResolver implements CollectionURIResolver {
      * or transformation.
      */
 
-    public SequenceIterator resolve(/*@Nullable*/ String href, String base, XPathContext context) throws XPathException {
+    public SequenceIterator<?> resolve(
+            String href, String base, XPathContext context) throws XPathException {
 
-        if (href == null) {
-            XPathException err = new XPathException("No default collection has been defined");
-            err.setErrorCode("FODC0002");
-            err.setXPathContext(context);
-            throw err;
-        }
+        StandardCollectionFinder.checkNotNull(href, context);
 
         URIQueryParameters params = null;
         URI relativeURI;
@@ -121,12 +119,7 @@ public class StandardCollectionURIResolver implements CollectionURIResolver {
 
         if ("file".equals(resolvedURI.getScheme())) {
             File file = new File(resolvedURI);
-            if (!file.exists()) {
-                XPathException err = new XPathException("The file or directory " + resolvedURI + " does not exist");
-                err.setErrorCode("FODC0002");
-                err.setXPathContext(context);
-                throw err;
-            }
+            checkFileExists(file, resolvedURI, context);
             if (file.isDirectory()) {
                 return directoryContents(file, params, context);
             }
@@ -135,6 +128,15 @@ public class StandardCollectionURIResolver implements CollectionURIResolver {
 
     }
 
+    public static void checkFileExists(File file, URI resolvedURI, XPathContext context) throws XPathException {
+        if (!file.exists()) {
+            XPathException err = new XPathException("The file or directory " + resolvedURI + " does not exist");
+            err.setErrorCode("FODC0002");
+            err.setXPathContext(context);
+            throw err;
+        }
+    }
+
     protected URI makeAbsoluteURI(String href, String base, XPathContext context, URI relativeURI) throws XPathException {
         URI resolvedURI;
         if (!relativeURI.isAbsolute()) {
@@ -165,7 +167,7 @@ public class StandardCollectionURIResolver implements CollectionURIResolver {
      * @return an iterator over the documents in the collection
      */
 
-    protected SequenceIterator directoryContents(File directory, URIQueryParameters params, XPathContext context) {
+    protected SequenceIterator<?> directoryContents(File directory, URIQueryParameters params, XPathContext context) {
 
         FilenameFilter filter = null;
 
@@ -185,7 +187,7 @@ public class StandardCollectionURIResolver implements CollectionURIResolver {
 
         ObjectValue<File>[] fileValues = new ObjectValue[files.length];
         for (int f = 0; f < files.length; f++) {
-            fileValues[f] = new ObjectValue<File>(files[f]);
+            fileValues[f] = new ObjectValue<>(files[f]);
         }
 
         // If the URI requested suppression of errors, or that errors should be treated
@@ -231,7 +233,7 @@ public class StandardCollectionURIResolver implements CollectionURIResolver {
             });
         }
         FileExpander expander = new FileExpander(params, newPipe);
-        SequenceIterator base = new ArrayIterator(fileValues);
+        SequenceIterator<?> base = new ArrayIterator<>(fileValues);
         return new MappingIterator(base, expander);
     }
 
@@ -246,7 +248,7 @@ public class StandardCollectionURIResolver implements CollectionURIResolver {
      * @throws XPathException if any failures occur
      */
 
-    protected SequenceIterator catalogContents(String href, String baseURI, String absURI, final XPathContext context)
+    protected SequenceIterator<?> catalogContents(String href, String baseURI, String absURI, final XPathContext context)
             throws XPathException {
 
         boolean stable = true;
@@ -348,7 +350,7 @@ public class StandardCollectionURIResolver implements CollectionURIResolver {
      */
 
 
-    protected static class FileExpander implements MappingFunction {
+    protected static class FileExpander implements MappingFunction<ExternalObject<File>, Item<?>> {
 
         private URIQueryParameters params;
         boolean recurse = false;
@@ -402,8 +404,8 @@ public class StandardCollectionURIResolver implements CollectionURIResolver {
          * sequence.
          */
 
-        public SequenceIterator map(Item item) throws XPathException {
-            File file = ((ObjectValue<File>)item).getObject();
+        public SequenceIterator<?> map(ExternalObject<File> item) throws XPathException {
+            File file = item.getObject();
             if (file.isDirectory()) {
                 if (recurse) {
                     File[] files;
@@ -413,13 +415,13 @@ public class StandardCollectionURIResolver implements CollectionURIResolver {
                         files = file.listFiles(filter);
                     }
 
-                    ObjectValue[] fileValues = new ObjectValue[files.length];
+                    ObjectValue<File>[] fileValues = new ObjectValue[files.length];
                     for (int f = 0; f < files.length; f++) {
-                        fileValues[f] = new ObjectValue<File>(files[f]);
+                        fileValues[f] = new ObjectValue<>(files[f]);
                     }
 
                     FileExpander expander = new FileExpander(params, pipe);
-                    return new MappingIterator(new ArrayIterator(fileValues), expander);
+                    return new MappingIterator<>(new ArrayIterator<>(fileValues), expander);
                 } else {
                     return null;
                 }
@@ -431,7 +433,6 @@ public class StandardCollectionURIResolver implements CollectionURIResolver {
                     String uri = file.toURI().toString();
                     TextFragmentValue doc = new TextFragmentValue(pipe.getConfiguration(), content, uri);
                     doc.setSystemId(file.toURI().toString());
-                    doc.setConfiguration(pipe.getConfiguration());
                     return SingletonIterator.makeIterator(doc);
                 } catch (IOException err) {
                     if (onError == URIQueryParameters.ON_ERROR_IGNORE) {
