@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,6 +8,7 @@
 package net.sf.saxon.lib;
 
 import net.sf.saxon.Configuration;
+import net.sf.saxon.expr.Expression;
 import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.expr.instruct.*;
 import net.sf.saxon.expr.parser.ExplicitLocation;
@@ -17,6 +18,11 @@ import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.om.StandardNames;
 import net.sf.saxon.om.StructuredQName;
+import net.sf.saxon.regex.BMPString;
+import net.sf.saxon.regex.GeneralUnicodeString;
+import net.sf.saxon.regex.LatinString;
+import net.sf.saxon.regex.UnicodeString;
+import net.sf.saxon.serialize.charcode.UTF16CharacterSet;
 import net.sf.saxon.trace.ContextStackFrame;
 import net.sf.saxon.trace.ContextStackIterator;
 import net.sf.saxon.trace.InstructionInfo;
@@ -25,6 +31,7 @@ import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.KeyDefinition;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.AttributeLocation;
+import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.tree.util.Navigator;
 import net.sf.saxon.type.Type;
 import net.sf.saxon.type.ValidationException;
@@ -49,6 +56,7 @@ public class StandardErrorListener implements UnfailingErrorListener {
     private int recoveryPolicy = Configuration.RECOVER_WITH_WARNINGS;
     private int warningCount = 0;
     private int maximumNumberOfWarnings = 25;
+    private int maxOrdinaryCharacter = 255;
     protected transient Logger logger = new StandardLogger();
 
     /**
@@ -151,13 +159,35 @@ public class StandardErrorListener implements UnfailingErrorListener {
         return this.maximumNumberOfWarnings;
     }
 
+    /**
+     * Set the maximum codepoint value for a character to be considered non-special.
+     * Special characters (codepoints above this value) will be expanded in hex for
+     * extra clarity in the error message.
+     * @param max the highest codepoint considered non-special (defaults to 255)
+     */
+
+    public void setMaxOrdinaryCharacter(int max) {
+        maxOrdinaryCharacter = max;
+    }
+
+    /**
+     * Set the maximum codepoint value for a character to be considered non-special.
+     * Special characters (codepoints above this value) will be expanded in hex for
+     * extra clarity in the error message.
+     *
+     * @param max the highest codepoint considered non-special (defaults to 255)
+     */
+
+    public int getMaxOrdinaryCharacter(int max) {
+        return maxOrdinaryCharacter;
+    }
+
+
     /**
      * Receive notification of a warning.
-     * <p/>
      * <p>Transformers can use this method to report conditions that
      * are not errors or fatal errors.  The default behaviour is to
      * take no action.</p>
-     * <p/>
      * <p>After invoking this method, the Transformer must continue with
      * the transformation. It should still be possible for the
      * application to process the document through to the end.</p>
@@ -197,14 +227,12 @@ public class StandardErrorListener implements UnfailingErrorListener {
 
     /**
      * Receive notification of a recoverable error.
-     * <p/>
      * <p>The transformer must continue to provide normal parsing events
      * after invoking this method.  It should still be possible for the
      * application to process the document through to the end.</p>
-     * <p/>
      * <p>The action of the standard error listener depends on the
      * recovery policy that has been set, which may be one of RECOVER_SILENTLY,
-     * RECOVER_WITH_WARNING, or DO_NOT_RECOVER
+     * RECOVER_WITH_WARNING, or DO_NOT_RECOVER</p>
      *
      * @param exception The error information encapsulated in a
      *                  transformer exception.
@@ -261,7 +289,6 @@ public class StandardErrorListener implements UnfailingErrorListener {
 
     /**
      * Receive notification of a non-recoverable error.
-     * <p/>
      * <p>The application must assume that the transformation cannot
      * continue after the Transformer has invoked this method,
      * and should continue (if at all) only to collect
@@ -321,6 +348,10 @@ public class StandardErrorListener implements UnfailingErrorListener {
 
     private String constructMessage(TransformerException exception, XPathException xe, String langText, String kind) {
         String message;
+        Expression failingExpression = null;
+        if (exception instanceof XPathException) {
+            failingExpression = ((XPathException)exception).getFailingExpression();
+        }
         if (xe.getLocator() instanceof AttributeLocation) {
             String line1 = kind + /*where +*/ langText + getLocationMessageText(xe.getLocator()) + "\n";
             String line2 = "  " + wordWrap(getExpandedMessage(exception));
@@ -340,9 +371,11 @@ public class StandardErrorListener implements UnfailingErrorListener {
                 String nearBy = nestedLoc.getNearbyText();
 
                 innerLoc = lineInfo + columnInfo;
-                if (nearBy != null && !nearBy.isEmpty()) {
-                    innerLoc += (nearBy.startsWith("...") ? "near" : "in") +
-                            ' ' + Err.wrap(nearBy) + ' ';
+                if (failingExpression != null) {
+                    innerLoc = lineInfo + "evaluating (" + failingExpression.toShortString() + ") ";
+                } else if (nearBy != null && !nearBy.isEmpty()) {
+                    innerLoc = lineInfo + (nearBy.startsWith("...") ? "near" : "in") +
+                            ' ' + Err.wrap(nearBy) + ' ' + columnInfo;
                 }
 
                 String where = innerLoc;
@@ -368,7 +401,9 @@ public class StandardErrorListener implements UnfailingErrorListener {
                     innerLoc += "of " + outerLoc.getSystemId() + " ";
                 }
                 String nearBy = nestedLoc.getNearbyText();
-                if (nearBy != null && !nearBy.isEmpty()) {
+                if (failingExpression != null) {
+                    innerLoc += "evaluating (" + failingExpression.toShortString() + ")";
+                } else if (nearBy != null && !nearBy.isEmpty()) {
                     innerLoc += (nearBy.startsWith("...") ? "near" : "in") +
                             ' ' + Err.wrap(nearBy) + ' ';
                 }
@@ -392,12 +427,13 @@ public class StandardErrorListener implements UnfailingErrorListener {
             message += getOffenderListText(failure);
         } else {
             message = kind +
+                    (failingExpression != null ? "evaluating (" + failingExpression.toShortString() + ") " : "") +
                     getLocationMessage(exception) +
                     "\n  " +
                     wordWrap(getExpandedMessage(exception));
 
         }
-        return message;
+        return expandSpecialCharacters(message).toString();
     }
 
     public static String getOffenderListText(ValidationFailure failure) {
@@ -599,7 +635,7 @@ public class StandardErrorListener implements UnfailingErrorListener {
         }
 
         if (err instanceof XPathException) {
-            Sequence errorObject = ((XPathException) err).getErrorObject();
+            Sequence<?> errorObject = ((XPathException) err).getErrorObject();
             if (errorObject != null) {
                 String errorObjectDesc = getErrorObjectString(errorObject);
                 if (errorObjectDesc != null) {
@@ -653,7 +689,7 @@ public class StandardErrorListener implements UnfailingErrorListener {
      */
 
     @SuppressWarnings({"UnusedParameters"})
-    public String getErrorObjectString(Sequence errorObject) {
+    public String getErrorObjectString(Sequence<?> errorObject) {
         return null;
     }
 
@@ -772,5 +808,68 @@ public class StandardErrorListener implements UnfailingErrorListener {
         }
     }
 
+    /**
+     * Expand any special characters appearing in a message. Special characters
+     * will be output as themselves, followed by a hex codepoint in the form [xHHHHH].
+     * Characters are considered special if they have a codepoint above the value set
+     * using {@link #setMaxOrdinaryCharacter(int)}
+     *
+     * @param in        the message to be expanded
+     */
+
+    public CharSequence expandSpecialCharacters(CharSequence in) {
+        if (logger.isUnicodeAware()) {
+            return in;
+        } else {
+            return expandSpecialCharacters(in, maxOrdinaryCharacter);
+        }
+    }
+
+    /**
+     * Expand any special characters appearing in a message. Special characters
+     * will be output as themselves, followed by a hex codepoint in the form [xHHHHH].
+     * @param in the message to be expanded
+     * @param threshold the codepoint above which characters are considered special.
+     */
+
+    public static CharSequence expandSpecialCharacters(CharSequence in, int threshold) {
+        if (threshold >= UTF16CharacterSet.NONBMP_MAX) {
+            return in;
+        }
+        int max = 0;
+        boolean isAstral = false;
+        for (int i=0; i<in.length(); i++) {
+            char c = in.charAt(i);
+            if (c > max) {
+                max = c;
+            }
+            if (UTF16CharacterSet.isSurrogate(c)) {
+                isAstral = true;
+            }
+        }
+        if (max <= threshold && !isAstral) {
+            return in;
+        }
+        UnicodeString str;
+        if (max <= 255) {
+            str = new LatinString(in);
+        } else if (!isAstral) {
+            str = new BMPString(in);
+        } else {
+            str = new GeneralUnicodeString(in);
+        }
+        FastStringBuffer fsb = new FastStringBuffer(str.uLength()*2);
+        for (int i=0; i<str.uLength(); i++) {
+            int ch = str.uCharAt(i);
+            fsb.appendWideChar(ch);
+            if (ch > threshold) {
+                fsb.append("[x");
+                fsb.append(Integer.toHexString(ch));
+                fsb.append("]");
+            }
+        }
+        return fsb;
+    }
+
 }
 
