@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,11 +7,15 @@
 
 package net.sf.saxon.lib;
 
+import net.sf.saxon.trans.SaxonErrorCode;
 import net.sf.saxon.trans.XPathException;
 
 import javax.xml.transform.Result;
 import javax.xml.transform.stream.StreamResult;
-import java.io.*;
+import java.io.File;
+import java.io.IOException;
+import java.io.OutputStream;
+import java.io.Writer;
 import java.net.*;
 
 
@@ -72,7 +76,7 @@ public class StandardOutputResolver implements OutputURIResolver {
             URI absoluteURI;
             if (href.isEmpty()) {
                 if (base == null) {
-                    throw new XPathException("The system identifier of the principal output file is unknown");
+                    throw new XPathException("The system identifier of the principal output file is unknown", SaxonErrorCode.SXRD0002);
                 }
                 absoluteURI = new URI(base);
             } else {
@@ -81,7 +85,7 @@ public class StandardOutputResolver implements OutputURIResolver {
             }
             if (!absoluteURI.isAbsolute()) {
                 if (base == null) {
-                    throw new XPathException("The system identifier of the principal output file is unknown");
+                    throw new XPathException("The system identifier of the principal output file is unknown", SaxonErrorCode.SXRD0002);
                 }
                 which = "base";
                 URI baseURI = new URI(base);
@@ -91,15 +95,25 @@ public class StandardOutputResolver implements OutputURIResolver {
 
             return createResult(absoluteURI);
         } catch (URISyntaxException err) {
-            throw new XPathException("Invalid syntax for " + which + " URI", err);
+            XPathException xe = new XPathException("Invalid syntax for " + which + " URI");
+            xe.setErrorCode(SaxonErrorCode.SXRD0001);
+            throw xe;
         } catch (IllegalArgumentException err2) {
-            throw new XPathException("Invalid " + which + " URI syntax", err2);
+            XPathException xe = new XPathException("Invalid " + which + " URI syntax");
+            xe.setErrorCode(SaxonErrorCode.SXRD0001);
+            throw xe;
         } catch (MalformedURLException err3) {
-            throw new XPathException("Resolved URL is malformed", err3);
-        } catch (UnknownServiceException err5) {
-            throw new XPathException("Specified protocol does not allow output", err5);
-        } catch (IOException err4) {
-            throw new XPathException("Cannot open connection to specified URL", err4);
+            XPathException xe = new XPathException("Resolved URL is malformed", err3);
+            xe.setErrorCode(SaxonErrorCode.SXRD0001);
+            throw xe;
+        } catch (UnknownServiceException err4) {
+            XPathException xe = new XPathException("Specified protocol does not allow output", err4);
+            xe.setErrorCode(SaxonErrorCode.SXRD0001);
+            throw xe;
+        } catch (IOException err5) {
+            XPathException xe = new XPathException("Cannot open connection to specified URL", err5);
+            xe.setErrorCode(SaxonErrorCode.SXRD0001);
+            throw xe;
         }
     }
 
@@ -129,14 +143,15 @@ public class StandardOutputResolver implements OutputURIResolver {
      *
      * @param absoluteURI the URI of the output file (which should use the "file" scheme
      * @return a Result object referencing this output file
-     * @throws XPathException
+     * @throws XPathException if the URI cannot be written to
      */
 
     public static synchronized Result makeOutputFile(URI absoluteURI) throws XPathException {
         try {
             return new StreamResult(new File(absoluteURI));
         } catch (IllegalArgumentException err) {
-            throw new XPathException("Cannot write to URI " + absoluteURI + " (" + err.getMessage() + ")");
+            throw new XPathException(
+                    "Cannot write to URI " + absoluteURI + " (" + err.getMessage() + ")", SaxonErrorCode.SXRD0001);
         }
     }
 
@@ -156,7 +171,9 @@ public class StandardOutputResolver implements OutputURIResolver {
                 try {
                     stream.close();
                 } catch (java.io.IOException err) {
-                    throw new XPathException("Failed while closing output file", err);
+                    XPathException xe = new XPathException("Failed while closing output file", err);
+                    xe.setErrorCode(SaxonErrorCode.SXRD0003);
+                    throw xe;
                 }
             }
             Writer writer = ((StreamResult) result).getWriter(); // Path not used, but there for safety
@@ -164,7 +181,9 @@ public class StandardOutputResolver implements OutputURIResolver {
                 try {
                     writer.close();
                 } catch (java.io.IOException err) {
-                    throw new XPathException("Failed while closing output file", err);
+                    XPathException xe = new XPathException("Failed while closing output file", err);
+                    xe.setErrorCode(SaxonErrorCode.SXRD0003);
+                    throw xe;
                 }
             }
         }
