@@ -86,10 +86,7 @@ public class HashTrieMap implements MapItem {
      * @return a singleton map
      */
 
-    public static HashTrieMap singleton(AtomicValue key, GroundedValue value) {
-//        if (Instrumentation.ACTIVE) {
-//            Instrumentation.count("singletons");
-//        }
+    public static HashTrieMap singleton(AtomicValue key, GroundedValue<?> value) {
         return new HashTrieMap().addEntry(key, value);
     }
 
@@ -99,9 +96,6 @@ public class HashTrieMap implements MapItem {
      */
 
     public HashTrieMap(ImmutableMap<AtomicMatchKey, KeyValuePair> imap) {
-//        if (Instrumentation.ACTIVE) {
-//            Instrumentation.count("new maps");
-//        }
         this.imap = imap;
         entries = -1;
     }
@@ -134,7 +128,7 @@ public class HashTrieMap implements MapItem {
      * @param wasEmpty true if the map was empty before adding these values
      */
 
-    private void updateTypeInformation(AtomicValue key, Sequence val, boolean wasEmpty) {
+    private void updateTypeInformation(AtomicValue key, Sequence<?> val, boolean wasEmpty) {
 //        if (Instrumentation.ACTIVE) {
 //            Instrumentation.count("updateTypeInformation");
 //        }
@@ -280,7 +274,7 @@ public class HashTrieMap implements MapItem {
         AtomicIterator keyIter = keys();
         AtomicValue key;
         while ((key = keyIter.next()) != null) {
-            Sequence val = get(key);
+            Sequence<?> val = get(key);
             if (keyType == null) {
                 keyType = key.getItemType();
                 valueType = SequenceTool.getItemType(val, th);
@@ -325,10 +319,7 @@ public class HashTrieMap implements MapItem {
      * @return the new map containing the additional entry
      */
 
-    public HashTrieMap addEntry(AtomicValue key, GroundedValue value) {
-//        if (Instrumentation.ACTIVE) {
-//            Instrumentation.count("addEntry");
-//        }
+    public HashTrieMap addEntry(AtomicValue key, GroundedValue<?> value) {
         boolean empty = isEmpty();
         ImmutableMap<AtomicMatchKey, KeyValuePair> imap2 = imap.put(makeKey(key), new KeyValuePair(key, value));
         HashTrieMap t2 = new HashTrieMap(imap2);
@@ -351,7 +342,7 @@ public class HashTrieMap implements MapItem {
      * @return true if an existing entry with the same key was replaced
      */
 
-    public boolean initialPut(AtomicValue key, GroundedValue value) {
+    public boolean initialPut(AtomicValue key, GroundedValue<?> value) {
 //        if (Instrumentation.ACTIVE) {
 //            Instrumentation.count("initialPut");
 //        }
@@ -400,10 +391,7 @@ public class HashTrieMap implements MapItem {
      * @return the value associated with the given key, or null if the key is not present in the map
      */
 
-    public GroundedValue get(AtomicValue key)  {
-//        if (Instrumentation.ACTIVE) {
-//            Instrumentation.count("get");
-//        }
+    public GroundedValue<?> get(AtomicValue key)  {
         KeyValuePair o = imap.get(makeKey(key));
         return o==null ? null : o.value;
     }
@@ -416,9 +404,6 @@ public class HashTrieMap implements MapItem {
      */
 
     public KeyValuePair getKeyValuePair(AtomicValue key) {
-//        if (Instrumentation.ACTIVE) {
-//            Instrumentation.count("getKVP");
-//        }
         return imap.get(makeKey(key));
     }
 
@@ -428,9 +413,6 @@ public class HashTrieMap implements MapItem {
      */
 
     public AtomicIterator keys() {
-//        if (Instrumentation.ACTIVE) {
-//            Instrumentation.count("keys");
-//        }
         return new AtomicIterator() {
 
             Iterator<Tuple2<AtomicMatchKey, KeyValuePair>> base = imap.iterator();
@@ -482,7 +464,7 @@ public class HashTrieMap implements MapItem {
             Tuple2<AtomicMatchKey, KeyValuePair> entry = iter.next();
             AtomicMatchKey k1 = entry._1;
             AtomicValue k2 = entry._2.key;
-            Sequence v = entry._2.value;
+            Sequence<?> v = entry._2.value;
             System.err.println(k1.getClass() + " " + k1 +
                                        " #:" + k1.hashCode() +
                                        " = (" + k2.getClass() + " " + k2 + " : " + v + ")");
@@ -503,7 +485,7 @@ public class HashTrieMap implements MapItem {
                 Tuple2<AtomicMatchKey, KeyValuePair> entry = iter.next();
                 AtomicMatchKey k1 = entry._1;
                 AtomicValue k2 = entry._2.key;
-                Sequence v = entry._2.value;
+                Sequence<?> v = entry._2.value;
                 buff.append(k2.toShortString());
                 buff.append(':');
                 buff.append(Err.depictSequence(v).toString().trim());
