@@ -123,7 +123,7 @@ public class MapFunctionSet extends BuiltInFunctionSet {
      */
     public static class MapContains extends SystemFunction {
 
-        public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+        public BooleanValue call(XPathContext context, Sequence[] arguments) throws XPathException {
             MapItem map = (MapItem) arguments[0].head();
             AtomicValue key = (AtomicValue) arguments[1].head();
             return BooleanValue.get(map.get(key) != null);
@@ -153,7 +153,7 @@ public class MapFunctionSet extends BuiltInFunctionSet {
                 if (arguments[1] instanceof Literal) {
                     String key = ((Literal)arguments[1]).getValue().getStringValue();
                     if (((TupleType)it).getFieldType(key) == null) {
-                        XPathException xe = new XPathException("Field " + key + " is not defined for tuple type " + it.toString(), "SXTT0001");
+                        XPathException xe = new XPathException("Field " + key + " is not defined for tuple type " + it, "SXTT0001");
                         xe.setIsTypeError(true);
                         throw xe;
                     }
@@ -248,11 +248,11 @@ public class MapFunctionSet extends BuiltInFunctionSet {
             }
         }
 
-        public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+        public Sequence<?> call(XPathContext context, Sequence[] arguments) throws XPathException {
             MapItem map = (MapItem) arguments[0].head();
             assert map != null;
             AtomicValue key = (AtomicValue) arguments[1].head();
-            Sequence value = map.get(key);
+            Sequence<?> value = map.get(key);
             if (value == null) {
                 return EmptySequence.getInstance();
             } else {
@@ -267,21 +267,21 @@ public class MapFunctionSet extends BuiltInFunctionSet {
      */
     public static class MapFind extends SystemFunction {
 
-        public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
-            List<GroundedValue<? extends Item>> result = new ArrayList<>();
+        public ArrayItem call(XPathContext context, Sequence[] arguments) throws XPathException {
+            List<GroundedValue<?>> result = new ArrayList<>();
             AtomicValue key = (AtomicValue) arguments[1].head();
             processSequence(arguments[0], key, result);
             return new SimpleArrayItem(result);
         }
 
-        private void processSequence(Sequence in, AtomicValue key, List<GroundedValue<? extends Item>> result) throws XPathException {
+        private void processSequence(Sequence<?> in, AtomicValue key, List<GroundedValue<?>> result) throws XPathException {
             in.iterate().forEachOrFail(item -> {
                 if (item instanceof ArrayItem) {
-                    for (Sequence sequence : ((ArrayItem) item).members()) {
+                    for (Sequence<?> sequence : ((ArrayItem) item).members()) {
                         processSequence(sequence, key, result);
                     }
                 } else if (item instanceof MapItem) {
-                    GroundedValue<? extends Item> value = ((MapItem) item).get(key);
+                    GroundedValue<?> value = ((MapItem) item).get(key);
                     if (value != null) {
                         result.add(value);
                     }
@@ -299,10 +299,10 @@ public class MapFunctionSet extends BuiltInFunctionSet {
      */
     public static class MapEntry extends SystemFunction {
 
-        public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+        public Sequence<?> call(XPathContext context, Sequence[] arguments) throws XPathException {
             AtomicValue key = (AtomicValue) arguments[0].head();
             assert key != null;
-            GroundedValue value = ((SequenceIterator<Item>) arguments[1].iterate()).materialize();
+            GroundedValue<?> value = ((Sequence<?>)arguments[1]).iterate().materialize();
             return HashTrieMap.singleton(key, value);
         }
 
@@ -336,13 +336,13 @@ public class MapFunctionSet extends BuiltInFunctionSet {
      */
     public static class MapForEach extends SystemFunction {
 
-        public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+        public Sequence<?> call(XPathContext context, Sequence[] arguments) throws XPathException {
             MapItem map = (MapItem) arguments[0].head();
             Function fn = (Function) arguments[1].head();
-            List<GroundedValue> results = new ArrayList<>();
+            List<GroundedValue<?>> results = new ArrayList<>();
             for (KeyValuePair pair : map.keyValuePairs()) {
-                Sequence seq = dynamicCall(fn, context, new Sequence[]{pair.key, pair.value});
-                results.add(((Sequence<Item>) seq).materialize());
+                Sequence<?> seq = dynamicCall(fn, context, new Sequence[]{pair.key, pair.value});
+                results.add(seq.materialize());
             }
             return new Chain(results);
         }
@@ -353,7 +353,7 @@ public class MapFunctionSet extends BuiltInFunctionSet {
      */
     public static class MapKeys extends SystemFunction {
 
-        public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+        public Sequence<? extends Item<AtomicValue>> call(XPathContext context, Sequence[] arguments) throws XPathException {
             MapItem map = (MapItem) arguments[0].head();
             assert map != null;
             return SequenceTool.toLazySequence(map.keys());
@@ -386,7 +386,7 @@ public class MapFunctionSet extends BuiltInFunctionSet {
         public Expression makeOptimizedFunctionCall(ExpressionVisitor visitor, ContextItemStaticInfo contextInfo, Expression... arguments) throws XPathException {
             if (arguments.length == 2 && arguments[1] instanceof Literal) {
                 MapItem options = (MapItem) ((Literal)arguments[1]).getValue().head();
-                Map<String, Sequence> values = getDetails().optionDetails.processSuppliedOptions(
+                Map<String, Sequence<?>> values = getDetails().optionDetails.processSuppliedOptions(
                         options, visitor.getStaticContext().makeEarlyEvaluationContext());
                 String duplicates = ((StringValue) values.get("duplicates")).getStringValue();
                 String duplicatesErrorCode = ((StringValue) values.get("duplicates-error-code")).getStringValue();
@@ -407,18 +407,40 @@ public class MapFunctionSet extends BuiltInFunctionSet {
         @Override
         public ItemType getResultItemType(Expression[] args) {
             ItemType it = args[0].getItemType();
-            return it == ErrorType.getInstance() ? MapType.EMPTY_MAP_TYPE : it;
+            if (it == ErrorType.getInstance()) {
+                return MapType.EMPTY_MAP_TYPE;
+            } else if (it instanceof MapType) {
+                boolean maybeCombined = true;  // see bug 3980
+                if (args.length == 1) {
+                    maybeCombined = false;
+                } else if (args[1] instanceof Literal) {
+                    MapItem options = (MapItem) ((Literal) args[1]).getValue().head();
+                    GroundedValue dupes = options.get(new StringValue("duplicates"));
+                    try {
+                        if (!"combine".equals(dupes.getStringValue())) {
+                            maybeCombined = false;
+                        }
+                    } catch (XPathException e) {
+                        //
+                    }
+                }
+                if (maybeCombined) {
+                    return new MapType(((MapType) it).getKeyType(),
+                                       SequenceType.makeSequenceType(((MapType) it).getValueType().getPrimaryType(), StaticProperty.ALLOWS_ZERO_OR_MORE));
+                } else {
+                    return it;
+                }
+            } else {
+                return super.getResultItemType(args);
+            }
         }
 
-        public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
-//            if (Instrumentation.ACTIVE) {
-//                Instrumentation.count("map:merge");
-//            }
+        public MapItem call(XPathContext context, Sequence[] arguments) throws XPathException {
             String duplicates = this.duplicates;
             String duplicatesErrorCode = this.duplicatesErrorCode;
             if (arguments.length > 1) {
                 MapItem options = (MapItem) arguments[1].head();
-                Map<String, Sequence> values = getDetails().optionDetails.processSuppliedOptions(options, context);
+                Map<String, Sequence<?>> values = getDetails().optionDetails.processSuppliedOptions(options, context);
                 duplicates = ((StringValue) values.get("duplicates")).getStringValue();
                 duplicatesErrorCode = ((StringValue) values.get("duplicates-error-code")).getStringValue();
             }
@@ -434,7 +456,7 @@ public class MapFunctionSet extends BuiltInFunctionSet {
                 MapItem next;
                 while ((next = (MapItem) iter.next()) != null) {
                     for (KeyValuePair pair : next.keyValuePairs()) {
-                        Sequence<? extends Item> existing = baseMap.get(pair.key);
+                        Sequence<?> existing = baseMap.get(pair.key);
                         if (existing != null) {
                             switch (duplicates) {
                                 case "use-first":
@@ -448,7 +470,7 @@ public class MapFunctionSet extends BuiltInFunctionSet {
                                 case "combine":
                                     InsertBefore.InsertIterator combinedIter =
                                             new InsertBefore.InsertIterator(pair.value.iterate(), existing.iterate(), 1);
-                                    GroundedValue combinedValue = combinedIter.materialize();
+                                    GroundedValue<?> combinedValue = combinedIter.materialize();
                                     baseMap = ((HashTrieMap) baseMap).addEntry(pair.key, combinedValue);
                                     break;
                                 default:
@@ -491,7 +513,7 @@ public class MapFunctionSet extends BuiltInFunctionSet {
 
     public static class MapPut extends SystemFunction {
 
-        public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+        public MapItem call(XPathContext context, Sequence[] arguments) throws XPathException {
 
             MapItem baseMap = (MapItem) arguments[0].head();
 
@@ -500,7 +522,8 @@ public class MapFunctionSet extends BuiltInFunctionSet {
             }
 
             AtomicValue key = (AtomicValue) arguments[1].head();
-            GroundedValue value = ((Sequence<Item>) arguments[2]).materialize();
+            GroundedValue<?> value =
+                    ((Sequence<?>)arguments[2]).materialize();
             KeyValuePair pair = new KeyValuePair(key, value);
             return ((HashTrieMap) baseMap).addEntry(pair.key, pair.value);
         }
@@ -512,7 +535,7 @@ public class MapFunctionSet extends BuiltInFunctionSet {
      */
     public static class MapRemove extends SystemFunction {
 
-        public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+        public MapItem call(XPathContext context, Sequence[] arguments) throws XPathException {
             MapItem map = (MapItem) arguments[0].head();
             SequenceIterator iter = arguments[1].iterate();
             AtomicValue key;
@@ -529,7 +552,7 @@ public class MapFunctionSet extends BuiltInFunctionSet {
      */
     public static class MapSize extends SystemFunction {
 
-        public Sequence call(XPathContext context, Sequence[] arguments) throws XPathException {
+        public IntegerValue call(XPathContext context, Sequence[] arguments) throws XPathException {
             MapItem map = (MapItem) arguments[0].head();
             return new Int64Value(map.size());
         }
