@@ -27,7 +27,7 @@ import net.sf.saxon.value.SequenceType;
 /**
  * Interface supported by different implementations of an XDM map item
  */
-public interface MapItem extends Function<MapItem> {
+public interface MapItem extends Function {
 
     /**
      * Get an entry from the Map
@@ -35,7 +35,7 @@ public interface MapItem extends Function<MapItem> {
      * @return the value associated with the given key, or null if the key is not present in the map.
      */
 
-    GroundedValue<? extends Item> get(AtomicValue key);
+    GroundedValue<?> get(AtomicValue key);
 
     /**
      * Get the size of the map
@@ -78,7 +78,7 @@ public interface MapItem extends Function<MapItem> {
      * @return the new map containing the additional entry
      */
 
-    MapItem addEntry(AtomicValue key, GroundedValue<? extends Item> value);
+    MapItem addEntry(AtomicValue key, GroundedValue<?> value);
 
     /**
      * Remove an entry from the map
@@ -201,13 +201,13 @@ public interface MapItem extends Function<MapItem> {
      * be efficiently determined
      */
 
-    static boolean isKnownToConform(Sequence value, ItemType itemType) {
+    static boolean isKnownToConform(Sequence<?> value, ItemType itemType) {
         // Problem is we don't have access to a TypeHierarchy object...
         if (itemType == AnyItemType.getInstance()) {
             return true;
         }
         try {
-            SequenceIterator iter = value.iterate();
+            SequenceIterator<?> iter = value.iterate();
             Item item;
             while ((item = iter.next()) != null) {
                 if (item instanceof AtomicValue) {
@@ -245,7 +245,7 @@ public interface MapItem extends Function<MapItem> {
      * values in the sequence are known to conform to this type; otherwise item().
      */
 
-    static ItemType getItemTypeOfSequence(Sequence val) {
+    static ItemType getItemTypeOfSequence(Sequence<?> val) {
         try {
             Item first = val.head();
             if (first == null) {
@@ -339,9 +339,9 @@ public interface MapItem extends Function<MapItem> {
      * @throws XPathException if an error occurs evaluating
      *                        the supplied argument
      */
-    default Sequence call(XPathContext context, Sequence[] args) throws XPathException {
+    default Sequence<?> call(XPathContext context, Sequence[] args) throws XPathException {
         AtomicValue key = (AtomicValue) args[0].head();
-        Sequence value = get(key);
+        Sequence<?> value = get(key);
         if (value == null) {
             return EmptySequence.getInstance();
         } else {
@@ -422,8 +422,8 @@ public interface MapItem extends Function<MapItem> {
             AtomicIterator keys = keys();
             AtomicValue key;
             while ((key = keys.next()) != null) {
-                Sequence thisValue = get(key);
-                Sequence otherValue = ((MapItem) other).get(key);
+                Sequence<?> thisValue = get(key);
+                Sequence<?> otherValue = ((MapItem) other).get(key);
                 if (otherValue == null) {
                     return false;
                 }
