@@ -32,7 +32,7 @@ import java.util.Iterator;
  *
  */
 
-public interface Item<T extends Item> extends GroundedValue<T> {
+public interface Item<T extends Item<?>> extends GroundedValue<T> {
 
     /**
      * Get the genre of this item (to distinguish the top-level categories of item,
@@ -180,6 +180,10 @@ public interface Item<T extends Item> extends GroundedValue<T> {
         return this;
     }
 
+    static <T extends Item<?>> GroundedValue<T> toGroundedValue(Item<T> item) {
+        return item.reduce();
+    }
+
     /**
      * Ask whether this is a node in a streamed document
      *
@@ -190,11 +194,6 @@ public interface Item<T extends Item> extends GroundedValue<T> {
         return false;
     }
 
-    static Sequence<? extends Item>[] asArgumentArray(Item item) {
-        return new Sequence[]{item};
-    }
-
-
 
 }
 
