@@ -25,7 +25,7 @@ import java.util.Arrays;
  * to the end) contains the entire value.
  */
 
-public class MemoSequence<T extends Item> implements Sequence<T> {
+public class MemoSequence<T extends Item<?>> implements Sequence<T> {
 
     SequenceIterator<T> inputIterator;
 
@@ -50,7 +50,6 @@ public class MemoSequence<T extends Item> implements Sequence<T> {
     // State in which we know that the value is an empty sequence
     protected static final int EMPTY = 5;
 
-
     public MemoSequence(SequenceIterator<T> iterator) {
         this.inputIterator = iterator;
     }
@@ -294,7 +293,8 @@ public class MemoSequence<T extends Item> implements Sequence<T> {
                 if (used == 0) {
                     return EmptySequence.getInstance();
                 } else if (used == 1) {
-                    return reservoir[0];
+                    T item = reservoir[0];
+                    return (Item<T>)reservoir[0];
                 } else {
                     return new SequenceExtent<>(reservoir);
                 }
