@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -14,14 +14,11 @@ import net.sf.saxon.tree.tiny.Statistics;
 import net.sf.saxon.tree.tiny.TinyBuilder;
 import net.sf.saxon.tree.tiny.TinyBuilderCondensed;
 
-import java.io.Serializable;
-
 /**
  * A TreeModel represents an implementation of the Saxon NodeInfo interface, which itself
  * is essentially an implementation of the XDM model defined in W3C specifications (except
  * that Saxon's NodeInfo understands the 13 XPath axes, rather than merely supporting
  * parent and child properties).
- * <p/>
  * <p>This class serves two purposes: it acts as a factory for obtaining a Builder which
  * can be used to build trees using this tree model; and it provides static constants
  * that can be used to identify the built-in tree models.</p>
