@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,6 +8,7 @@
 package net.sf.saxon.pattern;
 
 import net.sf.saxon.expr.parser.Token;
+import net.sf.saxon.om.Item;
 import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.om.NodeName;
 import net.sf.saxon.om.StructuredQName;
@@ -15,7 +16,11 @@ import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.tiny.NodeVectorTree;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.type.*;
-import net.sf.saxon.z.*;
+import net.sf.saxon.z.IntExceptPredicate;
+import net.sf.saxon.z.IntSet;
+
+import java.util.Optional;
+import java.util.function.IntPredicate;
 
 /**
  * A CombinedNodeTest combines two node tests using one of the operators
@@ -107,9 +112,9 @@ public class CombinedNodeTest extends NodeTest {
     public IntPredicate getMatcher(NodeVectorTree tree) {
         switch (operator) {
             case Token.UNION:
-                return new IntUnionPredicate(nodetest1.getMatcher(tree), nodetest2.getMatcher(tree));
+                return nodetest1.getMatcher(tree).or(nodetest2.getMatcher(tree));
             case Token.INTERSECT:
-                return new IntIntersectionPredicate(nodetest1.getMatcher(tree), nodetest2.getMatcher(tree));
+                return nodetest1.getMatcher(tree).and(nodetest2.getMatcher(tree));
             case Token.EXCEPT:
                 return new IntExceptPredicate(nodetest1.getMatcher(tree), nodetest2.getMatcher(tree));
             default:
@@ -144,12 +149,19 @@ public class CombinedNodeTest extends NodeTest {
     }
 
     public String toString() {
+        return makeString(false);
+    }
+
+    private String makeString(boolean forExport) {
         if (nodetest1 instanceof NameTest && operator == Token.INTERSECT) {
             int kind = nodetest1.getPrimitiveType();
             String skind = kind == Type.ELEMENT ? "element(" : "attribute(";
             String content = "";
             if (nodetest2 instanceof ContentTypeTest) {
-                final SchemaType schemaType = ((ContentTypeTest) nodetest2).getSchemaType();
+                SchemaType schemaType = ((ContentTypeTest) nodetest2).getSchemaType();
+                if (forExport) {
+                    schemaType = schemaType.getNearestNamedType();
+                }
                 content = ", " + schemaType.getEQName();
                 if (nodetest2.isNillable()) {
                     content += "?";
@@ -170,59 +182,19 @@ public class CombinedNodeTest extends NodeTest {
     }
 
     /**
-     * Return a string representation of this SequenceType suitable for use in export (SEF) files.
-     * The difference from the toString() method is that the representation will not contain
-     * references to anonymous types.
+     * Return a string representation of this ItemType suitable for use in stylesheet
+     * export files. This differs from the result of toString() in that it will not contain
+     * any references to anonymous types. Note that it may also use the Saxon extended syntax
+     * for union types and tuple types. The default implementation returns the result of
+     * calling {@code toString()}.
      *
-     * @return the string representation as an instance of the XPath
-     * SequenceType construct, using only names that will be in-scope if the target environment
-     * imports the same schema as the source environment
+     * @return the string representation as an instance of the XPath SequenceType construct
      */
     @Override
     public String toExportString() {
-        if (nodetest1 instanceof NameTest && operator == Token.INTERSECT) {
-            int kind = nodetest1.getPrimitiveType();
-            String skind = kind == Type.ELEMENT ? "element(" : "attribute(";
-            String content = "";
-            if (nodetest2 instanceof ContentTypeTest) {
-                final SchemaType schemaType = ((ContentTypeTest) nodetest2).getNearestNamedType();
-                content = ", " + schemaType.getEQName();
-                if (nodetest2.isNillable()) {
-                    content += "?";
-                }
-            }
-            String name;
-            if (nodetest1 instanceof NameTest) {
-                name = nodetest1.getMatchingNodeName().getEQName();
-            } else {
-                name = nodetest1.toString();
-            }
-            return skind + name + content + ')';
-        } else {
-            String nt1 = nodetest1 == null ? "item()" : nodetest1.toString();
-            String nt2 = nodetest2 == null ? "item()" : nodetest2.toString();
-            return '(' + nt1 + ' ' + Token.tokens[operator] + ' ' + nt2 + ')';
-        }
+        return makeString(true);
     }
 
-    /**
-     * Get a mask indicating which kinds of nodes this NodeTest can match. This is a combination
-     * of bits: 1&lt;&lt;Type.ELEMENT for element nodes, 1&lt;&lt;Type.TEXT for text nodes, and so on.
-     */
-
-    public int getNodeKindMask() {
-        switch (operator) {
-            case Token.UNION:
-                return nodetest1.getNodeKindMask() | nodetest2.getNodeKindMask();
-            case Token.INTERSECT:
-                return nodetest1.getNodeKindMask() & nodetest2.getNodeKindMask();
-            case Token.EXCEPT:
-                return nodetest1.getNodeKindMask();
-            default:
-                return 0;
-        }
-
-    }
 
     /**
      * Get the basic kind of object that this ItemType matches: for a NodeTest, this is the kind of node,
@@ -247,26 +219,31 @@ public class CombinedNodeTest extends NodeTest {
 
     /**
      * Get the set of node names allowed by this NodeTest. This is returned as a set of Integer fingerprints.
-     * A null value indicates that all names are permitted (i.e. that there are no constraints on the node name).
-     * The default implementation returns null.
+     * IntUniversalSet indicates that all names are permitted (i.e. that there are no constraints on the node name).
      */
 
     /*@NotNull*/
-    public IntSet getRequiredNodeNames() {
-        IntSet s1 = nodetest1.getRequiredNodeNames();
-        IntSet s2 = nodetest2.getRequiredNodeNames();
-        switch (operator) {
-            case Token.UNION: {
-                return s1.union(s2);
+    public Optional<IntSet> getRequiredNodeNames() {
+        Optional<IntSet> os1 = nodetest1.getRequiredNodeNames();
+        Optional<IntSet> os2 = nodetest2.getRequiredNodeNames();
+        if (os1.isPresent() && os2.isPresent()) {
+            IntSet s1 = os1.get();
+            IntSet s2 = os2.get();
+            switch (operator) {
+                case Token.UNION: {
+                    return Optional.of(s1.union(s2));
+                }
+                case Token.INTERSECT: {
+                    return Optional.of(s1.intersect(s2));
+                }
+                case Token.EXCEPT: {
+                    return Optional.of(s1.except(s2));
+                }
+                default:
+                    throw new IllegalStateException();
             }
-            case Token.INTERSECT: {
-                return s1.intersect(s2);
-            }
-            case Token.EXCEPT: {
-                return s1.except(s2);
-            }
-            default:
-                throw new IllegalStateException();
+        } else {
+            return Optional.empty();
         }
     }
 
@@ -278,7 +255,9 @@ public class CombinedNodeTest extends NodeTest {
     public SchemaType getContentType() {
         SchemaType type1 = nodetest1.getContentType();
         SchemaType type2 = nodetest2.getContentType();
-        if (type1.isSameType(type2)) return type1;
+        if (type1.isSameType(type2)) {
+            return type1;
+        }
         if (operator == Token.INTERSECT) {
             if (type2 instanceof AnyType || (type2 instanceof AnySimpleType && type1.isSimpleType())) {
                 return type1;
@@ -299,7 +278,9 @@ public class CombinedNodeTest extends NodeTest {
     public AtomicType getAtomizedItemType() {
         AtomicType type1 = nodetest1.getAtomizedItemType();
         AtomicType type2 = nodetest2.getAtomizedItemType();
-        if (type1.isSameType(type2)) return type1;
+        if (type1.isSameType(type2)) {
+            return type1;
+        }
         if (operator == Token.INTERSECT) {
             if (type2.equals(BuiltInAtomicType.ANY_ATOMIC)) {
                 return type1;
@@ -316,10 +297,20 @@ public class CombinedNodeTest extends NodeTest {
      *
      * @return true unless it is known that these items will be elements with element-only
      * content, in which case return false
+     * @param th the type hierarchy cache
      */
 
-    public boolean isAtomizable() {
-        return nodetest1.isAtomizable() && nodetest2.isAtomizable();
+    public boolean isAtomizable(TypeHierarchy th) {
+        switch (operator) {
+            case Token.UNION:
+                return nodetest1.isAtomizable(th) || nodetest2.isAtomizable(th);
+            case Token.INTERSECT:
+                return nodetest1.isAtomizable(th) && nodetest2.isAtomizable(th);
+            case Token.EXCEPT:
+                return nodetest1.isAtomizable(th);
+            default:
+                return true;
+        }
     }
 
     /**
@@ -330,9 +321,15 @@ public class CombinedNodeTest extends NodeTest {
     public int getFingerprint() {
         int fp1 = nodetest1.getFingerprint();
         int fp2 = nodetest2.getFingerprint();
-        if (fp1 == fp2) return fp1;
-        if (fp2 == -1 && operator == Token.INTERSECT) return fp1;
-        if (fp1 == -1 && operator == Token.INTERSECT) return fp2;
+        if (fp1 == fp2) {
+            return fp1;
+        }
+        if (fp2 == -1 && operator == Token.INTERSECT) {
+            return fp1;
+        }
+        if (fp1 == -1 && operator == Token.INTERSECT) {
+            return fp2;
+        }
         return -1;
     }
 
@@ -418,17 +415,6 @@ public class CombinedNodeTest extends NodeTest {
         return operator;
     }
 
-    /**
-     * Visit all the schema components used in this ItemType definition
-     *
-     * @param visitor the visitor class to be called when each component is visited
-     */
-
-    public void visitNamedSchemaComponents(SchemaComponentVisitor visitor) throws XPathException {
-        nodetest1.visitNamedSchemaComponents(visitor);
-        nodetest2.visitNamedSchemaComponents(visitor);
-    }
-
 
     /**
      * Generate Javascript code to test whether an item conforms to this item type
@@ -438,8 +424,10 @@ public class CombinedNodeTest extends NodeTest {
      * variable "item" is an instance of this item type.
      * @throws XPathException if JS code cannot be generated for this item type, for example because
      *                        the test is schema-aware.
-     * @param knownToBe
-     * @param targetVersion
+     * @param knownToBe An item type that the supplied item is known to conform to; the generated code
+     *                  can assume that the item is an instance of this type.
+     * @param targetVersion The version of Saxon-JS for which code is being generated. Currently either 1 or 2.
+
      */
     @Override
     public String generateJavaScriptItemTypeTest(ItemType knownToBe, int targetVersion) throws XPathException {
@@ -462,5 +450,32 @@ public class CombinedNodeTest extends NodeTest {
         return fsb.toString();
     }
 
+    /**
+     * Get extra diagnostic information about why a supplied item does not conform to this
+     * item type, if available. If extra information is returned, it should be in the form of a complete
+     * sentence, minus the closing full stop. No information should be returned for obvious cases.
+     *
+     * @param item the item that doesn't match this type
+     * @param th   the type hierarchy cache
+     * @return optionally, a message explaining why the item does not match the type
+     */
+    @Override
+    public Optional<String> explainMismatch(Item item, TypeHierarchy th) {
+        Optional<String> explanation = super.explainMismatch(item, th);
+        if (explanation.isPresent()) {
+            return explanation;
+        }
+        if (operator == Token.INTERSECT) {
+            // the most common case
+            if (!nodetest1.matchesNode((NodeInfo)item)) {
+                return nodetest1.explainMismatch(item, th);
+            } else if (!nodetest2.matchesNode((NodeInfo)item)) {
+                return nodetest2.explainMismatch(item, th);
+            }
+        }
+        return Optional.empty();
+    }
+
+
 }
 
