@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,16 +9,14 @@ package net.sf.saxon.pattern;
 
 import net.sf.saxon.om.*;
 import net.sf.saxon.trace.ExpressionPresenter;
-import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.tiny.NodeVectorTree;
-import net.sf.saxon.type.ItemType;
-import net.sf.saxon.type.SchemaType;
-import net.sf.saxon.type.Type;
-import net.sf.saxon.type.UType;
-import net.sf.saxon.z.IntPredicate;
+import net.sf.saxon.type.*;
 import net.sf.saxon.z.IntSet;
 import net.sf.saxon.z.IntSingletonSet;
 
+import java.util.Optional;
+import java.util.function.IntPredicate;
+
 /**
  * NodeTest is an interface that enables a test of whether a node has a particular
  * name and type. A NameTest matches the node kind and the namespace URI and the local
@@ -152,12 +150,8 @@ public class NameTest extends NodeTest implements QNameTest {
     public IntPredicate getMatcher(final NodeVectorTree tree) {
         final byte[] nodeKindArray = tree.getNodeKindArray();
         final int[] nameCodeArray = tree.getNameCodeArray();
-        return new IntPredicate() {
-            @Override
-            public boolean matches(int nodeNr) {
-                return (nameCodeArray[nodeNr] & 0xfffff) == fingerprint && nodeKindArray[nodeNr] == nodeKind;
-            }
-        };
+        return nodeNr -> (nameCodeArray[nodeNr] & 0xfffff) == fingerprint &&
+                (nodeKindArray[nodeNr] & 0x0f) == nodeKind;
     }
 
     /**
@@ -244,15 +238,6 @@ public class NameTest extends NodeTest implements QNameTest {
         return nodeKind;
     }
 
-    /**
-     * Get a mask indicating which kinds of nodes this NodeTest can match. This is a combination
-     * of bits: 1<<Type.ELEMENT for element nodes, 1<<Type.TEXT for text nodes, and so on.
-     */
-
-    public int getNodeKindMask() {
-        return 1 << nodeKind;
-    }
-
     /**
      * Get the set of node names allowed by this NodeTest. This is returned as a set of Integer fingerprints.
      * A null value indicates that all names are permitted (i.e. that there are no constraints on the node name.
@@ -260,8 +245,8 @@ public class NameTest extends NodeTest implements QNameTest {
      */
 
     /*@NotNull*/
-    public IntSet getRequiredNodeNames() {
-        return new IntSingletonSet(fingerprint);
+    public Optional<IntSet> getRequiredNodeNames() {
+        return Optional.of(new IntSingletonSet(fingerprint));
     }
 
     /**
@@ -326,7 +311,7 @@ public class NameTest extends NodeTest implements QNameTest {
      * as the body of a JS function in which the argument name "q" is an
      * XdmQName object holding the name. The XdmQName object has properties
      * uri and local.
-     * @param targetVersion
+     * @param targetVersion the version of Saxon-JS being targeted
      */
     public String generateJavaScriptNameTest(int targetVersion) {
         computeUriAndLocal();
@@ -336,16 +321,14 @@ public class NameTest extends NodeTest implements QNameTest {
     /**
      * Generate Javascript code to test whether an item conforms to this item type
      * @param knownToBe NodeTest that the item is known to conform to (no run-time test needed)
-     * @param targetVersion
+     * @param targetVersion the version of Saxon-JS being targeted
      * @return a Javascript instruction or sequence of instructions, which can be used as the body
      * of a Javascript function, and which returns a boolean indication whether the value of the
      * variable "item" is an instance of this item type.
-     * @throws XPathException if JS code cannot be generated for this item type, for example because
-     *                        the test is schema-aware.
      *
      */
     @Override
-    public String generateJavaScriptItemTypeTest(ItemType knownToBe, int targetVersion) throws XPathException {
+    public String generateJavaScriptItemTypeTest(ItemType knownToBe, int targetVersion) {
 
         boolean knownKind = knownToBe.getUType() == this.getUType();
         if (nodeKind == Type.ATTRIBUTE && getNamespaceURI().equals("")) {
@@ -374,5 +357,24 @@ public class NameTest extends NodeTest implements QNameTest {
                     ExpressionPresenter.jsEscape(uri) + "')";
         }
     }
+
+    /**
+     * Get extra diagnostic information about why a supplied item does not conform to this
+     * item type, if available. If extra information is returned, it should be in the form of a complete
+     * sentence, minus the closing full stop. No information should be returned for obvious cases.
+     *
+     * @param item the item that doesn't match this type
+     * @param th   the type hierarchy cache
+     * @return optionally, a message explaining why the item does not match the type
+     */
+    @Override
+    public Optional<String> explainMismatch(Item item, TypeHierarchy th) {
+        Optional<String> explanation = super.explainMismatch(item, th);
+        if (explanation.isPresent()) {
+            return explanation;
+        }
+        return Optional.of("The node has the wrong name");
+    }
+
 }
 
