@@ -298,7 +298,7 @@ public class XQueryExpression implements Location, ExpressionOwner {
      */
 
     /*@NotNull*/
-    public SequenceIterator<? extends Item> iterator(/*@NotNull*/ DynamicQueryContext env) throws XPathException {
+    public SequenceIterator<?> iterator(/*@NotNull*/ DynamicQueryContext env) throws XPathException {
         if (isUpdateQuery()) {
             throw new XPathException("Cannot call iterator() on an updating query");
         }
@@ -322,7 +322,7 @@ public class XQueryExpression implements Location, ExpressionOwner {
 
             context.openStackFrame(stackFrameMap);
 
-            SequenceIterator<? extends Item> iterator = expression.iterate(context);
+            SequenceIterator<?> iterator = expression.iterate(context);
             if ((iterator.getProperties() & SequenceIterator.GROUNDED) != 0) {
                 return iterator;
             } else {
@@ -567,11 +567,11 @@ public class XQueryExpression implements Location, ExpressionOwner {
 
     protected XPathContextMajor initialContext(DynamicQueryContext dynamicEnv, Controller controller) throws XPathException {
 
-        Item contextItem = controller.getGlobalContextItem();
+        Item<?> contextItem = controller.getGlobalContextItem();
         XPathContextMajor context = controller.newXPathContext();
 
         if (contextItem != null) {
-            ManualIterator<Item> single = new ManualIterator<Item>(contextItem);
+            ManualIterator<Item<?>> single = new ManualIterator<>(contextItem);
             context.setCurrentIterator(single);
             controller.setGlobalContextItem(contextItem);
         }
@@ -742,7 +742,7 @@ public class XQueryExpression implements Location, ExpressionOwner {
      * any exceptions that are raised to the ErrorListener
      */
 
-    private class ErrorReportingIterator<T extends Item> implements SequenceIterator<T> {
+    private class ErrorReportingIterator<T extends Item<?>> implements SequenceIterator<T> {
         private SequenceIterator<T> base;
         private UnfailingErrorListener listener;
 
