@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -33,7 +33,8 @@ package net.sf.saxon.regex;
 
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.z.IntIterator;
-import net.sf.saxon.z.IntPredicate;
+
+import java.util.function.IntPredicate;
 
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -47,32 +48,22 @@ import java.util.List;
  * match a string against a pattern, you can also extract parts of the
  * match.  This is especially useful in text parsing! Details on the
  * syntax of regular expression patterns are given below.
- * <p/>
- * <p/>
- * To compile a regular expression (RE), you can simply construct an RE
- * matcher object from the string specification of the pattern, like this:
- * <p/>
+ * <p>To compile a regular expression (RE), you can simply construct an RE
+ * matcher object from the string specification of the pattern, like this:</p>
  * <pre>
  *  RE r = new RE("a*b");
  * </pre>
- * <p/>
- * <p/>
- * Once you have done this, you can call either of the RE.match methods to
- * perform matching on a String.  For example:
- * <p/>
+ * <p>Once you have done this, you can call either of the RE.match methods to
+ * perform matching on a String.  For example:</p>
  * <pre>
  *  boolean matched = r.match("aaaab");
  * </pre>
- * <p/>
- * will cause the boolean matched to be set to true because the
- * pattern "a*b" matches the string "aaaab".
- * <p/>
- * <p/>
- * If you were interested in the <i>number</i> of a's which matched the
+ * <p>will cause the boolean matched to be set to true because the
+ * pattern "a*b" matches the string "aaaab".</p>
+ * <p>If you were interested in the <i>number</i> of a's which matched the
  * first part of our example expression, you could change the expression to
  * "(a*)b".  Then when you compiled the expression and matched it against
- * something like "xaaaab", you would get results like this:
- * <p/>
+ * something like "xaaaab", you would get results like this:</p>
  * <pre>
  *  RE r = new RE("(a*)b");                  // Compile expression
  *  boolean matched = r.match("xaaaab");     // Match against "xaaaab"
@@ -88,28 +79,20 @@ import java.util.List;
  *  int endInside = r.getParenEnd(1);        // endInside will be index 5
  *  int lenInside = r.getParenLength(1);     // lenInside will be 4
  * </pre>
- * <p/>
- * You can also refer to the contents of a parenthesized expression
+ * <p>You can also refer to the contents of a parenthesized expression
  * within a regular expression itself.  This is called a
  * 'backreference'.  The first backreference in a regular expression is
- * denoted by \1, the second by \2 and so on.  So the expression:
- * <p/>
+ * denoted by \1, the second by \2 and so on.  So the expression:</p>
  * <pre>
  *  ([0-9]+)=\1
  * </pre>
- * <p/>
- * will match any string of the form n=n (like 0=0 or 2=2).
- * <p/>
- * <p/>
- * The full regular expression syntax accepted by RE is as defined in the XSD 1.1
- * specification, modified by the XPath 2.0 or 3.0 specifications.
- * <p/>
- * <p/>
- * <b><font face="times roman">Line terminators</font></b>
- * <br>
- * A line terminator is a one- or two-character sequence that marks
+ * <p>will match any string of the form n=n (like 0=0 or 2=2).</p>
+ * <p>The full regular expression syntax accepted by RE is as defined in the XSD 1.1
+ * specification, modified by the XPath 2.0 or 3.0 specifications.</p>
+ * <p><b>Line terminators</b></p>
+ * <p>A line terminator is a one- or two-character sequence that marks
  * the end of a line of the input character sequence. The following
- * are recognized as line terminators:
+ * are recognized as line terminators:</p>
  * <ul>
  * <li>A newline (line feed) character ('\n'),</li>
  * <li>A carriage-return character followed immediately by a newline character ("\r\n"),</li>
@@ -118,14 +101,11 @@ import java.util.List;
  * <li>A line-separator character ('\u2028'), or</li>
  * <li>A paragraph-separator character ('\u2029).</li>
  * </ul>
- * <p/>
- * <p/>
- * RE runs programs compiled by the RECompiler class.  But the RE
+ * <p>RE runs programs compiled by the RECompiler class.  But the RE
  * matcher class does not include the actual regular expression compiler
  * for reasons of efficiency.  In fact, if you want to pre-compile one
  * or more regular expressions, the 'recompile' class can be invoked
- * from the command line to produce compiled output like this:
- * <p/>
+ * from the command line to produce compiled output like this:</p>
  * <pre>
  *    // Pre-compiled regular expression "a*b"
  *    char[] re1Instructions =
@@ -140,8 +120,7 @@ import java.util.List;
  *
  *    REProgram re1 = new REProgram(re1Instructions);
  * </pre>
- * <p/>
- * You can then construct a regular expression matcher (RE) object from
+ * <p>You can then construct a regular expression matcher (RE) object from
  * the pre-compiled expression re1 and thus avoid the overhead of
  * compiling the expression at runtime. If you require more dynamic
  * regular expressions, you can construct a single RECompiler object and
@@ -152,13 +131,9 @@ import java.util.List;
  * requirement), so you will need to construct a separate compiler or
  * matcher object for each thread (unless you do thread synchronization
  * yourself). Once expression compiled into the REProgram object, REProgram
- * can be safely shared across multiple threads and RE objects.
- * <p/>
- * <br><p><br>
- * <p/>
- * <font color="red">
- * <i>ISSUES:</i>
- * <p/>
+ * can be safely shared across multiple threads and RE objects.</p>
+ * <p><i>ISSUES:</i></p>
+ * <ul>
  * <li>Not *all* possibilities are considered for greediness when backreferences
  * are involved (as POSIX suggests should be the case).  The POSIX RE
  * "(ac*)c*d[ac]*\1", when matched against "acdacaa" should yield a match
@@ -167,12 +142,10 @@ import java.util.List;
  * actually complains about this, I'm not sure it's worth "fixing".
  * If it ever is fixed, test #137 in RETest.txt should be updated.</li>
  * </ul>
- * <p/>
  * <p>This library is based on the Apache Jakarta regex library as downloaded
  * on 3 January 2012. Changes have been made to make the grammar and semantics conform to XSD
  * and XPath rules; these changes are listed in source code comments in the
  * RECompiler source code module.</p>
- * </font>
  *
  * @author <a href="mailto:jonl@muppetlabs.com">Jonathan Locke</a>
  * @author <a href="mailto:ts@sch-fer.de">Tobias Sch&auml;fer</a>
@@ -441,7 +414,7 @@ public class REMatcher {
                 // no prefix known; but the first character must match a predicate
                 IntPredicate pred = program.initialCharClass;
                 for (; !search.isEnd(i); i++) {
-                    if (pred.matches(search.uCharAt(i))) {
+                    if (pred.test(search.uCharAt(i))) {
                         if (matchAt(i, false)) {
                             return true;
                         }
@@ -464,22 +437,28 @@ public class REMatcher {
         } else {
             // Prefix-anchored matching is possible
             UnicodeString prefix = program.prefix;
-            for (; !search.isEnd(i + prefix.uLength() - 1); i++) {
-                int j = i;
-                int k = 0;
-
-                if (program.flags.isCaseIndependent()) {
-                    do {
-                        // If there's a mismatch of any character in the prefix, give up
-                    } while (equalCaseBlind(search.uCharAt(j++), prefix.uCharAt(k++)) && k < prefix.uLength());
+            int prefixLength = prefix.uLength();
+            boolean ignoreCase = program.flags.isCaseIndependent();
+            for (; !search.isEnd(i + prefixLength - 1); i++) {
+                boolean prefixOK = true;
+                if (ignoreCase) {
+                    for (int j = i, k = 0; k < prefixLength; j++, k++) {
+                        if (!equalCaseBlind(search.uCharAt(j), prefix.uCharAt(k))) {
+                            prefixOK = false;
+                            break;
+                        }
+                    }
                 } else {
-                    do {
-                        // If there's a mismatch of any character in the prefix, give up
-                    } while ((search.uCharAt(j++) == prefix.uCharAt(k++)) && k < prefix.uLength());
+                    for (int j = i, k=0; k < prefixLength; j++, k++) {
+                        if (search.uCharAt(j) != prefix.uCharAt(k)) {
+                            prefixOK = false;
+                            break;
+                        }
+                    }
                 }
 
                 // See if the whole prefix string matched
-                if (k == prefix.uLength()) {
+                if (prefixOK) {
                     // We matched the full prefix at firstChar, so try it
                     if (matchAt(i, false)) {
                         return true;
@@ -542,10 +521,9 @@ public class REMatcher {
      * Given a regular expression of "[ab]+" and a string to split of
      * "xyzzyababbayyzabbbab123", the result would be the array of Strings
      * "[xyzzy, yyz, 123]".
-     * <p/>
      * <p>Please note that the first string in the resulting array may be an empty
      * string. This happens when the very first character of input string is
-     * matched by the pattern.
+     * matched by the pattern.</p>
      *
      * @param s String to split on this regular exression
      * @return Array of strings
@@ -592,15 +570,13 @@ public class REMatcher {
      * Given a regular expression of "a*b", a String to substituteIn of
      * "aaaabfooaaabgarplyaaabwackyb" and the substitution String "-", the
      * resulting String returned by subst would be "-foo-garply-wacky-".
-     * <p/>
-     * It is also possible to reference the contents of a parenthesized expression
-     * with $0, $1, ... $9. A regular expression of "http://[\\.\\w\\-\\?/~_@&=%]+",
+     * <p>It is also possible to reference the contents of a parenthesized expression
+     * with $0, $1, ... $9. A regular expression of "http://[\\.\\w\\-\\?/~_@&amp;=%]+",
      * a String to substituteIn of "visit us: http://www.apache.org!" and the
      * substitution String "&lt;a href=\"$0\"&gt;$0&lt;/a&gt;", the resulting String
      * returned by subst would be
-     * "visit us: &lt;a href=\"http://www.apache.org\"&gt;http://www.apache.org&lt;/a&gt;!".
-     * <p/>
-     * <i>Note:</i> $0 represents the whole match.
+     * "visit us: &lt;a href=\"http://www.apache.org\"&gt;http://www.apache.org&lt;/a&gt;!".</p>
+     * <p><i>Note:</i> $0 represents the whole match.</p>
      *
      * @param in          String to substitute within
      * @param replacement String to substitute for matches of this regular expression
