@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -58,7 +58,7 @@ public abstract class AbstractResourceCollection implements ResourceCollection {
         }
         Boolean stable = params.getStable();
         if (stable == null) {
-            return context.getConfiguration().getBooleanProperty(FeatureKeys.STABLE_COLLECTION_URI);
+            return context.getConfiguration().getBooleanProperty(Feature.STABLE_COLLECTION_URI);
         } else {
             return stable;
         }
@@ -165,7 +165,6 @@ public abstract class AbstractResourceCollection implements ResourceCollection {
             if ("file".equals(uri.getScheme())) {
                 File file = new File(uri);
                 inputDetails.inputStream = new BufferedInputStream(new FileInputStream(file));
-
             } else {
                 //TODO: check for redirects
                 URL url = uri.toURL();
@@ -182,11 +181,7 @@ public abstract class AbstractResourceCollection implements ResourceCollection {
                 }
 
             }
-        } catch (URISyntaxException e) {
-            throw new XPathException(e);
-        } catch (MalformedURLException e) {
-            throw new XPathException(e);
-        } catch (IOException e) {
+        } catch (URISyntaxException | IOException e) {
             throw new XPathException(e);
         }
         if (inputDetails.contentType == null || config.getResourceFactoryForMediaType(inputDetails.contentType) == null) {
@@ -287,6 +282,29 @@ public abstract class AbstractResourceCollection implements ResourceCollection {
         return factory.makeResource(config, resourceURI, contentType, details);
     }
 
+    public Resource makeTypedResource(Configuration config, Resource basicResource) throws XPathException {
+
+        String mediaType = basicResource.getContentType();
+        ResourceFactory factory = config.getResourceFactoryForMediaType(mediaType);
+        if (factory == null) {
+            return basicResource;
+        }
+        if (basicResource instanceof BinaryResource) {
+            InputDetails details = new InputDetails();
+            details.inputStream = new ByteArrayInputStream(((BinaryResource)basicResource).getData());
+            details.contentType = mediaType;
+            return factory.makeResource(config, basicResource.getResourceURI(), mediaType, details);
+        } else if (basicResource instanceof UnparsedTextResource) {
+            InputDetails details = new InputDetails();
+            details.inputStream = ((UnparsedTextResource) basicResource).getInputStream();
+            details.contentType = mediaType;
+            return factory.makeResource(config, basicResource.getResourceURI(), mediaType, details);
+        } else {
+            return basicResource;
+        }
+
+    }
+
     /**
      * Default method to make a resource, given a resource URI
      * @param resourceURI the resource URI
