@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,7 +11,7 @@ import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.functions.URIQueryParameters;
 import net.sf.saxon.lib.ParseOptions;
 import net.sf.saxon.lib.Resource;
-import net.sf.saxon.om.Sequence;
+import net.sf.saxon.om.GroundedValue;
 import net.sf.saxon.om.SpaceStrippingRule;
 import net.sf.saxon.trans.UncheckedXPathException;
 import net.sf.saxon.trans.XPathException;
@@ -202,7 +202,7 @@ public class JarCollection extends AbstractResourceCollection {
     }
 
 
-    private class JarIterator implements Iterator<Resource> {
+    private class JarIterator implements Iterator<Resource>, Closeable {
         private FilenameFilter filter;
         private Resource next = null;
         private XPathContext context;
@@ -212,7 +212,7 @@ public class JarCollection extends AbstractResourceCollection {
         private boolean metadata;
 
 
-        public JarIterator(XPathContext context, ZipInputStream zipInputStream, FilenameFilter filter) throws XPathException {
+        public JarIterator(XPathContext context, ZipInputStream zipInputStream, FilenameFilter filter) {
             this.context = context;
             this.filter = filter;
             this.zipInputStream = zipInputStream;
@@ -221,7 +221,6 @@ public class JarCollection extends AbstractResourceCollection {
             Boolean metadataParam = params == null ? null : params.getMetaData();
             metadata = metadataParam != null && metadataParam;
 
-            //this.resourceURIs = resourceURIs;
             advance();
         }
 
@@ -263,7 +262,6 @@ public class JarCollection extends AbstractResourceCollection {
                 }
                 if (entry.isDirectory()) {
                     dirStr = entry.getName();
-                    continue;
                 } else {
                     String entryName = entry.getName();
                     if (filter != null) {
@@ -305,7 +303,7 @@ public class JarCollection extends AbstractResourceCollection {
                         String resourceURI = makeResourceURI(entry.getName());
                         next = makeResource(context.getConfiguration(), details, resourceURI);
                         if (metadata) {
-                            Map<String, Sequence> properties = makeProperties(entry);
+                            Map<String, GroundedValue<?>> properties = makeProperties(entry);
                             next = new MetadataResource(resourceURI, next, properties);
                         }
                         return;
@@ -317,6 +315,11 @@ public class JarCollection extends AbstractResourceCollection {
             }
 
         }
+
+        @Override
+        public void close() throws IOException {
+            zipInputStream.close();
+        }
     }
 
     /**
@@ -326,8 +329,8 @@ public class JarCollection extends AbstractResourceCollection {
      * entry-name, size, and last-modified.
      */
 
-    protected Map<String, Sequence> makeProperties(ZipEntry entry) {
-        HashMap<String, Sequence> map = new HashMap<String, Sequence>(10);
+    protected Map<String, GroundedValue<?>> makeProperties(ZipEntry entry) {
+        HashMap<String, GroundedValue<?>> map = new HashMap<>(10);
         map.put("comment", StringValue.makeStringValue(entry.getComment()));
         map.put("compressed-size", new Int64Value(entry.getCompressedSize()));
         map.put("crc", new Int64Value(entry.getCrc()));
