@@ -45,6 +45,9 @@ abstract class AbstractXsltTransformer {
     protected XsltController controller;
     protected boolean baseOutputUriWasSet = false;
     private ResultDocumentResolver resultDocumentHandler = null;
+    private MessageListener messageListener;
+    private MessageListener2 messageListener2;
+
 
     AbstractXsltTransformer(Processor processor, XsltController controller) {
         this.processor = processor;
@@ -143,10 +146,11 @@ abstract class AbstractXsltTransformer {
      * to {@code xsl:result-document}, resolved against the base output URI of the transformation),
      * and returns a {@link Destination}, which will be used as the destination for the result document.
      * <p>If the {@code href} argument of the {@code xsl:result-document} instruction is absent or if
-     * it is set to a zero length string, then the callback function is called supplying the base output
-     * URI as its argument, and the resulting {@code Destination} is used in the normal way. (Note
-     * that this is allowed only if no output is written directly to the principal output destination;
-     * so the {@code Destination} supplied as the primary output destination will effectively be ignored.)</p>
+     * it is set to a zero length string, then the callback function is not normally called; instead
+     * a {@code Receiver} for the secondary output is obtained by making a second call on {@link Destination#getReceiver(PipelineConfiguration, SerializationProperties)}
+     * for the principal destination of the transformation. In that situation, this result document handler
+     * is invoked only if the call on {@link Destination#getReceiver(PipelineConfiguration, SerializationProperties)}
+     * returns null. </p>
      * <p>If the base output URI is absent (perhaps because the principal output destination for the
      * transformation was supplied as a {@link OutputStream} or {@link Writer} with no associated
      * URI or systemId), then the value of the {@code href} attribute is used <i>as is</i> if it
@@ -201,7 +205,8 @@ abstract class AbstractXsltTransformer {
      */
 
     public void setMessageListener(MessageListener listener) {
-        controller.setMessageEmitter(new MessageListenerProxy(listener, controller.makePipelineConfiguration()));
+        this.messageListener = listener;
+        controller.setMessageFactory(() -> new MessageListenerProxy(listener, controller.makePipelineConfiguration()));
     }
 
     /**
@@ -217,7 +222,8 @@ abstract class AbstractXsltTransformer {
      */
 
     public void setMessageListener(MessageListener2 listener) {
-        controller.setMessageEmitter(new MessageListener2Proxy(listener, controller.makePipelineConfiguration()));
+        this.messageListener2 = listener;
+        controller.setMessageFactory(() -> new MessageListener2Proxy(listener, controller.makePipelineConfiguration()));
     }
 
     /**
@@ -229,12 +235,19 @@ abstract class AbstractXsltTransformer {
      */
 
     public MessageListener getMessageListener() {
-        Receiver r = controller.getMessageEmitter();
-        if (r instanceof MessageListenerProxy) {
-            return ((MessageListenerProxy) r).getMessageListener();
-        } else {
-            return null;
-        }
+        return messageListener;
+    }
+
+    /**
+     * Get the MessageListener2 to be notified whenever the stylesheet evaluates an
+     * <code>xsl:message</code> instruction. If no MessageListener2 has been nominated,
+     * return null
+     *
+     * @return the user-supplied MessageListener2, or null if none has been supplied
+     */
+
+    public MessageListener2 getMessageListener2() {
+        return messageListener2;
     }
 
     /**
@@ -440,9 +453,13 @@ abstract class AbstractXsltTransformer {
 
     public Receiver getDestinationReceiver(XsltController controller, Destination destination) throws SaxonApiException {
         Receiver receiver;
+        controller.setPrincipalDestination(destination);
         PipelineConfiguration pipe = controller.makePipelineConfiguration();
         SerializationProperties params = controller.getExecutable().getPrimarySerializationProperties();
         receiver = destination.getReceiver(pipe, params);
+        if (Configuration.isAssertionsEnabled()) {
+            receiver = new RegularSequenceChecker(receiver);
+        }
         receiver.getPipelineConfiguration().setController(controller);
 
         if (baseOutputUriWasSet) {
@@ -461,27 +478,6 @@ abstract class AbstractXsltTransformer {
 
     }
 
-//    protected SequenceReceiver makeSequenceReceiver(Destination destination, Receiver out) {
-//        SequenceReceiver sOut;
-//        Properties stylesheetProps = controller.getExecutable().getDefaultOutputProperties();
-//        if (destination.isRequiresTree(stylesheetProps)) {
-//            sOut = getUnderlyingController().getConfiguration().getSerializerFactory().
-//                    makeSequenceNormalizer(out, destination.getCombinedOutputProperties(stylesheetProps));
-//            if (destination instanceof Serializer && controller.getExecutable().createsSecondaryResult()) {
-//                // This is for the case where the stylesheet writes no output to the primary destination,
-//                // and then calls xsl:result-document will a null or empty href, in which case the xsl:result-document
-//                // output is sent to the primary output destination, but with different serialization properties.
-//                sOut = new ReconfigurableSerializer(sOut, ((Serializer)destination).getLocalOutputProperties(),
-//                                                    ((Serializer)destination).getResult());
-//            }
-//        } else if (out instanceof SequenceReceiver) {
-//            sOut = (SequenceReceiver)out;
-//        } else {
-//            sOut = new TreeReceiver(out);
-//        }
-//        return sOut;
-//    }
-
     /**
      * Return a Receiver which can be used to supply the principal source document for the transformation.
      * This method is intended primarily for internal use, though it can also
