@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,9 +7,10 @@
 
 package net.sf.saxon.s9api;
 
-import net.sf.saxon.Configuration;
 import net.sf.saxon.dom.DOMWriter;
+import net.sf.saxon.event.PipelineConfiguration;
 import net.sf.saxon.event.Receiver;
+import net.sf.saxon.serialize.SerializationProperties;
 
 /**
  * This class represents a Destination (for example, the destination of the output of a transformation)
@@ -17,7 +18,7 @@ import net.sf.saxon.event.Receiver;
  * a Document node, which will be used as the root of the constructed tree
  */
 
-public class DOMDestination implements Destination {
+public class DOMDestination extends AbstractDestination {
 
     private DOMWriter domWriter;
 
@@ -37,15 +38,15 @@ public class DOMDestination implements Destination {
      * Return a Receiver. Saxon calls this method to obtain a Receiver, to which it then sends
      * a sequence of events representing the content of an XML document.
      *
-     * @param config The Saxon configuration. This is supplied so that the destination can
+     * @param pipe The Saxon configuration. This is supplied so that the destination can
      *               use information from the configuration (for example, a reference to the name pool)
      *               to construct or configure the returned Receiver.
+     * @param params
      * @return the Receiver to which events are to be sent.
-     * @throws SaxonApiException if the Receiver cannot be created
      */
 
-    public Receiver getReceiver(/*@NotNull*/ Configuration config) throws SaxonApiException {
-        domWriter.setPipelineConfiguration(config.makePipelineConfiguration());
+    public Receiver getReceiver(/*@NotNull*/ PipelineConfiguration pipe, SerializationProperties params) {
+        domWriter.setPipelineConfiguration(pipe);
         return domWriter;
     }
 
