@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -12,7 +12,9 @@ import net.sf.saxon.expr.parser.Token;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.ma.arrays.ArrayItemType;
 import net.sf.saxon.ma.map.MapType;
-import net.sf.saxon.om.*;
+import net.sf.saxon.om.NodeInfo;
+import net.sf.saxon.om.StandardNames;
+import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.pattern.*;
 import net.sf.saxon.type.*;
 import net.sf.saxon.value.AtomicValue;
@@ -22,7 +24,6 @@ import java.util.Map;
 
 /**
  * This class is used for creating ItemType objects.
- * <p/>
  * <p>The <code>ItemTypeFactory</code> class is thread-safe.</p>
  */
 public class ItemTypeFactory {
@@ -45,7 +46,6 @@ public class ItemTypeFactory {
      * Get an item type representing an atomic type. This may be a built-in type in the
      * XML Schema namespace, or a user-defined atomic type. The resulting type will use
      * the conversion rules configured for the supplied <code>Processor</code>.
-     * <p/>
      * <p>It is undefined whether two calls supplying the same QName will return the same ItemType
      * object.</p>
      *
@@ -229,11 +229,9 @@ public class ItemTypeFactory {
 
     /**
      * Get an item type that matches any node of a specified kind.
-     * <p/>
      * <p>This corresponds to the XPath syntactic forms element(), attribute(),
      * document-node(), text(), comment(), processing-instruction(). It also provides
      * an option, not available in the XPath syntax, that matches namespace nodes.</p>
-     * <p/>
      * <p>It is undefined whether two calls supplying the same argument value will
      * return the same ItemType object.</p>
      *
@@ -264,11 +262,9 @@ public class ItemTypeFactory {
 
     /**
      * Get an item type that matches nodes of a specified kind with a specified name.
-     * <p/>
      * <p>This corresponds to the XPath syntactic forms element(name), attribute(name),
      * and processing-instruction(name). In the case of processing-instruction, the supplied
      * QName must have no namespace.</p>
-     * <p/>
      * <p>It is undefined whether two calls supplying the same argument values will
      * return the same ItemType object.</p>
      *
@@ -296,7 +292,6 @@ public class ItemTypeFactory {
     /**
      * Make an ItemType representing an element declaration in the schema. This is the
      * equivalent of the XPath syntax <code>schema-element(element-name)</code>
-     * <p/>
      * <p>It is undefined whether two calls supplying the same argument values will
      * return the same ItemType object.</p>
      *
@@ -324,7 +319,6 @@ public class ItemTypeFactory {
     /**
      * Make an ItemType that tests an element name and/or schema type. This is the
      * equivalent of the XPath syntax <code>element(element-name, type)</code>
-     * <p/>
      * <p>It is undefined whether two calls supplying the same argument values will
      * return the same ItemType object.</p>
      *
@@ -377,7 +371,6 @@ public class ItemTypeFactory {
     /**
      * Get an ItemType representing an attribute declaration in the schema. This is the
      * equivalent of the XPath syntax <code>schema-attribute(attribute-name)</code>
-     * <p/>
      * <p>It is undefined whether two calls supplying the same argument values will
      * return the same ItemType object.</p>
      *
@@ -406,7 +399,6 @@ public class ItemTypeFactory {
     /**
      * Get an ItemType that tests an element name and/or schema type. This is the
      * equivalent of the XPath syntax <code>element(element-name, type)</code>
-     * <p/>
      * <p>It is undefined whether two calls supplying the same argument values will
      * return the same ItemType object.</p>
      *
@@ -458,7 +450,6 @@ public class ItemTypeFactory {
      * Make an ItemType representing a document-node() test with an embedded element test.
      * This reflects the XPath syntax <code>document-node(element(N, T))</code> or
      * <code>document-node(schema-element(N))</code>.
-     * <p/>
      * <p>It is undefined whether two calls supplying the same argument values will
      * return the same ItemType object.</p>
      *
@@ -485,7 +476,7 @@ public class ItemTypeFactory {
      */
 
     public ItemType getExternalObjectType(Class externalClass) {
-        JavaExternalObjectType type = new JavaExternalObjectType(externalClass, processor.getUnderlyingConfiguration());
+        JavaExternalObjectType type = processor.getUnderlyingConfiguration().getJavaExternalObjectType(externalClass);
         return new ConstructedItemType(type, processor);
     }
 
@@ -493,16 +484,15 @@ public class ItemTypeFactory {
      * Factory method to construct an "external object". This is an XDM value that wraps a Java
      * object. Such values can be passed as parameters to stylesheets or queries, for use in conjunction
      * with external (extension) functions.
-     * <p/>
      * <p>Changed in Saxon 9.5 to return XdmItem rather than XdmAtomicValue, since wrapped
      * external objects are now modelled as a separate type of item, rather than as an atomic value.</p>
      *
      * @param object the value to be wrapped as an external object. Must not be null.
-     * @return the object, wrapped as an XdmAtomicValue
+     * @return the object, wrapped as an XdmItem
      */
 
     public XdmItem getExternalObject(Object object) {
-        return (XdmItem) XdmItem.wrap(new ObjectValue<Object>(object));
+        return (XdmItem) XdmItem.wrap(new ObjectValue<>(object));
     }
 
     /**
