@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,10 +7,7 @@
 
 package net.sf.saxon.s9api;
 
-import net.sf.saxon.Configuration;
 import net.sf.saxon.event.*;
-import net.sf.saxon.expr.instruct.Executable;
-import net.sf.saxon.expr.instruct.ResultDocument;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.lib.OutputURIResolver;
 import net.sf.saxon.lib.SaxonOutputKeys;
@@ -19,6 +16,7 @@ import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.query.QueryResult;
 import net.sf.saxon.serialize.CharacterMap;
 import net.sf.saxon.serialize.CharacterMapIndex;
+import net.sf.saxon.serialize.SerializationProperties;
 import net.sf.saxon.trans.XPathException;
 
 import javax.xml.transform.OutputKeys;
@@ -32,75 +30,54 @@ import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.HashMap;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Properties;
 
 /**
  * A Serializer takes a tree representation of XML and turns it into lexical XML markup.
- * <p/>
  * <p><i>Note that this is XML serialization in the sense of the W3C XSLT and XQuery specifications.
  * This has nothing to do with the serialization of Java objects, or the {@link java.io.Serializable}
  * interface.</i></p>
- * <p/>
  * <p>The serialization may be influenced by a number of serialization parameters. A parameter has a name,
  * which is an instance of {@link Serializer.Property}, and a value, which is expressed as a string.
  * The effect of most of the properties is as described in the W3C specification
- * <a href="http://www.w3.org/TR/xslt-xquery-serialization/">XSLT 2.0 and XQuery 1.0 Serialization</a>.
+ * <a href="http://www.w3.org/TR/xslt-xquery-serialization/">XSLT 3.0 and XQuery 3.1 Serialization</a>.
  * Saxon supports all the serialization parameters defined in that specification, together with some
- * additional parameters, whose property names are prefixed "SAXON_".
- * <p/>
+ * additional parameters, whose property names are prefixed "SAXON_".</p>
  * <p>Serialization parameters defined via this interface take precedence over any serialization parameters
- * defined within the source of the query or stylesheet.
+ * defined within the source of the query or stylesheet. The {@link #getReceiver} method is supplied with
+ * parameters derived from the query or stylesheet, which are merged with the parameters set on this
+ * {@code Serializer}.</p>
+ * <p><i>Changed in 9.9 so the {@code Serializer} now only holds one set of output properties. Additional
+ * (default) serialization properties (taken from the source of the query or stylesheet) can be supplied
+ * when {@link #getReceiver(PipelineConfiguration, SerializationProperties)}
+ * is called, but these are not stored within the {@code Serializer} itself.</i></p>
  */
 @SuppressWarnings({"ForeachStatement"})
-public class Serializer implements Destination {
+public class Serializer extends AbstractDestination {
 
-    private Processor processor; // Beware: this will often be null
-    private Map<StructuredQName, String> properties = new HashMap<StructuredQName, String>(10);
+    private Processor processor; // never null
+    private Map<StructuredQName, String> properties = new HashMap<>(10);
     private StreamResult result = new StreamResult();
-    private Properties defaultOutputProperties = null;
     private CharacterMapIndex characterMap = null;
     private boolean mustClose = false;
 
-    private static Map<String, Property> standardProperties = new HashMap<String, Property>();
+    // Index of permitted property names
 
-    private static void z(Property prop) {
-        standardProperties.put(prop.name, prop);
-    }
+    private static Map<String, Property> standardProperties = new HashMap<>();
+
+//    private static void z(Property prop) {
+//        standardProperties.put(prop.name, prop);
+//    }
 
     static {
-        z(Property.METHOD);
-        z(Property.VERSION);
-        z(Property.ENCODING);
-        z(Property.OMIT_XML_DECLARATION);
-        z(Property.STANDALONE);
-        z(Property.DOCTYPE_PUBLIC);
-        z(Property.DOCTYPE_SYSTEM);
-        z(Property.CDATA_SECTION_ELEMENTS);
-        z(Property.INDENT);
-        z(Property.MEDIA_TYPE);
-        z(Property.USE_CHARACTER_MAPS);
-        z(Property.INCLUDE_CONTENT_TYPE);
-        z(Property.UNDECLARE_PREFIXES);
-        z(Property.ESCAPE_URI_ATTRIBUTES);
-        z(Property.BYTE_ORDER_MARK);
-        z(Property.NORMALIZATION_FORM);
-        z(Property.ITEM_SEPARATOR);
-        z(Property.SAXON_INDENT_SPACES);
-        z(Property.SAXON_ATTRIBUTE_ORDER);
-        z(Property.SAXON_CHARACTER_REPRESENTATION);
-        z(Property.SAXON_DOUBLE_SPACE);
-        z(Property.SAXON_IMPLICIT_RESULT_DOCUMENT);
-        z(Property.SAXON_LINE_LENGTH);
-        z(Property.SAXON_RECOGNIZE_BINARY);
-        z(Property.SAXON_REQUIRE_WELL_FORMED);
-        z(Property.SAXON_STYLESHEET_VERSION);
-        z(Property.SAXON_SUPPLY_SOURCE_LOCATOR);
-        z(Property.SAXON_SUPPRESS_INDENTATION);
-        z(Property.SAXON_WRAP);
+        for (Property p : Property.values()) {
+            standardProperties.put(p.name, p);
+        }
     }
 
     /**
-     * Enumerator over the defined serialization properties
+     * Enumeration class defining the permitted serialization properties
      */
 
     public enum Property {
@@ -204,17 +181,25 @@ public class Serializer implements Destination {
          * by namespace, then by local name).
          */
         SAXON_ATTRIBUTE_ORDER(SaxonOutputKeys.ATTRIBUTE_ORDER),
+        /**
+         * Saxon extension: request canonical XML output.
+         */
+        SAXON_CANONICAL(SaxonOutputKeys.CANONICAL),
         /**
          * Saxon extension: set to any string. Indicates the sequence of characters used to represent
          * a newline in the text output method, and in newlines used for indentation in any output
          * methods that use indentation.
          */
         SAXON_NEWLINE(SaxonOutputKeys.NEWLINE),
-
         /**
-         * Saxon extension: set to a space-separated list of element names, in Clark notation,
+         * Set to a space-separated list of element names, in Clark notation,
          * within which no content is to be indented. This is typically because the element contains
          * mixed content in which whitespace is significant.
+         * <p>
+         * This was originally a Saxon extension to the W3C specifications, but it then became
+         * a standard serialization attribute. The "SAXON_" prefix on the name of the enumeration
+         * constant reflects this history; the name is retained to avoid disrupting existing
+         * applications.
          */
         SAXON_SUPPRESS_INDENTATION(SaxonOutputKeys.SUPPRESS_INDENTATION),
         /**
@@ -261,7 +246,7 @@ public class Serializer implements Destination {
 
         private String name;
 
-        private Property(String name) {
+        Property(String name) {
             this.name = name;
         }
 
@@ -303,7 +288,7 @@ public class Serializer implements Destination {
     /**
      * Create a Serializer belonging to a specific processor
      *
-     * @param processor the processor associated with the Serializer
+     * @param processor the processor associated with the Serializer. Must not be null.
      */
 
     protected Serializer(Processor processor) {
@@ -312,25 +297,18 @@ public class Serializer implements Destination {
 
     /**
      * Set the Processor associated with this Serializer. This will be called automatically if the
-     * serializer is created using one of the <code>Processor.newSerializer()</code> methods. The Serializer
-     * currently needs to know about the Processor only if the method {@link #getXMLStreamWriter} is called.
-     *
+     * serializer is created using one of the <code>Processor.newSerializer()</code> methods.
      * @param processor the associated Processor (must not be null)
      * @since 9.3
      */
 
     public void setProcessor(Processor processor) {
-        if (processor == null) {
-            throw new NullPointerException();
-        }
-        this.processor = processor;
+        this.processor = Objects.requireNonNull(processor);
     }
 
     /**
-     * Get the Processor associated with this Serializer. This may be null if a deprecated
-     * constructor was used to create the Serializer.
-     *
-     * @return the associated Processor
+     * Get the Processor associated with this Serializer.
+     * @return the associated Processor. Never null.
      */
 
     public Processor getProcessor() {
@@ -338,17 +316,42 @@ public class Serializer implements Destination {
     }
 
     /**
-     * Set default output properties, for use when no explicit properties are set using setProperty().
+     * Set output properties, from a {@link Properties} object. The keys in the supplied
+     * {@code Properties} object are QNames in Clark format, that is "{uri}local"; any QNames
+     * within the values must also be in this format.
      * The values supplied are typically those specified in the stylesheet or query. In the case of XSLT,
-     * they are the properties associated with unnamed xsl:output declarations.
+     * they are typically the properties associated with unnamed xsl:output declarations.
      *
-     * @param defaultProperties the default output properties to be used
+     * @param suppliedProperties the output properties to be used. These overwrite any properties
+     *                           that have been individually specified using
+     *                           {@link #setOutputProperty(Property, String)} setOutputProperty}
      */
 
-    public void setDefaultOutputProperties(Properties defaultProperties) {
-        this.defaultOutputProperties = defaultProperties;
+    public void setOutputProperties(Properties suppliedProperties) {
+        for (String name : suppliedProperties.stringPropertyNames()) {
+            properties.put(StructuredQName.fromClarkName(name), suppliedProperties.getProperty(name));
+        }
     }
 
+    /**
+     * Set output properties, from an {@link SerializationProperties} object. The keys in the contained
+     * {@code Properties} object are QNames in Clark format, that is "{uri}local"; any QNames
+     * within the values must also be in this format.
+     * <p>The values supplied are typically those specified in the stylesheet or query. In the case of XSLT,
+     * they are typically the properties associated with unnamed xsl:output declarations.</p>
+     *
+     * @param suppliedProperties the output properties to be used. These overwrite any properties
+     *                           that have been individually specified using
+     *                           {@link #setOutputProperty(Property, String)} setOutputProperty}
+     * @since 9.9
+     */
+
+    public void setOutputProperties(SerializationProperties suppliedProperties) {
+        setOutputProperties(suppliedProperties.getProperties());
+        setCharacterMap(suppliedProperties.getCharacterMapIndex());
+    }
+
+
     /**
      * Say if the output stream should be closed on completion
      * By default the close method closes the output stream only when the serializer created the output stream itself
@@ -390,10 +393,8 @@ public class Serializer implements Destination {
     /**
      * Set the value of a serialization property. Any existing value of the property is overridden.
      * If the supplied value is null, any existing value of the property is removed.
-     * <p/>
      * <p>Example:</p>
      * <p><code>serializer.setOutputProperty(Serializer.Property.METHOD, "xml");</code></p>
-     * <p/>
      * <p>Any serialization properties supplied via this interface take precedence over serialization
      * properties defined in the source stylesheet or query, including properties set dynamically
      * using xsl:result-document. However, they only affect the principal output of a transformation;
@@ -442,10 +443,8 @@ public class Serializer implements Destination {
     /**
      * Set the value of a serialization property. Any existing value of the property is overridden.
      * If the supplied value is null, any existing value of the property is removed.
-     * <p/>
      * <p>Example:</p>
      * <p><code>serializer.setOutputProperty(new QName("method"), "xml");</code></p>
-     * <p/>
      * <p>Any serialization properties supplied via this interface take precedence over serialization
      * properties defined in the source stylesheet or query.</p>
      * <p>Unlike the method {@link #setOutputProperty(Property, String)}, this method allows properties
@@ -506,13 +505,10 @@ public class Serializer implements Destination {
 
     /**
      * Set the destination of the serialized output, as a Writer.
-     * <p/>
      * <p>Note that when this option is used, the serializer does not perform character
      * encoding. This also means that it never replaces special characters with XML numeric
      * character references. The final encoding is the responsibility of the supplied Writer.</p>
-     * <p/>
      * <p>Closing the writer after use is the responsibility of the caller.</p>
-     * <p/>
      * <p>Calling this method has the side-effect of setting the OutputStream and OutputFile to null.</p>
      *
      * @param writer the Writer to which the serialized XML output will be written.
@@ -527,9 +523,7 @@ public class Serializer implements Destination {
 
     /**
      * Set the destination of the serialized output, as an OutputStream.
-     * <p/>
      * <p>Closing the output stream after use is the responsibility of the caller.</p>
-     * <p/>
      * <p>Calling this method has the side-effect of setting the OutputWriter and OutputFile to null.</p>
      *
      * @param stream the OutputStream to which the serialized XML output will be written.
@@ -544,9 +538,10 @@ public class Serializer implements Destination {
 
     /**
      * Set the destination of the serialized output, as a File.
-     * <p/>
      * <p>Calling this method has the side-effect of setting the current OutputWriter
      * and OutputStream to null.</p>
+     * <p>This method sets the destination base URI to the URI corresponding to the name of the
+     * supplied file.</p>
      *
      * @param file the File to which the serialized XML output will be written.
      */
@@ -555,6 +550,7 @@ public class Serializer implements Destination {
         result.setOutputStream(null);
         result.setWriter(null);
         result.setSystemId(file);
+        setDestinationBaseURI(file.toURI());
         mustClose = true;
     }
 
@@ -594,16 +590,14 @@ public class Serializer implements Destination {
         if (value instanceof XdmNode) {
             serializeNode((XdmNode) value);
         } else {
-            if (processor == null) {
-                throw new IllegalStateException("The Serializer is not associated with any s9api Processor");
-            }
             try {
                 QueryResult.serializeSequence(value.getUnderlyingValue().iterate(),
-                    processor.getUnderlyingConfiguration(), result, getCombinedOutputProperties());
+                                              processor.getUnderlyingConfiguration(), result, getLocallyDefinedProperties());
             } catch (XPathException e) {
                 throw new SaxonApiException(e);
             }
         }
+        closeAndNotify();
 
     }
 
@@ -625,7 +619,7 @@ public class Serializer implements Destination {
 
     private void serializeNodeToResult(XdmNode node, Result res) throws SaxonApiException {
         try {
-            QueryResult.serialize(node.getUnderlyingNode(), res, getCombinedOutputProperties());
+            QueryResult.serialize(node.getUnderlyingNode(), res, getLocallyDefinedProperties());
         } catch (XPathException e) {
             throw new SaxonApiException(e);
         }
@@ -636,16 +630,13 @@ public class Serializer implements Destination {
      * to be output via this serializer.
      *
      * @return a newly constructed XMLStreamWriter that pipes events into this Serializer
-     * @throws IllegalStateException if no Processor has been set for this Serializer
      * @throws SaxonApiException     if any other failure occurs
      * @since 9.3
      */
 
     public StreamWriterToReceiver getXMLStreamWriter() throws SaxonApiException {
-        if (processor == null) {
-            throw new IllegalStateException("This method is available only if a Processor has been set");
-        }
-        Receiver r = getReceiver(processor.getUnderlyingConfiguration());
+        PipelineConfiguration pipe = processor.getUnderlyingConfiguration().makePipelineConfiguration();
+        Receiver r = getReceiver(pipe, getSerializationProperties());
         r = new NamespaceReducer(r);
         return new StreamWriterToReceiver(r);
     }
@@ -655,16 +646,14 @@ public class Serializer implements Destination {
      * source of SAX events) to this serializer.
      *
      * @return a newly constructed ContentHandler that pipes events into this Serializer
-     * @throws IllegalStateException if no Processor has been set for this Serializer
      * @throws SaxonApiException     if any other failure occurs
      * @since 9.7
      */
 
     public org.xml.sax.ContentHandler getContentHandler() throws SaxonApiException {
-        if (processor == null) {
-            throw new IllegalStateException("This method is available only if a Processor has been set");
-        }
-        Receiver r = getReceiver(processor.getUnderlyingConfiguration());
+        PipelineConfiguration pipe = processor.getUnderlyingConfiguration().makePipelineConfiguration();
+        Receiver r = getReceiver(pipe, getSerializationProperties());
+        //r = new SequenceNormalizerWithSpaceSeparator(r);
         r = new NamespaceReducer(r);
         ReceivingContentHandler rch = new ReceivingContentHandler();
         rch.setReceiver(r);
@@ -704,93 +693,29 @@ public class Serializer implements Destination {
      * primarily for internal use, though it could also be called by user applications
      * wanting to make use of the Saxon serializer.
      *
-     * @param config The Saxon configuration. This is an internal implementation object
+     * @param pipe The Saxon configuration. This is an internal implementation object
      *               held within the {@link Processor}
+     * @param params Serialization parameters originating from the query or stylesheet
+     *               (for example, xsl:output declarations or xsl:result-document attributes).
+     *               These parameters are combined with those held by this {@code Serializer} itself,
+     *               with properties set of the {@code Serializer} taking precedence.
      * @return a receiver to which XML events will be sent
      */
 
-    public Receiver getReceiver(Configuration config) throws SaxonApiException {
+    public Receiver getReceiver(PipelineConfiguration pipe, SerializationProperties params) throws SaxonApiException {
         try {
-            SerializerFactory sf = config.getSerializerFactory();
-            PipelineConfiguration pipe = config.makePipelineConfiguration();
-            Properties props = getCombinedOutputProperties();
-            Receiver target = sf.getReceiver(result, pipe, props, characterMap);
-            if (target.getSystemId() == null) {
-                target.setSystemId(result.getSystemId());
-            }
-            return target;
-        } catch (XPathException e) {
-            throw new SaxonApiException(e);
-        }
-    }
-
-    /**
-     * Return a receiver to which Saxon will send events. This method is provided
-     * primarily for internal use, though it could also be called by user applications
-     * wanting to make use of the Saxon serializer.
-     *
-     * @param pipe The Saxon pipeline configuration. This is an internal implementation object
-     *               held within the {@link Processor}
-     * @return a receiver to which XML events will be sent
-     */
-
-    public Receiver getReceiver(PipelineConfiguration pipe) throws SaxonApiException {
-        try {
-            Configuration config = pipe.getConfiguration();
-            SerializerFactory sf = config.getSerializerFactory();
-            Properties props = getCombinedOutputProperties();
-            Receiver target = sf.getReceiver(result, pipe, props, characterMap);
-            if (target.getSystemId() == null) {
-                target.setSystemId(result.getSystemId());
-            }
-            return target;
-        } catch (XPathException e) {
-            throw new SaxonApiException(e);
-        }
-    }
-
-
-    /**
-     * Return a receiver to which Saxon will send events. This method is provided
-     * primarily for internal use, though it could also be called by user applications
-     * wanting to make use of the Saxon serializer.
-     *
-     * @param executable The Saxon Executable for the transformation or query. The serialization
-     *                   properties defined in this Serializer are supplemented by properties that have been
-     *                   defined in the query or stylesheet associated with the Executable. The properties defined
-     *                   in this Serializer take precedence over those in the stylesheet or query.
-     * @return a receiver to which XML events will be sent
-     * @throws SaxonApiException if any failure occurs
-     */
-
-    protected Receiver getReceiver(Executable executable) throws SaxonApiException {
-        // Note: used only for XQuery. The equivalent logic for XSLT is in XsltTransformer.setDestination()
-        try {
-            Configuration config = executable.getConfiguration();
-            SerializerFactory sf = config.getSerializerFactory();
-            PipelineConfiguration pipe = config.makePipelineConfiguration();
-            pipe.setHostLanguage(executable.getHostLanguage());
-            Properties baseProps = new Properties(executable.getDefaultOutputProperties());
-
-            CharacterMapIndex charMapIndex = executable.getCharacterMapIndex();
-            if (charMapIndex.isEmpty()) {
-                charMapIndex = characterMap;
-            } else if (characterMap != null && !characterMap.isEmpty() && charMapIndex != characterMap) {
-                // Merge the character maps
-                charMapIndex = charMapIndex.copy();
-                for (CharacterMap map : characterMap) {
-                    charMapIndex.putCharacterMap(map.getName(), map);
+            SerializerFactory sf = pipe.getConfiguration().getSerializerFactory();
+            SerializationProperties mergedParams = getSerializationProperties().combineWith(params);
+            Receiver target = sf.getReceiver(result, mergedParams, pipe);
+            if (helper.getListeners() != null) {
+                if (target instanceof SequenceNormalizer) {
+                    ((SequenceNormalizer)target).onClose(helper.getListeners());
+                } else {
+                    target = new CloseNotifier(target, helper.getListeners());
                 }
             }
-
-            for (Map.Entry<StructuredQName, String> entry : properties.entrySet()) {
-                StructuredQName name = entry.getKey();
-                ResultDocument.setSerializationProperty(
-                    baseProps, name.getURI(), name.getLocalPart(), entry.getValue(), null, true, config);
-            }
-            Receiver target = sf.getReceiver(result, pipe, baseProps, charMapIndex);
-            if (target.getSystemId() == null) {
-                target.setSystemId(result.getSystemId());
+            if (target.getSystemId() == null && getDestinationBaseURI() != null) {
+                target.setSystemId(getDestinationBaseURI().toASCIIString());
             }
             return target;
         } catch (XPathException e) {
@@ -808,7 +733,7 @@ public class Serializer implements Destination {
      * in unnamed xsl:output declarations in the stylesheet, or output declarations in XQuery.
      */
 
-    public Properties getCombinedOutputProperties() {
+    public Properties getCombinedOutputProperties(Properties defaultOutputProperties) {
         Properties props = defaultOutputProperties == null ? new Properties() : new Properties(defaultOutputProperties);
         for (StructuredQName p : properties.keySet()) {
             String value = properties.get(p);
@@ -817,6 +742,7 @@ public class Serializer implements Destination {
         return props;
     }
 
+
     /**
      * Create a Properties object holding the serialization properties explicitly declared
      * within this Serializer object, and not including any defaults taken from the stylesheet or query.
@@ -825,7 +751,7 @@ public class Serializer implements Destination {
      * in named or unnamed xsl:output declarations in the stylesheet, or the equivalent in XQuery.
      */
 
-    protected Properties getLocalOutputProperties() {
+    protected Properties getLocallyDefinedProperties() {
         Properties props = new Properties();
         for (StructuredQName p : properties.keySet()) {
             String value = properties.get(p);
@@ -834,6 +760,21 @@ public class Serializer implements Destination {
         return props;
     }
 
+    /**
+     * Create a {@link SerializationProperties} object holding the serialization properties explicitly declared
+     * within this Serializer object (including any character map index), and not including any
+     * defaults taken from the stylesheet or query.
+     *
+     * @return a newly-constructed {@code SerializationProperties} object holding the declared
+     * serialization properties. Specifically, it holds the properties defined explicitly on this
+     * {@code Serializer} object, and excludes any properties defined in named or unnamed
+     * {@code xsl:output} declarations in the stylesheet, or the equivalent in XQuery.
+     */
+
+    public SerializationProperties getSerializationProperties() {
+        return new SerializationProperties(getLocallyDefinedProperties(), characterMap);
+    }
+
     /**
      * Get the JAXP StreamResult object representing the output destination
      * of this serializer
@@ -860,7 +801,7 @@ public class Serializer implements Destination {
                     throw new SaxonApiException("Failed while closing output file", err);
                 }
             }
-            Writer writer = result.getWriter();  // Path not used, but there for safety
+            Writer writer = result.getWriter();  
             if (writer != null) {
                 try {
                     writer.close();
