@@ -9,10 +9,7 @@ package net.sf.saxon.s9api;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.Controller;
-import net.sf.saxon.event.Builder;
-import net.sf.saxon.event.PipelineConfiguration;
-import net.sf.saxon.event.Receiver;
-import net.sf.saxon.event.SequenceNormalizer;
+import net.sf.saxon.event.*;
 import net.sf.saxon.expr.instruct.Executable;
 import net.sf.saxon.expr.instruct.GlobalContextRequirement;
 import net.sf.saxon.expr.instruct.UserFunction;
@@ -164,7 +161,7 @@ public class XQueryEvaluator extends AbstractDestination implements Iterable<Xdm
      */
 
     public XdmItem getContextItem() {
-        Item item = context.getContextItem();
+        Item<?> item = context.getContextItem();
         if (item == null) {
             return null;
         }
@@ -182,7 +179,7 @@ public class XQueryEvaluator extends AbstractDestination implements Iterable<Xdm
     public void setExternalVariable(QName name, XdmValue value) {
         try {
             context.setParameter(name.getStructuredQName(),
-                                 value == null ? null : ((Sequence<? extends Item>) value.getUnderlyingValue()).materialize());
+                                 value == null ? null : ((Sequence<?>) value.getUnderlyingValue()).materialize());
         } catch (XPathException e) {
             throw new SaxonApiUncheckedException(e);
         }
@@ -196,11 +193,11 @@ public class XQueryEvaluator extends AbstractDestination implements Iterable<Xdm
      */
 
     public XdmValue getExternalVariable(QName name) {
-        Object oval = context.getParameter(name.getStructuredQName());
+        GroundedValue<?> oval = context.getParameter(name.getStructuredQName());
         if (oval == null) {
             return null;
         }
-        return XdmValue.wrap((Sequence) oval);
+        return XdmValue.wrap(oval);
     }
 
     /**
@@ -428,9 +425,8 @@ public class XQueryEvaluator extends AbstractDestination implements Iterable<Xdm
             throw new IllegalStateException("Query is updating");
         }
         try {
-            SequenceIterator<? extends Item> iter = expression.iterator(context);
-            Sequence result = iter.materialize();
-            return XdmValue.wrap(result);
+            SequenceIterator<?> iter = expression.iterator(context);
+            return XdmValue.wrap(iter.materialize());
         } catch (XPathException e) {
             throw new SaxonApiException(e);
         }
@@ -450,8 +446,8 @@ public class XQueryEvaluator extends AbstractDestination implements Iterable<Xdm
 
     public XdmItem evaluateSingle() throws SaxonApiException {
         try {
-            SequenceIterator iter = expression.iterator(context);
-            Item next = iter.next();
+            SequenceIterator<?> iter = expression.iterator(context);
+            Item<?> next = iter.next();
             return next == null ? null : (XdmItem) XdmValue.wrap(next);
         } catch (XPathException e) {
             throw new SaxonApiException(e);
@@ -488,7 +484,12 @@ public class XQueryEvaluator extends AbstractDestination implements Iterable<Xdm
     private Receiver getDestinationReceiver(Destination destination) throws SaxonApiException {
         Executable exec = expression.getExecutable();
         PipelineConfiguration pipe = expression.getConfiguration().makePipelineConfiguration();
-        return destination.getReceiver(pipe, exec.getPrimarySerializationProperties());
+        Receiver out = destination.getReceiver(pipe, exec.getPrimarySerializationProperties());
+        if (Configuration.isAssertionsEnabled()) {
+            return new RegularSequenceChecker(out);
+        } else {
+            return out;
+        }
     }
 
     /**
@@ -505,7 +506,7 @@ public class XQueryEvaluator extends AbstractDestination implements Iterable<Xdm
      * @param pipe The Saxon configuration. This is supplied so that the destination can
      *               use information from the configuration (for example, a reference to the name pool)
      *               to construct or configure the returned Receiver.
-     * @param params
+     * @param params the output serialization properties
      * @return the Receiver to which events are to be sent.
      * @throws SaxonApiException     if the Receiver cannot be created
      * @throws IllegalStateException if no Destination has been supplied
@@ -551,9 +552,7 @@ public class XQueryEvaluator extends AbstractDestination implements Iterable<Xdm
      */
 
     public void close() throws SaxonApiException {
-//        if (sourceTreeBuilder != null) {
-//
-//        }
+
     }
 
 
@@ -603,8 +602,7 @@ public class XQueryEvaluator extends AbstractDestination implements Iterable<Xdm
                 context.initializeController(controller);
             }
             Configuration config = processor.getUnderlyingConfiguration();
-            Sequence[] vr = new Sequence[arguments.length];
-
+            Sequence<?>[] vr = SequenceTool.makeSequenceArray(arguments.length);
             for (int i = 0; i < arguments.length; i++) {
                 net.sf.saxon.value.SequenceType type = fn.getParameterDefinitions()[i].getRequiredType();
                 vr[i] = arguments[i].getUnderlyingValue();
@@ -616,7 +614,7 @@ public class XQueryEvaluator extends AbstractDestination implements Iterable<Xdm
                     vr[i] = th.applyFunctionConversionRules(vr[i], type, role, ExplicitLocation.UNKNOWN_LOCATION);
                 }
             }
-            Sequence result = fn.call(vr, controller);
+            Sequence<?> result = fn.call(vr, controller);
             return XdmValue.wrap(result);
         } catch (XPathException e) {
             throw new SaxonApiException(e);
