@@ -11,7 +11,6 @@ import net.sf.saxon.ma.arrays.ArrayFunctionSet;
 import net.sf.saxon.ma.arrays.ArrayItem;
 import net.sf.saxon.ma.arrays.SimpleArrayItem;
 import net.sf.saxon.om.GroundedValue;
-import net.sf.saxon.om.Item;
 import net.sf.saxon.om.Sequence;
 import net.sf.saxon.trans.XPathException;
 
@@ -54,7 +53,7 @@ public class XdmArray extends XdmFunctionItem {
      */
 
     public XdmArray(XdmValue[] members) {
-        List<GroundedValue<? extends Item>> values = new ArrayList<>();
+        List<GroundedValue<?>> values = new ArrayList<>();
         for (XdmValue member : members) {
             values.add(member.getUnderlyingValue());
         }
@@ -71,7 +70,7 @@ public class XdmArray extends XdmFunctionItem {
      */
 
     public XdmArray(Iterable<? extends XdmValue> members) {
-        List<GroundedValue<? extends Item>> values = new ArrayList<>();
+        List<GroundedValue<?>> values = new ArrayList<>();
         for (XdmValue member : members) {
             values.add(member.getUnderlyingValue());
         }
@@ -100,7 +99,7 @@ public class XdmArray extends XdmFunctionItem {
 
     public XdmValue get(int n) throws IndexOutOfBoundsException {
         try {
-            Sequence member = getUnderlyingValue().get(n);
+            Sequence<?> member = getUnderlyingValue().get(n);
             return XdmValue.wrap(member);
         } catch (XPathException e) {
             throw new IndexOutOfBoundsException();
@@ -133,10 +132,10 @@ public class XdmArray extends XdmFunctionItem {
      * @param value the new member
      * @return a new array, one item longer than the original
      * @throws SaxonApiUncheckedException if the value is lazily evaluated, and evaluation fails
-     * @since 9.9
+     * @since 9.9. (See bug 3968: on first release of 9.9, the method was mistakenly named <code>append</code>).
      */
 
-    public XdmArray append(XdmValue value) {
+    public XdmArray addMember(XdmValue value) {
         try {
             GroundedValue member = value.getUnderlyingValue();
             ArrayItem newArray = ArrayFunctionSet.ArrayAppend.append(getUnderlyingValue(), member);
@@ -167,7 +166,7 @@ public class XdmArray extends XdmFunctionItem {
      * @return a list of the members of this array.
      */
     public List<XdmValue> asList() {
-        Iterator<GroundedValue<? extends Item>> members = getUnderlyingValue().members().iterator();
+        Iterator<GroundedValue<?>> members = getUnderlyingValue().members().iterator();
         List<XdmValue> result = new ArrayList<XdmValue>(getUnderlyingValue().getLength());
         while (members.hasNext()) {
             result.add(XdmValue.wrap(members.next()));
