@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,8 +9,12 @@ package net.sf.saxon.s9api;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.PreparedStylesheet;
+import net.sf.saxon.expr.parser.OptimizerOptions;
 import net.sf.saxon.lib.ErrorGatherer;
 import net.sf.saxon.lib.StringCollator;
+import net.sf.saxon.om.Item;
+import net.sf.saxon.om.NodeInfo;
+import net.sf.saxon.om.Sequence;
 import net.sf.saxon.style.*;
 import net.sf.saxon.trace.XSLTTraceCodeInjector;
 import net.sf.saxon.trans.CompilerInfo;
@@ -25,22 +29,28 @@ import javax.xml.transform.ErrorListener;
 import javax.xml.transform.Source;
 import javax.xml.transform.URIResolver;
 import javax.xml.transform.stream.StreamSource;
+import java.io.InputStream;
 import java.net.URI;
 import java.util.ArrayList;
 import java.util.Comparator;
 import java.util.List;
+import java.util.Objects;
 
 /**
  * An XsltCompiler object allows XSLT 2.0 and XSLT 3.0 stylesheets to be compiled. The compiler holds information that
  * represents the static context for the compilation.
- * <p/>
  * <p>To construct an XsltCompiler, use the factory method {@link Processor#newXsltCompiler} on the Processor object.</p>
- * <p/>
  * <p>An XsltCompiler may be used repeatedly to compile multiple queries. Any changes made to the
  * XsltCompiler (that is, to the static context) do not affect queries that have already been compiled.
  * An XsltCompiler may in principle be used concurrently in multiple threads, but in practice this
  * is best avoided because all instances will share the same ErrorListener and it may therefore be
  * difficult to establish which error messages are associated with each compilation.</p>
+ * <p>If JIT compilation is enabled (this is the default for Saxon-EE), then static errors in template
+ *  rules might be detected not during execution of the <code>compile()</code> method, but rather
+ *  when the relevant code is first executed. In this situation the <code>compile()</code> method will
+ *  not throw an exception, but the errors will still (eventually) be notified to the <code>ErrorListener</code>
+ *  associated with the compiler. This is problematic if the compiler is used to perform multiple compilations,
+ *  because </p>
  *
  * @since 9.0
  */
@@ -64,7 +74,7 @@ public class XsltCompiler {
         compilerInfo = new CompilerInfo(config.getDefaultXsltCompilerInfo());
         compilerInfo.setGenerateByteCode(config.isGenerateByteCode(Configuration.XSLT));
         compilerInfo.setTargetEdition(config.getEditionCode());
-        compilerInfo.setJustInTimeCompilation(config.isJITEnabled(Configuration.XSLT));
+        compilerInfo.setJustInTimeCompilation(config.isJITEnabled());
     }
 
     /**
@@ -83,12 +93,13 @@ public class XsltCompiler {
      * is <b>not</b> used for resolving relative URIs against a base URI; it is used for dereferencing
      * an absolute URI (after resolution) to return a {@link javax.xml.transform.Source} representing the
      * location where a stylesheet module can be found.
-     * <p/>
      * <p>This URIResolver is used to dereference the URIs appearing in <code>xsl:import</code>,
      * <code>xsl:include</code>, and <code>xsl:import-schema</code> declarations. It is not used
      * for resolving the URI supplied for the main stylesheet module (as supplied to the
      * {@link #compile(javax.xml.transform.Source)} or {@link #compilePackage(javax.xml.transform.Source)} methods.
-     * It is not used at run-time for resolving requests to the <code>document()</code> or similar functions.</p>
+     * It is not used at run-time for resolving requests to the <code>document()</code> or similar functions.
+     * (Instead, such functions use the run-time URIResolver passed to the {@link XsltTransformer}
+     * or {@link Xslt30Transformer}).</p>
      *
      * @param resolver the URIResolver to be used during stylesheet compilation.
      */
@@ -103,11 +114,18 @@ public class XsltCompiler {
      * may also be provided using this method if their values will not vary from one transformation
      * to another.
      *
-     * @param name  the StructuredQName of the parameter
-     * @param value as a XdmValue of the parameter
+     * @param name  the qualified name identifying the parameter, as declared in a global <code>&lt;xsl:param&gt;</code>
+     *              element in the stylesheet
+     * @param value the value of the parameter
+     * @throws SaxonApiUncheckedException if the value is lazily evaluated, and evaluation fails
      */
     public void setParameter(QName name, XdmValue value) {
-        compilerInfo.setParameter(name.getStructuredQName(), value.getUnderlyingValue());
+        try {
+            compilerInfo.setParameter(name.getStructuredQName(),
+                                      ((Sequence<?>) value.getUnderlyingValue()).materialize());
+        } catch (XPathException e) {
+            throw new SaxonApiUncheckedException(e);
+        }
     }
 
     /**
@@ -231,31 +249,13 @@ public class XsltCompiler {
         return compilerInfo.getTargetEdition();
     }
 
-    /**
-     * Bind a collation URI to a collation
-     *
-     * @param uri       the absolute collation URI
-     * @param collation a {@link java.text.Collator} object that implements the required collation
-     * @throws IllegalArgumentException if an attempt is made to rebind the standard URI
-     *                                  for the Unicode codepoint collation
-     * @since 9.5
-     * @deprecated since 9.6. Collations are now held globally. If this method is called, the effect
-     * is to update the pool of collations held globally by the Processor.
-     */
-
-    public void declareCollation(String uri, final java.text.Collator collation) {
-        getProcessor().declareCollation(uri, collation);
-    }
-
     /**
      * Declare the default collation
      *
      * @param uri the absolute URI of the default collation. Either this URI must have been bound to a collation
-     *            using the method {@link Configuration#registerCollation(String, StringCollator)}
-     *            Collation(String, java.text.Collator)}, or it must be a
+     *            using the method {@link Configuration#registerCollation(String, StringCollator)}, or it must be a
      *            collation that is recognized implicitly, such as a UCA collation
-     * @throws IllegalStateException if the collation URI has not been registered, unless it is the standard
-     *                               Unicode codepoint collation which is registered implicitly
+     * @throws IllegalStateException if the collation URI is not a recognized collation
      * @since 9.5
      */
 
@@ -337,6 +337,42 @@ public class XsltCompiler {
         compilerInfo.setAssertionsEnabled(enabled);
     }
 
+    /**
+     * Request fast compilation. Fast compilation will generally be achieved at the expense of run-time performance
+     * and quality of diagnostics. Fast compilation is a good trade-off if (a) the stylesheet is known to be correct,
+     * and (b) once compiled, it is only executed once against a document of modest size.
+     * <p>Fast compilation may result in static errors going unreported, especially if they occur in code
+     * that is never executed.</p>
+     * <p><i>The current implementation is equivalent to switching off all optimizations other than just-in-time
+     * compilation of template rules. Setting this option, however, indicates an intent rather than a mechanism,
+     * and the implementation details may change in future to reflect the intent.</i></p>
+     *
+     * @param fast set to true to request fast compilation; set to false to revert to the optimization options
+     *             defined in the Configuration.
+     * @since 9.9
+     */
+
+    public void setFastCompilation(boolean fast) {
+        if (fast) {
+            // The only optimizer option that speeds up compilation is JIT.
+            compilerInfo.setOptimizerOptions(new OptimizerOptions(OptimizerOptions.JIT));
+        } else {
+            compilerInfo.setOptimizerOptions(getProcessor().getUnderlyingConfiguration().getOptimizerOptions());
+        }
+    }
+
+    /**
+     * Ask if fast compilation has been enabled.
+     *
+     * @return true if fast compilation has been enabled (technically, this returns true if the only
+     * optimizer option that has been enabled is JIT compilation of template rules).
+     * @since 9.9
+     */
+
+    public boolean isFastCompilation() {
+        return compilerInfo.getOptimizerOptions().getOptions() == OptimizerOptions.JIT;
+    }
+
 
     /**
      * Set whether trace hooks are to be included in the compiled code. To use tracing, it is necessary
@@ -411,7 +447,6 @@ public class XsltCompiler {
      * processing instructions, then the returned Source will identify
      * a synthesized stylesheet module that imports all the referenced
      * stylesheet module.
-     * <p/>
      * <p>The returned Source will have an absolute URI, created by resolving
      * any relative URI against the base URI of the supplied source document,
      * and redirected if necessary by using the URIResolver associated with this
@@ -450,11 +485,11 @@ public class XsltCompiler {
 
     /**
      * Compile a library package.
-     * <p/>
      * <p>The source argument identifies an XML file containing an &lt;xsl:package&gt; element. Any packages
      * on which this package depends must have been made available to the <code>XsltCompiler</code>
      * by importing them using {@link #importPackage}.</p>
      *
+     *
      * @param source identifies an XML document holding the the XSLT package to be compiled
      * @return the XsltPackage that results from the compilation. Note that this package
      * is not automatically imported to this <code>XsltCompiler</code>; if the package is required
@@ -463,6 +498,7 @@ public class XsltCompiler {
      *                           compilation. Any such errors will have been notified to the registered <code>ErrorListener</code>
      *                           if there is one, or reported on the <code>System.err</code> output stream otherwise.
      * @since 9.6
+     * @see #compile(Source) - especially the notes regarding error handling and just-in-time compilation.
      */
 
     public XsltPackage compilePackage(Source source) throws SaxonApiException {
@@ -471,12 +507,14 @@ public class XsltCompiler {
             if (source instanceof DocumentImpl && ((DocumentImpl)source).getDocumentElement() instanceof StyleElement) {
                 compilation = ((StyleElement)((DocumentImpl) source).getDocumentElement()).getCompilation();
             } else {
-                compilation = new Compilation(config, compilerInfo);
+                compilation = new Compilation(config, new CompilerInfo(compilerInfo));
             }
             compilation.setLibraryPackage(true);
             XsltPackage pack = new XsltPackage(processor, compilation.compilePackage(source).getStylesheetPackage());
-            if (compilation.getErrorCount() > 0) {
-                throw new SaxonApiException("Package compilation failed: " + compilation.getErrorCount() + " errors reported");
+            int errors = compilation.getErrorCount();
+            if (errors > 0) {
+                String count = errors == 1 ? "one error" : errors + " errors";
+                throw new SaxonApiException("Package compilation failed: " + count + " reported");
             }
             return pack;
         } catch (XPathException e) {
@@ -501,39 +539,29 @@ public class XsltCompiler {
      * with the underlying {@link CompilerInfo}, or may be listed in the configuration file. Alternatively they
      * can be imported explicitly (taking care over the order of importing) using {@link #importPackage(XsltPackage)}.
      */
-    public Iterable<XsltPackage> compilePackages(Iterable<Source> sources) throws SaxonApiException {
-        throw new UnsupportedOperationException("XsltCompiler#compilePackages() is dropped in Saxon 9.8");
+    public Iterable<XsltPackage> compilePackages(Iterable<Source> sources) {
+        throw new UnsupportedOperationException("XsltCompiler#compilePackages() was dropped in Saxon 9.8");
     }
 
     /**
      * Add new packages to a package library.
-     * It assumes that a library (possibly empty) already exists in the compilerInfo and adds to anything there.
-     * In particular stylesheets can exploit packages that are already compiled
-     * and they can be linked during this process
-     *
      * @param sources Sources for the packages
      * @param link    Link each of the packages when loaded if true
      * @return the collection of compiled packages, in the form of an Iterable
      * ..... note that unless link = true these will probably require link() processing before execution.
-     * @throws SaxonApiException if the source cannot be read or if static errors are found during the
-     *                           compilation. Any such errors will have been notified to the registered <code>ErrorListener</code>
-     *                           if there is one, or reported on the <code>System.err</code> output stream otherwise.
-     * @throws XPathException    if there are XPath errors in the stylesheet
      * @since 9.6
      * @deprecated since 9.8. Multiple packages may be supplied in the form of a {@link PackageLibrary} registered
      * with the underlying {@link CompilerInfo}, or may be listed in the configuration file. Alternatively they
      * can be imported explicitly (taking care over the order of importing) using {@link #importPackage(XsltPackage)}.
      */
-    public Iterable<XsltPackage> addCompilePackages(Iterable<Source> sources, boolean link) throws SaxonApiException, XPathException {
+    public Iterable<XsltPackage> addCompilePackages(Iterable<Source> sources, boolean link) {
         throw new UnsupportedOperationException("XsltCompiler.addCompilePackages is dropped from Saxon 9.8");
     }
 
     /**
      * Load a compiled package from a file or from a remote location.
-     * <p/>
      * <p>The supplied URI represents the location of a resource which must have been originally
      * created using {@link XsltPackage#save(java.io.File)}.</p>
-     * <p/>
      * <p>The result of loading the package is returned as an <code>XsltPackage</code> object.
      * Note that this package is not automatically imported to this <code>XsltCompiler</code>;
      * if the package is required for use in subsequent compilations then it must be explicitly
@@ -565,12 +593,9 @@ public class XsltCompiler {
     /**
      * Load a compiled package from a file or from a remote location, with the intent to use this as a complete
      * executable stylesheet, not as a library package.
-     * <p/>
      * <p>The supplied URI represents the location of a resource which must have been originally
-     * created using {@link XsltPackage#save(java.io.File)}.</p>
-     * <p/>
-     * <p>The result of loading the package is returned as an <code>XsltExecutable</code> object.
-     * </p>
+     * created using {@link XsltPackage#save(java.io.File)} or some equivalent.</p>
+     * <p>The result of loading the package is returned as an <code>XsltExecutable</code> object.</p>
      *
      * @param location the location from which the package is to be loaded, as a URI
      * @return the compiled package loaded from the supplied file or remote location
@@ -587,7 +612,7 @@ public class XsltCompiler {
 
     /**
      * Import a library package. Calling this method makes the supplied package available for reference
-     * in the <code>xsl:use-package</code> declaration of subsequent compilations performed using this
+     * in the <code>xsl:use-package</code> declarations of subsequent compilations performed using this
      * <code>XsltCompiler</code>.
      *
      * @param thePackage the package to be imported
@@ -645,8 +670,8 @@ public class XsltCompiler {
 
     /**
      * Import a named package, together with all the packages on which it depends, recursively.
-     * The package must be identified in the package library for this XsltCompiler, which defaults
-     * to the package library defined in the Configuration, typically set up by loading a configuration
+     * The package must be identified in the package library for this {@code XsltCompiler}, which defaults
+     * to the package library defined in the {@link Configuration}, typically set up by loading a configuration
      * file.
      * @param packageName the name of the required package. This is the name under which it is registered
      *                    in the package library, which is not necessarily the same as the name appearing
@@ -693,26 +718,22 @@ public class XsltCompiler {
             throw new SaxonApiException("No package with alias " + alias + " found in package library");
         }
         try {
-            StylesheetPackage pack = getPackageLibrary().obtainLoadedPackage(details, new ArrayList<VersionedPackageName>());
+            StylesheetPackage pack = getPackageLibrary().obtainLoadedPackage(details, new ArrayList<>());
             return new XsltPackage(processor, pack);
         } catch (XPathException e) {
             throw new SaxonApiException(e);
         }
     }
 
-
     /**
      * Compile a stylesheet.
-     * <p/>
      * <p><i>Note: the term "compile" here indicates that the stylesheet is converted into an executable
      * form. There is no implication that this involves code generation.</i></p>
-     * <p/>
      * <p>The source argument identifies the XML document holding the principal stylesheet module. Other
-     * modules will be located relative to this module by resolving against the base URI that is defined
-     * as the systemId property of the supplied Source.</p>
-     * <p/>
+     * modules will be located relative to this module by resolving relative URIs found in <code>&lt;xsl:include&gt;</code>
+     * and <code>&lt;xsl:import&gt;</code> declarations against the base URI that is defined
+     * as the {@code systemId} property of the supplied {@code Source}.</p>
      * <p>The following kinds of {@link javax.xml.transform.Source} are recognized:</p>
-     * <p/>
      * <ul>
      * <li>{@link javax.xml.transform.stream.StreamSource}, allowing the stylesheet to be supplied as a
      * URI, as a {@link java.io.File}, as an {@link java.io.InputStream}, or as a {@link java.io.Reader}</li>
@@ -720,23 +741,41 @@ public class XsltCompiler {
      * of SAX events from a SAX2-compliant XML parser (or any other source of SAX events)</li>
      * <li>{@link javax.xml.transform.dom.DOMSource}, allowing the stylesheet to be supplied as a
      * DOM tree</li>
-     * <li>Document wrappers for XOM, JDOM, or DOM4J trees</li>
-     * <li>A Saxon NodeInfo, representing the root of a tree in any of the native tree formats supported
+     * <li>Document wrappers for XOM, JDOM, DOM4J, or AXIOM trees</li>
+     * <li>A Saxon {@link NodeInfo}, representing the root of a tree in any of the native tree formats supported
      * by Saxon</li>
-     * <li>An {@link XdmNode} representing the document node of the stylesheet module</li>
      * </ul>
+     * <p>The compilation uses a snapshot of the properties of the <code>XsltCompiler</code> at the
+     * time this method is invoked. This is relevant when JIT compilation is enabled (which is the default
+     * under Saxon-EE). Under JIT compilation, templates are compiled when first executed, which will typically be after
+     * the <code>compile()</code> method has returned. Subsequent changes to the properties of the <code>XsltCompiler</code>,
+     * for example, setting a new <code>ErrorListener</code>, have no effect on the delayed compilation of
+     * template rules at execution time.</p>
+     * <p>If JIT compilation is enabled (this is the default for Saxon-EE), then static errors in template
+     * rules might be detected not during execution of the <code>compile()</code> method, but rather
+     * when the relevant code is first executed. In this situation the <code>compile()</code> method will
+     * not throw an exception, but the errors will still (eventually) be notified to the <code>ErrorListener</code>
+     * or <code>ErrorList</code> associated with the compiler: more specifically, the <code>ErrorListener</code>
+     * or <code>ErrorList</code> that was associated with the <code>XsltCompiler</code> at the time
+     * <code>compile()</code> was invoked.</p>
      *
-     * @param source Source object representing the principal stylesheet module to be compiled
-     * @return an XsltExecutable, which represents the compiled stylesheet.
+     * @param source Source object representing the principal stylesheet module to be compiled.
+     *               Must not be null. If the {@code Source} wraps a resource such as a {@link java.io.Reader}
+     *               or {@link InputStream} then the resource will be consumed by the method.
+     * @return an {@code XsltExecutable}, which represents the compiled stylesheet. The {@code XsltExecutable}
+     * is immutable and thread-safe; it may be used to run multiple transformations, in series or concurrently.
      * @throws SaxonApiException if the stylesheet contains static errors or if it cannot be read. Note that
-     *                           the exception that is thrown will <b>not</b> contain details of the actual errors found in the stylesheet. These
-     *                           will instead be notified to the registered ErrorListener. The default ErrorListener displays error messages
-     *                           on the standard error output.
+     * the exception that is thrown will <b>not</b> contain details of the actual errors found in the stylesheet. These
+     * will instead be notified to the registered {@code ErrorListener} or {@code ErrorList}.
+     * The default {@code ErrorListener} displays error messages on the standard error output.
+     *
      */
 
-    public XsltExecutable compile(/*@NotNull*/ Source source) throws SaxonApiException {
+    public XsltExecutable compile(Source source) throws SaxonApiException {
+        Objects.requireNonNull(source);
         try {
-            PreparedStylesheet pss = Compilation.compileSingletonPackage(config, compilerInfo, source);
+            CompilerInfo ci2 = new CompilerInfo(compilerInfo);
+            PreparedStylesheet pss = Compilation.compileSingletonPackage(config, ci2, source);
             return new XsltExecutable(processor, pss);
         } catch (XPathException e) {
             throw new SaxonApiException(e);
@@ -744,11 +783,11 @@ public class XsltCompiler {
     }
 
     /**
-     * Get the underlying CompilerInfo object, which provides more detailed (but less stable) control
+     * Get the underlying {@link CompilerInfo} object, which provides more detailed (but less stable) control
      * over some compilation options
      *
-     * @return the underlying CompilerInfo object, which holds compilation-time options. The methods on
-     * the CompilerInfo object are not guaranteed stable from release to release.
+     * @return the underlying {@code CompilerInfo} object, which holds compilation-time options. The methods on
+     * the {@code CompilerInfo} object are not guaranteed stable from release to release.
      */
 
     public CompilerInfo getUnderlyingCompilerInfo() {
@@ -756,8 +795,10 @@ public class XsltCompiler {
     }
 
     /**
-     * Supply a List object which will be populated with information about any static errors
+     * Supply a (typically empty) {@code List} which will be populated with information about any static errors
      * encountered during the transformation.
+     * <p>Calling this method overwrites the effect of any previous call on {@link #setErrorListener(ErrorListener)}
+     * as well as previous calls on {@code setErrorList}.</p>
      *
      * @param errorList a List (typically empty) to which information will be appended about
      *                  static errors found during the compilation. Each such error is represented by a
@@ -774,7 +815,16 @@ public class XsltCompiler {
      *            static analysis of a template rule is deferred until the first time that the
      *            template is matched. This can improve performance when many template
      *            rules are rarely used during the course of a particular transformation; however,
-     *            it means that static errors in the stylesheet may go undetected.
+     *            it means that static errors in the stylesheet will not necessarily cause the
+     *            {@link #compile(Source)} method to throw an exception (errors in code that is
+     *            actually executed will still be notified to the registered <code>ErrorListener</code>
+     *            or <code>ErrorList</code>, but this may happen after the {@link #compile(Source)}
+     *            method returns). This option is enabled by default in Saxon-EE, and is not available
+     *            in Saxon-HE or Saxon-PE.
+     *            <p><b>Recommendation:</b> disable this option unless you are confident that the
+     *            stylesheet you are compiling is error-free.</p>
+     * @throws UnsupportedOperationException if the argument is set to <code>true</code> and the
+     * configuration is not a licensed Saxon-EE configuration.
      */
 
     public void setJustInTimeCompilation(boolean jit){
