@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -123,7 +123,8 @@ public class XsltPackage {
      *
      * @param file the file to which the compiled package should be saved
      * @throws SaxonApiException if the compiled package cannot be saved to the specified
-     *                           location.
+     *                           location, or if the package was compiled with just-in-time
+     *                           compilation enabled
      * @since 9.7
      */
 
@@ -142,8 +143,9 @@ public class XsltPackage {
      * @param target the target environment. The only values currently recognized are "JS" and "JS2,
      *               which export the package for running under Saxon-JS 1.0 or 2.0 respectively.
      * @throws SaxonApiException if the compiled package cannot be saved to the specified
-     *                           location.
-     * @since 9.7.0.5 (experimental and subject to change)
+     *                           location, or if the package was compiled with just-in-time
+     *                           compilation enabled.
+     * @since 9.7.0.5
      */
 
     public void save(File file, String target) throws SaxonApiException {
@@ -156,14 +158,10 @@ public class XsltPackage {
                 file.createNewFile();
             }
             ExpressionPresenter presenter = getProcessor().getUnderlyingConfiguration()
-                    .newExpressionExporter(target, new FileOutputStream(file));
+                    .newExpressionExporter(target, new FileOutputStream(file), stylesheetPackage);
             presenter.setRelocatable(stylesheetPackage.isRelocatable());
             stylesheetPackage.export(presenter);
-        } catch (XPathException e) {
-            throw new SaxonApiException(e);
-        } catch (FileNotFoundException e) {
-            throw new SaxonApiException(e);
-        } catch (IOException e) {
+        } catch (XPathException | IOException e) {
             throw new SaxonApiException(e);
         }
     }
