@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -20,8 +20,6 @@ import java.util.Stack;
 
 /**
  * This class generates HTML output
- *
- * @author Michael H. Kay
  */
 
 public abstract class HTMLEmitter extends XMLEmitter {
@@ -43,7 +41,7 @@ public abstract class HTMLEmitter extends XMLEmitter {
     private String parentElement;
     private String uri;
     private boolean escapeNonAscii = false;
-    private Stack<NodeName> nodeNameStack = new Stack<NodeName>();
+    private Stack<NodeName> nodeNameStack = new Stack<>();
 
     /**
      * Decode preferred representation
@@ -53,11 +51,19 @@ public abstract class HTMLEmitter extends XMLEmitter {
      */
 
     private static int representationCode(String rep) {
-        if (rep.equalsIgnoreCase("native")) return REP_NATIVE;
-        if (rep.equalsIgnoreCase("entity")) return REP_ENTITY;
-        if (rep.equalsIgnoreCase("decimal")) return REP_DECIMAL;
-        if (rep.equalsIgnoreCase("hex")) return REP_HEX;
-        return REP_ENTITY;
+        rep = rep.toLowerCase();
+        switch (rep) {
+            case "native":
+                return REP_NATIVE;
+            case "entity":
+                return REP_ENTITY;
+            case "decimal":
+                return REP_DECIMAL;
+            case "hex":
+                return REP_HEX;
+            default:
+                return REP_ENTITY;
+        }
     }
 
     /**
