@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -159,12 +159,15 @@ public class MetaTagAdjuster extends ProxyReceiver {
      * Notify an attribute. Attributes are notified after the startElement event, and before any
      * children. Namespaces and attributes may be intermingled.
      *
-     * @param nameCode   The name of the attribute, as held in the name pool
-     * @param typeCode   The type of the attribute, as held in the name pool
-     * @param locationId
-     *@param properties Bit significant value. The following bits are defined:
+     * @param nameCode   The name of the attribute
+     * @param typeCode   The type of the attribute
+     * @param locationId the location of the node in the source, or of the instruction that created it
+     * @param properties Bit significant value. The following bits are defined:
+     *                   <dl>
      *                   <dd>DISABLE_ESCAPING</dd>    <dt>Disable escaping for this attribute</dt>
-     *                   <dd>NO_SPECIAL_CHARACTERS</dd>      <dt>Attribute value contains no special characters</dt>  @throws IllegalStateException: attempt to output an attribute when there is no open element
+     *                   <dd>NO_SPECIAL_CHARACTERS</dd>      <dt>Attribute value contains no special characters</dt>
+     *                   </dl>
+     * @throws IllegalStateException: attempt to output an attribute when there is no open element
      *                                start tag
      */
 
