@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -40,6 +40,7 @@ public class XMLEmitter extends Emitter {
     // into a single loop. Scrapped it because we couldn't measure any benefits - but there
     // ought to be, in theory. Perhaps we weren't buffering the writes carefully enough.
 
+    protected boolean canonical = false;
     protected boolean started = false;
     protected boolean startedElement = false;
     protected boolean openStartTag = false;
@@ -55,7 +56,7 @@ public class XMLEmitter extends Emitter {
     // have been started but not finished. It is used to obtain the element name
     // for the end tag.
 
-    protected Stack<String> elementStack = new Stack<String>();
+    protected Stack<String> elementStack = new Stack<>();
 
     // For other names we use a hashtable. It
 
@@ -176,11 +177,10 @@ public class XMLEmitter extends Emitter {
             outputProperties = new Properties();
         }
 
-        String rep = outputProperties.getProperty(SaxonOutputKeys.UNDECLARE_PREFIXES);
-        if (rep != null) {
-            undeclareNamespaces = (rep.equalsIgnoreCase("yes"));
-        }
+        undeclareNamespaces = "yes".equals(outputProperties.getProperty(SaxonOutputKeys.UNDECLARE_PREFIXES));
+        canonical = "yes".equals(outputProperties.getProperty(SaxonOutputKeys.CANONICAL));
         unfailing = "yes".equals(outputProperties.getProperty(SaxonOutputKeys.UNFAILING));
+
         if ("yes".equals(outputProperties.getProperty(SaxonOutputKeys.SINGLE_QUOTES))) {
             delimiter = '\'';
             attSpecials = specialInAttSingle;
@@ -202,11 +202,11 @@ public class XMLEmitter extends Emitter {
 
             String byteOrderMark = outputProperties.getProperty(SaxonOutputKeys.BYTE_ORDER_MARK);
             String encoding = outputProperties.getProperty(OutputKeys.ENCODING);
-            if (encoding == null || encoding.equalsIgnoreCase("utf8")) {
+            if (encoding == null || encoding.equalsIgnoreCase("utf8") || canonical) {
                 encoding = "UTF-8";
             }
 
-            if ("yes".equals(byteOrderMark) && (
+            if ("yes".equals(byteOrderMark) && !canonical && (
                     "UTF-8".equalsIgnoreCase(encoding) ||
                             "UTF-16LE".equalsIgnoreCase(encoding) ||
                             "UTF-16BE".equalsIgnoreCase(encoding))) {
@@ -218,6 +218,10 @@ public class XMLEmitter extends Emitter {
                 omitXMLDeclaration = "no";
             }
 
+            if (canonical) {
+                omitXMLDeclaration = "yes";
+            }
+
             String version = outputProperties.getProperty(OutputKeys.VERSION);
             if (version == null) {
                 version = getConfiguration().getXMLVersion() == Configuration.XML10 ? "1.0" : "1.1";
@@ -298,25 +302,27 @@ public class XMLEmitter extends Emitter {
 
     protected void writeDocType(NodeName name, String displayName, String systemId, String publicId) throws XPathException {
         try {
-            if (declarationIsWritten && !indenting) {
-                // don't add a newline if indenting, because the indenter will already have done so
-                writer.write("\n");
-            }
-            writer.write("<!DOCTYPE " + displayName + '\n');
-            String quotedSystemId = null;
-            if (systemId != null) {
-                if (systemId.contains("\"")) {
-                    quotedSystemId = "'" + systemId + "'";
+            if (!canonical) {
+                if (declarationIsWritten && !indenting) {
+                    // don't add a newline if indenting, because the indenter will already have done so
+                    writer.write("\n");
+                }
+                writer.write("<!DOCTYPE " + displayName + '\n');
+                String quotedSystemId = null;
+                if (systemId != null) {
+                    if (systemId.contains("\"")) {
+                        quotedSystemId = "'" + systemId + "'";
+                    } else {
+                        quotedSystemId = '"' + systemId + '"';
+                    }
+                }
+                if (systemId != null && publicId == null) {
+                    writer.write("  SYSTEM " + quotedSystemId + ">\n");
+                } else if (systemId == null && publicId != null) {     // handles the HTML case
+                    writer.write("  PUBLIC \"" + publicId + "\">\n");
                 } else {
-                    quotedSystemId = '"' + systemId + '"';
+                    writer.write("  PUBLIC \"" + publicId + "\" " + quotedSystemId + ">\n");
                 }
-            }
-            if (systemId != null && publicId == null) {
-                writer.write("  SYSTEM " + quotedSystemId + ">\n");
-            } else if (systemId == null && publicId != null) {     // handles the HTML case
-                writer.write("  PUBLIC \"" + publicId + "\">\n");
-            } else {
-                writer.write("  PUBLIC \"" + publicId + "\" " + quotedSystemId + ">\n");
             }
         } catch (java.io.IOException err) {
             throw new XPathException("Failure writing to " + getSystemId(), err);
@@ -387,7 +393,7 @@ public class XMLEmitter extends Emitter {
                     requireWellFormed = true;
                     writeDocType(elemName, displayName, systemId, publicId);
                 } else if (writeDocTypeWithNullSystemId()) {
-                    writeDocType(elemName, displayName, systemId, publicId);
+                    writeDocType(elemName, displayName, null, publicId);
                 }
                 started = true;
             }
@@ -430,7 +436,7 @@ public class XMLEmitter extends Emitter {
                         err.setErrorCode("SERE0008");
                         throw err;
                     }
-                    if (undeclareNamespaces || nsuri.length() != 0) {
+                    if (undeclareNamespaces || !nsuri.isEmpty()) {
                         writer.write(sep);
                         writeAttribute(elementCode, "xmlns:" + nsprefix, nsuri, 0);
                     }
@@ -485,6 +491,7 @@ public class XMLEmitter extends Emitter {
         } else {
             int indent = indentForNextAttribute;
             while (indent >= indentChars.length()) {
+                //noinspection StringConcatenationInLoop
                 indentChars += "                     ";
             }
             return indentChars.substring(0, indent);
@@ -522,7 +529,7 @@ public class XMLEmitter extends Emitter {
      */
 
     protected String emptyElementTagCloser(String displayName, NodeName nameCode) {
-        return "/>";
+        return canonical ? "></" + displayName + ">" : "/>";
     }
 
     /**
@@ -574,7 +581,7 @@ public class XMLEmitter extends Emitter {
      * first offending character if not
      */
 
-    protected int testCharacters(CharSequence chars) throws XPathException {
+    protected int testCharacters(CharSequence chars) {
         for (int i = 0; i < chars.length(); i++) {
             char c = chars.charAt(i);
             if (c > 127) {
@@ -613,7 +620,7 @@ public class XMLEmitter extends Emitter {
      */
 
     public void endElement() throws XPathException {
-        String displayName = (String) elementStack.pop();
+        String displayName = elementStack.pop();
         try {
             if (openStartTag) {
                 writer.write(emptyElementTagCloser(displayName, elementCode));
@@ -771,7 +778,7 @@ public class XMLEmitter extends Emitter {
 
     /**
      * Write contents of array to current writer, after escaping special characters.
-     * This method converts the XML special characters (such as < and &) into their
+     * This method converts the XML special characters (such as &lt; and &amp;) into their
      * predefined entities.
      *
      * @param chars       The character sequence containing the string
@@ -793,9 +800,8 @@ public class XMLEmitter extends Emitter {
         while (segstart < clength) {
             int i = segstart;
             // find a maximal sequence of "ordinary" characters
-            char c = 0;
             while (i < clength) {
-                c = chars.charAt(i);
+                final char c = chars.charAt(i);
                 if (c < 127) {
                     if (specialChars[c]) {
                         break;
@@ -831,7 +837,7 @@ public class XMLEmitter extends Emitter {
             }
 
             // examine the special character that interrupted the scan
-            //final char c = chars.charAt(i);
+            final char c = chars.charAt(i);
             if (c == 0) {
                 // used to switch escaping on and off
                 disabled = !disabled;
