@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -16,6 +16,7 @@ import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.lib.Validation;
 import net.sf.saxon.om.*;
 import net.sf.saxon.trace.LocationKind;
+import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.linked.DocumentImpl;
 import net.sf.saxon.tree.linked.LinkedTreeBuilder;
@@ -124,7 +125,8 @@ public class LiteralResultElement extends StyleElement {
                     } else if (forwardsCompatibleModeIsEnabled()) {
                         // the attribute is ignored
                     } else {
-                        compileError("Unknown XSLT attribute " + atts.getNodeName(i).getDisplayName(), "XTSE0805");
+                        compileError("Unknown XSLT attribute " +
+                                             Err.wrap(atts.getNodeName(i).getDisplayName(), Err.ATTRIBUTE), "XTSE0805");
                     }
                 } else {
                     attributeNames[numberOfAttributes] = new FingerprintedQName(atts.getPrefix(i), atts.getURI(i), atts.getLocalName(i), fp);
@@ -317,7 +319,7 @@ public class LiteralResultElement extends StyleElement {
                 att.setSelect(attributeValues[i]);
                 att.setLocation(allocateLocation());
                 Expression exp = att;
-                if (getConfiguration().isCompileWithTracing()) {
+                if (exec.getCompilerInfo().isCompileWithTracing()) {
                     TraceExpression trace = new TraceExpression(exp);
                     trace.setNamespaceResolver(getNamespaceResolver());
                     trace.setConstructType(LocationKind.LITERAL_RESULT_ATTRIBUTE);
@@ -350,6 +352,7 @@ public class LiteralResultElement extends StyleElement {
         }
         inst.setContentExpression(content);
         inst.setRetainedStaticContext(makeRetainedStaticContext());
+        inst.setLocation(allocateLocation());
         return inst;
     }
 
