@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -41,21 +41,12 @@ import java.util.*;
  * Represents both the stylesheet module at the root of the import tree, that is, the module
  * that includes or imports all the others, and also the XSLT package that has this stylesheet
  * module as its root.
- * <p/>
- * This version of the StylesheetPackage class represents a trivial package that is constructed
+ * <p>This version of the StylesheetPackage class represents a trivial package that is constructed
  * by the system to wrap an ordinary (non-package) stylesheet, as available in XSLT 2.0. A subclass
- * is used for "real" packages, currently available only in Saxon-EE.
+ * is used for "real" packages, currently available only in Saxon-EE.</p>
  */
 public class PrincipalStylesheetModule extends StylesheetModule implements GlobalVariableManager {
 
-    /**
-     * A class that simply encapsulates a callback action of any kind
-     */
-
-    public static abstract class Action {
-        public abstract void doAction() throws XPathException;
-    }
-
     private StylesheetPackage stylesheetPackage;
     private boolean declaredModes;
 
@@ -69,15 +60,14 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
     // index of global variables and parameters, by StructuredQName
     // (overridden variables are excluded).
     private HashMap<StructuredQName, ComponentDeclaration> globalVariableIndex =
-            new HashMap<StructuredQName, ComponentDeclaration>(20);
+            new HashMap<>(20);
 
 
     // index of templates - only includes those actually declared within this package
-    private HashMap<StructuredQName, ComponentDeclaration> templateIndex = new HashMap<StructuredQName, ComponentDeclaration>(20);
+    private HashMap<StructuredQName, ComponentDeclaration> templateIndex = new HashMap<>(20);
 
     // Table of named stylesheet functions.
-    private HashMap<SymbolicName, ComponentDeclaration> functionIndex =
-            new HashMap<SymbolicName, ComponentDeclaration>(8);
+    private HashMap<SymbolicName, ComponentDeclaration> functionIndex = new HashMap<>(8);
 
     // key manager for all the xsl:key definitions in this package
     private KeyManager keyManager;
@@ -93,26 +83,24 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
 
     // namespace aliases. This information is needed at compile-time only
     private int numberOfAliases = 0;
-    private List<ComponentDeclaration> namespaceAliasList = new ArrayList<ComponentDeclaration>(5);
+    private List<ComponentDeclaration> namespaceAliasList = new ArrayList<>(5);
     private HashMap<String, NamespaceBinding> namespaceAliasMap;
     private Set<String> aliasResultUriSet;
 
     // attribute sets. A package can contain several declarations attribute sets with the same name.
     // They are indexed in the order they will be applied, that is, highest precedence first
-    private Map<StructuredQName, List<ComponentDeclaration>> attributeSetDeclarations = new HashMap<StructuredQName, List<ComponentDeclaration>>();
+    private Map<StructuredQName, List<ComponentDeclaration>> attributeSetDeclarations = new HashMap<>();
 
     // cache of stylesheet documents. Note that multiple imports of the same URI
     // lead to the stylesheet tree being reused
-    private HashMap<DocumentURI, XSLModuleRoot> moduleCache = new HashMap<DocumentURI, XSLModuleRoot>(4);
+    private HashMap<DocumentURI, XSLModuleRoot> moduleCache = new HashMap<>(4);
 
     private TypeAliasManager typeAliasManager;
 
 
     private CharacterMapIndex characterMapIndex;
 
-    private List<Action> fixupActions = new ArrayList<Action>();
-    private List<Action> completionActions = new ArrayList<Action>();
-    //private boolean createsSecondaryResultDocuments = false;
+    private List<Action> fixupActions = new ArrayList<>();
     private boolean needsDynamicOutputProperties = false;
 
 
@@ -127,9 +115,9 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
         declaredModes = sourceElement.isDeclaredModes();
         stylesheetPackage = getConfiguration().makeStylesheetPackage();
         CompilerInfo compilerInfo = sourceElement.getCompilation().getCompilerInfo();
-        stylesheetPackage.setJustInTimeCompilation(compilerInfo.isJustInTimeCompilation());
         stylesheetPackage.setTargetEdition(compilerInfo.getTargetEdition());
         stylesheetPackage.setRelocatable(compilerInfo.isRelocatable());
+        stylesheetPackage.setJustInTimeCompilation(compilerInfo.isJustInTimeCompilation());
 
         keyManager = stylesheetPackage.getKeyManager();
         decimalFormatManager = stylesheetPackage.getDecimalFormatManager();
@@ -137,12 +125,13 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
         ruleManager = new RuleManager(stylesheetPackage, compilerInfo);
         ruleManager.getUnnamedMode().makeDeclaringComponent(Visibility.PRIVATE, stylesheetPackage);
         stylesheetPackage.setRuleManager(ruleManager);
+        stylesheetPackage.setDeclaredModes(declaredModes);
         StructuredQName defaultMode = sourceElement.getDefaultMode();
         stylesheetPackage.setDefaultMode(sourceElement.getDefaultMode());
         if (defaultMode != null) {
-            ruleManager.obtainMode(defaultMode, true);
+            ruleManager.obtainMode(defaultMode, !declaredModes);
         }
-        stylesheetPackage.setDeclaredModes(declaredModes);
+
 
         characterMapIndex = new CharacterMapIndex();
         stylesheetPackage.setCharacterMapIndex(characterMapIndex);
@@ -242,17 +231,6 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
         fixupActions.add(action);
     }
 
-    /**
-     * Register a callback action to be performed during the completion phase
-     *
-     * @param action the callback action
-     */
-
-    public void addCompletionAction(Action action) {
-        completionActions.add(action);
-    }
-
-
     /**
      * Say that this stylesheet package needs dynamic output properties
      *
@@ -404,9 +382,6 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
             StyleElement inst = decl.getSourceElement();
             if (!inst.isActionCompleted(StyleElement.ACTION_FIXUP)) {
                 inst.setActionCompleted(StyleElement.ACTION_FIXUP);
-//                if (inst instanceof XSLVariableDeclaration) {
-//                    System.err.println("Fixup global variable " + ((XSLVariableDeclaration)inst).getVariableQName());
-//                }
                 inst.fixupReferences();
             }
         }
@@ -423,8 +398,6 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
         ComponentDeclaration decl = new ComponentDeclaration(this, top);
         if (!top.isActionCompleted(StyleElement.ACTION_VALIDATE)) {
             top.setActionCompleted(StyleElement.ACTION_VALIDATE);
-            //top.validate(decl);
-            getRootElement().validate(decl);
             top.validate(null);
             for (ComponentDeclaration d : topLevel) {
                 d.getSourceElement().validateSubtree(d, false);
@@ -444,7 +417,7 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
 
         // Handle named output formats for use at run-time
 
-        HashSet<StructuredQName> outputNames = new HashSet<StructuredQName>(5);
+        HashSet<StructuredQName> outputNames = new HashSet<>(5);
         for (ComponentDeclaration outputDecl : topLevel) {
             if (outputDecl.getSourceElement() instanceof XSLOutput) {
                 XSLOutput out = (XSLOutput) outputDecl.getSourceElement();
@@ -476,7 +449,7 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
             if (inst instanceof XSLCharacterMap) {
                 XSLCharacterMap xcm = (XSLCharacterMap) inst;
                 StructuredQName qn = xcm.getCharacterMapName();
-                IntHashMap<String> map = new IntHashMap<String>();
+                IntHashMap<String> map = new IntHashMap<>();
                 xcm.assemble(map);
                 characterMapIndex.putCharacterMap(xcm.getCharacterMapName(), new CharacterMap(qn, map));
             }
@@ -504,11 +477,11 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
 //            compilation.reportWarning(w);
 //        }
 
-        List<XSLUsePackage> useDeclarations = new ArrayList<XSLUsePackage>();
+        List<XSLUsePackage> useDeclarations = new ArrayList<>();
         gatherUsePackageDeclarations(compilation, xslpackage, useDeclarations);
 
         // First pass: gather all the named overriding declarations and add them to the topLevel declaration list
-        Set<SymbolicName> overrides = new HashSet<SymbolicName>();
+        Set<SymbolicName> overrides = new HashSet<>();
         for (XSLUsePackage use : useDeclarations) {
             gatherOverridingDeclarations(use, compilation, overrides);
         }
@@ -533,9 +506,7 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
 
 
     private static void gatherUsePackageDeclarations(Compilation compilation, StyleElement wrapper, List<XSLUsePackage> declarations) throws XPathException {
-        AxisIterator iter = wrapper.iterateAxis(AxisInfo.CHILD);
-        NodeInfo use;
-        while ((use = iter.next()) != null) {
+        wrapper.iterateAxis(AxisInfo.CHILD).forEachOrFail(use -> {
             if (use instanceof XSLUsePackage) {
                 declarations.add((XSLUsePackage) use);
             } else if (use instanceof XSLInclude) {
@@ -546,7 +517,7 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
                 StyleElement incWrapper = (StyleElement) ((DocumentImpl) includedTree.getRootNode()).getDocumentElement();
                 gatherUsePackageDeclarations(compilation, incWrapper, declarations);
             }
-        }
+        });
     }
 
     private void gatherOverridingDeclarations(XSLUsePackage use, Compilation compilation, Set<SymbolicName> overrides) throws XPathException {
@@ -620,9 +591,8 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
      * Add a stylesheet function to the index
      *
      * @param decl The declaration wrapping an XSLFunction object
-     * @throws XPathException if errors are found
      */
-    protected void indexFunction(ComponentDeclaration decl) throws XPathException {
+    protected void indexFunction(ComponentDeclaration decl) {
         HashMap<SymbolicName, Component> componentIndex = stylesheetPackage.getComponentIndex();
         XSLFunction sourceFunction = (XSLFunction) decl.getSourceElement();
         UserFunction compiledFunction = sourceFunction.getCompiledFunction();
@@ -755,12 +725,9 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
      * @param decl the declaration of the Template object
      * @throws XPathException if an error occurs
      */
-    protected NamedTemplate indexNamedTemplate(ComponentDeclaration decl) throws XPathException {
+    protected void indexNamedTemplate(ComponentDeclaration decl) throws XPathException {
         HashMap<SymbolicName, Component> componentIndex = stylesheetPackage.getComponentIndex();
         XSLTemplate sourceTemplate = (XSLTemplate) decl.getSourceElement();
-        if (sourceTemplate.getTemplateName() == null) {
-            return null; // Not a named template
-        }
         SymbolicName sName = sourceTemplate.getSymbolicName();
         if (sName != null) {
             // see if there is already a named template with this precedence
@@ -771,9 +738,8 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
                 NamedTemplate compiledTemplate = ((XSLTemplate) decl.getSourceElement()).getCompiledNamedTemplate();
                 Component declaringComponent = compiledTemplate.makeDeclaringComponent(sourceTemplate.getVisibility(), getStylesheetPackage());
                 componentIndex.put(sName, declaringComponent);
-                templateIndex.put(sName.getComponentName(), decl);
                 setLocalParamDetails(sourceTemplate, compiledTemplate);
-                return compiledTemplate;
+                templateIndex.put(sName.getComponentName(), decl);
             } else {
                 if (other.getDeclaringPackage() == getStylesheetPackage()) {
                     // check the precedences
@@ -781,10 +747,11 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
                     ComponentDeclaration otherTemplate = templateIndex.get(sName.getComponentName());
                     int otherPrecedence = otherTemplate.getPrecedence();
                     if (thisPrecedence == otherPrecedence) {
+                        String errorCode = sourceTemplate.getParent() instanceof XSLOverride ? "XTSE3055" : "XTSE0660";
                         sourceTemplate.compileError("Duplicate named template (see line " +
-                                                            otherTemplate.getSourceElement().getLineNumber() + " of " + otherTemplate.getSourceElement().getSystemId() + ')', "XTSE0660");
+                                                            otherTemplate.getSourceElement().getLineNumber() + " of " + otherTemplate.getSourceElement().getSystemId() + ')', errorCode);
                     } else if (thisPrecedence < otherPrecedence) {
-                        return null;
+                        //return;
                     } else {
                         // can't happen, but we'll play safe
                         //other.setRedundantNamedTemplate();
@@ -793,7 +760,6 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
                         componentIndex.put(sName, declaringComponent);
                         templateIndex.put(sName.getComponentName(), decl);
                         setLocalParamDetails(sourceTemplate, compiledTemplate);
-                        return compiledTemplate;
                     }
                 } else if (sourceTemplate.findAncestorElement(StandardNames.XSL_OVERRIDE) != null) {
                     // the new one wins
@@ -801,7 +767,6 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
                     Component declaringComponent = compiledTemplate.makeDeclaringComponent(sourceTemplate.getVisibility(), getStylesheetPackage());
                     componentIndex.put(sName, declaringComponent);
                     templateIndex.put(sName.getComponentName(), decl);
-                    return compiledTemplate;
                 } else {
                     sourceTemplate.compileError("Named template " + sName.getComponentName().getDisplayName() +
                                                         " conflicts with a public named template in package " + other.getDeclaringPackage().getPackageName(), "XTSE3050");
@@ -809,16 +774,15 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
                 }
             }
         }
-        return null;
     }
 
+
     private static void setLocalParamDetails(XSLTemplate source, NamedTemplate nt) throws XPathException {
-        NodeInfo child;
         AxisIterator kids = source.iterateAxis(AxisInfo.CHILD, NodeKindTest.ELEMENT);
-        List<NamedTemplate.LocalParamInfo> details = new ArrayList<NamedTemplate.LocalParamInfo>();
-        while ((child = kids.next()) != null) {
+        List<NamedTemplate.LocalParamInfo> details = new ArrayList<>();
+        kids.forEachOrFail(child -> {
             if (child instanceof XSLLocalParam) {
-                XSLLocalParam lp = (XSLLocalParam)child;
+                XSLLocalParam lp = (XSLLocalParam) child;
                 lp.prepareTemplateSignatureAttributes();
                 NamedTemplate.LocalParamInfo info = new NamedTemplate.LocalParamInfo();
                 info.name = lp.getVariableQName();
@@ -827,10 +791,11 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
                 info.isTunnel = lp.isTunnelParam();
                 details.add(info);
             }
-        }
+        });
         nt.setLocalParamDetails(details);
     }
 
+
     /**
      * Get the named template with a given name
      *
@@ -859,12 +824,12 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
         StructuredQName name = sourceAttributeSet.getAttributeSetName();
         List<ComponentDeclaration> entries = attributeSetDeclarations.get(name);
         if (entries == null) {
-            entries = new ArrayList<ComponentDeclaration>();
+            entries = new ArrayList<>();
             attributeSetDeclarations.put(name, entries);
         } else {
             String thisVis = Whitespace.trim(sourceAttributeSet.getAttributeValue("", "visibility"));
             String firstVis = Whitespace.trim(entries.get(0).getSourceElement().getAttributeValue("", "visibility"));
-            if (!(thisVis == null ? firstVis == null : thisVis.equals(firstVis))) {
+            if (thisVis == null ? firstVis != null : !thisVis.equals(firstVis)) {
                 throw new XPathException("Visibility attributes on attribute-sets sharing the same name must all be the same", "XTSE0010");
             }
         }
@@ -892,7 +857,7 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
 
 
     public void combineAttributeSets(Compilation compilation) throws XPathException {
-        Map<StructuredQName, AttributeSet> index = new HashMap<StructuredQName, AttributeSet>();
+        Map<StructuredQName, AttributeSet> index = new HashMap<>();
         for (Map.Entry<StructuredQName, List<ComponentDeclaration>> entry : attributeSetDeclarations.entrySet()) {
             AttributeSet as = new AttributeSet();
             as.setName(entry.getKey());
@@ -909,7 +874,7 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
             stylesheetPackage.addComponent(declaringComponent);
         }
         for (Map.Entry<StructuredQName, List<ComponentDeclaration>> entry : attributeSetDeclarations.entrySet()) {
-            List<Expression> content = new ArrayList<Expression>();
+            List<Expression> content = new ArrayList<>();
             Visibility vis = null;
             boolean explicitVisibility = false;
             boolean streamable = false;
@@ -918,8 +883,8 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
             // once in the list, we need to remove all but the last occurrence, otherwise
             // the same expression will be present more than once in the tree. This can
             // happen when a stylesheet module is included/imported more than once.
-            List<ComponentDeclaration> entries = new ArrayList<ComponentDeclaration>();
-            Set<XSLAttributeSet> elements = new HashSet<XSLAttributeSet>();
+            List<ComponentDeclaration> entries = new ArrayList<>();
+            Set<XSLAttributeSet> elements = new HashSet<>();
             for (int i = entry.getValue().size() - 1; i >= 0; i--) {
                 ComponentDeclaration decl = entry.getValue().get(i);
                 XSLAttributeSet src = (XSLAttributeSet) decl.getSourceElement();
@@ -971,14 +936,9 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
      *
      * @param name the name of the required attribute set
      * @param list a list to hold the list of XSLAttributeSet elements in the stylesheet tree.
-     * @return true if any declarations were found and added to the list; false if none were found
-     * @throws net.sf.saxon.trans.XPathException if an error occurs
      */
 
-    protected boolean getAttributeSets(StructuredQName name, List<ComponentDeclaration> list)
-            throws XPathException {
-
-        boolean found = false;
+    protected void getAttributeSets(StructuredQName name, List<ComponentDeclaration> list) {
 
         // search for the named attribute set, using all of them if there are several with the
         // same name
@@ -988,35 +948,30 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
                 XSLAttributeSet t = (XSLAttributeSet) decl.getSourceElement();
                 if (t.getAttributeSetName().equals(name)) {
                     list.add(decl);
-                    found = true;
                 }
             }
         }
-        return found;
     }
 
     /**
      * Handle an explicitly-declared mode
      *
      * @param decl the declaration of the Mode object
-     * @throws XPathException if an error occurs
      */
-    public void indexMode(ComponentDeclaration decl) throws XPathException {
+    public void indexMode(ComponentDeclaration decl) {
         XSLMode sourceMode = (XSLMode) decl.getSourceElement();
         StructuredQName modeName = sourceMode.getObjectName();
         if (modeName == null) {
             return; // Not a named mode
         }
         SymbolicName sName = new SymbolicName(StandardNames.XSL_MODE, modeName);
-        if (sName != null) {
-            // see if there is already a named mode with this precedence
-            Mode other = getStylesheetPackage().getRuleManager().obtainMode(modeName, false);
-            if (other != null && other.getDeclaringComponent().getDeclaringPackage() != getStylesheetPackage()) {
-                sourceMode.compileError("Mode " + sName.getComponentName().getDisplayName() +
-                                                " conflicts with a public mode declared in package " +
-                                                other.getDeclaringComponent().getDeclaringPackage().getPackageName(), "XTSE3050");
+        // see if there is already a named mode with this precedence
+        Mode other = getStylesheetPackage().getRuleManager().obtainMode(modeName, false);
+        if (other != null && other.getDeclaringComponent().getDeclaringPackage() != getStylesheetPackage()) {
+            sourceMode.compileError("Mode " + sName.getComponentName().getDisplayName() +
+                                            " conflicts with a public mode declared in package " +
+                                            other.getDeclaringComponent().getDeclaringPackage().getPackageName(), "XTSE3050");
 
-            }
         }
     }
 
@@ -1025,11 +980,9 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
      *
      * @param template the template rule
      * @param mode     the mode
-     * @throws XPathException if the mode is declared in a used package and the template
-     *                        is not declared within the relevant xsl:mode declaration
      */
 
-    public boolean checkAcceptableModeForPackage(XSLTemplate template, Mode mode) throws XPathException {
+    public boolean checkAcceptableModeForPackage(XSLTemplate template, Mode mode) {
         StylesheetPackage templatePack = template.getPackageData();
         if (mode.getDeclaringComponent() == null) {
             return true;
@@ -1136,14 +1089,12 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
 
     /**
      * Collect any namespace aliases
-     *
-     * @throws net.sf.saxon.trans.XPathException if an error occurs
      */
 
-    private void collectNamespaceAliases() throws XPathException {
-        namespaceAliasMap = new HashMap<String, NamespaceBinding>(numberOfAliases);
-        aliasResultUriSet = new HashSet<String>(numberOfAliases);
-        HashSet<String> aliasesAtThisPrecedence = new HashSet<String>();
+    private void collectNamespaceAliases() {
+        namespaceAliasMap = new HashMap<>(numberOfAliases);
+        aliasResultUriSet = new HashSet<>(numberOfAliases);
+        HashSet<String> aliasesAtThisPrecedence = new HashSet<>();
         int currentPrecedence = -1;
         // Note that we are processing the list in reverse stylesheet order,
         // that is, highest precedence first.
@@ -1194,7 +1145,7 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
         boolean found = formatQName == null;
         Configuration config = getConfiguration();
         Properties details = new Properties(config.getDefaultSerializationProperties());
-        HashMap<String, Integer> precedences = new HashMap<String, Integer>(10);
+        HashMap<String, Integer> precedences = new HashMap<>(10);
         for (int i = topLevel.size() - 1; i >= 0; i--) {
             ComponentDeclaration decl = topLevel.get(i);
             if (decl.getSourceElement() instanceof XSLOutput) {
@@ -1237,14 +1188,10 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
             // If any XQuery functions were imported, fix up all function calls
             // registered against these functions.
             XQueryFunctionLibrary queryFunctions = stylesheetPackage.getXQueryFunctionLibrary();
-            try {
-                Iterator qf = queryFunctions.getFunctionDefinitions();
-                while (qf.hasNext()) {
-                    XQueryFunction f = (XQueryFunction) qf.next();
-                    f.fixupReferences();
-                }
-            } catch (XPathException e) {
-                getRootElement().compileError(e);
+            Iterator qf = queryFunctions.getFunctionDefinitions();
+            while (qf.hasNext()) {
+                XQueryFunction f = (XQueryFunction) qf.next();
+                f.fixupReferences();
             }
 
             if (Compilation.TIMING) {
@@ -1418,7 +1365,7 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
             AccumulatorRegistry accMan = getAccumulatorManager();
             if (accMan != null) {
                 for (Accumulator acc : accMan.getAllAccumulators()) {
-                    ((Actor) acc).allocateAllBindingSlots(stylesheetPackage);
+                    acc.allocateAllBindingSlots(stylesheetPackage);
                 }
             }
 
@@ -1451,7 +1398,7 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
     }
 
     private void registerImplicitModes(StyleElement element, RuleManager manager) {
-        if (element instanceof XSLTemplate || element instanceof XSLApplyTemplates) {
+        if (element instanceof XSLApplyTemplates || element instanceof XSLTemplate) {
             String modeAtt = element.getAttributeValue("mode");
             if (modeAtt != null) {
                 String[] tokens = Whitespace.trim(modeAtt).split("[ \t\n\r]+");
@@ -1465,16 +1412,14 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
                             if (existing != null && existing.getDeclaringPackage() != getStylesheetPackage()) {
                                 if (element instanceof XSLTemplate && !(element.getParent() instanceof XSLOverride)) {
                                     element.compileError("A template rule cannot be added to a mode declared in a used package " +
-                                                                  "unless the xsl:template declaration appears within an xsl:override child of the appropriate xsl:use-package element",
-                                                          "XTSE3050");
-                                } 
+                                                                 "unless the xsl:template declaration appears within an xsl:override child of the appropriate xsl:use-package element",
+                                                         "XTSE3050");
+                                }
                             } else {
                                 manager.obtainMode(modeName, true);
                             }
                         } catch (XPathException e) {
                             // skip invalid names
-                        } catch (NamespaceException e) {
-                            // skip invalid names
                         }
                     }
                 }
@@ -1553,7 +1498,7 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
      */
 
     public void adjustExposedVisibility() throws XPathException {
-        List<XSLExpose> exposeDeclarations = new ArrayList<XSLExpose>(); // xsl:expose declarations in reverse order
+        List<XSLExpose> exposeDeclarations = new ArrayList<>(); // xsl:expose declarations in reverse order
         for (ComponentDeclaration decl : topLevel) {
             if (decl.getSourceElement() instanceof XSLExpose) {
                 exposeDeclarations.add(0, (XSLExpose) decl.getSourceElement());
@@ -1645,11 +1590,9 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
      * Report an error with diagnostic information
      *
      * @param error contains information about the error
-     * @throws XPathException unconditionally, after reporting the error to the ErrorListener
      */
 
-    protected void compileError(XPathException error)
-            throws XPathException {
+    protected void compileError(XPathException error) {
         error.setIsStaticError(true);
         getRootElement().compileError(error);
     }
@@ -1662,11 +1605,6 @@ public class PrincipalStylesheetModule extends StylesheetModule implements Globa
     }
 
     protected void complete() throws XPathException {
-        // Perform the completion actions
-        for (Action a : completionActions) {
-            a.doAction();
-        }
-        //adjustExposedVisibility();
         stylesheetPackage.complete();
     }
 
