@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -21,7 +21,6 @@ import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.AttributeLocation;
 import net.sf.saxon.tree.iter.AxisIterator;
 import net.sf.saxon.type.ItemType;
-import net.sf.saxon.type.Type;
 import net.sf.saxon.type.TypeHierarchy;
 import net.sf.saxon.type.UType;
 import net.sf.saxon.value.Cardinality;
@@ -44,10 +43,9 @@ public class SourceBinding {
     private Expression select = null;
     private SequenceType declaredType = null;
     private SequenceType inferredType = null;
-    protected String constantText = null;
     protected SlotManager slotManager = null;  // used only for global variable declarations
     private Visibility visibility;
-    private GroundedValue constantValue = null;
+    private GroundedValue<?> constantValue = null;
 
     private int properties;
 
@@ -79,12 +77,12 @@ public class SourceBinding {
 
         String selectAtt = null;
         String asAtt = null;
+        String extraAsAtt = null;
         String requiredAtt = null;
         String tunnelAtt = null;
         String assignableAtt = null;
         String staticAtt = null;
         String visibilityAtt = null;
-        String extraAsAtt = null;
 
         for (int a = 0; a < atts.getLength(); a++) {
             String f = atts.getQName(a);
@@ -163,13 +161,13 @@ public class SourceBinding {
             }
         }
 
-        declaredType = combineTypeAttributes(asAtt, extraAsAtt);
+        declaredType = combineTypeDeclarations(asAtt, extraAsAtt);
+
 
         if (visibilityAtt != null) {
             if (hasProperty(PARAM)) {
                 sourceElement.compileErrorInAttribute("The visibility attribute is not allowed on xsl:param", "XTSE0020", "visibility");
             } else {
-                sourceElement.check30attribute("visibility");
                 visibility = sourceElement.interpretVisibilityValue(visibilityAtt, "");
             }
             if (!hasProperty(GLOBAL)) {
@@ -186,7 +184,8 @@ public class SourceBinding {
      * This method is called to establish basic signature information for local parameters of a named template,
      * so that this can be checked against xsl:call-template instructions. It is called while a named template
      * is being indexed: see bug 3722.
-     * @throws XPathException
+     *
+     * @throws XPathException if things go wrong
      */
 
     public void prepareTemplateSignatureAttributes() throws XPathException {
@@ -224,11 +223,13 @@ public class SourceBinding {
             name = errorName();
         }
 
-        declaredType = combineTypeAttributes(asAtt, extraAsAtt);
+        declaredType = combineTypeDeclarations(asAtt, extraAsAtt);
     }
 
-    private SequenceType combineTypeAttributes(String asAtt, String extraAsAtt) throws XPathException {
+
+    private SequenceType combineTypeDeclarations(String asAtt, String extraAsAtt) {
         SequenceType declaredType = null;
+
         if (asAtt != null) {
             try {
                 declaredType = sourceElement.makeSequenceType(asAtt);
@@ -258,8 +259,8 @@ public class SourceBinding {
         }
 
         return declaredType;
-    }
 
+    }
 
     /**
      * Get the declaration in the stylesheet
@@ -305,10 +306,6 @@ public class SourceBinding {
             if (nameAttribute != null) {
                 name = sourceElement.makeQName(nameAttribute);
             }
-        } catch (NamespaceException err) {
-            name = errorName();
-            StructuredQName n = StructuredQName.fromClarkName("name");
-            throw new XPathException("Invalid variable name: " + err.getMessage(), "XTSE0020", new AttributeLocation(sourceElement, n));
         } catch (XPathException err) {
             name = errorName();
             StructuredQName n = StructuredQName.fromClarkName("name");
@@ -373,6 +370,7 @@ public class SourceBinding {
                         if (!hasProperty(REQUIRED)) {
                             if (Cardinality.allowsZero(declaredType.getCardinality())) {
                                 select = Literal.makeEmptySequence();
+                                select.setRetainedStaticContext(sourceElement.makeRetainedStaticContext());
                             } else {
                                 // The implicit default value () is not valid for the required type, so
                                 // it is treated as if there is no default
@@ -388,15 +386,6 @@ public class SourceBinding {
                         }
                     }
                 }
-            } else {
-                if (kids.next() == null) {
-                    // there is exactly one child node
-                    if (first.getNodeKind() == Type.TEXT) {
-                        // it is a text node: optimize for this case
-                        constantText = first.getStringValue();
-                    }
-                }
-
             }
         }
         select = sourceElement.typeCheck("select", select);
@@ -411,11 +400,9 @@ public class SourceBinding {
      *
      * @param required The type required by the variable declaration, or in the case
      *                 of xsl:with-param, the signature of the called template
-     * @throws net.sf.saxon.trans.XPathException if an error is detected
      */
 
-    public void checkAgainstRequiredType(SequenceType required)
-            throws XPathException {
+    public void checkAgainstRequiredType(SequenceType required) {
         if (visibility != Visibility.ABSTRACT) {
             try {
                 if (required != null) {
@@ -531,8 +518,8 @@ public class SourceBinding {
                 }
                 boolean textonly = UType.TEXT.subsumes(b.getItemType().getUType());
                 String constant = null;  // bug 3748
-                if (textonly && b instanceof ValueOf && ((ValueOf)b).getSelect() instanceof StringLiteral) {
-                    constant = ((StringLiteral)((ValueOf) b).getSelect()).getStringValue();
+                if (textonly && b instanceof ValueOf && ((ValueOf) b).getSelect() instanceof StringLiteral) {
+                    constant = ((StringLiteral) ((ValueOf) b).getSelect()).getStringValue();
                 }
                 DocumentInstr doc = new DocumentInstr(textonly, constant);
                 doc.setContentExpression(b);
@@ -666,8 +653,8 @@ public class SourceBinding {
     }
 
 
-    public GroundedValue getConstantValue() {
-        if(constantValue == null) {
+    public GroundedValue<?> getConstantValue() {
+        if (constantValue == null) {
             final SequenceType type = getInferredType(true);
             final TypeHierarchy th = sourceElement.getConfiguration().getTypeHierarchy();
             if (!hasProperty(ASSIGNABLE) && !hasProperty(PARAM) && !(visibility == Visibility.PUBLIC || visibility == Visibility.ABSTRACT)) {
@@ -689,10 +676,9 @@ public class SourceBinding {
      * Notify all references to this variable of the data type
      *
      * @param compiledGlobalVariable null if this is a local variable; otherwise, the compiled global variable
-     * @throws XPathException if the declaration is invalid
      */
 
-    public void fixupReferences(GlobalVariable compiledGlobalVariable) throws XPathException {
+    public void fixupReferences(GlobalVariable compiledGlobalVariable) {
         final SequenceType type = getInferredType(true);
         final TypeHierarchy th = sourceElement.getConfiguration().getTypeHierarchy();
         GroundedValue constantValue = null;
