@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,6 +13,7 @@ import net.sf.saxon.expr.accum.AccumulatorRegistry;
 import net.sf.saxon.expr.parser.Location;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.om.*;
+import net.sf.saxon.trans.Err;
 import net.sf.saxon.trans.SaxonErrorCode;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.linked.ElementImpl;
@@ -135,35 +136,12 @@ public class StyleNodeFactory implements NodeFactory {
             e.setNamespaceDeclarations(namespaces, namespacesUsed);
             e.initialise(elemName, elemType, attlist, parent, sequence);
             e.setLocation(baseURI, lineNumber, columnNumber);
-            // We're not catching multiple errors in the following attributes, but catching each of the
-            // exceptions helps to ensure we don't report spurious errors through not processing some
-            // of the attributes when others are faulty.
-            try {
                 e.processExtensionElementAttribute("");
-            } catch (TransformerException err) {
-                e.setValidationError(err, StyleElement.REPORT_ALWAYS);
-            }
-            try {
                 e.processExcludedNamespaces("");
-            } catch (TransformerException err) {
-                e.setValidationError(err, StyleElement.REPORT_ALWAYS);
-            }
-            try {
                 e.processVersionAttribute("");
-            } catch (TransformerException err) {
-                e.setValidationError(err, StyleElement.REPORT_ALWAYS);
-            }
             e.processDefaultXPathNamespaceAttribute("");
-            try {
                 e.processExpandTextAttribute("");
-            } catch (XPathException err) {
-                e.setValidationError(err, StyleElement.REPORT_ALWAYS);
-            }
-            try {
                 e.processDefaultValidationAttribute("");
-            } catch (XPathException err) {
-                e.setValidationError(err, StyleElement.REPORT_ALWAYS);
-            }
 
 //            if (e.isInstruction() && e.getEffectiveVersion() == 10 &&
 //                    !config.isLicensedFeature(Configuration.LicenseFeature.PROFESSIONAL_EDITION)) {
@@ -283,7 +261,7 @@ public class StyleNodeFactory implements NodeFactory {
             Class actualClass;
 
             if (uri.equals(NamespaceConstant.XSLT)) {
-                reason = new XPathException("Unknown XSLT element: " + localname);
+                reason = new XPathException("Unknown XSLT element: " + Err.wrap(localname, Err.ELEMENT));
                 ((XPathException) reason).setErrorCode("XTSE0010");
                 ((XPathException) reason).setIsStaticError(true);
                 actualClass = AbsentExtensionElement.class;
@@ -301,7 +279,7 @@ public class StyleNodeFactory implements NodeFactory {
                     reason = new XPathException("Cannot use a reserved namespace for extension instructions", "XTSE0800");
                     temp.setValidationError(reason, StyleElement.REPORT_ALWAYS);
                 } else {
-                    reason = new XPathException("Unknown extension instruction " + elemName.getDisplayName(), "XTDE1450", temp);
+                    reason = new XPathException("Unknown extension instruction " + Err.wrap(elemName.getDisplayName(), Err.ELEMENT), "XTDE1450", temp);
                     temp.setValidationError(reason, StyleElement.REPORT_DYNAMICALLY_UNLESS_FALLBACK_AVAILABLE);
                 }
 
@@ -468,8 +446,6 @@ public class StyleNodeFactory implements NodeFactory {
                 return new XSLSequence();
             case StandardNames.XSL_SORT:
                 return new XSLSort();
-            case StandardNames.XSL_STREAM:
-                return new XSLSourceDocument(); // TODO: drop this eventually
             case StandardNames.XSL_SOURCE_DOCUMENT:
                 return new XSLSourceDocument();
             case StandardNames.XSL_STRIP_SPACE:
@@ -504,6 +480,7 @@ public class StyleNodeFactory implements NodeFactory {
     /**
      * Make a text node
      *
+     * @param parent  the parent element
      * @param content the content of the text node
      * @return the constructed text node
      */
