@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -18,6 +18,8 @@ import net.sf.saxon.om.Function;
 import net.sf.saxon.trans.SymbolicName;
 import net.sf.saxon.trans.XPathException;
 
+import java.util.List;
+
 
 /**
  * A StylesheetFunctionLibrary contains functions defined by the user in a stylesheet. This library is used at
@@ -74,17 +76,13 @@ public class StylesheetFunctionLibrary implements FunctionLibrary {
      *                     that the static type of the arguments (obtainable via getItemType() and getCardinality() may
      *                     be used as part of the binding algorithm.
      * @param env          The static context
+     * @param reasons In the event that a function cannot be bound, this output parameter may be populated with one
+     *                or more diagnostic messages indicating possible reasons why no function binding was possible.
      * @return An object representing the extension function to be called, if one is found;
      *         null if no extension function was found matching the required name and arity.
-     * @throws net.sf.saxon.trans.XPathException
-     *          if a function is found with the required name and arity, but
-     *          the implementation of the function cannot be loaded or used; or if an error occurs
-     *          while searching for the function; or if this function library "owns" the namespace containing
-     *          the function call, but no function was found.
      */
 
-    public Expression bind(SymbolicName.F functionName, Expression[] staticArgs, StaticContext env)
-            throws XPathException {
+    public Expression bind(SymbolicName.F functionName, Expression[] staticArgs, StaticContext env, List<String> reasons) {
         UserFunction fn = pack.getFunction(functionName);
         if (fn == null) {
             return null;
@@ -99,24 +97,18 @@ public class StylesheetFunctionLibrary implements FunctionLibrary {
             // compile-time binding of a static function call in XSLT
             final PrincipalStylesheetModule psm = ((ExpressionContext) env).getStyleElement().getCompilation().getPrincipalStylesheetModule();
             final ExpressionVisitor visitor = ExpressionVisitor.make(env);
-            psm.addFixupAction(new PrincipalStylesheetModule.Action() {
-                public void doAction() throws XPathException {
-                    if (fc.getFunction() == null) {
-                        Component target = psm.getComponent(fc.getSymbolicName());
-                        UserFunction fn = (UserFunction) target.getActor();
-                        if (fn != null) {
-
-                            // TODO: next line is needed only in odd cases like key-089 where the function name isn't known during ufCall.typeCheck()
-                            // See bug 2879
-                            fc.checkFunctionCall(fn, visitor);
-                            fc.computeArgumentEvaluationModes();
-                            fc.setStaticType(fn.getResultType());
-                        } else {
-                            XPathException err = new XPathException("There is no available function named " + fc.getDisplayName() +
-                                                                            " with " + fc.getArity() + " arguments", "XPST0017");
-                            err.setLocator(fc.getLocation());
-                            throw err;
-                        }
+            psm.addFixupAction(() -> {
+                if (fc.getFunction() == null) {
+                    Component target = psm.getComponent(fc.getSymbolicName());
+                    UserFunction fn1 = (UserFunction) target.getActor();
+                    if (fn1 != null) {
+                        fc.allocateArgumentEvaluators();
+                        fc.setStaticType(fn1.getResultType());
+                    } else {
+                        XPathException err = new XPathException("There is no available function named " + fc.getDisplayName() +
+                                                                        " with " + fc.getArity() + " arguments", "XPST0017");
+                        err.setLocator(fc.getLocation());
+                        throw err;
                     }
                 }
             });
@@ -133,7 +125,6 @@ public class StylesheetFunctionLibrary implements FunctionLibrary {
     /**
      * Test whether a function with a given name and arity is available; if so, return a function
      * item that can be dynamically called.
-     * <p/>
      * <p>This supports the function-lookup() function in XPath 3.0.</p>
      *
      * @param functionName  the qualified name of the function being called
