@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -123,7 +123,7 @@ public class StylesheetModule {
         options.setErrorListener(pipe.getErrorListener());
         try {
             if (options.getXMLReader() == null && options.getXMLReaderMaker() == null && Version.platform.isJava()
-                    && !(styleSource instanceof SAXSource && ((SAXSource)styleSource).getXMLReader() != null)) {
+                    && !(styleSource instanceof SAXSource && ((SAXSource) styleSource).getXMLReader() != null)) {
                 XMLReader styleParser = config.getStyleParser();
                 options.setXMLReader(styleParser);
                 Sender.send(styleSource, commentStripper, options);
@@ -164,7 +164,7 @@ public class StylesheetModule {
             Source styleSource, Compilation compilation) throws XPathException {
 
         if (styleSource instanceof SAXSource &&
-                compilation.getConfiguration().getBooleanProperty(FeatureKeys.IGNORE_SAX_SOURCE_PARSER)) {
+                compilation.getConfiguration().getBooleanProperty(Feature.IGNORE_SAX_SOURCE_PARSER)) {
             // This option is provided to allow the parser set by applications such as Ant to be overridden by
             // the parser requested using FeatureKeys.SOURCE_PARSER
             ((SAXSource) styleSource).setXMLReader(null);
@@ -219,7 +219,7 @@ public class StylesheetModule {
         } else {
             options = new ParseOptions();
         }
-        options.setSchemaValidationMode(Validation.STRIP);
+        options.setSchemaValidationMode(Validation.PRESERVE);
         options.setDTDValidationMode(Validation.STRIP);
         options.setLineNumbering(true);
         options.setSpaceStrippingRule(NoElementsSpaceStrippingRule.getInstance());
