@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -18,6 +18,7 @@ import net.sf.saxon.functions.ExecutableFunctionLibrary;
 import net.sf.saxon.functions.FunctionLibrary;
 import net.sf.saxon.functions.FunctionLibraryList;
 import net.sf.saxon.functions.registry.ConstructorFunctionLibrary;
+import net.sf.saxon.om.Action;
 import net.sf.saxon.om.SpaceStrippingRule;
 import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.pattern.NodeTest;
@@ -42,17 +43,9 @@ public class StylesheetPackage extends PackageData {
 
     private final static boolean TRACING = false;
 
-    /**
-     * A class that simply encapsulates a callback action of any kind
-     */
-
-    public static abstract class Action {
-        public abstract void doAction() throws XPathException;
-    }
-
     private PackageVersion packageVersion = null;
     private String packageName;
-    private List<StylesheetPackage> usedPackages = new ArrayList<StylesheetPackage>();
+    private List<StylesheetPackage> usedPackages = new ArrayList<>();
     private int xsltVersion;
 
     private RuleManager ruleManager;
@@ -60,7 +53,7 @@ public class StylesheetPackage extends PackageData {
 
     private boolean createsSecondaryResultDocuments;
 
-    private List<Action> completionActions = new ArrayList<Action>();
+    private List<Action> completionActions = new ArrayList<>();
     private boolean isRootPackage = true;
     protected GlobalContextRequirement globalContextRequirement = null;
     private boolean containsGlobalContextItemDeclaration = false;
@@ -70,10 +63,10 @@ public class StylesheetPackage extends PackageData {
     protected Properties defaultOutputProperties;
     private StructuredQName defaultMode;
     private boolean declaredModes;
-    protected Map<StructuredQName, Properties> namedOutputProperties = new HashMap<StructuredQName, Properties>(4);
+    protected Map<StructuredQName, Properties> namedOutputProperties = new HashMap<>(4);
 
     // table of imported schemas. The members of this set are strings holding the target namespace.
-    protected Set<String> schemaIndex = new HashSet<String>(10);
+    protected Set<String> schemaIndex = new HashSet<>(10);
 
 
     private FunctionLibraryList functionLibrary;
@@ -88,14 +81,13 @@ public class StylesheetPackage extends PackageData {
     // component; value is the component itself. Hidden components are not included in this table because their names
     // need not be unique, and because they are not available for reference by name.
 
-    private HashMap<SymbolicName, Component> componentIndex =
-        new HashMap<SymbolicName, Component>(20);
+    private HashMap<SymbolicName, Component> componentIndex = new HashMap<>(20);
 
-    protected List<Component> hiddenComponents = new ArrayList<Component>();
+    protected List<Component> hiddenComponents = new ArrayList<>();
 
-    protected HashMap<SymbolicName, Component> overriddenComponents = new HashMap<SymbolicName, Component>();
+    protected HashMap<SymbolicName, Component> overriddenComponents = new HashMap<>();
 
-    private HashMap<SymbolicName, Component> abstractComponents = new HashMap<SymbolicName, Component>();
+    private HashMap<SymbolicName, Component> abstractComponents = new HashMap<>();
 
     /**
      * Create a stylesheet package
@@ -139,6 +131,21 @@ public class StylesheetPackage extends PackageData {
         usedPackages.add(pack);
     }
 
+    /**
+     * Ask whether a given library package is within the package subtree rooted at this package
+     * @param pack the library package
+     * @return true if the given library package is in the transitive contents of this package
+     */
+
+    public boolean contains(StylesheetPackage pack) {
+        for (StylesheetPackage p : usedPackages) {
+            if (p == pack || p.contains(pack)) {
+                return true;
+            }
+        }
+        return false;
+    }
+
     /**
      * Set the version of the XSLT language specification to which the package manifest conforms
      *
@@ -201,6 +208,28 @@ public class StylesheetPackage extends PackageData {
         return isRootPackage;
     }
 
+    /**
+     * Ask whether the package was compiled with just-in-time compilation of template rules enabled
+     *
+     * @return false (always) for Saxon-HE
+     */
+
+    public boolean isJustInTimeCompilation() {
+        return false;
+    }
+
+    /**
+     * Say whether the package was compiled with just-in-time compilation of template rules enabled.
+     * This method is called by the compiler itself and should not be called by users. The setting is ignored
+     * in Saxon-HE.
+     *
+     * @param justInTimeCompilation set to true if just-in-time compilation was enabled when the package was compiled
+     */
+
+    public void setJustInTimeCompilation(boolean justInTimeCompilation) {
+        // no action
+    }
+
     /**
      * Get the rule manager, which knows about all the modes present in the package
      * @return the rule manager
@@ -232,7 +261,7 @@ public class StylesheetPackage extends PackageData {
 
     /**
      * Set the default mode for the package
-     * @param defaultMode
+     * @param defaultMode the default mode for the package
      */
 
     public void setDefaultMode(StructuredQName defaultMode) {
@@ -598,7 +627,7 @@ public class StylesheetPackage extends PackageData {
         // Create a mapping from components in the used package to their corresponding components
         // in the using package, so that we can re-bind the component bindings
 
-        final Map<Component, Component> correspondence = new HashMap<Component, Component>();
+        final Map<Component, Component> correspondence = new HashMap<>();
 
         for (Map.Entry<SymbolicName, Component> namedComponentEntry : usedPackage.componentIndex.entrySet()) {
             SymbolicName name = namedComponentEntry.getKey();
@@ -614,8 +643,8 @@ public class StylesheetPackage extends PackageData {
                 Visibility acceptedVisibility = explicitAcceptedVisibility(name, acceptors);
                 if (acceptedVisibility != null) {
                     if (!XSLAccept.isCompatible(oldV, acceptedVisibility)) {
-                        throw new XPathException("Cannot accept a " + oldV.toString() +
-                            " component with visibility " + acceptedVisibility.toString() +
+                        throw new XPathException("Cannot accept a " + oldV +
+                            " component with visibility " + acceptedVisibility +
                         " (" + name + ")", "XTSE3040");
                     }
                     newV = acceptedVisibility;
@@ -669,64 +698,38 @@ public class StylesheetPackage extends PackageData {
             }
 
             if (newC.getActor() instanceof Mode && overrides.contains(name)) {
-                addCompletionAction(new Action() {
-                    @Override
-                    public void doAction() throws XPathException {
-                        trace("Doing mode completion for " + newC.getActor().getSymbolicName());
-                        List<ComponentBinding> oldBindings = newC.getBaseComponent().getComponentBindings();
-                        List<ComponentBinding> newBindings = newC.getComponentBindings();
-                        for (int i=0; i<oldBindings.size(); i++) {
-                            SymbolicName name = oldBindings.get(i).getSymbolicName();
-                            Component target;
-                            if (overrides.contains(name)) {
-                                // if there is an override in this package, we bind to it
-                                target = getComponent(name);
-                                if (target == null) {
-                                    throw new AssertionError("We know there's an override for " + name + ", but we can't find it");
-                                }
-                            } else {
-                                // otherwise we bind to the component in this package that corresponds to the component in the used package
-                                target = correspondence.get(oldBindings.get(i).getTarget());
-                                if (target == null) {
-                                    throw new AssertionError("Saxon can't find the new component corresponding to " + name);
-                                }
+                addCompletionAction(() -> {
+                    trace("Doing mode completion for " + newC.getActor().getSymbolicName());
+                    List<ComponentBinding> oldBindings = newC.getBaseComponent().getComponentBindings();
+                    List<ComponentBinding> newBindings = newC.getComponentBindings();
+                    for (int i=0; i<oldBindings.size(); i++) {
+                        SymbolicName name12 = oldBindings.get(i).getSymbolicName();
+                        Component target;
+                        if (overrides.contains(name12)) {
+                            // if there is an override in this package, we bind to it
+                            target = getComponent(name12);
+                            if (target == null) {
+                                throw new AssertionError("We know there's an override for " + name12 + ", but we can't find it");
+                            }
+                        } else {
+                            // otherwise we bind to the component in this package that corresponds to the component in the used package
+                            target = correspondence.get(oldBindings.get(i).getTarget());
+                            if (target == null) {
+                                throw new AssertionError("Saxon can't find the new component corresponding to " + name12);
                             }
-                            ComponentBinding newBinding = new ComponentBinding(name, target);
-                            newBindings.set(i, newBinding);
                         }
-
+                        ComponentBinding newBinding = new ComponentBinding(name12, target);
+                        newBindings.set(i, newBinding);
                     }
+
                 });
             } else {
-
-                addCompletionAction(new Action() {
-                    @Override
-                    public void doAction() throws XPathException {
-                        trace("Doing normal completion for " + newC.getActor().getSymbolicName());
-                        List<ComponentBinding> oldBindings = newC.getBaseComponent().getComponentBindings();
-                        List<ComponentBinding> newBindings = new ArrayList<ComponentBinding>(oldBindings.size());
-                        for (ComponentBinding oldBinding : oldBindings) {
-                            SymbolicName name = oldBinding.getSymbolicName();
-                            Component target;
-                            if (overrides.contains(name)) {
-                                // if there is an override in this package, we bind to it
-                                target = getComponent(name);
-                                if (target == null) {
-                                    throw new AssertionError("We know there's an override for " + name + ", but we can't find it");
-                                }
-                            } else {
-                                // otherwise we bind to the component in this package that corresponds to the component in the used package
-                                target = correspondence.get(oldBinding.getTarget());
-                                if (target == null) {
-                                    throw new AssertionError("Saxon can't find the new component corresponding to " + name);
-                                }
-                            }
-                            ComponentBinding newBinding = new ComponentBinding(name, target);
-                            newBindings.add(newBinding);
-                        }
-                        newC.setComponentBindings(newBindings);
-
-                    }
+                addCompletionAction(() -> {
+                    trace("Doing normal completion for " + newC.getActor().getSymbolicName());
+                    List<ComponentBinding> oldBindings = newC.getBaseComponent().getComponentBindings();
+                    List<ComponentBinding> newBindings = new ArrayList<>(oldBindings.size());
+                    makeNewComponentBindings(overrides, correspondence, oldBindings, newBindings);
+                    newC.setComponentBindings(newBindings);
                 });
             }
 
@@ -742,33 +745,11 @@ public class StylesheetPackage extends PackageData {
 
             hiddenComponents.add(newC);
 
-            addCompletionAction(new Action() {
-                // TODO: common code with above
-                @Override
-                public void doAction() throws XPathException {
-                    List<ComponentBinding> oldBindings = newC.getBaseComponent().getComponentBindings();
-                    List<ComponentBinding> newBindings = new ArrayList<ComponentBinding>(oldBindings.size());
-                    for (ComponentBinding oldBinding : oldBindings) {
-                        SymbolicName name = oldBinding.getSymbolicName();
-                        Component target;
-                        if (overrides.contains(name)) {
-                            // if there is an override in this package, we bind to it
-                            target = getComponent(name);
-                            if (target == null) {
-                                throw new AssertionError("We know there's an override for " + name + ", but we can't find it");
-                            }
-                        } else {
-                            // otherwise we bind to the component in this package that corresponds to the component in the used package
-                            target = correspondence.get(oldBinding.getTarget());
-                            if (target == null) {
-                                throw new AssertionError("Saxon can't find the new component corresponding to " + name);
-                            }
-                        }
-                        ComponentBinding newBinding = new ComponentBinding(name, target);
-                        newBindings.add(newBinding);
-                    }
-                    newC.setComponentBindings(newBindings);
-                }
+            addCompletionAction(() -> {
+                List<ComponentBinding> oldBindings = newC.getBaseComponent().getComponentBindings();
+                List<ComponentBinding> newBindings = new ArrayList<>(oldBindings.size());
+                makeNewComponentBindings(overrides, correspondence, oldBindings, newBindings);
+                newC.setComponentBindings(newBindings);
             });
 
         }
@@ -778,6 +759,28 @@ public class StylesheetPackage extends PackageData {
         }
     }
 
+    private void makeNewComponentBindings(Set<SymbolicName> overrides, Map<Component, Component> correspondence, List<ComponentBinding> oldBindings, List<ComponentBinding> newBindings) {
+        for (ComponentBinding oldBinding : oldBindings) {
+            SymbolicName name = oldBinding.getSymbolicName();
+            Component target;
+            if (overrides.contains(name)) {
+                // if there is an override in this package, we bind to it
+                target = getComponent(name);
+                if (target == null) {
+                    throw new AssertionError("We know there's an override for " + name + ", but we can't find it");
+                }
+            } else {
+                // otherwise we bind to the component in this package that corresponds to the component in the used package
+                target = correspondence.get(oldBinding.getTarget());
+                if (target == null) {
+                    throw new AssertionError("Saxon can't find the new component corresponding to " + name);
+                }
+            }
+            ComponentBinding newBinding = new ComponentBinding(name, target);
+            newBindings.add(newBinding);
+        }
+    }
+
     private void trace(String message) {
         if (TRACING) {
             System.err.println(message);
@@ -821,11 +824,9 @@ public class StylesheetPackage extends PackageData {
 
     /**
      * Create the function library containing stylesheet functions declared in this package
-     *
-     * @return the resulting function library
      */
 
-    public FunctionLibraryList createFunctionLibrary() {
+    public void createFunctionLibrary() {
 
         FunctionLibraryList functionLibrary = new FunctionLibraryList();
         functionLibrary.addFunctionLibrary(config.getXSLT30FunctionSet());
@@ -841,7 +842,7 @@ public class StylesheetPackage extends PackageData {
         functionLibrary.addFunctionLibrary(config.getIntegratedFunctionLibrary());
         config.addExtensionBinders(functionLibrary);
 
-        return this.functionLibrary = functionLibrary;
+        this.functionLibrary = functionLibrary;
     }
 
     protected void addIxslFunctionLibrary(FunctionLibraryList functionLibrary) {}
@@ -979,7 +980,7 @@ public class StylesheetPackage extends PackageData {
         pss.setKeyManager(keyManager);
 
         if (!pss.createsSecondaryResult()) {
-            pss.setCreatesSecondaryResult(createsSecondaryResultDocuments);
+            pss.setCreatesSecondaryResult(mayCreateSecondaryResultDocuments());
         }
         pss.setDefaultOutputProperties(defaultOutputProperties);
         for (Map.Entry<StructuredQName, Properties> entry : namedOutputProperties.entrySet()) {
@@ -1035,6 +1036,18 @@ public class StylesheetPackage extends PackageData {
 
     }
 
+    private boolean mayCreateSecondaryResultDocuments() {
+        if (createsSecondaryResultDocuments) {
+            return true;
+        }
+        for (StylesheetPackage p : usedPackages) {
+            if (p.mayCreateSecondaryResultDocuments()) {
+                return true;
+            }
+        }
+        return false;
+    }
+
     public Map<SymbolicName, Component> getAbstractComponents() {
         return abstractComponents;
     }
@@ -1074,12 +1087,19 @@ public class StylesheetPackage extends PackageData {
         }
     }
 
-    public boolean isJustInTimeCompilation() {
-        return false; // not available in Saxon-HE
+    /**
+     * Ask whether a non-streamable construct has been found, forcing the entire stylesheet
+     * to fall back to unstreamed processing
+     *
+     * @return true if the stylesheet must fall back to unstreamed processing
+     */
+
+    public boolean isFallbackToNonStreaming() {
+        return true;
     }
 
-    public void setJustInTimeCompilation(boolean justInTimeCompilation) {
-        // no action for Saxon-HE
+    public void setFallbackToNonStreaming() {
+        // No action except in EE
     }
 
 
