@@ -55,12 +55,12 @@ public class UseWhenFilter extends ProxyReceiver {
     private StartTagBuffer startTag;
     private int depthOfHole = 0;
     private boolean emptyStylesheetElement = false;
-    private Stack<String> defaultNamespaceStack = new Stack<String>();
-    private Stack<Integer> versionStack = new Stack<Integer>();
+    private Stack<String> defaultNamespaceStack = new Stack<>();
+    private Stack<Integer> versionStack = new Stack<>();
     private DateTimeValue currentDateTime = DateTimeValue.getCurrentDateTime(null);
     private Compilation compilation;
-    private Stack<String> systemIdStack = new Stack<String>();
-    private Stack<URI> baseUriStack = new Stack<URI>();
+    private Stack<String> systemIdStack = new Stack<>();
+    private Stack<URI> baseUriStack = new Stack<>();
     private NestedIntegerValue precedence;
     private int importCount = 0;
     private boolean dropUnderscoredAttributes;
@@ -118,9 +118,9 @@ public class UseWhenFilter extends ProxyReceiver {
 
     /**
      * Notify the start of an element.
-     *  @param elemName   the name of the element.
+     * @param elemName   the name of the element.
      * @param typeCode   integer code identifying the element's type within the name pool.
-     * @param location
+     * @param location   location where the event originated
      * @param properties bit-significant properties of the element node
      */
 
@@ -348,16 +348,16 @@ public class UseWhenFilter extends ProxyReceiver {
             }
 
             if (isSupplied) {
-                Sequence suppliedValue = compilation.getParameters()
+                Sequence<?> suppliedValue = compilation.getParameters()
                         .convertParameterValue(varName, requiredType, true, staticContext.makeEarlyEvaluationContext());
 
-                compilation.declareStaticVariable(varName, ((Sequence<Item>) suppliedValue).materialize(), precedence, isParam);
+                compilation.declareStaticVariable(varName, suppliedValue.materialize(), precedence, isParam);
             }
         }
 
         if (isVariable || !isSupplied) {
             String selectStr = startTag.getAttribute("", "select");
-            GroundedValue value;
+            GroundedValue<?> value;
             if (selectStr == null) {
                 if (isVariable) {
                     throw createXPathException(
@@ -374,8 +374,8 @@ public class UseWhenFilter extends ProxyReceiver {
             } else {
                 try {
                     staticContext.setContainingLocation(attLoc);
-                    Sequence seq = evaluateStatic(selectStr, location, staticContext);
-                    value = ((Sequence<Item>) seq).materialize();
+                    Sequence<?> seq = evaluateStatic(selectStr, location, staticContext);
+                    value = seq.materialize();
                 } catch (XPathException e) {
                     throw createXPathException("Error in " + elemName.getLocalPart() + " expression. " + e.getMessage(),
                             e.getErrorCodeLocalPart(), attLoc);
@@ -384,8 +384,8 @@ public class UseWhenFilter extends ProxyReceiver {
             RoleDiagnostic role = new RoleDiagnostic(RoleDiagnostic.VARIABLE, varName.getDisplayName(), 0);
             role.setErrorCode("XTDE0050");
             TypeHierarchy th = getConfiguration().getTypeHierarchy();
-            Sequence seq = th.applyFunctionConversionRules(value, requiredType, role, attLoc);
-            value = ((Sequence<Item>) seq).materialize();
+            Sequence<?> seq = th.applyFunctionConversionRules(value, requiredType, role, attLoc);
+            value = seq.materialize();
             try {
                 compilation.declareStaticVariable(varName, value, precedence, isParam);
             } catch (XPathException e) {
@@ -471,7 +471,7 @@ public class UseWhenFilter extends ProxyReceiver {
         return expr.evaluateAsString(dynamicContext).toString();
     }
 
-    public XPathException createXPathException(String message, String errorCode, Location location) throws XPathException {
+    public XPathException createXPathException(String message, String errorCode, Location location) {
         XPathException err = new XPathException(message);
         err.setErrorCode(errorCode);
         err.setIsStaticError(true);
@@ -653,7 +653,7 @@ public class UseWhenFilter extends ProxyReceiver {
      * @throws XPathException if evaluation of the expression fails
      */
 
-    public Sequence evaluateStatic(String expression, Location locationId, UseWhenStaticContext staticContext) throws XPathException {
+    public Sequence<?> evaluateStatic(String expression, Location locationId, UseWhenStaticContext staticContext) throws XPathException {
         setNamespaceBindings(staticContext);
         Expression expr = ExpressionTool.make(expression, staticContext,
             0, Token.EOF, null);
