@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -42,6 +42,7 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
     private boolean doneAttributes = false;
     /*@Nullable*/ private String nameAtt = null;
     private String asAtt = null;
+    private String extraAsAtt = null;
     private SequenceType resultType = SequenceType.ANY_SEQUENCE;
     private SlotManager stackFrameMap;
     private boolean memoFunction = false;
@@ -92,61 +93,73 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
             String uri = atts.getURI(a);
             String local = atts.getLocalName(a);
             if ("".equals(uri)) {
-                if (local.equals("name")) {
-                    nameAtt = Whitespace.trim(atts.getValue(a));
-                    assert nameAtt != null;
-                    if (nameAtt.indexOf(':') < 0) {
-                        nameAtt = "Q{" + NamespaceConstant.SAXON + "}error-function-name";
-                        compileError("Function name must have a namespace prefix", "XTSE0740");
-                    }
-                    try {
-                        setObjectName(makeQName(nameAtt));
-                    } catch (NamespaceException err) {
-                        compileError(err.getMessage(), "XTSE0280");
-                    } catch (XPathException err) {
-                        compileError(err);
-                    }
-                } else if (local.equals("as")) {
-                    asAtt = atts.getValue(a);
-                } else if (local.equals("visibility")) {
-                    visibilityAtt = Whitespace.trim(atts.getValue(a));
-                } else if (local.equals("streamability")) {
-                    streamabilityAtt = Whitespace.trim(atts.getValue(a));
-                } else if (local.equals("override")) {
-                    String overrideAtt = Whitespace.trim(atts.getValue(a));
-                    boolean override = processBooleanAttribute("override", overrideAtt);
-                    if (overrideExtensionFunctionAtt != null) {
-                        if (override != overrideExtensionFunction) {
-                            compileError("Attributes override-extension-function and override are both used, but do not match", "XTSE0020");
+                switch (local) {
+                    case "name":
+                        nameAtt = Whitespace.trim(atts.getValue(a));
+                        assert nameAtt != null;
+                        if (nameAtt.indexOf(':') < 0) {
+                            nameAtt = "Q{" + NamespaceConstant.SAXON + "}error-function-name";
+                            compileError("Function name must have a namespace prefix", "XTSE0740");
                         }
-                    } else {
-                        overrideExtensionFunctionAtt = overrideAtt;
-                        overrideExtensionFunction = override;
-                    }
-                    compileWarning("The xsl:function/@override attribute is deprecated; use override-extension-function", SaxonErrorCode.SXWN9014);
-                } else if (local.equals("override-extension-function")) {
-                    String overrideExtAtt = Whitespace.trim(atts.getValue(a));
-                    boolean overrideExt = processBooleanAttribute("override-extension-function", overrideExtAtt);
-                    if (overrideExtensionFunctionAtt != null) {
-                        if (overrideExt != overrideExtensionFunction) {
-                            compileError("Attributes override-extension-function and override are both used, but do not match", "XTSE0020");
+                        try {
+                            setObjectName(makeQName(nameAtt));
+                        } catch (XPathException err) {
+                            compileError(err);
+                        }
+                        break;
+                    case "as":
+                        asAtt = atts.getValue(a);
+                        break;
+                    case "visibility":
+                        visibilityAtt = Whitespace.trim(atts.getValue(a));
+                        break;
+                    case "streamability":
+                        streamabilityAtt = Whitespace.trim(atts.getValue(a));
+                        break;
+                    case "override":
+                        String overrideAtt = Whitespace.trim(atts.getValue(a));
+                        boolean override = processBooleanAttribute("override", overrideAtt);
+                        if (overrideExtensionFunctionAtt != null) {
+                            if (override != overrideExtensionFunction) {
+                                compileError("Attributes override-extension-function and override are both used, but do not match", "XTSE0020");
+                            }
+                        } else {
+                            overrideExtensionFunctionAtt = overrideAtt;
+                            overrideExtensionFunction = override;
                         }
-                    } else {
-                        overrideExtensionFunctionAtt = overrideExtAtt;
-                        overrideExtensionFunction = overrideExt;
-                    }
-                    if (local.equals("override")) {
                         compileWarning("The xsl:function/@override attribute is deprecated; use override-extension-function", SaxonErrorCode.SXWN9014);
-                    }
-                } else if (local.equals("cache")) {
-                    cacheAtt = Whitespace.trim(atts.getValue(a));
-                } else if (local.equals("new-each-time")) {
-                    newEachTimeAtt = Whitespace.trim(atts.getValue(a));
-                } else {
-                    checkUnknownAttribute(atts.getNodeName(a));
+                        break;
+                    case "override-extension-function":
+                        String overrideExtAtt = Whitespace.trim(atts.getValue(a));
+                        boolean overrideExt = processBooleanAttribute("override-extension-function", overrideExtAtt);
+                        if (overrideExtensionFunctionAtt != null) {
+                            if (overrideExt != overrideExtensionFunction) {
+                                compileError("Attributes override-extension-function and override are both used, but do not match", "XTSE0020");
+                            }
+                        } else {
+                            overrideExtensionFunctionAtt = overrideExtAtt;
+                            overrideExtensionFunction = overrideExt;
+                        }
+                        if (local.equals("override")) {
+                            compileWarning("The xsl:function/@override attribute is deprecated; use override-extension-function", SaxonErrorCode.SXWN9014);
+                        }
+                        break;
+                    case "cache":
+                        cacheAtt = Whitespace.trim(atts.getValue(a));
+                        break;
+                    case "new-each-time":
+                        newEachTimeAtt = Whitespace.trim(atts.getValue(a));
+                        break;
+                    default:
+                        checkUnknownAttribute(atts.getNodeName(a));
+                        break;
+                }
+            } else if (uri.equals(NamespaceConstant.SAXON)) {
+                if (local.equals("memo-function")) {
+                    memoFunction = processBooleanAttribute("saxon:memo-function", atts.getValue(a));
+                } else if (local.equals("as")) {
+                    extraAsAtt = atts.getValue(a);
                 }
-            } else if (local.equals("memo-function") && uri.equals(NamespaceConstant.SAXON)) {
-                memoFunction = processBooleanAttribute("saxon:memo-function", atts.getValue(a));
             } else {
                 checkUnknownAttribute(atts.getNodeName(a));
             }
@@ -165,17 +178,35 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
             }
         }
 
+        if (extraAsAtt != null) {
+            SequenceType extraResultType = null;
+            try {
+                extraResultType = makeExtendedSequenceType(extraAsAtt);
+            } catch (XPathException e) {
+                compileErrorInAttribute(e.getMessage(), e.getErrorCodeLocalPart(), "saxon:as");
+                extraResultType = resultType;
+            }
+            if (asAtt != null) {
+                int rel = getConfiguration().getTypeHierarchy().sequenceTypeRelationship(extraResultType, resultType);
+                if (rel == TypeHierarchy.SAME_TYPE || rel == TypeHierarchy.SUBSUMED_BY) {
+                    resultType = extraResultType;
+                } else {
+                    compileErrorInAttribute("When both are present, @saxon:as must be a subtype of @as", "SXER7TBA", "saxon:as");
+                }
+            } else {
+                resultType = extraResultType;
+            }
+        }
+
         if (visibilityAtt == null) {
             visibility = Visibility.PRIVATE;
         } else {
-            check30attribute("visibility");
             visibility = interpretVisibilityValue(visibilityAtt, "");
         }
 
         if (streamabilityAtt == null) {
             streamability = FunctionStreamability.UNCLASSIFIED;
         } else {
-            check30attribute("streamability");
             streamability = getStreamabilityValue(streamabilityAtt);
             if (streamability.isStreaming()) {
                 boolean streamable = processStreamableAtt("yes");
@@ -189,12 +220,8 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
             if ("maybe".equals(newEachTimeAtt)) {
                 determinism = UserFunction.Determinism.ELIDABLE;
             } else {
-                try {
-                    boolean b = processBooleanAttribute("new-each-time", newEachTimeAtt);
-                    determinism = b ? UserFunction.Determinism.PROACTIVE : UserFunction.Determinism.DETERMINISTIC;
-                } catch (XPathException e) {
-                    invalidAttribute("new-each-time", "yes|no|maybe");
-                }
+                boolean b = processBooleanAttribute("new-each-time", newEachTimeAtt);
+                determinism = b ? UserFunction.Determinism.PROACTIVE : UserFunction.Determinism.DETERMINISTIC;
             }
         }
 
@@ -211,11 +238,7 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
     private FunctionStreamability getStreamabilityValue(String s) throws XPathException {
         if (s.contains(":")) {
             // QNames are allowed but not recognized by Saxon
-            try {
-                makeQName(s);
-            } catch (NamespaceException e) {
-                throw new XPathException(e);
-            }
+            makeQName(s);
             return FunctionStreamability.UNCLASSIFIED;
         }
         for (FunctionStreamability v : FunctionStreamability.values()) {
@@ -244,8 +267,6 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
             try {
                 qn = makeQName(nameAtt);
                 setObjectName(qn);
-            } catch (NamespaceException err) {
-                return new StructuredQName("saxon", NamespaceConstant.SAXON, "badly-named-function" + generateId());
             } catch (XPathException err) {
                 return new StructuredQName("saxon", NamespaceConstant.SAXON, "badly-named-function" + generateId());
             }
@@ -263,8 +284,8 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
         return true;
     }
 
-    protected boolean mayContainParam(String attName) {
-        return !"required".equals(attName);
+    protected boolean mayContainParam() {
+        return true;
     }
 
     /**
@@ -277,12 +298,8 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
 
     public Visibility getVisibility() {
         if (visibility == null) {
-            try {
-                String vAtt = getAttributeValue("", "visibility");
-                return vAtt == null ? Visibility.PRIVATE : interpretVisibilityValue(Whitespace.trim(vAtt), "");
-            } catch (XPathException e) {
-                return Visibility.PRIVATE;
-            }
+            String vAtt = getAttributeValue("", "visibility");
+            return vAtt == null ? Visibility.PRIVATE : interpretVisibilityValue(Whitespace.trim(vAtt), "");
         }
         return visibility;
     }
@@ -291,7 +308,7 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
         return new SymbolicName.F(getObjectName(), getNumberOfArguments());
     }
 
-    public void checkCompatibility(Component component) throws XPathException {
+    public void checkCompatibility(Component component) {
         if (compiledFunction == null) {
             getCompiledFunction();
         }
@@ -342,7 +359,7 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
         return overrideExtensionFunction;
     }
 
-    public void index(ComponentDeclaration decl, PrincipalStylesheetModule top) throws XPathException {
+    public void index(ComponentDeclaration decl, PrincipalStylesheetModule top) {
         //getSkeletonCompiledFunction();
         getCompiledFunction();
         top.indexFunction(decl);
@@ -371,10 +388,6 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
 
         checkTopLevel("XTSE0010", true);
         int arity = getNumberOfArguments();
-//        if (compiledFunction != null) {
-//            completeCompiledFunction();
-//        }
-
         if (arity == 0 && streamability != FunctionStreamability.UNCLASSIFIED) {
             compileError("A function with no arguments must have streamability=unclassified", "XTSE3155");
         }
@@ -643,7 +656,7 @@ public class XSLFunction extends StyleElement implements StylesheetComponent {
                 fn.setDeclaredVisibility(getDeclaredVisibility());
                 fn.setDeclaredStreamability(streamability);
                 fn.setDeterminism(determinism);
-                List<Annotation> annotations = new ArrayList<Annotation>();
+                List<Annotation> annotations = new ArrayList<>();
                 if (memoFunction) {
                     annotations.add(new Annotation(new StructuredQName("saxon", NamespaceConstant.SAXON, "memo-function")));
                 }
