@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -40,20 +40,15 @@ public class XSLGlobalParam extends XSLGlobalVariable {
     /**
      * Default visibility for xsl:param is public
      * @return the declared visibility, or "public" if not declared
-     * @throws XPathException
      */
 
-    public Visibility getVisibility() throws XPathException {
+    public Visibility getVisibility()  {
         String statik = getAttributeValue("static");
         if (statik == null) {
             return Visibility.PUBLIC;
         } else {
-            try {
-                boolean isStatic = processBooleanAttribute("static", statik);
-                return isStatic ? Visibility.PRIVATE : Visibility.PUBLIC;
-            } catch (XPathException err) {
-                return Visibility.PUBLIC;
-            }
+            boolean isStatic = processBooleanAttribute("static", statik);
+            return isStatic ? Visibility.PRIVATE : Visibility.PUBLIC;
         }
     }
 
