@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -19,7 +19,6 @@ import net.sf.saxon.expr.sort.CodepointCollator;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.lib.StringCollator;
 import net.sf.saxon.om.AttributeCollection;
-import net.sf.saxon.om.NamespaceException;
 import net.sf.saxon.om.StandardNames;
 import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.pattern.NodeTestPattern;
@@ -33,6 +32,7 @@ import net.sf.saxon.value.Whitespace;
 
 import java.net.URI;
 import java.net.URISyntaxException;
+import java.util.function.Predicate;
 
 /**
  * Handler for xsl:key elements in stylesheet. <br>
@@ -65,7 +65,7 @@ public class XSLKey extends StyleElement implements StylesheetComponent {
         return null;
     }
 
-    public void checkCompatibility(Component component) throws XPathException {
+    public void checkCompatibility(Component component) {
         // no action: keys cannot be overridden
     }
 
@@ -104,29 +104,29 @@ public class XSLKey extends StyleElement implements StylesheetComponent {
             String uri = atts.getURI(a);
             String local = atts.getLocalName(a);
             if ("".equals(uri)) {
-                if (local.equals("name")) {
-                    nameAtt = Whitespace.trim(atts.getValue(a));
-                } else if (local.equals("use")) {
-                    useAtt = atts.getValue(a);
-                    use = makeExpression(useAtt, a);
-                } else if (local.equals("match")) {
-                    matchAtt = atts.getValue(a);
-                } else if (local.equals("collation")) {
-                    collationName = Whitespace.trim(atts.getValue(a));
-                } else if (local.equals("composite")) {
-                    composite = processBooleanAttribute("composite", atts.getValue(a));
-                } else {
-                    checkUnknownAttribute(atts.getNodeName(a));
+                switch (local) {
+                    case "name":
+                        nameAtt = Whitespace.trim(atts.getValue(a));
+                        break;
+                    case "use":
+                        useAtt = atts.getValue(a);
+                        use = makeExpression(useAtt, a);
+                        break;
+                    case "match":
+                        matchAtt = atts.getValue(a);
+                        break;
+                    case "collation":
+                        collationName = Whitespace.trim(atts.getValue(a));
+                        break;
+                    case "composite":
+                        composite = processBooleanAttribute("composite", atts.getValue(a));
+                        break;
+                    default:
+                        checkUnknownAttribute(atts.getNodeName(a));
+                        break;
                 }
             } else if (local.equals("range-key") && uri.equals(NamespaceConstant.SAXON)) {
-                String rangeKeyAtt = Whitespace.trim(atts.getValue(a));
-                if ("yes".equals(rangeKeyAtt)) {
-                    rangeKey = true;
-                } else if ("no".equals(rangeKeyAtt)) {
-                    rangeKey = false;
-                } else {
-                    compileError("saxon:range-key must be 'yes' or 'no'", "XTSE0020");
-                }
+                rangeKey = processBooleanAttribute("range-key", atts.getValue(a));
             } else {
                 checkUnknownAttribute(atts.getNodeName(a));
             }
@@ -139,8 +139,6 @@ public class XSLKey extends StyleElement implements StylesheetComponent {
         try {
             keyName = makeQName(nameAtt);
             setObjectName(keyName);
-        } catch (NamespaceException err) {
-            compileErrorInAttribute(err.getMessage(), "XTSE0280", "name");
         } catch (XPathException err) {
             compileErrorInAttribute(err.getMessage(), err.getErrorCodeLocalPart(), "name");
         }
@@ -168,10 +166,8 @@ public class XSLKey extends StyleElement implements StylesheetComponent {
                 }
             }
             return getObjectName();
-        } catch (NamespaceException err) {
-            return null;          // the errors will be picked up later
         } catch (XPathException err) {
-            return null;
+            return null;          // the errors will be picked up later
         }
     }
 
@@ -228,8 +224,8 @@ public class XSLKey extends StyleElement implements StylesheetComponent {
 
     }
 
-    private static class ContainsGlobalVariable implements ExpressionTool.ExpressionPredicate {
-        public boolean matches(Expression e) {
+    private static class ContainsGlobalVariable implements Predicate<Expression> {
+        public boolean test(Expression e) {
             return e instanceof GlobalVariableReference ||
                     e instanceof UserFunctionCall ||
                     e instanceof CallTemplate ||
@@ -239,7 +235,7 @@ public class XSLKey extends StyleElement implements StylesheetComponent {
 
     private static ContainsGlobalVariable containsGlobalVariable = new ContainsGlobalVariable();
 
-    public void index(ComponentDeclaration decl, PrincipalStylesheetModule top) throws XPathException {
+    public void index(ComponentDeclaration decl, PrincipalStylesheetModule top) {
         StructuredQName keyName = getKeyName();
         if (keyName != null) {
             top.getKeyManager().preRegisterKeyDefinition(keyName);
@@ -282,7 +278,7 @@ public class XSLKey extends StyleElement implements StylesheetComponent {
                 //role.setSourceLocator(new ExpressionLocation(this));
                 use = config.getTypeChecker(false).staticTypeCheck(
                         use,
-                        SequenceType.OPTIONAL_ATOMIC,
+                        SequenceType.ATOMIC_SEQUENCE,
                         role, makeExpressionVisitor());
                 // Do a further check that the use expression makes sense in the context of the match pattern
                 assert match != null;
@@ -359,8 +355,6 @@ public class XSLKey extends StyleElement implements StylesheetComponent {
      * Generate byte code if appropriate
      *
      * @param opt the optimizer
-     * @throws net.sf.saxon.trans.XPathException
-     *          if bytecode generation fails
      */
     public void generateByteCode(Optimizer opt) {}
 }
\ No newline at end of file
