@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -148,9 +148,8 @@ public abstract class XSLLeafNodeConstructor extends StyleElement {
      * @return an expression that returns a string containing the string value of the constructed node
      */
 
-    public static Expression makeSimpleContentConstructor(Expression select, Expression separator, StaticContext env)
-    throws XPathException {
-        RetainedStaticContext rsc = select.getRetainedStaticContext();
+    public static Expression makeSimpleContentConstructor(Expression select, Expression separator, StaticContext env) {
+        RetainedStaticContext rsc = select.getLocalRetainedStaticContext();
         if (rsc == null) {
             rsc = env.makeRetainedStaticContext();
         }
@@ -161,7 +160,7 @@ public abstract class XSLLeafNodeConstructor extends StyleElement {
         // Convert each atomic value to a string
         select = new AtomicSequenceConverter(select, BuiltInAtomicType.STRING);
         select.setRetainedStaticContext(rsc);
-        ((AtomicSequenceConverter) select).allocateConverter(env.getConfiguration(), false);
+        ((AtomicSequenceConverter) select).allocateConverterStatically(env.getConfiguration(), false);
         // Join the resulting strings with a separator
         if (select.getCardinality() != StaticProperty.EXACTLY_ONE) {
             select = SystemFunction.makeCall("string-join", rsc, select, separator);
