@@ -1,16 +1,14 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// This Source Code Form is “Incompatible With Secondary Licenses”, as defined by the Mozilla Public License, v. 2.0.
+// This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 package net.sf.saxon.style;
 
 import net.sf.saxon.om.AxisInfo;
-import net.sf.saxon.om.NodeInfo;
 import net.sf.saxon.trans.XPathException;
-import net.sf.saxon.tree.iter.AxisIterator;
 
 /**
  * Class representing xsl:stylesheet, xsl:transform, or xsl:package
@@ -64,9 +62,7 @@ public abstract class XSLModuleRoot extends StyleElement {
 
     public void processAllAttributes() throws XPathException {
         prepareAttributes();
-        AxisIterator iter = iterateAxis(AxisInfo.CHILD);
-        NodeInfo node;
-        while ((node = iter.next()) != null) {
+        iterateAxis(AxisInfo.CHILD).forEach(node -> {
             if (node instanceof StyleElement) {
                 try {
                     ((StyleElement) node).processAllAttributes();
@@ -74,7 +70,7 @@ public abstract class XSLModuleRoot extends StyleElement {
                     ((StyleElement) node).compileError(err);
                 }
             }
-        }
+        });
     }
 
 
