@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -228,8 +228,13 @@ public class XSLPackage extends XSLModuleRoot {
             } else if (child instanceof AbsentExtensionElement && ((StyleElement) child).forwardsCompatibleModeIsEnabled()) {
                 // this is OK: an unknown XSLT element is allowed in forwards compatibility mode
             } else if (NamespaceConstant.XSLT.equals(child.getURI())) {
-                ((StyleElement) child).compileError("Element " + child.getDisplayName() +
-                        " must not appear directly within " + getDisplayName(), "XTSE0010");
+                if (child instanceof AbsentExtensionElement) {
+                    // then the error will be reported later
+                } else {
+                    ((StyleElement) child).compileError(
+                            "Element " + child.getDisplayName() +
+                                    " must not appear directly within " + getDisplayName(), "XTSE0010");
+                }
             } else {
                 ((StyleElement) child).compileError("Element " + child.getDisplayName() +
                         " must not appear directly within " + getDisplayName() +
@@ -237,6 +242,13 @@ public class XSLPackage extends XSLModuleRoot {
             }
         }
 
+        if (declaredModes) {
+            String defaultMode = getAttributeValue("default-mode");
+            if (defaultMode != null && getPrincipalStylesheetModule().getRuleManager().obtainMode(getDefaultMode(), false) == null) {
+                compileError("The default mode " + defaultMode + " has not been declared in an xsl:mode declaration", "XTSE3085");
+            }
+        }
+
     }
 
 
