@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -8,10 +8,13 @@
 package net.sf.saxon.style;
 
 import net.sf.saxon.Configuration;
-import net.sf.saxon.expr.*;
+import net.sf.saxon.expr.Component;
+import net.sf.saxon.expr.ErrorExpression;
+import net.sf.saxon.expr.Expression;
+import net.sf.saxon.expr.StaticProperty;
 import net.sf.saxon.expr.instruct.*;
 import net.sf.saxon.expr.parser.*;
-import net.sf.saxon.lib.FeatureKeys;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.Logger;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.om.*;
@@ -54,7 +57,7 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
     // A compiled named template exists if the template has a name
     private NamedTemplate compiledNamedTemplate;
     // A set of compiled template rules exists if the template has a match pattern: one TemplateRule for each mode
-    private Map<StructuredQName, TemplateRule> compiledTemplateRules = new HashMap<StructuredQName, TemplateRule>();
+    private Map<StructuredQName, TemplateRule> compiledTemplateRules = new HashMap<>();
     private SequenceType requiredType = SequenceType.ANY_SEQUENCE;
     private boolean hasRequiredParams = false;
     private boolean isTailRecursive = false;
@@ -63,8 +66,8 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
     private boolean mayOmitContextItem = true;
     //private boolean maySupplyContextItem = true;
     private boolean absentFocus = false;
-    //private Expression body;
     private boolean jitCompilationDone = false;
+    //private Expression body;
 
     /**
      * Get the corresponding NamedTemplate object that results from the compilation of this
@@ -98,6 +101,7 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
 
     /**
      * Ask whether the compilation of the template should be deferred
+     *
      * @param compilation the compilation
      * @return true if compilation should be deferred
      */
@@ -116,7 +120,7 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
         return true;
     }
 
-    protected boolean mayContainParam(String attName) {
+    protected boolean mayContainParam() {
         return true;
     }
 
@@ -176,10 +180,8 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
                 }
             }
             return getObjectName();
-        } catch (NamespaceException err) {
-            return null;          // the errors will be picked up later
         } catch (XPathException err) {
-            return null;
+            return null;          // the errors will be picked up later
         }
     }
 
@@ -200,7 +202,7 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
     }
 
 
-    public void checkCompatibility(Component component) throws XPathException {
+    public void checkCompatibility(Component component) {
         NamedTemplate other = (NamedTemplate) component.getActor();
         if (!getSymbolicName().equals(other.getSymbolicName())) {
             throw new IllegalArgumentException();
@@ -209,49 +211,51 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
         SequenceType req = requiredType == null ? SequenceType.ANY_SEQUENCE : requiredType;
         if (!req.equals(other.getRequiredType())) {
             compileError("The overriding template has a different required type from the overridden template", "XTSE3070");
+            return;
         }
 
         if (!requiredContextItemType.equals(other.getRequiredContextItemType()) ||
                 mayOmitContextItem != other.isMayOmitContextItem() ||
                 absentFocus != other.isAbsentFocus()) {
             compileError("The required context item for the overriding template differs from that of the overridden template", "XTSE3070");
+            return;
         }
 
-        List<LocalParam> otherParams = other.getLocalParams();
-        Set<StructuredQName> overriddenParams = new HashSet<StructuredQName>();
-        for (LocalParam lp0 : otherParams) {
-            XSLLocalParam lp1 = getParam(lp0.getVariableQName());
+        List<NamedTemplate.LocalParamInfo> otherParams = other.getLocalParamDetails();
+        Set<StructuredQName> overriddenParams = new HashSet<>();
+        for (NamedTemplate.LocalParamInfo lp0 : otherParams) {
+            XSLLocalParam lp1 = getParam(lp0.name);
             if (lp1 == null) {
-                if (!lp0.isTunnelParam()) {
+                if (!lp0.isTunnel) {
                     compileError("The overridden template declares a parameter " +
-                                         lp0.getVariableQName().getDisplayName() + " which is not declared in the overriding template", "XTSE3070");
+                                         lp0.name.getDisplayName() + " which is not declared in the overriding template", "XTSE3070");
                 }
                 return;
             }
-            if (!lp1.getRequiredType().equals(lp0.getRequiredType())) {
+            if (!lp1.getRequiredType().equals(lp0.requiredType)) {
                 lp1.compileError("The parameter " +
-                                         lp0.getVariableQName().getDisplayName() + " has a different required type in the overridden template", "XTSE3070");
+                                         lp0.name.getDisplayName() + " has a different required type in the overridden template", "XTSE3070");
                 return;
             }
-            if (lp1.isRequiredParam() != lp0.isRequiredParam() && !lp0.isTunnelParam()) {
+            if (lp1.isRequiredParam() != lp0.isRequired && !lp0.isTunnel) {
                 lp1.compileError("The parameter " +
-                                         lp0.getVariableQName().getDisplayName() + " is " +
+                                         lp0.name.getDisplayName() + " is " +
                                          (lp1.isRequiredParam() ? "required" : "optional") +
                                          " in the overriding template, but " +
-                                         (lp0.isRequiredParam() ? "required" : "optional") +
+                                         (lp0.isRequired ? "required" : "optional") +
                                          " in the overridden template", "XTSE3070");
                 return;
             }
-            if (lp1.isTunnelParam() != lp0.isTunnelParam()) {
+            if (lp1.isTunnelParam() != lp0.isTunnel) {
                 lp1.compileError("The parameter " +
-                                         lp0.getVariableQName().getDisplayName() + " is a " +
+                                         lp0.name.getDisplayName() + " is a " +
                                          (lp1.isTunnelParam() ? "tunnel" : "non-tunnel") +
                                          " parameter in the overriding template, but " +
-                                         (lp0.isTunnelParam() ? "tunnel" : "non-tunnel") +
+                                         (lp0.isTunnel ? "tunnel" : "non-tunnel") +
                                          " parameter in the overridden template", "XTSE3070");
                 return;
             }
-            overriddenParams.add(lp0.getVariableQName());
+            overriddenParams.add(lp0.name);
         }
 
         AxisIterator params = iterateAxis(AxisInfo.CHILD);
@@ -281,9 +285,8 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
 
     public void prepareAttributes() throws XPathException {
 
-        String extraAsAtt = null;
-
         AttributeCollection atts = getAttributeList();
+        String extraAsAtt = null;
 
         for (int a = 0; a < atts.getLength(); a++) {
             String f = atts.getQName(a);
@@ -337,13 +340,11 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
             if (nameAtt != null) {
                 StructuredQName qName = makeQName(nameAtt);
                 setObjectName(qName);
-                if(compiledNamedTemplate != null /*bug3549*/) {
+                if (compiledNamedTemplate != null /*bug3549*/) {
                     compiledNamedTemplate.setTemplateName(qName);
                 }
                 diagnosticId = nameAtt;
             }
-        } catch (NamespaceException err) {
-            compileError(err.getMessage(), "XTSE0280");
         } catch (XPathException err) {
             err.maybeSetErrorCode("XTSE0280");
             err.setIsStaticError(true);
@@ -409,7 +410,6 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
         }
 
         if (visibilityAtt != null) {
-            check30attribute("visibility");
             visibility = interpretVisibilityValue(visibilityAtt, "");
             if (nameAtt == null) {
                 compileError("xsl:template/@visibility can be specified only if the template has a @name attribute", "XTSE0020");
@@ -475,8 +475,8 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
                 } else {
                     try {
                         mname = makeQName(s);
-                    } catch (NamespaceException e) {
-                        compileError(e.getMessage(), "XTSE0280");
+                    } catch (XPathException e) {
+                        compileError(e.getMessage(), "XTSE0550");
                         mname = Mode.UNNAMED_MODE_NAME;
                     }
                 }
@@ -496,13 +496,14 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
 
     /**
      * Get the modes to which this template rule applies
+     *
      * @return the set of modes to which it applies
      * @throws XPathException should not happen
      */
 
     public Set<Mode> getApplicableModes() throws XPathException {
         StructuredQName[] names = getModeNames();
-        Set<Mode> modes = new HashSet<Mode>(names.length);
+        Set<Mode> modes = new HashSet<>(names.length);
         RuleManager mgr = getPrincipalStylesheetModule().getRuleManager();
         for (StructuredQName name : names) {
             if (name.equals(Mode.OMNI_MODE)) {
@@ -604,22 +605,8 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
         if (getTemplateName() != null) {
             if (compiledNamedTemplate == null) {
                 compiledNamedTemplate = new NamedTemplate(getTemplateName());
-                top.indexNamedTemplate(decl);
-                Item child;
-                SequenceIterator kids = iterateAxis(AxisInfo.CHILD);
-                while ((child = kids.next()) != null) {
-                    if (child instanceof XSLLocalParam) {
-                        LocalParam lp = new LocalParam();
-                        lp.setVariableQName(((XSLLocalParam) child).getVariableQName());
-                        lp.setRequiredType(((XSLLocalParam) child).getRequiredType());
-                        lp.setTunnel(((XSLLocalParam) child).isTunnelParam());
-                        compiledNamedTemplate.addLocalParam(lp);
-                    }
-                }
-            } else {
-                // Typically happens when a stylesheet module is imported twice by different routes
-                top.indexNamedTemplate(decl);
             }
+            top.indexNamedTemplate(decl);
         }
 
     }
@@ -653,7 +640,7 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
         if (body.getRetainedStaticContext() == null) {
             body.setRetainedStaticContext(rsc); // bug 2608
         }
-        if (match != null && compilation.getConfiguration().getBooleanProperty(FeatureKeys.STRICT_STREAMABILITY) &&
+        if (match != null && compilation.getConfiguration().getBooleanProperty(Feature.STRICT_STREAMABILITY) &&
                 isWithinDeclaredStreamableConstruct()) {
             checkStrictStreamability(body);
         }
@@ -687,14 +674,15 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
             checkCompatibility(overridden);
         }
 
-        ContextItemStaticInfo ciso = getConfiguration().makeContextItemStaticInfo(requiredContextItemType, mayOmitContextItem);
-        Expression body2 = refineTemplateBody(body, ciso);
+        ContextItemStaticInfo cisi = getConfiguration().makeContextItemStaticInfo(requiredContextItemType, mayOmitContextItem);
+        Expression body2 = refineTemplateBody(body, cisi);
 
         compiledNamedTemplate.setBody(body2);
 
     }
 
-    private Expression refineTemplateBody(Expression body, ContextItemStaticInfo cisi) throws XPathException {
+    private Expression refineTemplateBody(Expression body, ContextItemStaticInfo cisi) {
+        Expression old = body;
         try {
             body = body.simplify();
         } catch (XPathException e) {
@@ -702,25 +690,25 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
                 compileError(e);
             } else {
                 body = new ErrorExpression(e);
+                ExpressionTool.copyLocationInfo(old, body);
             }
         }
 
         Configuration config = getConfiguration();
         if (visibility != Visibility.ABSTRACT) {
             try {
-                if (requiredType != null) {
+                if (requiredType != null && requiredType != SequenceType.ANY_SEQUENCE) {
                     RoleDiagnostic role =
                             new RoleDiagnostic(RoleDiagnostic.TEMPLATE_RESULT, diagnosticId, 0);
-                    //role.setSourceLocator(new ExpressionLocation(this));
                     role.setErrorCode("XTTE0505");
                     body = config.getTypeChecker(false).staticTypeCheck(body, requiredType, role, makeExpressionVisitor());
                 }
             } catch (XPathException err) {
                 if (err.isReportableStatically()) {
                     compileError(err);
-                } else {
-                    body = new ErrorExpression(err);
                 }
+                body = new ErrorExpression(err);
+                ExpressionTool.copyLocationInfo(old, body);
             }
         }
 
@@ -743,7 +731,7 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
         return body;
     }
 
-    public void compileTemplateRule(Compilation compilation, Expression body, ComponentDeclaration decl) throws XPathException {
+    public void compileTemplateRule(Compilation compilation, Expression body, ComponentDeclaration decl) {
 
         Configuration config = getConfiguration();
 
@@ -779,13 +767,13 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
     private void createSkeletonTemplate(Compilation compilation, ComponentDeclaration decl) throws XPathException {
         StructuredQName[] modes = modeNames;
         if (isOmniMode()) {
-            List<StructuredQName> all = new ArrayList<StructuredQName>();
+            List<StructuredQName> all = new ArrayList<>();
             all.add(Mode.UNNAMED_MODE_NAME);
             RuleManager mgr = getCompilation().getPrincipalStylesheetModule().getRuleManager();
             for (Mode m : mgr.getAllNamedModes()) {
                 all.add(m.getModeName());
             }
-            modes = all.toArray(new StructuredQName[all.size()]);
+            modes = all.toArray(new StructuredQName[0]);
         }
         for (StructuredQName modeName : modes) {
             TemplateRule templateRule = compiledTemplateRules.get(modeName);
@@ -811,8 +799,19 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
         templateRule.setContextItemRequirements(requiredContextItemType, absentFocus);
     }
 
+    /**
+     * Code executed when the template is first executed under JIT. If the template is defined in several
+     * modes, then this may be called several times, but it only does anything the first time. Mode-specific
+     * processing is done in the TemplateRuleInitializer.
+     *
+     * @param compilation the compilation episode
+     * @param decl        the template rule declaration
+     * @throws XPathException if anything goes wrong
+     */
+
     public synchronized void jitCompile(Compilation compilation, ComponentDeclaration decl) throws XPathException {
         if (!jitCompilationDone) {
+            jitCompilationDone = true;
             compilation.setPreScan(false);
             processAllAttributes();
 
@@ -838,11 +837,11 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
                 e.setHasBeenReported(true); // only intended as an exception message, not something to report to ErrorListener
                 throw e;
             }
-            jitCompilationDone = true;
         }
 
     }
 
+
     /**
      * Registers the template rule with each Mode that it belongs to.
      *
@@ -885,10 +884,7 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
                     if (match2 != match1) {
                         ContextItemStaticInfo info = getConfiguration().makeContextItemStaticInfo(AnyItemType.getInstance(), false);
                         ExpressionTool.copyLocationInfo(match, match2);
-                        //match2.setPackageData(match.getPackageData());
                         match2.setOriginalText(match.toString());
-                        //match2.setSystemId(match.getSystemId());
-                        //match2.setLineNumber(match.getLineNumber());
                         match2 = match2.typeCheck(visitor, info);
                         match1 = match2;
                     }
@@ -912,12 +908,16 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
                 if (mode.isDeclaredStreamable()) {
                     rule.setDeclaredStreamable(true);
                     if (!match1.isMotionless()) {
-                        boolean fallback = getConfiguration().getBooleanProperty(FeatureKeys.STREAMING_FALLBACK);
+                        boolean fallback = getConfiguration().getBooleanProperty(Feature.STREAMING_FALLBACK);
                         String message = "Template rule is declared streamable but the match pattern is not motionless";
                         if (fallback) {
-                            compileWarning("Streaming fallback is not available (see issue 3584)", "SXWN0001");
+                            message += "\n  * Falling back to non-streaming implementation";
+                            getStaticContext().issueWarning(message, this);
+                            rule.setDeclaredStreamable(false);
+                            getCompilation().setFallbackToNonStreaming(true);
+                        } else {
+                            throw new XPathException(message, "XTSE3430", this);
                         }
-                        throw new XPathException(message, "XTSE3430", this);
                     }
                 }
 
@@ -946,11 +946,9 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
 
     /**
      * Allocate slot numbers to any local variables declared within a predicate within the match pattern
-     *
-     * @throws XPathException if a failure occurs
      */
 
-    public void allocatePatternSlotNumbers() throws XPathException {
+    public void allocatePatternSlotNumbers() {
         if (match != null) {
             for (TemplateRule templateRule : compiledTemplateRules.values()) {
                 for (Rule r : templateRule.getRules()) {
@@ -998,7 +996,7 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
      *
      * @param declaration Associates this template with a stylesheet module (in principle an xsl:template
      *                    element can be in a document that is imported more than once; these are separate declarations)
-     * @throws XPathException
+     * @throws XPathException if errors are found
      */
 
     public void optimize(ComponentDeclaration declaration) throws XPathException {
@@ -1028,25 +1026,16 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
             }
         }
         if (match != null) {
-            ItemType contextItemType = match.getItemType();
-            if (contextItemType.equals(ErrorType.getInstance())) {
-                // if the match pattern can't match anything, we produce a warning, not a hard error
-                contextItemType = AnyItemType.getInstance();
-            }
+            ItemType contextItemType = getContextItemTypeForTemplateRule();
             ContextItemStaticInfo cisi = config.makeContextItemStaticInfo(contextItemType, false);
-            // TODO: use the xsl:context-item declaration within the template for additional type information
             cisi.setContextPostureStriding();
             ExpressionVisitor visitor = makeExpressionVisitor();
             match.resetLocalStaticProperties();
             match = match.optimize(visitor, cisi);
 
             if (!isDeferredCompilation(getCompilation())) {
-                Expression body = null;
-                for (TemplateRule templateRule : compiledTemplateRules.values()) {
-                    // Until now, all template rules share the same body.
-                    body = templateRule.getBody();
-                    break;
-                }
+                Expression body = compiledTemplateRules.values().stream().findFirst().map(TemplateRule::getBody).orElse(null);
+                // Until now, all template rules share the same body.
 
                 ExpressionTool.resetPropertiesWithinSubtree(body);
                 //        visitor.setOptimizeForStreaming(compiledNamedTemplate.isDeclaredStreamable());
@@ -1061,6 +1050,7 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
                     for (TemplateRule compiledTemplateRule : compiledTemplateRules.values()) {
                         Expression templateRuleBody = compiledTemplateRules.size() > 1 ? body.copy(new RebindingMap()) : body;
                         visitor.setOptimizeForStreaming(compiledTemplateRule.isDeclaredStreamable());
+                        templateRuleBody = templateRuleBody.typeCheck(visitor, cisi);
                         templateRuleBody = ExpressionTool.optimizeComponentBody(templateRuleBody, getCompilation(), visitor, cisi, true);
                         compiledTemplateRule.setBody(templateRuleBody);
                         opt.checkStreamability(this, compiledTemplateRule);
@@ -1099,6 +1089,33 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
 
     }
 
+    public ItemType getContextItemTypeForTemplateRule() throws XPathException {
+        Configuration config = getConfiguration();
+        ItemType contextItemType = match.getItemType();
+        if (contextItemType.equals(ErrorType.getInstance())) {
+            // if the match pattern can't match anything, we produce a warning, not a hard error
+            contextItemType = AnyItemType.getInstance();
+        }
+        if (requiredContextItemType != AnyItemType.getInstance()) {
+            int rel = config.getTypeHierarchy().relationship(contextItemType, requiredContextItemType);
+            switch (rel) {
+                case TypeHierarchy.DISJOINT:
+                    XPathException e = new XPathException("The declared context item type is inconsistent with the match pattern", "XPTY0004", this);
+                    e.setIsTypeError(true);
+                    throw e;
+                case TypeHierarchy.SUBSUMED_BY:
+                case TypeHierarchy.OVERLAPS:
+                case TypeHierarchy.SAME_TYPE:
+                    // no action
+                    break;
+                case TypeHierarchy.SUBSUMES:
+                    contextItemType = requiredContextItemType;
+                    break;
+            }
+        }
+        return contextItemType;
+    }
+
 
     /**
      * Generate byte code for the template (if appropriate)
