@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -34,7 +34,6 @@ import java.util.Map;
 /**
  * An abstract and configurable implementation of the StaticContext interface,
  * which defines the static context of an XPath expression.
- * <p/>
  * <p>This class implements those parts of the functionality of a static context
  * that tend to be common to most implementations: simple-valued properties such
  * as base URI and default element namespace; availability of the standard
@@ -132,17 +131,25 @@ public abstract class AbstractStaticContext implements StaticContext {
     }
 
     public final void setDefaultFunctionLibrary(int version) {
-        if (version == 20) {
-            FunctionLibraryList lib = new FunctionLibraryList();
-            lib.addFunctionLibrary(XPath20FunctionSet.getInstance());
-            lib.addFunctionLibrary(getConfiguration().getBuiltInExtensionLibraryList());
-            lib.addFunctionLibrary(new ConstructorFunctionLibrary(getConfiguration()));
-            lib.addFunctionLibrary(config.getIntegratedFunctionLibrary());
-            config.addExtensionBinders(lib);
-            setFunctionLibrary(lib);
-        } else {
-            setDefaultFunctionLibrary();
+        FunctionLibraryList lib = new FunctionLibraryList();
+        switch (version) {
+            case 20:
+            default:
+                lib.addFunctionLibrary(XPath20FunctionSet.getInstance());
+                break;
+            case 30:
+            case 305:
+                lib.addFunctionLibrary(config.getXPath30FunctionSet());
+                break;
+            case 31:
+                lib.addFunctionLibrary(config.getXPath31FunctionSet());
+                break;
         }
+        lib.addFunctionLibrary(getConfiguration().getBuiltInExtensionLibraryList());
+        lib.addFunctionLibrary(new ConstructorFunctionLibrary(getConfiguration()));
+        lib.addFunctionLibrary(config.getIntegratedFunctionLibrary());
+        config.addExtensionBinders(lib);
+        setFunctionLibrary(lib);
     }
     /**
      * Add a function library to the list of function libraries
@@ -352,11 +359,12 @@ public abstract class AbstractStaticContext implements StaticContext {
     }
 
     /**
-     * Get the XPath language level supported, as an integer value.
-     * The current levels supported are 20 ("2.0"), and 31 ("3.1"). The default is "3.1".
+     * Get the XPath language level supported, as an integer (being the actual version
+     * number times ten). In Saxon 9.9 the possible values are 20 (XPath 2.0), 30 (XPath 3.0),
+     * 31 (XPath 3.1), and 305 (XPath 3.0 plus the extensions defined in XSLT 3.0).
      *
-     * @return the XPath language level
-     * @since 9.3
+     * @return the XPath language level; the return value will be either 20, 30, 305, or 31
+     * @since 9.7
      */
 
     public int getXPathVersion() {
