@@ -1,8 +1,8 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// This Source Code Form is “Incompatible With Secondary Licenses”, as defined by the Mozilla Public License, v. 2.0.
+// This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 package net.sf.saxon.testdriver;
@@ -61,7 +61,7 @@ public abstract class TestReport {
     public void startTestSetElement(XdmNode funcSetNode) {
         try {
             results.writeStartElement(getReportNamespace(), "test-set");
-            results.writeAttribute("name", funcSetNode.getAttributeValue(new QName("name")));
+            results.writeAttribute("name", funcSetNode.attribute("name"));
         } catch (XMLStreamException e) {
             throw new RuntimeException(e);
         }
