@@ -347,8 +347,8 @@ public class Xslt30TestSuiteDriverHE extends TestDriver {
             compiler.setErrorListener(collector);
             compiler.clearParameters();
             testInput.select(child("param").where(isTrue("static"))).forEach(param -> {
-                String name = ((XdmNode) param).attribute("name");
-                String select = ((XdmNode) param).attribute("select");
+                String name = param.attribute("name");
+                String select = param.attribute("select");
                 XdmValue value;
                 try {
                     value = xpath.evaluate(select, null);
@@ -557,8 +557,8 @@ public class Xslt30TestSuiteDriverHE extends TestDriver {
                 }
             }
             testInput.select(child("param").where(not(isTrue("static")))).forEach(param -> {
-                String name = ((XdmNode) param).attribute("name");
-                String select = ((XdmNode) param).attribute("select");
+                String name = param.attribute("name");
+                String select = param.attribute("select");
                 XdmValue value;
                 try {
                     value = xpath.evaluate(select, null);
@@ -710,9 +710,9 @@ public class Xslt30TestSuiteDriverHE extends TestDriver {
             }
 
             Map<QName, XdmValue> caseGlobalParams = getNamedParameters(xpath, testInput, false, false);
-            Map<QName, XdmValue> caseStaticParams = getNamedParameters(xpath, testInput, true, false);
-            Map<QName, XdmValue> globalParams = new HashMap<QName, XdmValue>(env.params);
-            globalParams.putAll(caseStaticParams);
+            //Map<QName, XdmValue> caseStaticParams = getNamedParameters(xpath, testInput, true, false);
+            Map<QName, XdmValue> globalParams = new HashMap<>(env.params);
+            //globalParams.putAll(caseStaticParams);
             globalParams.putAll(caseGlobalParams);  // has to be this way to ensure test-local overrides global
             transformer.setStylesheetParameters(globalParams);
 
@@ -994,14 +994,18 @@ public class Xslt30TestSuiteDriverHE extends TestDriver {
      * @throws SaxonApiException
      */
     protected Map<QName, XdmValue> getNamedParameters(XPathCompiler xpath, XdmNode node, boolean getStatic, boolean tunnel) throws SaxonApiException {
-        Map<QName, XdmValue> params = new HashMap<QName, XdmValue>();
+        Map<QName, XdmValue> params = new HashMap<>();
         int i = 1;
-        String staticTest = getStatic ? "@static='yes'" : "not(@static='yes')";
-        for (XdmItem param : xpath.evaluate("param[" + staticTest + "]", node)) {
-            QName name = getQNameAttribute(xpath, (XdmNode)param, attribute("name"));
-            String select = ((XdmNode) param).attribute("select");
-            String tunnelled = ((XdmNode) param).attribute("tunnel");
-            QName as = getQNameAttribute(xpath, (XdmNode)param, attribute("as"));
+        Predicate<? super XdmNode> staticTest = isTrue("static");
+        if (!getStatic) {
+            staticTest = not(staticTest);
+        }
+        List<XdmNode> paramElements = node.select(child("param").where(staticTest)).asListOfNodes();
+        for (XdmNode param : paramElements) {
+            QName name = getQNameAttribute(xpath, param, attribute("name"));
+            String select = param.attribute("select");
+            String tunnelled = param.attribute("tunnel");
+            QName as = getQNameAttribute(xpath, param, attribute("as"));
                 // TODO: it won't always be a QName, could be a sequence type
             boolean required = tunnel == (tunnelled != null && tunnelled.equals("yes"));
             XdmValue value;
@@ -1035,10 +1039,10 @@ public class Xslt30TestSuiteDriverHE extends TestDriver {
      * @return Array of the parameter values
      */
     protected XdmValue[] getParameters(XPathCompiler xpath, XdmNode node) {
-        ArrayList<XdmValue> params = new ArrayList<XdmValue>();
+        ArrayList<XdmValue> params = new ArrayList<>();
 
         node.select(child("param").where(not(isTrue("static")))).forEach(param -> {
-            String select = ((XdmNode) param).attribute("select");
+            String select = param.attribute("select");
             XdmValue value;
             try {
                 value = xpath.evaluate(select, null);
@@ -1048,7 +1052,7 @@ public class Xslt30TestSuiteDriverHE extends TestDriver {
             }
             params.add(value);
         });
-        return params.toArray(new XdmValue[params.size()]);
+        return params.toArray(new XdmValue[0]);
     }
 
     @SuppressWarnings({"ResultOfMethodCallIgnored"})
@@ -1309,14 +1313,14 @@ public class Xslt30TestSuiteDriverHE extends TestDriver {
                 env.xsltCompiler.getUnderlyingCompilerInfo().setRecoveryPolicy(Configuration.RECOVER_SILENTLY);
             }
             return true;
-        } else if ("ignore-doc-failure".equals(type)) {
+        } else if ("ignore_doc_failure".equals(type)) {
             env.resetActions.add(new Environment.ResetAction() {
                 @Override
                 public void reset(Environment env) {
                     env.xsltCompiler.getUnderlyingCompilerInfo().setRecoveryPolicy(Configuration.RECOVER_WITH_WARNINGS);
                 }
             });
-            if (value.equals("false")) {
+            if (inverse) {
                 env.xsltCompiler.getUnderlyingCompilerInfo().setRecoveryPolicy(Configuration.DO_NOT_RECOVER);
             } else {
                 env.xsltCompiler.getUnderlyingCompilerInfo().setRecoveryPolicy(Configuration.RECOVER_SILENTLY);
