@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,6 +11,7 @@ import net.sf.saxon.Configuration;
 import net.sf.saxon.event.CheckSumFilter;
 import net.sf.saxon.event.NamespaceReducer;
 import net.sf.saxon.event.Receiver;
+import net.sf.saxon.expr.Component;
 import net.sf.saxon.expr.Expression;
 import net.sf.saxon.expr.parser.ExplicitLocation;
 import net.sf.saxon.expr.parser.RetainedStaticContext;
@@ -18,7 +19,9 @@ import net.sf.saxon.lib.Logger;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.lib.SaxonOutputKeys;
 import net.sf.saxon.om.*;
+import net.sf.saxon.serialize.SerializationProperties;
 import net.sf.saxon.serialize.charcode.UTF16CharacterSet;
+import net.sf.saxon.style.StylesheetPackage;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.util.FastStringBuffer;
 import net.sf.saxon.type.BuiltInAtomicType;
@@ -28,7 +31,9 @@ import net.sf.saxon.value.Whitespace;
 
 import javax.xml.transform.OutputKeys;
 import javax.xml.transform.stream.StreamResult;
-import java.util.*;
+import java.util.Iterator;
+import java.util.Map;
+import java.util.Stack;
 
 /**
  * This class handles the display of an abstract expression tree in an XML format
@@ -44,7 +49,7 @@ public class ExpressionPresenter {
     private Stack<Expression> expressionStack = new Stack<Expression>();
     private Stack<String> nameStack = new Stack<String>();
     private String defaultNamespace;
-    private Map<String, String> options = new HashMap<String, String>();
+    private Options options = new ExportOptions();
     private boolean relocatable = false;
 
     /**
@@ -98,19 +103,16 @@ public class ExpressionPresenter {
      */
 
     public void init(Configuration config, StreamResult out, boolean checksum) {
-        Properties props = makeDefaultProperties(config);
+        SerializationProperties props = makeDefaultProperties(config);
         if (config.getXMLVersion() == Configuration.XML11) {
-            if ("JS".equals(getOption("target"))) {
+            if ("JS".equals(((ExportOptions) getOptions()).target)) {
                 config.getLogger().warning("For target=JS, the SEF file will use XML 1.0, which disallows control characters");
             } else {
                 props.setProperty(OutputKeys.VERSION, "1.1");
             }
         }
         try {
-            receiver = config.getSerializerFactory().getReceiver(
-                    out,
-                    config.makePipelineConfiguration(),
-                    props);
+            receiver = config.getSerializerFactory().getReceiver(out, props);
             receiver = new NamespaceReducer(receiver);
             if (checksum) {
                 receiver = new CheckSumFilter(receiver);
@@ -140,11 +142,11 @@ public class ExpressionPresenter {
 
     public void init(Configuration config, Receiver out, boolean checksum) {
 
-            receiver = out;
-            receiver = new NamespaceReducer(receiver);
-            if (checksum) {
-                receiver = new CheckSumFilter(receiver);
-            }
+        receiver = out;
+        receiver = new NamespaceReducer(receiver);
+        if (checksum) {
+            receiver = new CheckSumFilter(receiver);
+        }
 
         this.config = config;
         try {
@@ -200,47 +202,21 @@ public class ExpressionPresenter {
     }
 
     /**
-     * Set an option (a keyword=value pair, where both are strings)
-     *
-     * @param key   the option name
-     * @param value the option value
+     * Set the options
      */
 
-    public void setOption(String key, String value) {
-        options.put(key, value);
+    public void setOptions(Options options) {
+        this.options = options;
     }
 
     /**
-     * Get the value of an option
+     * Get the options
      *
-     * @param key the option name
-     * @return the option value, or null if the option has not been set
+     * @return the options, or null if none have been set
      */
 
-    public String getOption(String key) {
-        return options.get(key);
-    }
-
-    /**
-     * Get the value of an integer-valued option
-     *
-     * @param key the option name
-     * @param defaultValue the default value to be used if the option is not present or if it cannot
-     *                be parsed as an integer
-     * @return the option value as an integer, or -1 if the option
-     */
-
-    public int getIntOption(String key, int defaultValue) {
-        String s = options.get(key);
-        if (s != null) {
-            try {
-                return Integer.parseInt(s);
-            } catch (NumberFormatException err) {
-                return defaultValue;
-            }
-        } else {
-            return defaultValue;
-        }
+    public Options getOptions() {
+        return options;
     }
 
     /**
@@ -278,11 +254,8 @@ public class ExpressionPresenter {
 
     /*@Nullable*/
     public static Receiver defaultDestination(/*@NotNull*/ Configuration config, Logger out) throws XPathException {
-        Properties props = makeDefaultProperties(config);
-        return config.getSerializerFactory().getReceiver(
-                out.asStreamResult(),
-                config.makePipelineConfiguration(),
-                props);
+        SerializationProperties props = makeDefaultProperties(config);
+        return config.getSerializerFactory().getReceiver(out.asStreamResult(), props);
     }
 
 
@@ -293,12 +266,14 @@ public class ExpressionPresenter {
      */
 
     /*@NotNull*/
-    public static Properties makeDefaultProperties(Configuration config) {
-        Properties props = new Properties();
+    public static SerializationProperties makeDefaultProperties(Configuration config) {
+        SerializationProperties props = new SerializationProperties();
         props.setProperty(OutputKeys.METHOD, "xml");
         props.setProperty(OutputKeys.INDENT, "yes");
-        props.setProperty(SaxonOutputKeys.INDENT_SPACES, "1");
-        props.setProperty(SaxonOutputKeys.LINE_LENGTH, "4096");
+        if (config.isLicensedFeature(Configuration.LicenseFeature.PROFESSIONAL_EDITION)) {
+            props.setProperty(SaxonOutputKeys.INDENT_SPACES, "1");
+            props.setProperty(SaxonOutputKeys.LINE_LENGTH, "4096");
+        }
         props.setProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
         props.setProperty(OutputKeys.ENCODING, "utf-8");
         props.setProperty(OutputKeys.VERSION, "1.0");
@@ -469,26 +444,45 @@ public class ExpressionPresenter {
      */
 
     public void emitAttribute(String name, StructuredQName value) {
+        String attVal = value.getEQName();
+//        if (value.hasURI("") && value.getPrefix().isEmpty()) {
+//            attVal = value.getLocalPart();
+//        } else {
+//            String uri = value.getURI();
+//            String prefix = value.getPrefix();
+//            if (uri.equals(NamespaceConstant.getUriForConventionalPrefix(prefix))) {
+//                uri = "~";
+//            }
+//            attVal = prefix + "{" + uri + "}" + value.getLocalPart();
+//        }
         try {
-            if (value.getURI().isEmpty()) {
-                receiver.attribute(new NoNamespaceName(name), BuiltInAtomicType.UNTYPED_ATOMIC, value.getLocalPart(), ExplicitLocation.UNKNOWN_LOCATION, 0);
-            } else if (value.getPrefix().isEmpty()) {
-                String prefix = config.getNamePool().suggestPrefixForURI(value.getURI());
-                if (prefix == null) {
-                    prefix = allocatePrefix(value.getURI());
-                }
-                receiver.namespace(new NamespaceBinding(prefix, value.getURI()), 0);
-                receiver.attribute(new NoNamespaceName(name),
-                                   BuiltInAtomicType.UNTYPED_ATOMIC, prefix + ":" + value.getLocalPart(), ExplicitLocation.UNKNOWN_LOCATION, 0);
-            } else {
-                receiver.namespace(new NamespaceBinding(value.getPrefix(), value.getURI()), 0);
-                receiver.attribute(new NoNamespaceName(name),
-                                   BuiltInAtomicType.UNTYPED_ATOMIC, value.getDisplayName(), ExplicitLocation.UNKNOWN_LOCATION, 0);
-            }
+            receiver.attribute(new NoNamespaceName(name), BuiltInAtomicType.UNTYPED_ATOMIC,
+                               attVal, ExplicitLocation.UNKNOWN_LOCATION, 0);
         } catch (XPathException err) {
             err.printStackTrace();
             throw new InternalError(err.getMessage());
         }
+//
+//        try {
+//            if (value.hasURI("")) {
+//                receiver.attribute(new NoNamespaceName(name), BuiltInAtomicType.UNTYPED_ATOMIC, value.getLocalPart(), ExplicitLocation.UNKNOWN_LOCATION, 0);
+//            } else if (value.getPrefix().isEmpty()) {
+//                String prefix = config.getNamePool().suggestPrefixForURI(value.getURI());
+//                if (prefix == null) {
+//                    prefix = allocatePrefix(value.getURI());
+//                }
+//                receiver.namespace(new NamespaceBinding(prefix, value.getURI()), 0);
+//                receiver.attribute(new NoNamespaceName(name),
+//                                   BuiltInAtomicType.UNTYPED_ATOMIC, prefix + ":" + value.getLocalPart(), ExplicitLocation.UNKNOWN_LOCATION, 0);
+//            } else {
+//                receiver.namespace(new NamespaceBinding(value.getPrefix(), value.getURI()), 0);
+//                receiver.attribute(new NoNamespaceName(name),
+//                                   BuiltInAtomicType.UNTYPED_ATOMIC, value.getDisplayName(), ExplicitLocation.UNKNOWN_LOCATION, 0);
+//            }
+//        } catch (XPathException err) {
+//            err.printStackTrace();
+//            throw new InternalError(err.getMessage());
+//        }
     }
 
     private String allocatePrefix(String uri) {
@@ -671,5 +665,15 @@ public class ExpressionPresenter {
         }
         return out.toString();
     }
+
+    public interface Options {};
+
+    public static class ExportOptions implements Options {
+        public String target;
+        public int targetVersion;
+        public StylesheetPackage rootPackage;
+        public Map<Component, Integer> componentMap;
+        public Map<StylesheetPackage, Integer> packageMap;
+    }
 }
 
