@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -40,7 +40,7 @@ public class SimpleMode extends Mode {
     // TODO:PERF the data structure does not cater well for a stylesheet making heavy use of
     // match="schema-element(X)". We should probably expand the substitution group.
 
-    protected RuleChain genericRuleChain = new RuleChain();
+    protected final RuleChain genericRuleChain = new RuleChain();
     protected RuleChain atomicValueRuleChain = new RuleChain();
     protected RuleChain functionItemRuleChain = new RuleChain();
     protected RuleChain documentRuleChain = new RuleChain();
@@ -50,8 +50,8 @@ public class SimpleMode extends Mode {
     protected RuleChain namespaceRuleChain = new RuleChain();
     protected RuleChain unnamedElementRuleChain = new RuleChain();
     protected RuleChain unnamedAttributeRuleChain = new RuleChain();
-    protected IntHashMap<RuleChain> namedElementRuleChains = new IntHashMap<RuleChain>(32);
-    protected IntHashMap<RuleChain> namedAttributeRuleChains = new IntHashMap<RuleChain>(8);
+    protected IntHashMap<RuleChain> namedElementRuleChains = new IntHashMap<>(32);
+    protected IntHashMap<RuleChain> namedAttributeRuleChains = new IntHashMap<>(8);
     protected Map<StructuredQName, RuleChain> qNamedElementRuleChains;
     protected Map<StructuredQName, RuleChain> qNamedAttributeRuleChains;
 
@@ -62,8 +62,8 @@ public class SimpleMode extends Mode {
     private int highestRank;
 
 
-    private Map<String, Integer> explicitPropertyPrecedences = new HashMap<String, Integer>();
-    private Map<String, String> explicitPropertyValues = new HashMap<String, String>();
+    private Map<String, Integer> explicitPropertyPrecedences = new HashMap<>();
+    private Map<String, String> explicitPropertyValues = new HashMap<>();
 
 
 
@@ -132,20 +132,28 @@ public class SimpleMode extends Mode {
                 mustBeUntyped = "no".equals(value);
             } else if (prop.equals("on-no-match")) {
                 BuiltInRuleSet base = null;
-                if (value.equals("text-only-copy")) {
-                    base = TextOnlyCopyRuleSet.getInstance();
-                } else if (value.equals("shallow-copy")) {
-                     base = ShallowCopyRuleSet.getInstance();
-                } else if (value.equals("deep-copy")) {
-                     base = DeepCopyRuleSet.getInstance();
-                } else if (value.equals("shallow-skip")) {
-                     base = ShallowSkipRuleSet.getInstance();
-                } else if (value.equals("deep-skip")) {
-                    base = DeepSkipRuleSet.getInstance();
-                } else if (value.equals("fail")) {
-                    base = FailRuleSet.getInstance();
-                } else {
-                    // already validated
+                switch (value) {
+                    case "text-only-copy":
+                        base = TextOnlyCopyRuleSet.getInstance();
+                        break;
+                    case "shallow-copy":
+                        base = ShallowCopyRuleSet.getInstance();
+                        break;
+                    case "deep-copy":
+                        base = DeepCopyRuleSet.getInstance();
+                        break;
+                    case "shallow-skip":
+                        base = ShallowSkipRuleSet.getInstance();
+                        break;
+                    case "deep-skip":
+                        base = DeepSkipRuleSet.getInstance();
+                        break;
+                    case "fail":
+                        base = FailRuleSet.getInstance();
+                        break;
+                    default:
+                        // already validated
+                        break;
                 }
                 if ("yes".equals(explicitPropertyValues.get("warning-on-no-match"))) {
                     base = new RuleSetWithWarnings(base);
@@ -175,11 +183,9 @@ public class SimpleMode extends Mode {
 
     public static void copyRules(final SimpleMode from, final SimpleMode to) {
         try {
-            from.processRules(new RuleAction() {
-                public void processRule(Rule r) throws XPathException {
-                    Rule r2 = r.copy(false);
-                    to.addRule(r2.getPattern(), r2);
-                }
+            from.processRules(r -> {
+                Rule r2 = r.copy(false);
+                to.addRule(r2.getPattern(), r2);
             });
         } catch (XPathException e) {
             throw new AssertionError(e);
@@ -195,7 +201,7 @@ public class SimpleMode extends Mode {
      * @return a new object capable of holding the state of a search for a rule
      */
     protected RuleSearchState makeRuleSearchState(RuleChain chain, XPathContext context) {
-        return new RuleSearchState(this);
+        return new RuleSearchState();
     }
 
     /**
@@ -221,10 +227,10 @@ public class SimpleMode extends Mode {
     public void setExplicitProperty(String name, String value, int precedence) {
         Integer p = explicitPropertyPrecedences.get(name);
         if (p != null) {
-            if (p != null && p < precedence) {
+            if (p < precedence) {
                 explicitPropertyPrecedences.put(name, precedence);
                 explicitPropertyValues.put(name, value);
-            } else if (p != null && p == precedence) {
+            } else if (p == precedence) {
                 String v = explicitPropertyValues.get(name);
                 if (v != null & !v.equals(value)) {
                     // We don't throw an exception, because the conflict is an error only if this
@@ -267,7 +273,7 @@ public class SimpleMode extends Mode {
 
     @Override
     public Set<String> getExplicitNamespaces(NamePool pool) {
-        Set<String> namespaces = new HashSet<String>();
+        Set<String> namespaces = new HashSet<>();
         IntIterator ii = namedElementRuleChains.keyIterator();
         while (ii.hasNext()) {
             int fp = ii.next();
@@ -359,30 +365,12 @@ public class SimpleMode extends Mode {
         UType uType = pattern.getUType();
         if (uType.equals(UType.ELEMENT)) {
             int fp = pattern.getFingerprint();
-            if (fp == -1) {
-                addRuleToList(newRule, unnamedElementRuleChain);
-            } else {
-                RuleChain chain = namedElementRuleChains.get(fp);
-                if (chain == null) {
-                    chain = new RuleChain(newRule);
-                    namedElementRuleChains.put(fp, chain);
-                } else {
-                    addRuleToList(newRule, chain);
-                }
-            }
+            addRuleToNamedOrUnnamedChain(
+                    newRule, fp, unnamedElementRuleChain, namedElementRuleChains);
         } else if (uType.equals(UType.ATTRIBUTE)) {
             int fp = pattern.getFingerprint();
-            if (fp == -1) {
-                addRuleToList(newRule, unnamedAttributeRuleChain);
-            } else {
-                RuleChain chain = namedAttributeRuleChains.get(fp);
-                if (chain == null) {
-                    chain = new RuleChain(newRule);
-                    namedAttributeRuleChains.put(fp, chain);
-                } else {
-                    addRuleToList(newRule, chain);
-                }
-            }
+            addRuleToNamedOrUnnamedChain(
+                    newRule, fp, unnamedAttributeRuleChain, namedAttributeRuleChains);
         } else if (uType.equals(UType.DOCUMENT)) {
             addRuleToList(newRule, documentRuleChain);
         } else if (uType.equals(UType.TEXT)) {
@@ -393,23 +381,26 @@ public class SimpleMode extends Mode {
             addRuleToList(newRule, processingInstructionRuleChain);
         } else if (uType.equals(UType.NAMESPACE)) {
             addRuleToList(newRule, namespaceRuleChain);
-        } else if (UType.ANY_NODE.subsumes(uType)) {
-            addRuleToList(newRule, genericRuleChain);
+        } else if (UType.ANY_ATOMIC.subsumes(uType)) {
+            addRuleToList(newRule, atomicValueRuleChain);
+        } else if (UType.FUNCTION.subsumes(uType)) {
+            addRuleToList(newRule, functionItemRuleChain);
         } else {
-            // TODO: we really shouldn't have to enumerate all possible kinds of pattern here!
-            if (pattern instanceof NodeTestPattern || pattern instanceof BasePatternWithPredicate) {
-                ItemType type = pattern.getItemType();
-                if (type instanceof AtomicType) {
-                    addRuleToList(newRule, atomicValueRuleChain);
-                } else if (type instanceof FunctionItemType) {
-                    addRuleToList(newRule, functionItemRuleChain);
-                } else {
-                    throw new UnsupportedOperationException("XSLT 3.0 '.[]' syntax not recognized with node type or external object types");
-                }
-            } else if (pattern instanceof BooleanExpressionPattern || pattern instanceof UniversalPattern) {
-                addRuleToList(newRule, genericRuleChain);
+            addRuleToList(newRule, genericRuleChain);
+        }
+    }
+
+    protected void addRuleToNamedOrUnnamedChain(
+            Rule newRule, int fp, RuleChain unnamedRuleChain, IntHashMap<RuleChain> namedRuleChains) {
+        if (fp == -1) {
+            addRuleToList(newRule, unnamedRuleChain);
+        } else {
+            RuleChain chain = namedRuleChains.get(fp);
+            if (chain == null) {
+                chain = new RuleChain(newRule);
+                namedRuleChains.put(fp, chain);
             } else {
-                throw new UnsupportedOperationException("Unrecognized pattern " + pattern.getClass());
+                addRuleToList(newRule, chain);
             }
         }
     }
@@ -419,8 +410,6 @@ public class SimpleMode extends Mode {
      *
      * @param newRule the new rule to be added into the list
      * @param list    the Rule at the head of the list, or null if the list is empty
-     * @return the new head of the list (which might be the old head, or the new rule if it
-     * was inserted at the start)
      */
 
 
@@ -448,6 +437,7 @@ public class SimpleMode extends Mode {
                 }
             }
             if (rule == null) {
+                assert prev != null;
                 prev.setNext(newRule);
                 newRule.setNext(null);
             }
@@ -552,25 +542,19 @@ public class SimpleMode extends Mode {
 
             // Search the list for rules for nodes of unknown node kind
 
-            if (genericRuleChain != null) {
-                bestRule = searchRuleChain(node, context, bestRule, genericRuleChain);
-            }
+            bestRule = searchRuleChain(node, context, bestRule, genericRuleChain);
 
         } else if (item instanceof AtomicValue) {
             if (atomicValueRuleChain != null) {
                 bestRule = searchRuleChain(item, context, bestRule, atomicValueRuleChain);
             }
-            if (genericRuleChain != null) {
-                bestRule = searchRuleChain(item, context, bestRule, genericRuleChain);
-            }
+            bestRule = searchRuleChain(item, context, bestRule, genericRuleChain);
 
         } else if (item instanceof Function) {
             if (functionItemRuleChain != null) {
                 bestRule = searchRuleChain(item, context, bestRule, functionItemRuleChain);
             }
-            if (genericRuleChain != null) {
-                bestRule = searchRuleChain(item, context, bestRule, genericRuleChain);
-            }
+            bestRule = searchRuleChain(item, context, bestRule, genericRuleChain);
         }
 
         return bestRule;
@@ -592,8 +576,8 @@ public class SimpleMode extends Mode {
         // to the rule chains based on StructuredQName rather than fingerprint
         synchronized(this) {
             if (qNamedElementRuleChains == null) {
-                qNamedElementRuleChains = new HashMap<StructuredQName, RuleChain>(namedElementRuleChains.size());
-                qNamedAttributeRuleChains = new HashMap<StructuredQName, RuleChain>(namedAttributeRuleChains.size());
+                qNamedElementRuleChains = new HashMap<>(namedElementRuleChains.size());
+                qNamedAttributeRuleChains = new HashMap<>(namedAttributeRuleChains.size());
                 NamePool pool = c.getNamePool();
                 indexByQName(pool, namedElementRuleChains, qNamedElementRuleChains);
                 indexByQName(pool, namedAttributeRuleChains, qNamedAttributeRuleChains);
@@ -679,7 +663,7 @@ public class SimpleMode extends Mode {
      * @param context the static context for evaluation
      * @param pre     An appropriate matcher for preconditions in this mode
      * @return true if this rule does match
-     * @throws XPathException
+     * @throws XPathException if a dynamic error occurs while matching the pattern
      */
 
     protected boolean ruleMatches(Rule r, Item item, XPathContextMajor context, RuleSearchState pre) throws XPathException {
@@ -785,10 +769,8 @@ public class SimpleMode extends Mode {
                 ruleSearchState = makeRuleSearchState(atomicValueRuleChain, context);
                 bestRule = searchRuleChain(item, context, bestRule, atomicValueRuleChain, ruleSearchState, filter);
             }
-            if (genericRuleChain != null) {
-                ruleSearchState = makeRuleSearchState(genericRuleChain, context);
-                bestRule = searchRuleChain(item, context, bestRule, genericRuleChain, ruleSearchState, filter);
-            }
+            ruleSearchState = makeRuleSearchState(genericRuleChain, context);
+            bestRule = searchRuleChain(item, context, bestRule, genericRuleChain, ruleSearchState, filter);
             return bestRule;
 
         } else if (item instanceof Function) {
@@ -796,10 +778,8 @@ public class SimpleMode extends Mode {
                 ruleSearchState = makeRuleSearchState(functionItemRuleChain, context);
                 bestRule = searchRuleChain(item, context, bestRule, functionItemRuleChain, ruleSearchState, filter);
             }
-            if (genericRuleChain != null) {
-                ruleSearchState = makeRuleSearchState(genericRuleChain, context);
-                bestRule = searchRuleChain(item, context, bestRule, genericRuleChain, ruleSearchState, filter);
-            }
+            ruleSearchState = makeRuleSearchState(genericRuleChain, context);
+            bestRule = searchRuleChain(item, context, bestRule, genericRuleChain, ruleSearchState, filter);
             return bestRule;
 
         } else {
@@ -888,7 +868,7 @@ public class SimpleMode extends Mode {
         if (item instanceof NodeInfo) {
             path = Navigator.getPath((NodeInfo) item);
         } else {
-            path = Err.depict(item).toString();
+            path = item.toShortString();
         }
 
         Pattern pat1 = r1.getPattern();
@@ -924,7 +904,7 @@ public class SimpleMode extends Mode {
     /**
      * Prepare for possible streamability - null here, but can be subclassed
      *
-     * @throws XPathException
+     * @throws XPathException if a failure occurs
      */
     public void prepareStreamability() throws XPathException {
     }
@@ -945,22 +925,21 @@ public class SimpleMode extends Mode {
 
     public static void forceAllocateAllBindingSlots(
             final StylesheetPackage pack, final SimpleMode mode, final List<ComponentBinding> bindings) {
-        final Set<TemplateRule> rulesProcessed = new HashSet<TemplateRule>();
-        final IdentityHashMap<Pattern, Boolean> patternsProcessed = new IdentityHashMap<Pattern, Boolean>();
+        final Set<TemplateRule> rulesProcessed = new HashSet<>();
+        final IdentityHashMap<Pattern, Boolean> patternsProcessed = new IdentityHashMap<>();
         try {
-            mode.processRules(new RuleAction() {
-                public void processRule(Rule r) {
-                    Pattern pattern = r.getPattern();
-                    if (!patternsProcessed.containsKey(pattern)) {
-                        allocateBindingSlotsRecursive(pack, mode, pattern, bindings);
-                        patternsProcessed.put(pattern, true);
-                    }
-                    TemplateRule tr = (TemplateRule) r.getAction();
-                    if (tr.getBody() != null && !rulesProcessed.contains(tr)) {
-                        // A rule can appear twice if it uses a union pattern; only process it once
-                        allocateBindingSlotsRecursive(pack, mode, tr.getBody(), bindings);
-                        rulesProcessed.add(tr);
-                    }
+            mode.processRules(r -> {
+                // A rule can appear twice, for example at different import precedences or
+                // because the match pattern is a union pattern; only allocate slots once
+                Pattern pattern = r.getPattern();
+                if (!patternsProcessed.containsKey(pattern)) {
+                    allocateBindingSlotsRecursive(pack, mode, pattern, bindings);
+                    patternsProcessed.put(pattern, true);
+                }
+                TemplateRule tr = (TemplateRule) r.getAction();
+                if (tr.getBody() != null && !rulesProcessed.contains(tr)) {
+                    allocateBindingSlotsRecursive(pack, mode, tr.getBody(), bindings);
+                    rulesProcessed.add(tr);
                 }
             });
         } catch (XPathException e) {
@@ -998,11 +977,7 @@ public class SimpleMode extends Mode {
      */
 
     public void explainTemplateRules(final ExpressionPresenter out) throws XPathException {
-        RuleAction action = new RuleAction() {
-            public void processRule(Rule r) throws XPathException {
-                r.export(out, isDeclaredStreamable());
-            }
-        };
+        RuleAction action = r -> r.export(out, isDeclaredStreamable());
         RuleGroupAction group = new RuleGroupAction() {
             String type;
 
@@ -1045,12 +1020,10 @@ public class SimpleMode extends Mode {
         //final Set<RuleTarget> processedRules = new HashSet<RuleTarget>();
         // TODO: if two rules share the same template, avoid duplicate output. This can happen with union patterns, and also
         // when a template is present in more than one mode.
-        RuleAction action = new RuleAction() {
-            public void processRule(Rule r) throws XPathException {
-                // if (processedRules.add(r.getAction())) {
-                r.export(out, isDeclaredStreamable());
-                // }
-            }
+        RuleAction action = r -> {
+            // if (processedRules.add(r.getAction())) {
+            r.export(out, isDeclaredStreamable());
+            // }
         };
 
         processRules(action);
@@ -1077,7 +1050,6 @@ public class SimpleMode extends Mode {
     public void processRules(RuleAction action, RuleGroupAction group) throws XPathException {
         processRuleChain(documentRuleChain, action, setGroup(group, "document-node()"));
         processRuleChain(unnamedElementRuleChain, action, setGroup(group, "element()"));
-        // TODO - think about using qNamedElementRuleChain and attribute here
         processRuleChains(namedElementRuleChains, action, setGroup(group, "namedElements"));
         processRuleChain(unnamedAttributeRuleChain, action, setGroup(group, "attribute()"));
         processRuleChains(namedAttributeRuleChains, action, setGroup(group, "namedAttributes"));
@@ -1159,6 +1131,10 @@ public class SimpleMode extends Mode {
         }
     }
 
+    /**
+     * Perform optimization on the complete set of rules comprising this Mode. This is a null operation
+     * in Saxon-HE.
+     */
 
     public void optimizeRules() {
     }
@@ -1195,13 +1171,8 @@ public class SimpleMode extends Mode {
     public void computeRankings(int start) throws XPathException {
         // Now sort the rules into ranking order
         final RuleSorter sorter = new RuleSorter(start);
-        RuleAction addToSorter = new RuleAction() {
-            public void processRule(Rule r) {
-                sorter.addRule(r);
-            }
-        };
         // add all the rules in this Mode to the sorter
-        processRules(addToSorter);
+        processRules(sorter::addRule);
         // now allocate ranks to all the rules in this Mode
         sorter.allocateRanks();
         highestRank = start + sorter.getNumberOfRules();
@@ -1218,7 +1189,7 @@ public class SimpleMode extends Mode {
      */
 
     private static class RuleSorter implements Sortable {
-        public ArrayList<Rule> rules = new ArrayList<Rule>(100);
+        public ArrayList<Rule> rules = new ArrayList<>(100);
         private int start;
 
         public RuleSorter(int start) {
@@ -1275,38 +1246,30 @@ public class SimpleMode extends Mode {
 
         /**
          * Start of a group characterised by some integer parameter
-         *
          * @param i characteristic of this group
-         * @throws XPathException
          */
-        void start(int i) throws XPathException;
+        void start(int i);
 
         /**
          * Invoked at the end of a group
-         *
-         * @throws XPathException
          */
-        void end() throws XPathException;
+        void end();
 
     }
 
     /**
      * Allocate slots for local variables in all patterns used by the rules in this mode.
      * Currently used only for accumulator rules
-     *
-     * @throws XPathException if anything goes wrong
      */
 
     public void allocateAllPatternSlots() {
-        final List<Integer> count = new ArrayList<Integer>(1);  // used to allow inner class to have side-effects
+        final List<Integer> count = new ArrayList<>(1);  // used to allow inner class to have side-effects
         count.add(0);
         final SlotManager slotManager = new SlotManager();
-        final RuleAction slotAllocator = new RuleAction() {
-            public void processRule(Rule r) throws XPathException {
-                int slots = r.getPattern().allocateSlots(slotManager, 0);
-                int max = Math.max(count.get(0), slots);
-                count.set(0, max);
-            }
+        final RuleAction slotAllocator = r -> {
+            int slots = r.getPattern().allocateSlots(slotManager, 0);
+            int max = Math.max(count.get(0), slots);
+            count.set(0, max);
         };
         try {
             processRules(slotAllocator);
