@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,7 +9,6 @@ package net.sf.saxon.tree.iter;
 
 import net.sf.saxon.expr.LastPositionFinder;
 import net.sf.saxon.om.*;
-import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.value.AtomicValue;
 import net.sf.saxon.value.EmptySequence;
 
@@ -19,11 +18,11 @@ import net.sf.saxon.value.EmptySequence;
  * getInstance() method.
  */
 
-public class EmptyIterator implements SequenceIterator,
-        ReversibleIterator, LastPositionFinder, GroundedIterator,
-        LookaheadIterator, UnfailingIterator, AtomizedValueIterator {
+public class EmptyIterator<T extends Item<?>> implements SequenceIterator<T>,
+        ReversibleIterator<T>, LastPositionFinder, GroundedIterator<T>,
+        LookaheadIterator<T>, UnfailingIterator<T>, AtomizedValueIterator<T> {
 
-    private static EmptyIterator theInstance = new EmptyIterator();
+    private static EmptyIterator<?> theInstance = new EmptyIterator<>();
 
     /**
      * Get an EmptyIterator, an iterator over an empty sequence.
@@ -36,9 +35,9 @@ public class EmptyIterator implements SequenceIterator,
         return theInstance;
     }
 
-
-    public static EmptyIterator emptyIterator() {
-        return theInstance;
+    @SuppressWarnings("unchecked")
+    public static <T extends Item<?>> EmptyIterator<T> emptyIterator() {
+        return (EmptyIterator<T>)theInstance;
     }
 
     /**
@@ -52,10 +51,8 @@ public class EmptyIterator implements SequenceIterator,
      * Deliver the atomic value that is next in the atomized result
      *
      * @return the next atomic value
-     * @throws net.sf.saxon.trans.XPathException
-     *          if a failure occurs reading or atomizing the next value
      */
-    public AtomicSequence nextAtomizedValue() throws XPathException {
+    public AtomicSequence nextAtomizedValue() {
         return null;
     }
 
@@ -64,7 +61,7 @@ public class EmptyIterator implements SequenceIterator,
      *
      * @return the next item. For the EmptyIterator this is always null.
      */
-    public Item next() {
+    public T next() {
         return null;
     }
 
@@ -78,9 +75,6 @@ public class EmptyIterator implements SequenceIterator,
         return 0;
     }
 
-    public void close() {
-    }
-
     /**
      * Get another iterator over the same items, in reverse order.
      *
@@ -88,7 +82,7 @@ public class EmptyIterator implements SequenceIterator,
      *         returns the same iterator each time)
      */
     /*@NotNull*/
-    public EmptyIterator getReverseIterator() {
+    public EmptyIterator<T> getReverseIterator() {
         return this;
     }
 
@@ -113,12 +107,12 @@ public class EmptyIterator implements SequenceIterator,
      * @return the corresponding Value
      */
 
-    public GroundedValue materialize() {
+    public GroundedValue<T> materialize() {
         return EmptySequence.getInstance();
     }
 
     @Override
-    public GroundedValue getResidue() throws XPathException {
+    public GroundedValue<T> getResidue() {
         return EmptySequence.getInstance();
     }
 
@@ -139,7 +133,7 @@ public class EmptyIterator implements SequenceIterator,
      * An empty iterator for use where a sequence of nodes is required
      */
 
-    public static class OfNodes extends EmptyIterator implements AxisIterator {
+    public static class OfNodes extends EmptyIterator<NodeInfo> implements AxisIterator {
 
         public final static OfNodes THE_INSTANCE = new OfNodes();
         /**
@@ -157,7 +151,7 @@ public class EmptyIterator implements SequenceIterator,
      * An empty iterator for use where a sequence of atomic values is required
      */
 
-    public static class OfAtomic extends EmptyIterator implements AtomicIterator {
+    public static class OfAtomic extends EmptyIterator<AtomicValue> implements AtomicIterator {
 
         public final static OfAtomic THE_INSTANCE = new OfAtomic();
 
