@@ -24,7 +24,7 @@ import net.sf.saxon.trans.XPathException;
  * the iterator can raise an exception.</p>
  */
 
-public interface GroundedIterator<T extends Item> extends SequenceIterator<T> {
+public interface GroundedIterator<T extends Item<?>> extends SequenceIterator<T> {
 
     /**
      * Return a GroundedValue containing all the items in the sequence returned by this
@@ -50,6 +50,6 @@ public interface GroundedIterator<T extends Item> extends SequenceIterator<T> {
      *                        which cause evaluation of expressions while materializing the value.
      */
 
-    GroundedValue<? extends Item> getResidue() throws XPathException;
+    GroundedValue<T> getResidue() throws XPathException;
 }
 
