@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -18,13 +18,11 @@ import net.sf.saxon.type.*;
 
 /**
  * A node in the "linked" tree representing an attribute. Note that this is
- * generated only "on demand", when the attribute is selected by a path expression.<P>
- * <p/>
+ * generated only "on demand", when the attribute is selected by a path expression.
  * <p>It is possible for multiple AttributeImpl objects to represent the same attribute node.
  * The identity of an attribute node is determined by the identity of the element, and the index
  * position of the attribute within the element. Index positions are not reused when an attribute
  * is deleted, and are retained when an attribute is renamed.</p>
- * <p/>
  * <p>This object no longer caches information such as the name code and string value, because
  * these would become invalid when the element node is modified.</p>
  *
@@ -115,16 +113,6 @@ public class AttributeImpl extends NodeImpl {
         }
     }
 
-    /**
-     * Determine whether the node has the is-nilled property
-     *
-     * @return true if the node has the is-nilled property
-     */
-
-    public boolean isNilled() {
-        return false;
-    }
-
     /**
      * Determine whether this is the same node as another node
      *
@@ -132,7 +120,7 @@ public class AttributeImpl extends NodeImpl {
      *         same node in the tree.
      */
 
-    public boolean isSameNodeInfo(NodeInfo other) {
+    public boolean equals(Object other) {
         if (!(other instanceof AttributeImpl)) {
             return false;
         }
@@ -140,16 +128,12 @@ public class AttributeImpl extends NodeImpl {
             return true;
         }
         AttributeImpl otherAtt = (AttributeImpl) other;
-        return getRawParent().isSameNodeInfo(otherAtt.getRawParent()) && getSiblingPosition() == otherAtt.getSiblingPosition();
+        return getRawParent().equals(otherAtt.getRawParent()) && getSiblingPosition() == otherAtt.getSiblingPosition();
     }
 
     /**
      * The hashCode() method obeys the contract for hashCode(): that is, if two objects are equal
      * (represent the same node) then they must have the same hashCode()
-     *
-     * @since 8.7 Previously, the effect of the equals() and hashCode() methods was not defined. Callers
-     *        should therefore be aware that third party implementations of the NodeInfo interface may
-     *        not implement the correct semantics.
      */
 
     public int hashCode() {
@@ -316,7 +300,7 @@ public class AttributeImpl extends NodeImpl {
         if (getRawParent() != null) {
             ((AttributeCollectionImpl) ((ElementImpl) getRawParent()).getAttributeList()).renameAttribute(getSiblingPosition(), newNameCode);
             String newURI = newNameCode.getURI();
-            if (newURI.length() != 0) {
+            if (!newURI.isEmpty()) {
                 // new attribute name is in a namespace
                 String newPrefix = newNameCode.getPrefix();
                 NamespaceBinding newBinding = new NamespaceBinding(newPrefix, newURI);
