@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -21,15 +21,15 @@ import net.sf.saxon.value.AtomicValue;
 
 
 /**
- * A node in the XML parse tree representing an XML element.<P>
- * This class is an implementation of NodeInfo. The object is a wrapper around
+ * A node in the XML parse tree representing an XML element.
+ * <p>This class is an implementation of NodeInfo. The object is a wrapper around
  * one entry in the arrays maintained by the TinyTree. Note that the same node
- * might be represented by different TinyElementImpl objects at different times.
+ * might be represented by different TinyElementImpl objects at different times.</p>
  *
  * @author Michael H. Kay
  */
 
-public final class TinyElementImpl extends TinyParentNodeImpl {
+public class TinyElementImpl extends TinyParentNodeImpl {
 
     /**
      * Constructor - create a tiny element node
@@ -59,13 +59,13 @@ public final class TinyElementImpl extends TinyParentNodeImpl {
      */
 
     public String getBaseURI() {
-        return Navigator.getBaseURI(this);
+        return Navigator.getBaseURI(this,
+                                    n -> ((TinyElementImpl) n).getTree().isTopWithinEntity(((TinyElementImpl) n).getNodeNumber()));
     }
 
     /**
      * Get the type annotation of this node, if any. The type annotation is represented as
      * SchemaType object.
-     * <p/>
      * <p>Types derived from a DTD are not reflected in the result of this method.</p>
      *
      * @return For element and attribute nodes: the type annotation derived from schema
@@ -209,7 +209,7 @@ public final class TinyElementImpl extends TinyParentNodeImpl {
      * Copy this node to a given receiver
      *
      * @param copyOptions determines handling of namespaces, etc
-     * @param location
+     * @param location location information associated with the event
      */
 
     public void copy(/*@NotNull*/ Receiver receiver, int copyOptions, Location location) throws XPathException {
@@ -222,12 +222,12 @@ public final class TinyElementImpl extends TinyParentNodeImpl {
             Receiver r1 = receiver;
             if (isSkipValidator(r1)) {
                 copyTypes = false;
-                r1 = ((ProxyReceiver) r1).getUnderlyingReceiver();
+                r1 = ((ProxyReceiver) r1).getNextReceiver();
             }
             if (r1 instanceof ComplexContentOutputter && !copyTypes) {
                 Receiver r2 = ((ComplexContentOutputter) r1).getReceiver();
                 if (r2 instanceof NamespaceReducer) {
-                    Receiver r3 = ((NamespaceReducer) r2).getUnderlyingReceiver();
+                    Receiver r3 = ((NamespaceReducer) r2).getNextReceiver();
                     if (r3 instanceof TinyBuilder) {
                         ((ComplexContentOutputter) r1).beforeBulkCopy();
                         TinyBuilder target = (TinyBuilder) r3;
@@ -237,6 +237,29 @@ public final class TinyElementImpl extends TinyParentNodeImpl {
                     }
                 }
             }
+        } else if (TinyTree.useGraft && (copyOptions & CopyOptions.FOR_UPDATE) == 0) {
+            boolean copyTypes = typed;
+            Receiver r1 = receiver;
+            if (isSkipValidator(r1)) {
+                copyTypes = false;
+                r1 = ((ProxyReceiver) r1).getNextReceiver();
+            }
+            if (r1 instanceof ComplexContentOutputter && !copyTypes) {
+                Receiver r2 = ((ComplexContentOutputter) r1).getReceiver();
+                if (r2 instanceof NamespaceReducer) {
+                    if (!((NamespaceReducer)r2).isDisinheritingNamespaces()) {
+                        Receiver r3 = ((NamespaceReducer) r2).getNextReceiver();
+                        if (r3 instanceof TinyBuilder && ((TinyBuilder) r3).isPositionedAtElement()) {
+                            ((ComplexContentOutputter) r1).beforeBulkCopy();
+                            TinyBuilder target = (TinyBuilder) r3;
+                            boolean copyNamespaces = CopyOptions.includes(copyOptions, CopyOptions.ALL_NAMESPACES);
+                            target.graft(this, copyNamespaces);
+                            ((ComplexContentOutputter) r1).afterBulkCopy();
+                            return;
+                        }
+                    }
+                }
+            }
         }
 
         // control vars
@@ -247,7 +270,7 @@ public final class TinyElementImpl extends TinyParentNodeImpl {
         boolean disallowNamespaceSensitiveContent =
                 ((copyOptions & CopyOptions.TYPE_ANNOTATIONS) != 0) &&
                         ((copyOptions & CopyOptions.SOME_NAMESPACES) == 0);
-
+        boolean foundElementInDefaultNamespace = false;
         Configuration config = tree.getConfiguration();
         NamePool pool = config.getNamePool();
         int next = nodeNr;
@@ -274,8 +297,10 @@ public final class TinyElementImpl extends TinyParentNodeImpl {
             level = nodeLevel;
 
             // output depends on node kind
-            switch (tree.nodeKind[next]) {
-                case Type.ELEMENT: {
+            int kind = tree.nodeKind[next];
+            switch (kind) {
+                case Type.ELEMENT:
+                case Type.TEXTUAL_ELEMENT: {
 
                     // start element
                     if (typed) {
@@ -311,66 +336,88 @@ public final class TinyElementImpl extends TinyParentNodeImpl {
                     receiver.startElement(new CodedName(fp, prefix, pool),
                             elementType, location, options);
 
-                    // there is an element to close
-                    closePending = true;
-
-                    // output namespaces
-                    if ((copyOptions & CopyOptions.SOME_NAMESPACES) != 0 && tree.usesNamespaces) {
-                        if (first) {
-                            if ((copyOptions & CopyOptions.LOCAL_NAMESPACES) != 0) {
-                                NamespaceBinding[] localNamespaces = getDeclaredNamespaces(null);
-                                for (NamespaceBinding ns : localNamespaces) {
-                                    if (ns == null) {
-                                        break;
+                    if (kind == Type.TEXTUAL_ELEMENT) {
+                        closePending = false;
+                        receiver.startContent();
+                        // output characters
+                        final CharSequence value = TinyTextImpl.getStringValue(tree, next);
+                        receiver.characters(value, location, ReceiverOptions.WHOLE_TEXT_NODE);
+                        receiver.endElement();
+                    } else {
+                        // there is an element to close
+                        closePending = true;
+                        
+                        // output namespaces
+                        if ((copyOptions & CopyOptions.SOME_NAMESPACES) != 0 && tree.usesNamespaces) {
+                            String defaultNS = null;
+                            if (prefix.isEmpty()) {
+                                defaultNS = getNamePool().getURI(fp);
+                                if (!defaultNS.isEmpty()) {
+                                    foundElementInDefaultNamespace = true;
+                                }
+                            }
+                            if (first) {
+                                if ((copyOptions & CopyOptions.LOCAL_NAMESPACES) != 0) {
+                                    NamespaceBinding[] localNamespaces = getDeclaredNamespaces(null);
+                                    for (NamespaceBinding ns : localNamespaces) {
+                                        if (ns == null) {
+                                            break;
+                                        }
+                                        receiver.namespace(ns, 0);
                                     }
-                                    receiver.namespace(ns, 0);
+                                } else if ((copyOptions & CopyOptions.ALL_NAMESPACES) != 0) {
+                                    NamespaceIterator.sendNamespaces(this, receiver);
                                 }
-                            } else if ((copyOptions & CopyOptions.ALL_NAMESPACES) != 0) {
-                                NamespaceIterator.sendNamespaces(this, receiver);
-                            }
-                        } else {
-                            int ns = tree.beta[next]; // by convention
-                            if (ns > 0) {
-                                while (ns < tree.numberOfNamespaces &&
-                                        tree.namespaceParent[ns] == next) {
-                                    NamespaceBinding nscode = tree.namespaceBinding[ns];
-                                    receiver.namespace(nscode, 0);
-                                    ns++;
-                                }
-                            }
-                        }
-                    }
-                    first = false;
-
-                    // output attributes
-
-                    int att = tree.alpha[next];
-                    if (att >= 0) {
-                        while (att < tree.numberOfAttributes && tree.attParent[att] == next) {
-                            int attCode = tree.attCode[att];
-                            int attfp = attCode & NamePool.FP_MASK;
-                            if (typed) {
-                                attributeType = tree.getAttributeType(att);
-                                if (disallowNamespaceSensitiveContent) {
-                                    try {
-                                        checkNotNamespaceSensitiveAttribute(attributeType, att);
-                                    } catch (CopyNamespaceSensitiveException e) {
-                                        int lang = receiver.getPipelineConfiguration().getHostLanguage();
-                                        e.setErrorCode(lang == Configuration.XSLT ? "XTTE0950" : "XQTY0086");
-                                        throw e;
+                            } else {
+                                int ns = tree.beta[next]; // by convention
+                                if (ns > 0) {
+                                    while (ns < tree.numberOfNamespaces &&
+                                            tree.namespaceParent[ns] == next) {
+                                        NamespaceBinding nscode = tree.namespaceBinding[ns];
+                                        receiver.namespace(nscode, 0);
+                                        ns++;
                                     }
                                 }
                             }
-                            String attPrefix = tree.prefixPool.getPrefix(attCode >> 20);
-                            receiver.attribute(new CodedName(attfp, attPrefix, pool), attributeType, tree.attValue[att], location, 0);
-                            att++;
                         }
-                    }
+                        first = false;
 
-                    // start content
-                    receiver.startContent();
+                        // output attributes
+
+                        int att = tree.alpha[next];
+                        if (att >= 0) {
+                            while (att < tree.numberOfAttributes && tree.attParent[att] == next) {
+                                int attCode = tree.attCode[att];
+                                int attfp = attCode & NamePool.FP_MASK;
+                                if (typed) {
+                                    attributeType = tree.getAttributeType(att);
+                                    if (disallowNamespaceSensitiveContent) {
+                                        try {
+                                            checkNotNamespaceSensitiveAttribute(attributeType, att);
+                                        } catch (CopyNamespaceSensitiveException e) {
+                                            int lang = receiver.getPipelineConfiguration().getHostLanguage();
+                                            e.setErrorCode(lang == Configuration.XSLT ? "XTTE0950" : "XQTY0086");
+                                            throw e;
+                                        }
+                                    }
+                                }
+                                String attPrefix = tree.prefixPool.getPrefix(attCode >> 20);
+                                receiver.attribute(new CodedName(attfp, attPrefix, pool), attributeType, tree.attValue[att], location, 0);
+                                att++;
+                            }
+                        }
+
+                        // start content
+                        receiver.startContent();
+                    }
                     break;
                 }
+
+                case Type.EXTERNAL_NODE_REFERENCE:
+                    closePending = false;
+                    tree.externalNodes.get(tree.alpha[next]).copy(receiver, copyOptions, location);
+                    break;
+
                 case Type.TEXT: {
 
                     // don't close text nodes
@@ -446,7 +493,7 @@ public final class TinyElementImpl extends TinyParentNodeImpl {
      * @throws XPathException if an error occurs
      */
 
-    private void checkNotNamespaceSensitiveElement(SchemaType type, int nodeNr) throws XPathException {
+    protected void checkNotNamespaceSensitiveElement(SchemaType type, int nodeNr) throws XPathException {
         if (type instanceof SimpleType && ((SimpleType) type).isNamespaceSensitive()) {
             if (type.isAtomicType()) {
                 throw new CopyNamespaceSensitiveException(
