@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -24,7 +24,7 @@ public abstract class TinyParentNodeImpl extends TinyNodeImpl {
      * Determine if the node has children.
      */
 
-    public final boolean hasChildNodes() {
+    public boolean hasChildNodes() {
         return nodeNr + 1 < tree.numberOfNodes &&
                 tree.depth[nodeNr + 1] > tree.depth[nodeNr];
     }
@@ -36,7 +36,7 @@ public abstract class TinyParentNodeImpl extends TinyNodeImpl {
      * @return the accumulated character content of the element, including descendant elements.
      */
 
-    public final String getStringValue() {
+    public String getStringValue() {
         return getStringValueCS(tree, nodeNr).toString();
     }
 
@@ -45,7 +45,7 @@ public abstract class TinyParentNodeImpl extends TinyNodeImpl {
      * the version of the method that returns a String.
      */
 
-    public final CharSequence getStringValueCS() {
+    public CharSequence getStringValueCS() {
         return getStringValueCS(tree, nodeNr);
     }
 
@@ -72,7 +72,12 @@ public abstract class TinyParentNodeImpl extends TinyNodeImpl {
         // we optimize two special cases: firstly, where the node has no children, and secondly,
         // where it has a single text node as a child.
 
-        if (tree.depth[next] <= level) {
+        if (tree.nodeKind[nodeNr] == Type.TEXTUAL_ELEMENT) {
+            return TinyTextImpl.getStringValue(tree, nodeNr);
+        } else if (tree.nodeKind[nodeNr] == Type.EXTERNAL_NODE_REFERENCE) {
+            GraftedElement graft = tree.externalNodes.get(tree.alpha[nodeNr]);
+            return graft.getStringValueCS();
+        } else if (tree.depth[next] <= level) {
             return "";
         } else if (tree.nodeKind[next] == Type.TEXT && (next + 1 >= tree.numberOfNodes || tree.depth[next + 1] <= level)) {
             return TinyTextImpl.getStringValue(tree, next);
@@ -83,14 +88,10 @@ public abstract class TinyParentNodeImpl extends TinyNodeImpl {
         FastStringBuffer sb = null;
         while (next < tree.numberOfNodes && tree.depth[next] > level) {
             final byte kind = tree.nodeKind[next];
-            if (kind == Type.TEXT) {
-//                int length = tree.beta[next];
-//                int start = tree.alpha[next];
+            if (kind == Type.TEXT || kind == Type.TEXTUAL_ELEMENT) {
                 if (sb == null) {
                     sb = new FastStringBuffer(FastStringBuffer.C256);
                 }
-                //sb.append(tree.charBuffer, start, length);
-                //sb.append(tree.charBuffer.subSequence(start, start+length));
                 sb.append(TinyTextImpl.getStringValue(tree, next));
             } else if (kind == Type.WHITESPACE_TEXT) {
                 if (sb == null) {
@@ -100,7 +101,9 @@ public abstract class TinyParentNodeImpl extends TinyNodeImpl {
             }
             next++;
         }
-        if (sb == null) return "";
+        if (sb == null) {
+            return "";
+        }
         return sb.condense();
     }
 
