@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,6 +9,7 @@ package net.sf.saxon.tree.tiny;
 
 import net.sf.saxon.Configuration;
 import net.sf.saxon.event.ReceiverOptions;
+import net.sf.saxon.lib.Feature;
 import net.sf.saxon.lib.FeatureKeys;
 import net.sf.saxon.om.*;
 import net.sf.saxon.trans.XPathException;
@@ -104,6 +105,10 @@ public final class TinyTree extends GenericTreeInfo implements NodeVectorTree {
     // defaultedAttribute is a set holding the attribute node numbers of attributes that resulted from expansion of schema defaults
     protected IntSet defaultedAttributes = null;
 
+    // topWithinEntity is a set holding the node numbers of elements that do not have a parent within
+    // the same external entity
+    protected IntSet topWithinEntity = null;
+
     // boolean switch to disable the typed value caching
     private boolean allowTypedValueCache = true;
 
@@ -167,12 +172,15 @@ public final class TinyTree extends GenericTreeInfo implements NodeVectorTree {
 
     private NodeInfo copiedFrom;
 
+    protected List<GraftedElement> externalNodes;
+
     /**
      * Temporary flag introduced in Saxon 9.8.0.5 to enable or disable fast-path code for copying
      * element nodes from one TinyTree to another. This code is currently disabled by default until
      * it has been more thoroughly tested.
      */
     public static boolean useFastCopy = false;
+    public static boolean useGraft = true;
 
     /**
      * Create a tree with a specified initial size
@@ -181,8 +189,10 @@ public final class TinyTree extends GenericTreeInfo implements NodeVectorTree {
      * @param statistics the size parameters for the tree
      */
 
+    //private static int counter = 0;
     public TinyTree(/*@NotNull*/ Configuration config, Statistics statistics) {
         super(config);
+        //if (counter++ % 1000 == 0) System.err.println("TinyTree instances: " + counter);
 
         int nodes = (int) statistics.getAverageNodes() + 1;
         int attributes = (int) statistics.getAverageAttributes() + 1;
@@ -220,7 +230,7 @@ public final class TinyTree extends GenericTreeInfo implements NodeVectorTree {
     public void setConfiguration(/*@NotNull*/ Configuration config) {
         super.setConfiguration(config);
         allowTypedValueCache = config.isLicensedFeature(Configuration.LicenseFeature.SCHEMA_VALIDATION) &&
-                config.getBooleanProperty(FeatureKeys.USE_TYPED_VALUE_CACHE);
+                config.getBooleanProperty(Feature.USE_TYPED_VALUE_CACHE);
         addNamespace(0, NamespaceBinding.XML);
     }
 
@@ -582,8 +592,11 @@ public final class TinyTree extends GenericTreeInfo implements NodeVectorTree {
             } else if (annotation == StandardNames.XS_ANY_URI) {
                 CharSequence stringValue = TinyParentNodeImpl.getStringValueCS(this, nodeNr);
                 return new AnyURIValue(stringValue);
+            } else if (annotation == StandardNames.XS_ID) {
+                CharSequence stringValue = TinyParentNodeImpl.getStringValueCS(this, nodeNr);
+                return new StringValue(stringValue, BuiltInAtomicType.ID);
             } else {
-                TinyElementImpl element = new TinyElementImpl(this, nodeNr);
+                TinyNodeImpl element = getNode(nodeNr);
                 AtomicSequence value = stype.atomize(element);
                 if (allowTypedValueCache) {
                     if (typedValueArray == null) {
@@ -927,6 +940,10 @@ public final class TinyTree extends GenericTreeInfo implements NodeVectorTree {
                 return (TinyDocumentImpl) getRootNode();
             case Type.ELEMENT:
                 return new TinyElementImpl(this, nr);
+            case Type.TEXTUAL_ELEMENT:
+                return new TinyTextualElement(this, nr);
+            case Type.EXTERNAL_NODE_REFERENCE:
+                return externalNodes.get(alpha[nr]);
             case Type.TEXT:
                 return new TinyTextImpl(this, nr);
             case Type.WHITESPACE_TEXT:
@@ -937,8 +954,10 @@ public final class TinyTree extends GenericTreeInfo implements NodeVectorTree {
                 return new TinyProcInstImpl(this, nr);
             case Type.PARENT_POINTER:
                 throw new IllegalArgumentException("Attempting to treat a parent pointer as a node");
+            case Type.STOPPER:
+                throw new IllegalArgumentException("Attempting to treat a stopper entry as a node");
             default:
-                throw new IllegalStateException("Unknown node kind");
+                throw new IllegalStateException("Unknown node kind " + nodeKind[nr]);
         }
     }
 
@@ -1091,17 +1110,16 @@ public final class TinyTree extends GenericTreeInfo implements NodeVectorTree {
      * Ask whether an element is an ID element. (The represents the
      * is-id property in the data model)
      *
-     * @param nr the element node whose is-idref property is required
-     * @return true if the node has the is-idref property
+     * @param nr the element node whose is-id property is required
+     * @return true if the node has the is-id property
      */
 
     public boolean isIdElement(int nr) {
         try {
-            return getSchemaType(nr).isIdType();
-        } catch (MissingComponentException e) {
+            return getSchemaType(nr).isIdType() && getTypedValueOfElement(nr).getLength() == 1;
+        } catch (XPathException e) {
             return false;
         }
-        // TODO: there is an additional condition, which is that the typed value must be a singleton
     }
 
     /**
@@ -1285,14 +1303,11 @@ public final class TinyTree extends GenericTreeInfo implements NodeVectorTree {
 
     void registerID(NodeInfo e, String id) {
         if (idTable == null) {
-            idTable = new HashMap<String, NodeInfo>(256);
+            idTable = new HashMap<>(256);
         }
 
         // the XPath spec (5.2.1) says ignore the second ID if it's not unique
-        NodeInfo old = idTable.get(id);
-        if (old == null) {
-            idTable.put(id, e);
-        }
+        idTable.putIfAbsent(id, e);
 
     }
 
@@ -1329,7 +1344,7 @@ public final class TinyTree extends GenericTreeInfo implements NodeVectorTree {
 
     void setUnparsedEntity(String name, String uri, String publicId) {
         if (entityTable == null) {
-            entityTable = new HashMap<String, String[]>(20);
+            entityTable = new HashMap<>(20);
         }
         String[] ids = new String[2];
         ids[0] = uri;
@@ -1375,6 +1390,17 @@ public final class TinyTree extends GenericTreeInfo implements NodeVectorTree {
         return getConfiguration().getNamePool();
     }
 
+    public void markTopWithinEntity(int nodeNr) {
+        if (topWithinEntity == null) {
+            topWithinEntity = new IntHashSet();
+        }
+        topWithinEntity.add(nodeNr);
+    }
+
+    public boolean isTopWithinEntity(int nodeNr) {
+        return topWithinEntity != null && topWithinEntity.contains(nodeNr);
+    }
+
 
     /**
      * Produce diagnostic print of main tree arrays
@@ -1698,7 +1724,7 @@ public final class TinyTree extends GenericTreeInfo implements NodeVectorTree {
                             int attNameCode = attCode[aTo] = source.attCode[aFrom];
                             if (NamePool.isPrefixed(attNameCode)) {
                                 String prefix = source.prefixPool.getPrefix(attNameCode >> 20);
-                                attCode[aTo] |= (prefixPool.obtainPrefixCode(prefix) << 20);
+                                attCode[aTo] |= prefixPool.obtainPrefixCode(prefix) << 20;
                             }
                             if (source.isIdAttribute(aFrom)) {
                                 registerID(getNode(to), source.attValue[aFrom].toString());
@@ -1785,5 +1811,32 @@ public final class TinyTree extends GenericTreeInfo implements NodeVectorTree {
         numberOfNodes += length;
     }
 
+    /**
+     * Graft an element node from an external tree
+     */
+
+    public void graft(NodeInfo externalNode, int nodeNr, int depth, boolean copyNamespaces) {
+        assert externalNode.getNodeKind() == Type.ELEMENT;
+        if (externalNodes == null) {
+            externalNodes = new ArrayList<>();
+        }
+        int externalNodeNr = externalNodes.size();
+        int fp = getNamePool().allocateFingerprint(externalNode.getURI(), externalNode.getLocalPart());
+        int prefixCode = prefixPool.obtainPrefixCode(externalNode.getPrefix());
+        int nameCode = (prefixCode << 20) | fp;
+        addNode(Type.EXTERNAL_NODE_REFERENCE, depth, externalNodeNr, -1, nameCode);
+        externalNodes.add(new GraftedElement(this, nodeNr, externalNode, copyNamespaces));
+
+    }
+
+    /**
+     * Ask if the tree contains grafted subtrees
+     * @return true if there are references to subtrees of other trees
+     */
+
+    public boolean containsGraftedSubtrees() {
+        return externalNodes != null;
+    }
+
 }
 
