@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -9,6 +9,7 @@ package net.sf.saxon.type;
 
 import net.sf.saxon.expr.Expression;
 import net.sf.saxon.expr.parser.RoleDiagnostic;
+import net.sf.saxon.om.Genre;
 import net.sf.saxon.query.AnnotationList;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.value.SequenceType;
@@ -21,6 +22,16 @@ import net.sf.saxon.value.SequenceType;
 
 public interface FunctionItemType extends ItemType {
 
+    /**
+     * Determine the Genre (top-level classification) of this type
+     *
+     * @return the Genre to which this type belongs, specifically {@link Genre#FUNCTION}
+     */
+    @Override
+    default Genre getGenre() {
+        return Genre.FUNCTION;
+    }
+
     /**
      * Get the argument types of the function
      *
