@@ -2,11 +2,12 @@
 // Copyright (c) 2013 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// This Source Code Form is “Incompatible With Secondary Licenses”, as defined by the Mozilla Public License, v. 2.0.
+// This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 package net.sf.saxon.type;
 
+import com.saxonica.ee.schema.UserSimpleType;
 import net.sf.saxon.expr.Expression;
 import net.sf.saxon.lib.ConversionRules;
 import net.sf.saxon.lib.NamespaceConstant;
@@ -17,6 +18,8 @@ import net.sf.saxon.value.*;
 import java.util.ArrayList;
 import java.util.List;
 
+import static net.sf.saxon.type.SchemaComponent.ValidationStatus.VALIDATED;
+
 /**
  * Singleton class representing the class xs:numeric as defined in XPath 3.1: a union type
  * whose members are xs:double, xs:decimal, and xs:float
@@ -37,6 +40,17 @@ public class NumericType implements UnionType, PlainType {
         return new StructuredQName("xs", NamespaceConstant.SCHEMA, "numeric");
     }
 
+    /**
+     * Get the genre of this item
+     *
+     * @return the genre
+     */
+    @Override
+    public Genre getGenre() {
+        return Genre.ATOMIC;
+    }
+
+
     /**
      * Ask whether the union contains a list type among its member types
      *
@@ -55,7 +69,7 @@ public class NumericType implements UnionType, PlainType {
     public Iterable<PlainType> getPlainMemberTypes() {
         if (members == null) {
             synchronized(NumericType.class) {
-                members = new ArrayList<PlainType>();
+                members = new ArrayList<>();
                 members.add(BuiltInAtomicType.DOUBLE);
                 members.add(BuiltInAtomicType.FLOAT);
                 members.add(BuiltInAtomicType.DECIMAL);
@@ -91,7 +105,7 @@ public class NumericType implements UnionType, PlainType {
      *
      *
      * @param item    The item to be tested
-     * @param th
+     * @param th      The type hierarchy cache
      * @return true if the item is an instance of this type; false otherwise
      */
     public boolean matches(Item item, TypeHierarchy th) {
@@ -153,22 +167,12 @@ public class NumericType implements UnionType, PlainType {
      *
      * @return true unless it is known that these items will be elements with element-only
      *         content, or function items, in which case return false
+     * @param th The type hierarchy cache
      */
-    public boolean isAtomizable() {
+    public boolean isAtomizable(TypeHierarchy th) {
         return true;
     }
 
-    /**
-     * Visit all the schema components used in this ItemType definition
-     *
-     * @param visitor the visitor class to be called when each component is visited
-     * @throws net.sf.saxon.trans.XPathException
-     *          if an error occurs
-     */
-    public void visitNamedSchemaComponents(SchemaComponentVisitor visitor) throws XPathException {
-        // no action
-    }
-
     /**
      * Test whether this Simple Type is an atomic type
      *
@@ -314,10 +318,8 @@ public class NumericType implements UnionType, PlainType {
      *
      * @param input the value to be preprocessed
      * @return the value after preprocessing
-     * @throws net.sf.saxon.type.ValidationException
-     *          if preprocessing detects that the value is invalid
      */
-    public CharSequence preprocess(CharSequence input) throws ValidationException {
+    public CharSequence preprocess(CharSequence input) {
         return input;
     }
 
@@ -329,10 +331,8 @@ public class NumericType implements UnionType, PlainType {
      * @param input the value to be postprocessed: this is the "ordinary" result of converting
      *              the value to a string
      * @return the value after postprocessing
-     * @throws net.sf.saxon.type.ValidationException
-     *          if postprocessing detects that the value is invalid
      */
-    public CharSequence postprocess(CharSequence input) throws ValidationException {
+    public CharSequence postprocess(CharSequence input) {
         return input;
     }
 
@@ -487,7 +487,7 @@ public class NumericType implements UnionType, PlainType {
      *          if the expression will never deliver a value of the correct type
      */
     public void analyzeContentExpression(Expression expression, int kind) throws XPathException {
-        // TODO
+        BuiltInAtomicType.analyzeContentExpression(this, expression, kind);
     }
 
     /**
@@ -527,20 +527,6 @@ public class NumericType implements UnionType, PlainType {
         return "xs:numeric";
     }
 
-    /**
-     * Check that this type is validly derived from a given type, following the rules for the Schema Component
-     * Constraint "Is Type Derivation OK (Simple)" (3.14.6) or "Is Type Derivation OK (Complex)" (3.4.6) as
-     * appropriate.
-     *
-     * @param base  the base type; the algorithm tests whether derivation from this type is permitted
-     * @param block the derivations that are blocked by the relevant element declaration
-     * @throws net.sf.saxon.type.SchemaException
-     *          if the derivation is not allowed
-     */
-    public void checkTypeDerivationIsOK(SchemaType base, int block) throws SchemaException {
-        // TODO
-    }
-
     /**
      * Get the URI of the schema document where the type was originally defined.
      *
@@ -577,11 +563,8 @@ public class NumericType implements UnionType, PlainType {
 
     /**
      * Get the validation status of this component.
-     *
-     * @return one of the values {@link #UNVALIDATED}, {@link #VALIDATING},
-     *         {@link #VALIDATED}, {@link #INVALID}, {@link #INCOMPLETE}
      */
-    public int getValidationStatus() {
+    public ValidationStatus getValidationStatus() {
         return VALIDATED;
     }
 
@@ -599,17 +582,17 @@ public class NumericType implements UnionType, PlainType {
     }
 
     /**
-     * Get the schema component in the form of a function item. This allows schema information
-     * to be made visible to XSLT or XQuery code. The function makes available the contents of the
-     * schema component as defined in the XSD specification. The function takes a string as argument
-     * representing a property name, and returns the corresponding property of the schema component.
-     * There is also a property "class" which returns the kind of schema component, for example
-     * "Attribute Declaration".
+     * Return a string representation of this ItemType suitable for use in stylesheet
+     * export files. This differs from the result of toString() in that it will not contain
+     * any references to anonymous types. Note that it may also use the Saxon extended syntax
+     * for union types and tuple types. The default implementation returns the result of
+     * calling {@code toString()}.
      *
-     * @return the schema component represented as a function from property names to property values.
+     * @return the string representation as an instance of the XPath SequenceType construct
      */
-    public Function getComponentAsFunction() {
-        return null;  // TODO
+    @Override
+    public String toExportString() {
+        return toString();
     }
 
     @Override
@@ -617,28 +600,16 @@ public class NumericType implements UnionType, PlainType {
         return "xs:numeric";
     }
 
-    /**
-     * Return a string representation of this SequenceType suitable for use in export (SEF) files.
-     * The difference from the toString() method is that the representation will not contain
-     * references to anonymous types.
-     *
-     * @return the string representation as an instance of the XPath
-     * SequenceType construct, using only names that will be in-scope if the target environment
-     * imports the same schema as the source environment
-     */
-    @Override
-    public String toExportString() {
-        return toString();
-    }
-
     /**
      * Generate Javascript code to test whether an item conforms to this item type
      *
      * @return a Javascript instruction or sequence of instructions, which can be used as the body
      * of a Javascript function, and which returns a boolean indication whether the value of the
      * variable "item" is an instance of this item type.
-     * @param knownToBe
-     * @param targetVersion
+     * @param knownToBe An item type that the supplied item is known to conform to; the generated code
+     *                  can assume that the item is an instance of this type.
+     * @param targetVersion The version of Saxon-JS for which code is being generated. Currently either 1 or 2.
+
      */
     @Override
     public String generateJavaScriptItemTypeTest(ItemType knownToBe, int targetVersion) {
@@ -654,7 +625,7 @@ public class NumericType implements UnionType, PlainType {
      * if conversion is possible, or throw an error otherwise.
      *
      * @param errorCode the error to be thrown if conversion is not possible
-     * @param targetVersion
+     * @param targetVersion the version of Saxon-JS: 1 or 2.
      * @return a Javascript instruction or sequence of instructions, which can be used as the body
      * of a Javascript function, and which returns the result of conversion to this type, or throws
      * an error if conversion is not possible. The variable "val" will hold the supplied Javascript
@@ -663,5 +634,42 @@ public class NumericType implements UnionType, PlainType {
     public String generateJavaScriptItemTypeAcceptor(String errorCode, int targetVersion) throws XPathException {
         return BuiltInAtomicType.DOUBLE.generateJavaScriptItemTypeAcceptor(errorCode, targetVersion);
     }
+
+    /**
+     * Check that this type is validly derived from a given type, following the rules for the Schema Component
+     * Constraint "Is Type Derivation OK (Simple)" (3.14.6) or "Is Type Derivation OK (Complex)" (3.4.6) as
+     * appropriate.
+     *
+     * @param base  the base type; the algorithm tests whether derivation from this type is permitted
+     * @param block the derivations that are blocked by the relevant element declaration
+     * @throws net.sf.saxon.type.SchemaException if the derivation is not allowed
+     */
+    public void checkTypeDerivationIsOK(SchemaType base, int block) throws SchemaException {
+        //#ifdefined  SCHEMA
+        String s = UserSimpleType.isTypeDerivationOK(this, base, block);
+        if (s != null) {
+            throw new SchemaException(s);
+        }
+        //#endif
+    }
+
+    //#ifdefined  SCHEMA
+
+    /**
+     * Get the schema component in the form of a function item. This allows schema information
+     * to be made visible to XSLT or XQuery code. The function makes available the contents of the
+     * schema component as defined in the XSD specification. The function takes a string as argument
+     * representing a property name, and returns the corresponding property of the schema component.
+     * There is also a property "class" which returns the kind of schema component, for example
+     * "Attribute Declaration".
+     *
+     * @return the schema component represented as a function from property names to property values.
+     */
+    public Function getComponentAsFunction() {
+        return UserSimpleType.getComponentAsFunction(this);
+    }
+
+    //#endif
+
 }
 
