@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -15,6 +15,7 @@ import net.sf.saxon.type.*;
 
 import java.math.BigDecimal;
 import java.math.BigInteger;
+import java.math.RoundingMode;
 import java.util.regex.Pattern;
 
 /**
@@ -31,7 +32,8 @@ public final class BigDecimalValue extends DecimalValue {
     private BigDecimal value;
     private Double doubleValue;
 
-    public static final BigDecimal BIG_DECIMAL_ONE_MILLION = BigDecimal.valueOf(1000000);
+    public static final BigDecimal BIG_DECIMAL_ONE_MILLION = BigDecimal.valueOf(1_000_000);
+    public static final BigDecimal BIG_DECIMAL_ONE_BILLION = BigDecimal.valueOf(1_000_000_000);
 
     public static final BigDecimalValue ZERO = new BigDecimalValue(BigDecimal.valueOf(0));
     public static final BigDecimalValue ONE = new BigDecimalValue(BigDecimal.valueOf(1));
@@ -64,92 +66,7 @@ public final class BigDecimalValue extends DecimalValue {
     public static ConversionResult makeDecimalValue(CharSequence in, boolean validate) {
 
         try {
-            FastStringBuffer digits = new FastStringBuffer(in.length());
-            int scale = 0;
-            int state = 0;
-            // 0 - in initial whitespace; 1 - after sign
-            // 3 - after decimal point; 5 - in final whitespace
-            boolean foundDigit = false;
-            int len = in.length();
-            for (int i = 0; i < len; i++) {
-                char c = in.charAt(i);
-                switch (c) {
-                    case ' ':
-                    case '\t':
-                    case '\r':
-                    case '\n':
-                        if (state != 0) {
-                            state = 5;
-                        }
-                        break;
-                    case '+':
-                        if (state != 0) {
-                            throw new NumberFormatException("unexpected sign");
-                        }
-                        state = 1;
-                        break;
-                    case '-':
-                        if (state != 0) {
-                            throw new NumberFormatException("unexpected sign");
-                        }
-                        state = 1;
-                        digits.append(c);
-                        break;
-                    case '0':
-                    case '1':
-                    case '2':
-                    case '3':
-                    case '4':
-                    case '5':
-                    case '6':
-                    case '7':
-                    case '8':
-                    case '9':
-                        if (state == 0) {
-                            state = 1;
-                        } else if (state >= 3) {
-                            scale++;
-                        }
-                        if (state == 5) {
-                            throw new NumberFormatException("contains embedded whitespace");
-                        }
-                        digits.append(c);
-                        foundDigit = true;
-                        break;
-                    case '.':
-                        if (state == 5) {
-                            throw new NumberFormatException("contains embedded whitespace");
-                        }
-                        if (state >= 3) {
-                            throw new NumberFormatException("more than one decimal point");
-                        }
-                        state = 3;
-                        break;
-                    default:
-                        throw new NumberFormatException("invalid character '" + c + "'");
-                }
-
-            }
-
-            if (!foundDigit) {
-                throw new NumberFormatException("no digits in value");
-            }
-
-            // remove insignificant trailing zeroes
-            while (scale > 0) {
-                if (digits.charAt(digits.length() - 1) == '0') {
-                    digits.setLength(digits.length() - 1);
-                    scale--;
-                } else {
-                    break;
-                }
-            }
-            if (digits.length() == 0 || (digits.length() == 1 && digits.charAt(0) == '-')) {
-                return BigDecimalValue.ZERO;
-            }
-            BigInteger bigInt = new BigInteger(digits.toString());
-            BigDecimal bigDec = new BigDecimal(bigInt, scale);
-            return new BigDecimalValue(bigDec);
+            return parse(in);
         } catch (NumberFormatException err) {
             ValidationFailure e = new ValidationFailure(
                     "Cannot convert string " + Err.wrap(Whitespace.trim(in), Err.VALUE) +
@@ -159,6 +76,104 @@ public final class BigDecimalValue extends DecimalValue {
         }
     }
 
+    /**
+     * Factory method to construct a DecimalValue from a string, throwing an unchecked exception
+     * if the value is invalid
+     *
+     * @param in       the lexical representation of the DecimalValue
+     * @return the required DecimalValue
+     * @throws NumberFormatException if the value is invalid
+     */
+
+    public static BigDecimalValue parse(CharSequence in) throws NumberFormatException {
+        FastStringBuffer digits = new FastStringBuffer(in.length());
+        int scale = 0;
+        int state = 0;
+        // 0 - in initial whitespace; 1 - after sign
+        // 3 - after decimal point; 5 - in final whitespace
+        boolean foundDigit = false;
+        int len = in.length();
+        for (int i = 0; i < len; i++) {
+            char c = in.charAt(i);
+            switch (c) {
+                case ' ':
+                case '\t':
+                case '\r':
+                case '\n':
+                    if (state != 0) {
+                        state = 5;
+                    }
+                    break;
+                case '+':
+                    if (state != 0) {
+                        throw new NumberFormatException("unexpected sign");
+                    }
+                    state = 1;
+                    break;
+                case '-':
+                    if (state != 0) {
+                        throw new NumberFormatException("unexpected sign");
+                    }
+                    state = 1;
+                    digits.append(c);
+                    break;
+                case '0':
+                case '1':
+                case '2':
+                case '3':
+                case '4':
+                case '5':
+                case '6':
+                case '7':
+                case '8':
+                case '9':
+                    if (state == 0) {
+                        state = 1;
+                    } else if (state >= 3) {
+                        scale++;
+                    }
+                    if (state == 5) {
+                        throw new NumberFormatException("contains embedded whitespace");
+                    }
+                    digits.append(c);
+                    foundDigit = true;
+                    break;
+                case '.':
+                    if (state == 5) {
+                        throw new NumberFormatException("contains embedded whitespace");
+                    }
+                    if (state >= 3) {
+                        throw new NumberFormatException("more than one decimal point");
+                    }
+                    state = 3;
+                    break;
+                default:
+                    throw new NumberFormatException("invalid character '" + c + "'");
+            }
+
+        }
+
+        if (!foundDigit) {
+            throw new NumberFormatException("no digits in value");
+        }
+
+        // remove insignificant trailing zeroes
+        while (scale > 0) {
+            if (digits.charAt(digits.length() - 1) == '0') {
+                digits.setLength(digits.length() - 1);
+                scale--;
+            } else {
+                break;
+            }
+        }
+        if (digits.isEmpty() || (digits.length() == 1 && digits.charAt(0) == '-')) {
+            return BigDecimalValue.ZERO;
+        }
+        BigInteger bigInt = new BigInteger(digits.toString());
+        BigDecimal bigDec = new BigDecimal(bigInt, scale);
+        return new BigDecimalValue(bigDec);
+    }
+
     /**
      * Test whether a string is castable to a decimal value
      *
@@ -282,12 +297,12 @@ public final class BigDecimalValue extends DecimalValue {
      */
 
     public int hashCode() {
-        BigDecimal round = value.setScale(0, BigDecimal.ROUND_DOWN);
+        BigDecimal round = value.setScale(0, RoundingMode.DOWN);
         long value = round.longValue();
         if (value > Integer.MIN_VALUE && value < Integer.MAX_VALUE) {
             return (int) value;
         } else {
-            return new Double(getDoubleValue()).hashCode();
+            return Double.valueOf(getDoubleValue()).hashCode();
         }
     }
 
@@ -338,8 +353,8 @@ public final class BigDecimalValue extends DecimalValue {
      */
 
     public static FastStringBuffer decimalToString(BigDecimal value, FastStringBuffer fsb) {
-        // Can't use the plain BigDecimal#toString() under JDK 1.5 because this produces values like "1E-5".
-        // JDK 1.5 offers BigDecimal#toPlainString() which might do the job directly
+        // Can't use BigDecimal#toString() under JDK 1.5 because this produces values like "1E-5".
+        // Can't use BigDecimal#toPlainString() because it retains trailing zeroes to represent the scale
         int scale = value.scale();
         if (scale == 0) {
             fsb.append(value.toString());
@@ -398,7 +413,7 @@ public final class BigDecimalValue extends DecimalValue {
      */
 
     public NumericValue floor() {
-        return new BigDecimalValue(value.setScale(0, BigDecimal.ROUND_FLOOR));
+        return new BigDecimalValue(value.setScale(0, RoundingMode.FLOOR));
     }
 
     /**
@@ -406,7 +421,7 @@ public final class BigDecimalValue extends DecimalValue {
      */
 
     public NumericValue ceiling() {
-        return new BigDecimalValue(value.setScale(0, BigDecimal.ROUND_CEILING));
+        return new BigDecimalValue(value.setScale(0, RoundingMode.CEILING));
     }
 
     /**
@@ -423,11 +438,11 @@ public final class BigDecimalValue extends DecimalValue {
 
         switch (value.signum()) {
             case -1:
-                return new BigDecimalValue(value.setScale(scale, BigDecimal.ROUND_HALF_DOWN));
+                return new BigDecimalValue(value.setScale(scale, RoundingMode.HALF_DOWN));
             case 0:
                 return this;
             case +1:
-                return new BigDecimalValue(value.setScale(scale, BigDecimal.ROUND_HALF_UP));
+                return new BigDecimalValue(value.setScale(scale, RoundingMode.HALF_UP));
             default:
                 // can't happen
                 return this;
@@ -440,7 +455,7 @@ public final class BigDecimalValue extends DecimalValue {
      */
 
     public NumericValue roundHalfToEven(int scale) {
-        BigDecimal scaledValue = value.setScale(scale, BigDecimal.ROUND_HALF_EVEN);
+        BigDecimal scaledValue = value.setScale(scale, RoundingMode.HALF_EVEN);
         return new BigDecimalValue(scaledValue.stripTrailingZeros());
     }
 
@@ -461,7 +476,7 @@ public final class BigDecimalValue extends DecimalValue {
 
     public boolean isWholeNumber() {
         return value.scale() == 0 ||
-                value.compareTo(value.setScale(0, BigDecimal.ROUND_DOWN)) == 0;
+                value.compareTo(value.setScale(0, RoundingMode.DOWN)) == 0;
     }
 
     /**
@@ -502,7 +517,7 @@ public final class BigDecimalValue extends DecimalValue {
      * Compare the value to another numeric value
      */
 
-    public int compareTo(Object other) {
+    public int compareTo(NumericValue other) {
         if (NumericValue.isInteger((NumericValue) other)) {
             // deliberately triggers a ClassCastException if other value is the wrong type
             try {
@@ -538,7 +553,6 @@ public final class BigDecimalValue extends DecimalValue {
      * Returns null if the value is not comparable according to XML Schema rules. The default implementation
      * returns the value itself if it is comparable, or null otherwise. This is modified for types such as
      * xs:duration which allow ordering comparisons in XML Schema, but not in XPath.
-     * <p/>
      * <p>In the case of data types that are partially ordered, the returned Comparable extends the standard
      * semantics of the compareTo() method by returning the value {@link net.sf.saxon.om.SequenceTool#INDETERMINATE_ORDERING} when there
      * is no defined order relationship between two given values.</p>
@@ -575,7 +589,7 @@ public final class BigDecimalValue extends DecimalValue {
             } else if (o instanceof Int64Value.Int64Comparable) {
                 return asBigDecimal().compareTo(BigDecimal.valueOf(((Int64Value.Int64Comparable) o).asLong()));
             } else if (o instanceof BigIntegerValue.BigIntegerComparable) {
-                return value.compareTo(new BigDecimal(((BigIntegerValue.BigIntegerComparable) o).asBigInteger()));
+                return asBigDecimal().compareTo(new BigDecimal(((BigIntegerValue.BigIntegerComparable) o).asBigInteger()));
             } else {
                 return SequenceTool.INDETERMINATE_ORDERING;
             }
@@ -588,12 +602,9 @@ public final class BigDecimalValue extends DecimalValue {
         public int hashCode() {
             // Must align with hashCodes for other subtypes of xs:decimal
             if (value.isWholeNumber()) {
-                try {
-                    IntegerValue iv = (IntegerValue) Converter.DECIMAL_TO_INTEGER.convert(value).asAtomic();
-                    return iv.getSchemaComparable().hashCode();
-                } catch (ValidationException e) {
-                    return 12345678;    // can't happen
-                }
+                IntegerValue iv = Converter.DecimalToInteger.INSTANCE.convert(value);
+                return iv.getSchemaComparable().hashCode();
+                
             }
             return value.hashCode();
         }
