@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -17,12 +17,10 @@ import net.sf.saxon.tree.iter.ManualIterator;
  * A Closure represents a value that has not yet been evaluated: the value is represented
  * by an expression, together with saved values of all the context variables that the
  * expression depends on.
- * <p/>
  * <p>This Closure is designed for use when the value is only read once. If the value
  * is read more than once, a new iterator over the underlying expression is obtained
  * each time: this may (for example in the case of a filter expression) involve
  * significant re-calculation.</p>
- * <p/>
  * <p>The expression may depend on local variables and on the context item; these values
  * are held in the saved XPathContext object that is kept as part of the Closure, and they
  * will always be read from that object. The expression may also depend on global variables;
@@ -33,7 +31,7 @@ import net.sf.saxon.tree.iter.ManualIterator;
  * of these context variables.</p>
  */
 
-public class Closure implements Sequence, ContextOriginator {
+public class Closure implements Sequence<Item<?>>, ContextOriginator {
 
     protected Expression expression;
     /*@Nullable*/ protected XPathContextMajor savedXPathContext;
@@ -43,7 +41,7 @@ public class Closure implements Sequence, ContextOriginator {
     // to the reservoir. It only ever has one instance (for each Closure) and each
     // item is read only once.
 
-    protected SequenceIterator inputIterator;
+    protected SequenceIterator<?> inputIterator;
 
     /**
      * Constructor should not be called directly, instances should be made using the make() method.
@@ -65,7 +63,8 @@ public class Closure implements Sequence, ContextOriginator {
      */
 
     /*@NotNull*/
-    public static Sequence make(/*@NotNull*/ Expression expression, /*@NotNull*/ XPathContext context, int ref) throws XPathException {
+    public static Sequence<?> make(
+            Expression expression, XPathContext context, int ref) throws XPathException {
         return context.getConfiguration().makeClosure(expression, ref, context);
     }
 
@@ -79,7 +78,7 @@ public class Closure implements Sequence, ContextOriginator {
 
         if ((expression.getDependencies() & StaticProperty.DEPENDS_ON_LOCAL_VARIABLES) != 0) {
             StackFrame localStackFrame = context.getStackFrame();
-            Sequence[] local = localStackFrame.getStackFrameValues();
+            Sequence<?>[] local = localStackFrame.getStackFrameValues();
             int[] slotsUsed = expression.getSlotsUsed();  // computed on first call
             if (local != null) {
                 final SlotManager stackFrameMap = localStackFrame.getStackFrameMap();
@@ -89,7 +88,7 @@ public class Closure implements Sequence, ContextOriginator {
                     if (local[i] instanceof Closure) {
                         int cdepth = ((Closure) local[i]).depth;
                         if (cdepth >= 10) {
-                            local[i] = SequenceExtent.makeSequenceExtent(local[i].iterate());
+                            local[i] = local[i].iterate().materialize();
                         } else if (cdepth + 1 > depth) {
                             depth = cdepth + 1;
                         }
@@ -104,8 +103,8 @@ public class Closure implements Sequence, ContextOriginator {
         // Make a copy of the context item
         FocusIterator currentIterator = context.getCurrentIterator();
         if (currentIterator != null) {
-            Item contextItem = currentIterator.current();
-            ManualIterator single = new ManualIterator(contextItem);
+            Item<?> contextItem = currentIterator.current();
+            ManualIterator<?> single = new ManualIterator<>(contextItem);
             savedXPathContext.setCurrentIterator(single);
             // we don't save position() and last() because we have no way
             // of restoring them. So the caller must ensure that a Closure is not
@@ -124,7 +123,7 @@ public class Closure implements Sequence, ContextOriginator {
      *          in the situation where the sequence is evaluated lazily, and
      *          evaluation of the first item causes a dynamic error.
      */
-    public Item head() throws XPathException {
+    public Item<?> head() throws XPathException {
         return iterate().next();
     }
 
@@ -150,10 +149,11 @@ public class Closure implements Sequence, ContextOriginator {
      */
 
     /*@NotNull*/
-    public SequenceIterator iterate() throws XPathException {
+    public SequenceIterator<Item<?>> iterate() throws XPathException {
 
         if (inputIterator == null) {
-            return inputIterator = expression.iterate(savedXPathContext);
+            inputIterator = expression.iterate(savedXPathContext);
+            return (SequenceIterator<Item<?>>)inputIterator;
         } else {
             // In an ideal world this shouldn't happen: if the value is needed more than once, we should
             // have chosen a MemoClosure.
@@ -176,9 +176,13 @@ public class Closure implements Sequence, ContextOriginator {
      * @throws XPathException if an error occurs doing the lazy evaluation
      */
 
-    public GroundedValue reduce() throws XPathException {
-        return SequenceExtent.makeSequenceExtent(iterate());
+    public GroundedValue<?> reduce() throws XPathException {
+        return iterate().materialize();
     }
 
+    @Override
+    public Sequence<Item<?>> makeRepeatable() throws XPathException {
+        return materialize();
+    }
 }
 
