@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,11 +11,9 @@ import net.sf.saxon.expr.XPathContext;
 import net.sf.saxon.lib.ConversionRules;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.tree.util.FastStringBuffer;
-import net.sf.saxon.type.AtomicType;
-import net.sf.saxon.type.BuiltInAtomicType;
-import net.sf.saxon.type.ConversionResult;
-import net.sf.saxon.type.ValidationException;
+import net.sf.saxon.type.*;
 
+import java.time.format.DateTimeParseException;
 import java.util.Calendar;
 import java.util.GregorianCalendar;
 
@@ -42,7 +40,7 @@ public class DateValue extends GDateValue implements Comparable {
      */
 
     public DateValue(int year, byte month, byte day) {
-        this.xsd10rules = true;
+        this.hasNoYearZero = true;
         this.year = year;
         this.month = month;
         this.day = day;
@@ -61,7 +59,7 @@ public class DateValue extends GDateValue implements Comparable {
      */
 
     public DateValue(int year, byte month, byte day, boolean xsd10) {
-        this.xsd10rules = xsd10;
+        this.hasNoYearZero = xsd10;
         this.year = year;
         this.month = month;
         this.day = day;
@@ -83,7 +81,7 @@ public class DateValue extends GDateValue implements Comparable {
 
     public DateValue(int year, byte month, byte day, int tz, boolean xsd10) {
         // Method is called by generated Java code.
-        this.xsd10rules = xsd10;
+        this.hasNoYearZero = xsd10;
         this.year = year;
         this.month = month;
         this.day = day;
@@ -172,6 +170,27 @@ public class DateValue extends GDateValue implements Comparable {
         return setLexicalValue(d, in, rules.isAllowYearZero());
     }
 
+    /**
+     * Factory method: create an xs:date value from a supplied string, in ISO 8601 format, allowing
+     * a year value of 0 to represent the year before year 1 (that is, following the XSD 1.1 rules).
+     * <p>The {@code hasNoYearZero} property in the result is set to false.</p>
+     *
+     * @param s a string in the lexical space of xs:date
+     * @return a DateValue representing the xs:date supplied, including a timezone offset if
+     * present in the lexical representation
+     * @throws DateTimeParseException if the format of the supplied string is invalid.
+     * @since 9.9
+     */
+
+    public static DateValue parse(CharSequence s) throws DateTimeParseException {
+        ConversionResult result = makeDateValue(s, ConversionRules.DEFAULT);
+        if (result instanceof ValidationFailure) {
+            throw new DateTimeParseException(((ValidationFailure) result).getMessage(), s, 0);
+        } else {
+            return (DateValue) result;
+        }
+    }
+
     /**
      * Determine the primitive type of the value. This delivers the same answer as
      * getItemType().getPrimitiveItemType(). The primitive types are
@@ -236,7 +255,7 @@ public class DateValue extends GDateValue implements Comparable {
         FastStringBuffer sb = new FastStringBuffer(FastStringBuffer.C16);
         int yr = year;
         if (year <= 0) {
-            yr = -yr + (xsd10rules ? 1 : 0);           // no year zero in lexical space for XSD 1.0
+            yr = -yr + (hasNoYearZero ? 1 : 0);           // no year zero in lexical space for XSD 1.0
             if (yr != 0) {
                 sb.append('-');
             }
@@ -285,7 +304,7 @@ public class DateValue extends GDateValue implements Comparable {
 
     /*@NotNull*/
     public AtomicValue copyAsSubType(AtomicType typeLabel) {
-        DateValue v = new DateValue(year, month, day, getTimezoneInMinutes(), xsd10rules);
+        DateValue v = new DateValue(year, month, day, getTimezoneInMinutes(), hasNoYearZero);
         v.typeLabel = typeLabel;
         return v;
     }
@@ -301,7 +320,7 @@ public class DateValue extends GDateValue implements Comparable {
 
     public DateValue adjustTimezone(int timezone) {
         DateTimeValue dt = toDateTime().adjustTimezone(timezone);
-        return new DateValue(dt.getYear(), dt.getMonth(), dt.getDay(), dt.getTimezoneInMinutes(), xsd10rules);
+        return new DateValue(dt.getYear(), dt.getMonth(), dt.getDay(), dt.getTimezoneInMinutes(), hasNoYearZero);
     }
 
     /**
@@ -329,7 +348,7 @@ public class DateValue extends GDateValue implements Comparable {
                 }
             }
             d.setTimezoneInMinutes(getTimezoneInMinutes());
-            d.xsd10rules = this.xsd10rules;
+            d.hasNoYearZero = this.hasNoYearZero;
             return d;
         } else if (duration instanceof YearMonthDurationValue) {
             int months = ((YearMonthDurationValue) duration).getLengthInMonths();
@@ -345,7 +364,7 @@ public class DateValue extends GDateValue implements Comparable {
             while (!isValidDate(y, m, d)) {
                 d -= 1;
             }
-            return new DateValue(y, (byte) m, (byte) d, getTimezoneInMinutes(), xsd10rules);
+            return new DateValue(y, (byte) m, (byte) d, getTimezoneInMinutes(), hasNoYearZero);
         } else {
             XPathException err = new XPathException("Date arithmetic is not available for xs:duration, only for its subtypes");
             err.setIsTypeError(true);
@@ -399,7 +418,6 @@ public class DateValue extends GDateValue implements Comparable {
      * http://vsg.cape.com/~pbaum/date/jdalg.htm and
      * http://vsg.cape.com/~pbaum/date/jdalg2.htm
      * (adjusted to handle BC dates correctly)
-     * <p/>
      * <p>Note that this assumes dates in the proleptic Gregorian calendar</p>
      *
      * @param year  the year
@@ -524,7 +542,6 @@ public class DateValue extends GDateValue implements Comparable {
      * The days of the week are numbered from 1 (Monday) to 7 (Sunday), and week 1
      * in any calendar month is the week (from Monday to Sunday) that includes the first Thursday
      * of that month.
-     * <p/>
      * <p>See bug 21370 which clarified the specification. This caused a change to the Saxon
      * implementation such that the days before the start of week 1 go in the last week of the previous
      * month, not week zero.</p>
