@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -31,7 +31,7 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
     protected boolean negative = false;
     protected int months = 0;
     protected long seconds = 0;
-    protected int microseconds = 0;
+    protected int nanoseconds = 0;
 
     /**
      * Private constructor for internal use
@@ -44,6 +44,8 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
      * Constructor for xs:duration taking the components of the duration. There is no requirement
      * that the values are normalized, for example it is acceptable to specify months=18. The values of
      * the individual components must all be non-negative.
+     * <p>Note: For historic reasons this constructor only supports microsecond precision. To get nanosecond
+     * precision, use the constructor {@link DurationValue#DurationValue(int, int, int, int, int, long, int, AtomicType)}.</p>
      *
      * @param positive     true if the duration is positive, false if negative. For a negative duration
      *                     the components are all supplied as positive integers (or zero).
@@ -70,6 +72,9 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
      * type which must be a subtype of xs:duration. There is no requirement
      * that the values are normalized, for example it is acceptable to specify months=18. The values of
      * the individual components must all be non-negative.
+     * <p>Note: for historic reasons this constructor was written to expect microseconds rather than nanoseconds.
+     * To supply nanoseconds, use the alternative constructor
+     * {@link DurationValue#DurationValue(int, int, int, int, int, long, int, AtomicType)}.</p>
      *
      * @param positive     true if the duration is positive, false if negative. For a negative duration
      *                     the components are all supplied as positive integers (or zero).
@@ -93,28 +98,84 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
         if (years < 0 || months < 0 || days < 0 || hours < 0 || minutes < 0 || seconds < 0 || microseconds < 0) {
             throw new IllegalArgumentException("Negative component value");
         }
-        if (((double) years) * 12 + (double) months > Integer.MAX_VALUE) {
+        if ((double) years * 12 + (double) months > Integer.MAX_VALUE) {
             throw new IllegalArgumentException("Duration months limit exceeded");
         }
-        if (((double) days) * (24 * 60 * 60) + ((double) hours) * (60 * 60) +
-                ((double) minutes) * 60 + (double) seconds > Long.MAX_VALUE) {
+        if ((double) days * (24 * 60 * 60) + (double) hours * (60 * 60) +
+                (double) minutes * 60 + (double) seconds > Long.MAX_VALUE) {
             throw new IllegalArgumentException("Duration seconds limit exceeded");
         }
         this.months = years * 12 + months;
         long h = days * 24L + hours;
         long m = h * 60L + minutes;
         this.seconds = m * 60L + seconds;
-        this.microseconds = microseconds;
+        this.nanoseconds = microseconds * 1000;
         normalizeZeroDuration();
         typeLabel = type;
     }
 
+    /**
+     * Constructor for xs:duration taking the components of the duration, plus a user-specified
+     * type which must be a subtype of xs:duration. There is no requirement
+     * that the values are normalized, for example it is acceptable to specify months=18. The values of
+     * the individual components must all be non-negative.
+     * <p>If the duration is positive, all the components must be supplied as positive (or zero) integers.
+     * If the duration is negative, all the components must be supplied as negative (or zero) integers.</p>
+     *
+     * @param years        the number of years
+     * @param months       the number of months
+     * @param days         the number of days
+     * @param hours        the number of hours
+     * @param minutes      the number of minutes
+     * @param seconds      the number of seconds (long to allow copying)
+     * @param nanoseconds  the number of nanoseconds
+     * @param type         the user-defined subtype of xs:duration. Note that this constructor cannot
+     *                     be used to create an instance of xs:dayTimeDuration or xs:yearMonthDuration.
+     * @throws IllegalArgumentException if the size of the duration exceeds implementation-defined
+     *                                  limits: specifically, if the total number of months exceeds 2^31, or if the total number
+     *                                  of seconds exceeds 2^63.
+     */
+
+    public DurationValue(int years, int months, int days,
+                         int hours, int minutes, long seconds, int nanoseconds, AtomicType type) {
+        boolean somePositive = years > 0 || months > 0 || days > 0 || hours > 0 || minutes > 0 || seconds > 0 || nanoseconds > 0;
+        boolean someNegative = years < 0 || months < 0 || days < 0 || hours < 0 || minutes < 0 || seconds < 0 || nanoseconds < 0;
+        if (somePositive && someNegative) {
+            throw new IllegalArgumentException("Some component values are positive and some negative");
+        }
+        if (someNegative) {
+            years = -years;
+            months = -months;
+            days = -days;
+            hours = -hours;
+            minutes = -minutes;
+            seconds = -seconds;
+            nanoseconds = -nanoseconds;
+        }
+        if ((double) years * 12 + (double) months > Integer.MAX_VALUE) {
+            throw new IllegalArgumentException("Duration months limit exceeded");
+        }
+        if ((double) days * (24 * 60 * 60) + (double) hours * (60 * 60) +
+                (double) minutes * 60 + (double) seconds > Long.MAX_VALUE) {
+            throw new IllegalArgumentException("Duration seconds limit exceeded");
+        }
+        this.months = years * 12 + months;
+        long h = days * 24L + hours;
+        long m = h * 60L + minutes;
+        this.seconds = m * 60L + seconds;
+        this.nanoseconds = nanoseconds;
+        negative = someNegative;
+        normalizeZeroDuration();
+        typeLabel = type;
+    }
+
+
     /**
      * Ensure that a zero duration is considered positive
      */
 
     protected void normalizeZeroDuration() {
-        if (months == 0 && seconds == 0L && microseconds == 0) {
+        if (months == 0 && seconds == 0L && nanoseconds == 0) {
             negative = false;
         }
     }
@@ -135,7 +196,7 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
 
     /*@NotNull*/
     protected static ConversionResult makeDuration(CharSequence s, boolean allowYM, boolean allowDT) {
-        int years = 0, months = 0, days = 0, hours = 0, minutes = 0, seconds = 0, microseconds = 0;
+        int years = 0, months = 0, days = 0, hours = 0, minutes = 0, seconds = 0, nanoseconds = 0;
         boolean negative = false;
         StringTokenizer tok = new StringTokenizer(Whitespace.trimWhitespace(s).toString(), "-+.PYMDTHS", true);
         int components = 0;
@@ -243,17 +304,19 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
                         return badDuration("Seconds component is not allowed in yearMonthDuration", s);
                     }
                     if (state == 7) {
-                        while (part.length() < 6) {
-                            part += "0";
+                        StringBuilder frac = new StringBuilder(part);
+                        while (frac.length() < 9) {
+                            frac.append("0");
                         }
-                        if (part.length() > 6) {
-                            part = part.substring(0, 6);
+                        part = frac.toString();
+                        if (part.length() > 9) {
+                            part = part.substring(0, 9);
                         }
                         value = simpleInteger(part);
                         if (value < 0) {
                             return badDuration("non-numeric fractional seconds", s);
                         }
-                        microseconds = value;
+                        nanoseconds = value;
                     } else {
                         seconds = value;
                     }
@@ -269,9 +332,19 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
             return badDuration("Duration specifies no components", s);
         }
 
+        if (negative) {
+            years = -years;
+            months = -months;
+            days = -days;
+            hours = -hours;
+            minutes = -minutes;
+            seconds = -seconds;
+            nanoseconds = -nanoseconds;
+        }
+
         try {
             return new DurationValue(
-                    !negative, years, months, days, hours, minutes, seconds, microseconds, BuiltInAtomicType.DURATION);
+                    years, months, days, hours, minutes, seconds, nanoseconds, BuiltInAtomicType.DURATION);
         } catch (IllegalArgumentException err) {
             // catch values that exceed limits
             return new ValidationFailure(err.getMessage());
@@ -325,7 +398,11 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
      */
 
     public AtomicValue copyAsSubType(AtomicType typeLabel) {
-        return new DurationValue(!negative, 0, months, 0, 0, 0, seconds, microseconds, typeLabel);
+        if (negative) {
+            return new DurationValue(0, -months, 0, 0, 0, -seconds, -nanoseconds, typeLabel);
+        } else {
+            return new DurationValue(0, months, 0, 0, 0, seconds, nanoseconds, typeLabel);
+        }
     }
 
     /**
@@ -349,7 +426,7 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
         if (negative) {
             return -1;
         }
-        if (months == 0 && seconds == 0L && microseconds == 0) {
+        if (months == 0 && seconds == 0L && nanoseconds == 0) {
             return 0;
         }
         return +1;
@@ -423,11 +500,22 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
     /**
      * Get the microseconds component
      *
-     * @return the number of microseconds in the normalized duration; always positive, in the range 0-999999
+     * @return the number of nanoseconds in the normalized duration, divided by one thousand and rounded down;
+     * always positive, in the range 0-999999
      */
 
     public int getMicroseconds() {
-        return microseconds;
+        return nanoseconds / 1000;
+    }
+
+    /**
+     * Get the nanoseconds component
+     *
+     * @return the number of nanoseconds in the normalized duration; always positive, in the range 0-999,999,999
+     */
+
+    public int getNanoseconds() {
+        return nanoseconds;
     }
 
     /**
@@ -438,36 +526,25 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
      */
 
     public int getTotalMonths() {
-        return (negative ? -months : months);
+        return negative ? -months : months;
     }
 
     /**
      * Get the total number of seconds (ignoring the years/months)
      *
      * @return the total number of seconds, as a positive
-     *         or negative number according as the duration is positive or negative
+     *         or negative number according as the duration is positive or negative,
+     *         with the fractional part indicating parts of a second to nanosecond precision
      */
 
     public BigDecimal getTotalSeconds() {
         BigDecimal dec = new BigDecimal(negative ? -seconds : seconds);
-        if (microseconds != 0) {
-            dec = dec.add(new BigDecimal(BigInteger.valueOf(negative ? -microseconds : microseconds), -6));
+        if (nanoseconds != 0) {
+            dec = dec.add(new BigDecimal(BigInteger.valueOf(negative ? -nanoseconds : nanoseconds), 9));
         }
         return dec;
     }
 
-
-    /**
-     * Convert the value to a string, using the serialization rules.
-     * For atomic values this is the same as a cast; for sequence values
-     * it gives a space-separated list. This method is refined for AtomicValues
-     * so that it never throws an Exception.
-     */
-
-//    public String getStringValue() {
-//        return getStringValueCS().toString();
-//    }
-
     /**
      * Convert to string
      *
@@ -479,7 +556,7 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
         // Note, Schema does not define a canonical representation. We omit all zero components, unless
         // the duration is zero-length, in which case we output PT0S.
 
-        if (months == 0 && seconds == 0L && microseconds == 0) {
+        if (months == 0 && seconds == 0L && nanoseconds == 0) {
             return "PT0S";
         }
 
@@ -504,7 +581,7 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
         if (days != 0) {
             sb.append(days + "D");
         }
-        if (hours != 0 || minutes != 0 || seconds != 0 || microseconds != 0) {
+        if (hours != 0 || minutes != 0 || seconds != 0 || nanoseconds != 0) {
             sb.append("T");
         }
         if (hours != 0) {
@@ -513,23 +590,23 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
         if (minutes != 0) {
             sb.append(minutes + "M");
         }
-        if (seconds != 0 || microseconds != 0) {
-            if (seconds != 0 && microseconds == 0) {
+        if (seconds != 0 || nanoseconds != 0) {
+            if (seconds != 0 && nanoseconds == 0) {
                 sb.append(seconds + "S");
             } else {
-                long ms = (seconds * 1000000) + microseconds;
+                long ms = (seconds * 1_000_000_000L) + nanoseconds;
                 String mss = ms + "";
                 if (seconds == 0) {
-                    mss = "0000000" + mss;
-                    mss = mss.substring(mss.length() - 7);
+                    mss = "0000000000" + mss;
+                    mss = mss.substring(mss.length() - 10);
                 }
-                sb.append(mss.substring(0, mss.length() - 6));
+                sb.append(mss.substring(0, mss.length() - 9));
                 sb.append('.');
                 int lastSigDigit = mss.length() - 1;
                 while (mss.charAt(lastSigDigit) == '0') {
                     lastSigDigit--;
                 }
-                sb.append(mss.substring(mss.length() - 6, lastSigDigit + 1));
+                sb.append(mss.substring(mss.length() - 9, lastSigDigit + 1));
                 sb.append('S');
             }
         }
@@ -548,37 +625,16 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
      */
 
     public double getLengthInSeconds() {
-        double a = months * (365.242199 / 12.0) * 24 * 60 * 60 + seconds + ((double) microseconds / 1000000);
-        return (negative ? -a : a);
+        double a = months * (365.242199 / 12.0) * 24 * 60 * 60 + seconds + ((double) nanoseconds / 1_000_000_000);
+        return negative ? -a : a;
     }
 
-    /**
-     * Convert to Java object (for passing to external functions)
-     */
-
-//    public Object convertAtomicToJava(Class target, XPathContext context) throws XPathException {
-//        if (target.isAssignableFrom(DurationValue.class)) {
-//            return this;
-//        } else if (target == Object.class) {
-//            return getStringValue();
-//        } else {
-//            Object o = super.convertSequenceToJava(target, context);
-//            if (o == null) {
-//                XPathException err = new XPathException("Conversion of xs:duration to " + target.getName() +
-//                        " is not supported");
-//                err.setXPathContext(context);
-//                err.setErrorCode(SaxonErrorCode.SXJE0003);
-//            }
-//            return o;
-//        }
-//    }
-
     /**
      * Get a component of the normalized value
      * @param component the required component
      */
 
-    public AtomicValue getComponent(AccessorFn.Component component) throws XPathException {
+    public AtomicValue getComponent(AccessorFn.Component component) {
         switch (component) {
             case YEAR:
                 return Int64Value.makeIntegerValue(negative ? -getYears() : getYears());
@@ -592,14 +648,16 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
                 return Int64Value.makeIntegerValue(negative ? -getMinutes() : getMinutes());
             case SECONDS:
                 FastStringBuffer sb = new FastStringBuffer(FastStringBuffer.C16);
-                String ms = "000000" + microseconds;
-                ms = ms.substring(ms.length() - 6);
+                String ms = "000000000" + nanoseconds;
+                ms = ms.substring(ms.length() - 9);
                 sb.append((negative ? "-" : "") + getSeconds() + '.' + ms);
-                return (AtomicValue) BigDecimalValue.makeDecimalValue(sb, false);
+                return BigDecimalValue.parse(sb);
             case WHOLE_SECONDS:
                 return Int64Value.makeIntegerValue(negative ? -seconds : seconds);
             case MICROSECONDS:
-                return new Int64Value(negative ? -microseconds : microseconds);
+                return new Int64Value((negative ? -nanoseconds : nanoseconds) / 1000);
+            case NANOSECONDS:
+                return new Int64Value(negative ? -nanoseconds : nanoseconds);
             default:
                 throw new IllegalArgumentException("Unknown component for duration: " + component);
         }
@@ -641,14 +699,14 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
             return d1.negative == d2.negative &&
                     d1.months == d2.months &&
                     d1.seconds == d2.seconds &&
-                    d1.microseconds == d2.microseconds;
+                    d1.nanoseconds == d2.nanoseconds;
         } else {
             return false;
         }
     }
 
     public int hashCode() {
-        return new Double(getLengthInSeconds()).hashCode();
+        return Double.valueOf(getLengthInSeconds()).hashCode();
     }
 
     /**
@@ -686,9 +744,25 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
      */
 
     public DurationValue negate() {
-        return new DurationValue(negative, 0, months, 0, 0, 0, seconds, microseconds, typeLabel);
+        if (negative) {
+            return new DurationValue(0, months, 0, 0, 0, seconds, nanoseconds, typeLabel);
+        } else {
+            return new DurationValue(0, -months, 0, 0, 0, -seconds, -nanoseconds, typeLabel);
+        }
     }
 
+    /**
+     * Multiply a duration by an integer
+     *
+     * @param factor the number to multiply by
+     * @return the result of the multiplication
+     */
+
+    public DurationValue multiply(long factor) throws XPathException {
+        return multiply((double)factor);
+    }
+
+
     /**
      * Multiply a duration by a number
      *
@@ -703,6 +777,20 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
         throw err;
     }
 
+    /**
+     * Divide a duration by a number
+     *
+     * @param factor the number to divide by
+     * @return the result of the division
+     */
+
+    public DurationValue divide(double factor) throws XPathException {
+        XPathException err = new XPathException("Only subtypes of xs:duration can be divided by a number");
+        err.setErrorCode("XPTY0004");
+        err.setIsTypeError(true);
+        throw err;
+    }
+
     /**
      * Divide a duration by a another duration
      *
@@ -741,12 +829,14 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
     /*@NotNull*/
     public static Comparable getSchemaComparable(/*@NotNull*/ DurationValue value) {
         int m = value.months;
-        double s = value.seconds + ((double) value.microseconds / 1000000);
+        long s = value.seconds;
+        int n = value.nanoseconds;
         if (value.negative) {
             s = -s;
             m = -m;
+            n = -n;
         }
-        return new DurationComparable(m, s);
+        return new DurationComparable(m, s, n);
     }
 
     /**
@@ -754,40 +844,46 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
      * having ordering rules that implement the XML Schema specification.
      */
 
-    private static class DurationComparable implements Comparable {
+    private static class DurationComparable implements Comparable<DurationComparable> {
 
         private int months;
-        private double seconds;
+        private long seconds;
+        private int nanoseconds;
 
-        public DurationComparable(int m, double s) {
+        public DurationComparable(int m, long s, int nanos) {
             months = m;
             seconds = s;
+            nanoseconds = nanos;
         }
 
         /**
          * Compare two durations according to the XML Schema rules.
          *
-         * @param o the other duration
+         * @param other the other duration
          * @return -1 if this duration is smaller; 0 if they are equal; +1 if this duration is greater;
          *         {@link net.sf.saxon.om.SequenceTool#INDETERMINATE_ORDERING} if there is no defined order
          */
 
-        public int compareTo(Object o) {
-            DurationComparable other;
-            if (o instanceof DurationComparable) {
-                other = (DurationComparable) o;
-            } else if (o instanceof YearMonthDurationValue) {
-                other = (DurationComparable) getSchemaComparable((YearMonthDurationValue) o);
-            } else if (o instanceof DayTimeDurationValue) {
-                other = (DurationComparable) getSchemaComparable((DayTimeDurationValue) o);
-            } else {
-                return SequenceTool.INDETERMINATE_ORDERING;
-            }
+        public int compareTo(DurationComparable other) {
+            //DurationComparable other = ;
+//            if (o instanceof DurationComparable) {
+//                other = (DurationComparable) o;
+//            } else if (o instanceof YearMonthDurationValue) {
+//                other = (DurationComparable) getSchemaComparable((YearMonthDurationValue) o);
+//            } else if (o instanceof DayTimeDurationValue) {
+//                other = (DurationComparable) getSchemaComparable((DayTimeDurationValue) o);
+//            } else {
+//                return SequenceTool.INDETERMINATE_ORDERING;
+//            }
             if (months == other.months) {
-                return Double.compare(seconds, other.seconds);
-            } else if (seconds == other.seconds) {
-                return (months == other.months ? 0 : (months < other.months ? -1 : +1));
+                if (seconds == other.seconds) {
+                    return Integer.compare(nanoseconds, other.nanoseconds);
+                } else {
+                    return Long.compare(seconds, other.seconds);
+                }
             } else {
+                // The months figure varies, but the seconds figure might tip things over if it's high
+                // enough. We make the assumption, however, that the nanoseconds won't affect things.
                 double oneDay = 24e0 * 60e0 * 60e0;
                 double min0 = monthsToDaysMinimum(months) * oneDay + seconds;
                 double max0 = monthsToDaysMaximum(months) * oneDay + seconds;
@@ -804,7 +900,7 @@ public class DurationValue extends AtomicValue implements AtomicMatchKey {
         }
 
         public boolean equals(Object o) {
-            return compareTo(o) == 0;
+            return o instanceof DurationComparable && compareTo((DurationComparable)o) == 0;
         }
 
         public int hashCode() {
