@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -20,6 +20,7 @@ import net.sf.saxon.type.ConversionResult;
 import net.sf.saxon.type.ValidationFailure;
 
 import java.math.BigDecimal;
+import java.math.RoundingMode;
 import java.util.*;
 
 /**
@@ -31,7 +32,7 @@ public final class TimeValue extends CalendarValue implements Comparable {
     private byte hour;
     private byte minute;
     private byte second;
-    private int microsecond;
+    private int nanosecond;
 
     private TimeValue() {
     }
@@ -52,11 +53,53 @@ public final class TimeValue extends CalendarValue implements Comparable {
         this.hour = hour;
         this.minute = minute;
         this.second = second;
-        this.microsecond = microsecond;
+        this.nanosecond = microsecond * 1000;
         setTimezoneInMinutes(tz);
         typeLabel = BuiltInAtomicType.TIME;
     }
 
+    /**
+     * Private constructor to construct a time value given the hour, minute, second, and nanosecond components.
+     * This constructor performs no validation.
+     *
+     * @param hour        the hour value, 0-23
+     * @param minute      the minutes value, 0-59
+     * @param second      the seconds value, 0-59
+     * @param nanosecond  the number of microseconds, 0-999999
+     * @param tz          the timezone displacement in minutes from UTC. Supply the value
+     *                    {@link CalendarValue#NO_TIMEZONE} if there is no timezone component.
+     * @param flag        used to disambiguate this constructor. Must be set to "".
+     */
+
+    public TimeValue(byte hour, byte minute, byte second, int nanosecond, int tz, String flag) {
+        if (!flag.isEmpty()) {
+            throw new IllegalArgumentException();
+        }
+        this.hour = hour;
+        this.minute = minute;
+        this.second = second;
+        this.nanosecond = nanosecond;
+        setTimezoneInMinutes(tz);
+        typeLabel = BuiltInAtomicType.TIME;
+    }
+
+    /**
+     * Factory method to construct a time value given the hour, minute, second, and nanosecond components.
+     * This constructor performs no validation.
+     *
+     * @param hour        the hour value, 0-23
+     * @param minute      the minutes value, 0-59
+     * @param second      the seconds value, 0-59
+     * @param nanosecond  the number of nanoseconds, 0-999_999_999
+     * @param tz          the timezone displacement in minutes from UTC. Supply the value
+     *                    {@link CalendarValue#NO_TIMEZONE} if there is no timezone component.
+     */
+
+    public TimeValue makeTimeValue(byte hour, byte minute, byte second, int nanosecond, int tz) {
+        return new TimeValue(hour, minute, second, nanosecond, tz, "");
+    }
+
+
     /**
      * Constructor: create a time value given a Java calendar object
      *
@@ -65,10 +108,10 @@ public final class TimeValue extends CalendarValue implements Comparable {
      */
 
     public TimeValue(/*@NotNull*/ GregorianCalendar calendar, int tz) {
-        hour = (byte) (calendar.get(Calendar.HOUR_OF_DAY));
-        minute = (byte) (calendar.get(Calendar.MINUTE));
-        second = (byte) (calendar.get(Calendar.SECOND));
-        microsecond = calendar.get(Calendar.MILLISECOND) * 1000;
+        hour = (byte) calendar.get(Calendar.HOUR_OF_DAY);
+        minute = (byte) calendar.get(Calendar.MINUTE);
+        second = (byte) calendar.get(Calendar.SECOND);
+        nanosecond = calendar.get(Calendar.MILLISECOND) * 1_000_000;
         setTimezoneInMinutes(tz);
         typeLabel = BuiltInAtomicType.TIME;
     }
@@ -179,8 +222,8 @@ public final class TimeValue extends CalendarValue implements Comparable {
                     return badTime("Non-numeric fractional seconds component", s);
                 }
                 double fractionalSeconds = Double.parseDouble('.' + part);
-                tv.microsecond = (int) (Math.round(fractionalSeconds * 1000000));
-                if (tv.hour == 24 && tv.microsecond != 0) {
+                tv.nanosecond = (int) Math.round(fractionalSeconds * 1_000_000_000);
+                if (tv.hour == 24 && tv.nanosecond != 0) {
                     return badTime("If hour is 24, fractional seconds must be 0", s);
                 }
                 state = 1;
@@ -305,13 +348,23 @@ public final class TimeValue extends CalendarValue implements Comparable {
     }
 
     /**
-     * Get the microsecond component, 0-999999
+     * Get the microsecond component, 0-999_999
      *
-     * @return the microseconds
+     * @return the nanoseconds component divided by 1000, rounded towards zero
      */
 
     public int getMicrosecond() {
-        return microsecond;
+        return nanosecond / 1000;
+    }
+
+    /**
+     * Get the nanosecond component, 0-999_999
+     *
+     * @return the nanoseconds
+     */
+
+    public int getNanosecond() {
+        return nanosecond;
     }
 
 
@@ -332,10 +385,10 @@ public final class TimeValue extends CalendarValue implements Comparable {
         appendTwoDigits(sb, minute);
         sb.append(':');
         appendTwoDigits(sb, second);
-        if (microsecond != 0) {
+        if (nanosecond != 0) {
             sb.append('.');
-            int ms = microsecond;
-            int div = 100000;
+            int ms = nanosecond;
+            int div = 100_000_000;
             while (ms > 0) {
                 int d = ms / div;
                 sb.append((char) (d + '0'));
@@ -377,7 +430,7 @@ public final class TimeValue extends CalendarValue implements Comparable {
 
     /*@NotNull*/
     public DateTimeValue toDateTime() {
-        return new DateTimeValue(1972, (byte) 12, (byte) 31, hour, minute, second, microsecond, getTimezoneInMinutes(), true);
+        return new DateTimeValue(1972, (byte) 12, (byte) 31, hour, minute, second, nanosecond, getTimezoneInMinutes());
     }
 
     /**
@@ -387,20 +440,18 @@ public final class TimeValue extends CalendarValue implements Comparable {
     /*@NotNull*/
     public GregorianCalendar getCalendar() {
         // create a calendar using the specified timezone
-        int tz = (hasTimezone() ? getTimezoneInMinutes() : 0);
-        TimeZone zone = new SimpleTimeZone(tz * 60000, "LLL");
+        int tz = hasTimezone() ? getTimezoneInMinutes()*60000 : 0;
+        TimeZone zone = new SimpleTimeZone(tz, "LLL");
         GregorianCalendar calendar = new GregorianCalendar(zone);
         calendar.setLenient(false);
+        if (tz < calendar.getMinimum(Calendar.ZONE_OFFSET) || tz > calendar.getMaximum(Calendar.ZONE_OFFSET)) {
+            return adjustTimezone(0).getCalendar();
+        }
 
         // use a reference date of 1972-12-31
-        int year = 1972;
-        int month = 11;
-        int day = 31;
-
-
-        calendar.set(year, month, day, hour, minute, second);
-        calendar.set(Calendar.MILLISECOND, microsecond / 1000);
-        calendar.set(Calendar.ZONE_OFFSET, tz * 60000);
+        calendar.set(1972, Calendar.DECEMBER, 31, hour, minute, second);
+        calendar.set(Calendar.MILLISECOND, nanosecond / 1_000_000);
+        calendar.set(Calendar.ZONE_OFFSET, tz);
         calendar.set(Calendar.DST_OFFSET, 0);
 
         calendar.getTime();
@@ -416,7 +467,7 @@ public final class TimeValue extends CalendarValue implements Comparable {
 
     /*@NotNull*/
     public AtomicValue copyAsSubType(AtomicType typeLabel) {
-        TimeValue v = new TimeValue(hour, minute, second, microsecond, getTimezoneInMinutes());
+        TimeValue v = new TimeValue(hour, minute, second, nanosecond, getTimezoneInMinutes());
         v.typeLabel = typeLabel;
         return v;
     }
@@ -437,29 +488,6 @@ public final class TimeValue extends CalendarValue implements Comparable {
                 dt.getMicrosecond(), dt.getTimezoneInMinutes());
     }
 
-
-    /**
-     * Convert to Java object (for passing to external functions)
-     */
-
-//    public Object convertAtomicToJava(Class target, XPathContext context) throws XPathException {
-//        if (target.isAssignableFrom(TimeValue.class)) {
-//            return this;
-//        } else if (target == String.class) {
-//            return getStringValue();
-//        } else if (target == Object.class) {
-//            return getStringValue();
-//        } else {
-//            Object o = super.convertSequenceToJava(target, context);
-//            if (o == null) {
-//                throw new XPathException("Conversion of time to " + target.getName() +
-//                        " is not supported");
-//            }
-//            return o;
-//        }
-//    }
-//
-
     /**
      * Get a component of the value. Returns null if the timezone component is
      * requested and is not present.
@@ -474,14 +502,16 @@ public final class TimeValue extends CalendarValue implements Comparable {
             case MINUTES:
                 return Int64Value.makeIntegerValue(minute);
             case SECONDS:
-                BigDecimal d = BigDecimal.valueOf(microsecond);
-                d = d.divide(BigDecimalValue.BIG_DECIMAL_ONE_MILLION, 6, BigDecimal.ROUND_HALF_UP);
+                BigDecimal d = BigDecimal.valueOf(nanosecond);
+                d = d.divide(BigDecimalValue.BIG_DECIMAL_ONE_BILLION, 6, RoundingMode.HALF_UP);
                 d = d.add(BigDecimal.valueOf(second));
                 return new BigDecimalValue(d);
             case WHOLE_SECONDS: //(internal use only)
                 return Int64Value.makeIntegerValue(second);
             case MICROSECONDS:
-                return new Int64Value(microsecond);
+                return new Int64Value(nanosecond / 1000);
+            case NANOSECONDS:
+                return new Int64Value(nanosecond);
             case TIMEZONE:
                 if (hasTimezone()) {
                     return DayTimeDurationValue.fromMilliseconds(60000L * getTimezoneInMinutes());
@@ -493,20 +523,6 @@ public final class TimeValue extends CalendarValue implements Comparable {
         }
     }
 
-    /**
-     * Get a Comparable value that implements the XPath ordering comparison semantics for this value.
-     * Returns null if the value is not comparable according to XPath rules. The default implementation
-     * returns null. This is overridden for types that allow ordered comparisons in XPath: numeric, boolean,
-     * string, date, time, dateTime, yearMonthDuration, dayTimeDuration, and anyURI.
-     * @param ordered true if an ordered comparison is required
-     * @param collator collation to be used for strings
-     * @param context XPath dynamic evaluation context
-     */
-
-//    public Object getXPathComparable(boolean ordered, StringCollator collator, XPathContext context) {
-//        return this;
-//    }
-
     /**
      * Compare the value to another dateTime value
      *
@@ -528,8 +544,8 @@ public final class TimeValue extends CalendarValue implements Comparable {
                 return IntegerValue.signum(minute - otherTime.minute);
             } else if (second != otherTime.second) {
                 return IntegerValue.signum(second - otherTime.second);
-            } else if (microsecond != otherTime.microsecond) {
-                return IntegerValue.signum(microsecond - otherTime.microsecond);
+            } else if (nanosecond != otherTime.nanosecond) {
+                return IntegerValue.signum(nanosecond - otherTime.nanosecond);
             } else {
                 return 0;
             }
@@ -605,7 +621,7 @@ public final class TimeValue extends CalendarValue implements Comparable {
 
     public int hashCode() {
         return DateTimeValue.hashCode(
-                1951, (byte) 10, (byte) 11, hour, minute, second, microsecond, getTimezoneInMinutes());
+                1951, (byte) 10, (byte) 11, hour, minute, second, nanosecond, getTimezoneInMinutes());
     }
 
     /**
