@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2017 Saxonica Limited.
+// Copyright (c) 2018 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -214,7 +214,7 @@ public class Whitespace {
      * @return true if the character is a whitespace character
      */
 
-    public final static boolean isWhite(char c) {
+    public static boolean isWhite(char c) {
         return c <= 32 && C0WHITE[c];
     }
 
@@ -351,7 +351,7 @@ public class Whitespace {
      * An iterator that splits a string on whitespace boundaries, corresponding to the XPath 3.1 function tokenize#1
      */
 
-    public static class Tokenizer implements UnfailingIterator {
+    public static class Tokenizer implements UnfailingIterator<StringValue> {
 
         private char[] input;
         private int position;
